/*
 Licensed to the Apache Software Foundation (ASF) under one
 or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
 regarding copyright ownership.  The ASF licenses this file
 to you under the Apache License, Version 2.0 (the
 "License"); you may not use this file except in compliance
 with the License.  You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing,
 software distributed under the License is distributed on an
 "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
 */

#import "AppDelegate.h"
#import "MainViewController.h"

#import <Cordova/CDVPlugin.h>

@implementation AppDelegate

@synthesize window, viewController;
@synthesize managedObjectContext = _managedObjectContext;
@synthesize managedObjectModel = _managedObjectModel;
@synthesize persistentStoreCoordinator = _persistentStoreCoordinator;

-(id)init {
    /** If you need to do any extra app-specific initialization, you can do it here
     *  -jm
     **/
    NSHTTPCookieStorage* cookieStorage = [NSHTTPCookieStorage sharedHTTPCookieStorage];

    [cookieStorage setCookieAcceptPolicy:NSHTTPCookieAcceptPolicyAlways];

    int cacheSizeMemory = 8 * 1024 * 1024; // 8MB
    int cacheSizeDisk = 32 * 1024 * 1024; // 32MB
#if __has_feature(objc_arc)
        NSURLCache *sharedCache = [[NSURLCache alloc] initWithMemoryCapacity:cacheSizeMemory diskCapacity:cacheSizeDisk diskPath:@"nsurlcache"];
#else
        NSURLCache *sharedCache = [[[NSURLCache alloc] initWithMemoryCapacity:cacheSizeMemory diskCapacity:cacheSizeDisk diskPath:@"nsurlcache"] autorelease];
#endif
    [NSURLCache setSharedURLCache:sharedCache];

    self = [super init];
    return self;
}

#pragma mark UIApplicationDelegate implementation

/**
 * This is main kick off after the app inits, the views and Settings are setup here.
 */
-(BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    CGRect screenBounds = [[UIScreen mainScreen] bounds];
    
#if __has_feature(objc_arc)
        self.window = [[UIWindow alloc] initWithFrame:screenBounds];
#else
        self.window = [[[UIWindow alloc] initWithFrame:screenBounds] autorelease];
#endif
    self.window.autoresizesSubviews = YES;

#if __has_feature(objc_arc)
        self.viewController = [[MainViewController alloc] init];
#else
        self.viewController = [[[MainViewController alloc] init] autorelease];
#endif

    // Set your app's start page by setting the <content src='foo.html' /> tag in config.xml.
    // If necessary, uncomment the line below to override it.
    // self.viewController.startPage = @"index.html";

    // NOTE: To customize the view's frame size (which defaults to full screen), override
    // [self.viewController viewWillAppear:] in your view controller.

    self.window.rootViewController = self.viewController;
    [self.window makeKeyAndVisible];

    return YES;
}

//Not needed for Release 1.0

//// this happens while we are running ( in the background, or from within our own app )
//// only valid if mHOMES-Info.plist specifies a protocol to handle
//-(BOOL)application:(UIApplication *)application handleOpenURL:(NSURL *)url {
//    
//    if (!url) {
//        return NO;
//    }
//    NSLog(@"parameters from URL %@", url);
//    NSString *redirectQuery = [url query];
//    NSString *token = nil;
//    NSString *where = nil;
//    
//    if (redirectQuery != nil) {
//        NSArray *parameters = [redirectQuery componentsSeparatedByString:@"?"];
//        
//        where = [[parameters objectAtIndex:0] substringFromIndex:[[parameters objectAtIndex:1] rangeOfString:@"="].location + 1];
//        token = [[parameters objectAtIndex:1] substringFromIndex:[[parameters objectAtIndex:0] rangeOfString:@"="].location + 1];
//    }
//    
//    NSLog(@"token %@. where=%@", token, where);
//    
////        NSString* jsStringA = [NSString stringWithFormat:@"alert('url:%@ token:%@ where:%@');", url, token, where];
////        [self.viewController.webView stringByEvaluatingJavaScriptFromString:jsStringA];
//        
//        NSString* jsString = [NSString stringWithFormat:@"window.sessionStorage['token']=\"%@\";", token];
//        [self.viewController.webView stringByEvaluatingJavaScriptFromString:jsString];
//        
////        jsString = [NSString stringWithFormat:@"localStorage.setItem('tokenA',\"%@\");", token];
////        [self.viewController.webView stringByEvaluatingJavaScriptFromString:jsString];
//
//        jsString = [NSString stringWithFormat:@"$('#move-btn').click();"];
//        [self.viewController.webView stringByEvaluatingJavaScriptFromString:jsString];
//        
////        jsString = [NSString stringWithFormat:@"alert('tokenA=' + localStorage.getItem('tokenA'));"];
////        [self.viewController.webView stringByEvaluatingJavaScriptFromString:jsString];
//        
//        //         jsString = [NSString stringWithFormat:@"window.location=\"index.html\";"];
////        [self.viewController.webView stringByEvaluatingJavaScriptFromString:jsString];
//        
////        jsString = [NSString stringWithFormat:@"window.location.reload();"];
////        [self.viewController.webView stringByEvaluatingJavaScriptFromString:jsString];
//
////        jsString = [NSString stringWithFormat:@"window.history.back();"];
////        [self.viewController.webView stringByEvaluatingJavaScriptFromString:jsString];
//        
////        jsString = [NSString stringWithFormat:@"window.location.replace(\"%@\");", where];
////        [self.viewController.webView stringByEvaluatingJavaScriptFromString:jsString];
////
////        jsString = [NSString stringWithFormat:@"alert('hi');"];
////        [self.viewController.webView stringByEvaluatingJavaScriptFromString:jsString];
//
////        jsString = [NSString stringWithFormat:@"window.location.reload();"];
////        [self.viewController.webView stringByEvaluatingJavaScriptFromString:jsString];
//       
////        NSString* jsString = [NSString stringWithFormat:@"app.state = %@;", token];
////        [self.viewController.webView stringByEvaluatingJavaScriptFromString:jsString];
////        jsString = [NSString stringWithFormat:@"alert(app.getState());"];
////        [self.viewController.webView stringByEvaluatingJavaScriptFromString:jsString];
//
//    
//    // calls into javascript global function 'handleOpenURL'
////    NSString* jsString = [NSString stringWithFormat:@"handleOpenURL(\"%@\");", url];
////    [self.viewController.webView stringByEvaluatingJavaScriptFromString:jsString];
//
//    // all plugins will get the notification, and their handlers will be called
//    [[NSNotificationCenter defaultCenter] postNotification:[NSNotification notificationWithName:CDVPluginHandleOpenURLNotification object:url]];
//
//    return YES;
//}


// repost the localnotification using the default NSNotificationCenter so multiple plugins may respond
-(void)application:(UIApplication *)application didReceiveLocalNotification:(UILocalNotification *)notification {
    // re-post ( broadcast )
    [[NSNotificationCenter defaultCenter] postNotificationName:CDVLocalNotification object:notification];
}

-(NSUInteger)application:(UIApplication *)application supportedInterfaceOrientationsForWindow:(UIWindow *)window {
    // iPhone doesn't support upside down by default, while the iPad does.  Override to allow all orientations always, and let the root view controller decide what's allowed (the supported orientations mask gets intersected).
    NSUInteger supportedInterfaceOrientations = (1 << UIInterfaceOrientationPortrait) | (1 << UIInterfaceOrientationLandscapeLeft) | (1 << UIInterfaceOrientationLandscapeRight) | (1 << UIInterfaceOrientationPortraitUpsideDown);

    return supportedInterfaceOrientations;
}

-(void)applicationDidReceiveMemoryWarning:(UIApplication*)application {
    [[NSURLCache sharedURLCache] removeAllCachedResponses];
}




#pragma mark - Core Data stack
-(void)saveContext {

    NSError *error = nil;
    NSManagedObjectContext *managedObjectContext = self.managedObjectContext;
    if (managedObjectContext != nil) {
        if ([managedObjectContext hasChanges] && ![managedObjectContext save:&error]) {
            // Replace this implementation with code to handle the error appropriately.
            // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
            NSLog(@"Unresolved error while saving Context %@, %@", error, [error userInfo]);
            abort();
        }
    }
}

// Returns the managed object context for the application.
// If the context doesn't already exist, it is created and bound to the persistent store coordinator for the application.
-(NSManagedObjectContext *)managedObjectContext {

    if (_managedObjectContext != nil) {
        return _managedObjectContext;
    }
    
    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (coordinator != nil) {
        _managedObjectContext = [[NSManagedObjectContext alloc] init];
        [_managedObjectContext setPersistentStoreCoordinator:coordinator];
    }
    return _managedObjectContext;
}

// Returns the managed object model for the application.
// If the model doesn't already exist, it is created from the application's model.
-(NSManagedObjectModel *)managedObjectModel {

    if (_managedObjectModel != nil) {
        return _managedObjectModel;
    }
    NSURL *modelURL = [[NSBundle mainBundle] URLForResource:@"HomesModel" withExtension:@"momd"];
    _managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
    return _managedObjectModel;
}

// Returns the persistent store coordinator for the application.
// If the coordinator doesn't already exist, it is created and the application's store added to it.
-(NSPersistentStoreCoordinator *)persistentStoreCoordinator {

    if (_persistentStoreCoordinator != nil) {
        return _persistentStoreCoordinator;
    }
    
    NSString *storePath = [[self applicationDocumentsDirectory].path stringByAppendingPathComponent: @"HomelessData.sqlite"];
    NSURL *storeURL = [NSURL fileURLWithPath:storePath ];
    
    NSError *error = nil;
    _persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:[self managedObjectModel]];
    if (![_persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:storeURL options:nil error:&error]) {
        NSLog(@"Unresolved error: %@, %@.\nAborting app now.", error, [error userInfo]);
        abort();
    }
    NSDictionary *fileProtection = @{ NSFileProtectionKey: NSFileProtectionComplete };
    if(![[NSFileManager defaultManager] setAttributes:fileProtection ofItemAtPath:storePath error:&error]){
        NSLog(@"Unable to encrypt core data: %@, %@.\nAborting app now.", error, [error userInfo]);
        abort();
    }
    
    return _persistentStoreCoordinator;
}

#pragma mark - Application's Documents directory

// Returns the URL to the application's Documents directory.
-(NSURL *)applicationDocumentsDirectory {
    return [[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask] lastObject];
}

@end

//@implementation NSURLRequest(DataController)
//+ (BOOL)allowsAnyHTTPSCertificateForHost:(NSString *)host {
//    return YES;
//}
//@end
