#import <Foundation/Foundation.h>
#import <Security/Security.h>

#import "MainViewController.h"
#import "RNEncryptor.h"
#import "RNDecryptor.h"


@implementation MainViewController


-(id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    
    if (self) {
        // Uncomment to override the CDVCommandDelegateImpl used
        // _commandDelegate = [[MainCommandDelegate alloc] initWithViewController:self];
        // Uncomment to override the CDVCommandQueue used
        // _commandQueue = [[MainCommandQueue alloc] initWithViewController:self];
    }
    
    return self;
}

-(id)init {
    self = [super init];
    
    if (self) {
        // Uncomment to override the CDVCommandDelegateImpl used
        // _commandDelegate = [[MainCommandDelegate alloc] initWithViewController:self];
        // Uncomment to override the CDVCommandQueue used
        // _commandQueue = [[MainCommandQueue alloc] initWithViewController:self];
    }
    
    return self;
}

#pragma mark View lifecycle

-(void)viewWillAppear:(BOOL)animated {
    if ([[[UIDevice currentDevice] systemVersion] floatValue] >= 7) {
//        CGRect viewBounds = [self.webView bounds];
//        viewBounds.origin.y = 20;
//        viewBounds.size.height = viewBounds.size.height - 20;
//        self.webView.frame = viewBounds;
        [self.webView scrollView].bounces = NO;
    }
    
    [super viewWillAppear:animated];
}

-(void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
}

-(void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

-(BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return [super shouldAutorotateToInterfaceOrientation:interfaceOrientation];
}

/* Comment out the block below to over-ride */

/*
-(UIWebView *)newCordovaViewWithFrame:(CGRect)bounds {
    return [super newCordovaViewWithFrame:bounds];
}
*/

#pragma mark UIWebDelegate implementation

-(void)webViewDidFinishLoad:(UIWebView *)theWebView {
    // Black base color for background matches the native apps
    theWebView.backgroundColor = [UIColor blackColor];
    
    return [super webViewDidFinishLoad:theWebView];
}

-(NSString *)getKey:(NSDictionary *)parameters{
    NSString        *secretKey;
    NSString        *serviceName;
    NSString        *accountName;
    NSData          *secret;
    NSDictionary    *keychainQuery;
    OSStatus        status;
    NSData          *returnedData;
    CFTypeRef       dataTypeRef;
    NSString        *cryptoKey;
    
    
    serviceName = [parameters valueForKey:@"userId"];
    accountName = [parameters valueForKey:@"homesId"];
    
    //
    // Get the data from the keychain
    //
    keychainQuery = @{(id)(__bridge id)kSecClass: (__bridge id)kSecClassGenericPassword,
                      (id)(__bridge id)kSecAttrService: serviceName,
                      (id)(__bridge id)kSecAttrAccount: accountName,
                      (id)(__bridge id)kSecReturnData: @YES,
                      };
    

    returnedData = NULL;
    dataTypeRef = (__bridge CFTypeRef)returnedData;
    
    status = SecItemCopyMatching((__bridge CFDictionaryRef)keychainQuery, &dataTypeRef);
    
    if(status == errSecItemNotFound) {
        //
        // Generate a unique secretKey for this user.
        //
        secretKey = [[NSString alloc]  initWithString:serviceName];
        secretKey = [secretKey stringByAppendingString:accountName];
        secretKey = [secretKey stringByAppendingFormat:@"%u", arc4random()];
        secret = [secretKey dataUsingEncoding:NSUTF8StringEncoding];

        //
        // Add the crypto key to the keychain.
        //
        keychainQuery = @{
                          (__bridge id)kSecClass: (__bridge id)kSecClassGenericPassword,
                          (__bridge id)kSecAttrService: serviceName,
                          (__bridge id)kSecAttrAccount: accountName,
                          (__bridge id)kSecValueData: secret,
                          };
        
        status = SecItemAdd((__bridge CFDictionaryRef)keychainQuery, NULL);
        
        NSAssert(status == errSecSuccess, @"Could not add crypto key to the keychain.");
        
        return secretKey;
    }
    else {
        if (status == errSecSuccess) {
            //
            // The call was successful, but we should make sure a crypto key was returned.
            //
            NSAssert(dataTypeRef != nil, @"No crypto key was returned from the keychain.");
            
            //
            // Now we can set the cryptoKey variable and return it.
            //
            returnedData = (__bridge NSData *)dataTypeRef;
            
            cryptoKey = [NSString stringWithUTF8String:[returnedData bytes]];
        }
        
        return cryptoKey;
    }
}


- (BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType {
    NSURL           *url = request.URL;
    NSError         *error = nil;
    NSString        *incomingDataBase64;
    NSString        *dataWithoutLeadingSlash;
    NSString        *key;
    NSString        *jsonKey;
    NSDictionary    *jsonToProcess;
    NSDictionary    *processingParameters;
    NSData          *base64EncodedPRocessingParameters;
    
    
    //
    // The format of the URL for the encryption and decryption calls is"
    // <encrypt | decrypt>://<fake host name>/<json string to process>?<processing parameters json string>
    //
    // The processing parameters are:
    //      - userId = user id
    //      - homesId = homes id associated with the user id
    //      - eventNAme = name of the event to fire
    //
    if ([[url scheme] isEqualToString:@"encrypt"]) {
        NSData    *encryptedResult;
        NSString  *encryptedDataStringBase64;
        NSData    *dataToEncrypt;
        
        
        //
        // Convert the Json string containing the processing pareamters into an NSDictionary object.
        //
        base64EncodedPRocessingParameters = [[NSData alloc] initWithBase64EncodedString:[url query] options:NSDataBase64DecodingIgnoreUnknownCharacters];
        
        processingParameters = [NSJSONSerialization JSONObjectWithData:base64EncodedPRocessingParameters options:NSJSONReadingMutableContainers error:&error];
        
        NSAssert(error == nil, @"Could not deserialize the incoming JSON data." );

        //
        // Get the key to use based on the data in the processing parameters Json object.
        //
        key = [self getKey:processingParameters];
        
        incomingDataBase64 = [url path];
        
        //
        // Strip the leading "/", which is not part of the data that was sent.  We get it because the
        // data is sent in base64 encoded format as the path of the URL and when we call [url path]
        // we get the data, prepended with the slash because it is part of the URL path.
        //
        dataWithoutLeadingSlash = [incomingDataBase64 substringWithRange:NSMakeRange(1, [incomingDataBase64 length]-1)];
        
        
        
        dataToEncrypt = [[NSData alloc] initWithBase64EncodedString:dataWithoutLeadingSlash options:NSDataBase64DecodingIgnoreUnknownCharacters];
        
        jsonToProcess = [NSJSONSerialization JSONObjectWithData:dataToEncrypt options:NSJSONReadingMutableContainers error:&error];
        
        //
        // Get values from jsonToProcess encrypt them and replace with encrypted values.
        //
        for(jsonKey in jsonToProcess.allKeys) {
            
            NSString  *value = [jsonToProcess valueForKey:jsonKey];
            
            if([value isKindOfClass:[NSNumber class]]) {
                if([(NSNumber *)value boolValue]) {
                    value = @"true";
                }
                else {
                    value = @"false";
                }
            }
            
            encryptedResult = [RNEncryptor  encryptData: [value dataUsingEncoding:NSUTF8StringEncoding]
                                           withSettings:kRNCryptorAES256Settings
                                               password:key
                                                  error:&error];
            
            NSAssert(error == nil, @"Could not encrypt the data." );

            [jsonToProcess setValue:[encryptedResult base64EncodedStringWithOptions:0] forKey:jsonKey];
        }

        //
        // Turn the NSDictionary with encrypted values base64 encoded json string.
        //
        encryptedDataStringBase64 = [[NSJSONSerialization dataWithJSONObject:jsonToProcess
                                                                     options:NSJSONWritingPrettyPrinted
                                                                       error:&error] base64EncodedStringWithOptions:0];
        
        NSAssert(error == nil, @"Could not deserialize the JSON data." );
        
        NSString  *eventFiringString = [[NSString alloc] initWithFormat:@"{var eventObject; if(document.createEventObject){eventObject = document.createEventObject();eventObject.detail = '%@';document.fireEvent('%@', event);}else{eventObject = document.createEvent('Events');eventObject.detail = '%@';eventObject.initEvent('%@',true,true);document.dispatchEvent(eventObject); } }",
                                         encryptedDataStringBase64, [processingParameters valueForKey:@"eventName"],encryptedDataStringBase64, [processingParameters valueForKey:@"eventName"]];
        
        
        [webView stringByEvaluatingJavaScriptFromString:eventFiringString];
        
        return YES;
    }
    
    //
    // See above for the format of the URL.
    //
    if ([[url scheme] isEqualToString:@"decrypt"]) {
        NSData    *decryptedResult;
        NSString  *decryptedDataStringBase64;
        NSData    *dataToDecrypt;
        
        
        //
        // Convert the Json string containing the processing pareamters into an NSDictionary object.
        //
        base64EncodedPRocessingParameters = [[NSData alloc] initWithBase64EncodedString:[url query] options:NSDataBase64DecodingIgnoreUnknownCharacters];
        
        processingParameters = [NSJSONSerialization JSONObjectWithData:base64EncodedPRocessingParameters options:NSJSONReadingMutableContainers error:&error];
        
        NSAssert(error == nil, @"Could not deserialize the incoming JSON data." );

        //
        // Get the key to use based on the data in the processing parameters Json object.
        //
        key = [self getKey:processingParameters];
        
        incomingDataBase64 = [url path];
        
        //
        // Strip the leading "/", which is not part of the data that was sent.  We get it because the
        // data is sent in base64 encoded format as the path of the URL and when we call [url path]
        // we get the data, prepended with the slash because it is part of the URL path.
        //
        dataWithoutLeadingSlash = [incomingDataBase64 substringWithRange:NSMakeRange(1, [incomingDataBase64 length]-1)];
        
        
        
        dataToDecrypt = [[NSData alloc] initWithBase64EncodedString:dataWithoutLeadingSlash options:NSDataBase64DecodingIgnoreUnknownCharacters];
        
        jsonToProcess = [NSJSONSerialization JSONObjectWithData:dataToDecrypt options:NSJSONReadingMutableContainers error:&error];
        
        //
        // Get values from jsonToProcess encrypt them and replace with encrypted values.
        //
        for(jsonKey in jsonToProcess.allKeys) {
            
            NSString  *value = [jsonToProcess valueForKey:jsonKey];
            
            decryptedResult = [RNDecryptor  decryptData:[[NSData alloc] initWithBase64EncodedString:value options:0]
                                            withSettings:kRNCryptorAES256Settings
                                            password:key
                                            error:&error];
            
            NSAssert(error == nil, @"Could not decrypt the data." );

            NSString  *tmpStr = [[NSString alloc] initWithData:decryptedResult encoding:NSASCIIStringEncoding];
            
            //
            // Look for "true" or "false" and convert to a boolean value.
            //
            if([tmpStr isEqualToString: @"true"]) {
                [jsonToProcess setValue:[[NSNumber alloc] initWithBool:YES] forKey:jsonKey];
            }
            else {
                if([tmpStr isEqualToString: @"false"]) {
                    [jsonToProcess setValue:[[NSNumber alloc] initWithBool:NO] forKey:jsonKey];
                }
                else {
                    [jsonToProcess setValue:[[NSString alloc] initWithData:decryptedResult encoding:NSASCIIStringEncoding] forKey:jsonKey];
                }
            }
        }
        
        decryptedDataStringBase64 = [[NSJSONSerialization dataWithJSONObject:jsonToProcess
                                                                     options:NSJSONWritingPrettyPrinted
                                                                       error:&error] base64EncodedStringWithOptions:0];
        
        NSAssert(error == nil, @"Could not serialize the JSON data." );

        NSString  *eventFiringString = [[NSString alloc] initWithFormat:@"{var eventObject; if(document.createEventObject){eventObject = document.createEventObject();eventObject.detail = '%@';document.fireEvent('%@', event);}else{eventObject = document.createEvent('Events');eventObject.detail = '%@';eventObject.initEvent('%@',true,true);document.dispatchEvent(eventObject); } }",
                                        decryptedDataStringBase64, [processingParameters valueForKey:@"eventName"], decryptedDataStringBase64, [processingParameters valueForKey:@"eventName"]];
        
        
        [webView stringByEvaluatingJavaScriptFromString:eventFiringString];
        
        return YES;
    }
    return YES;
}


/* Comment out the block below to over-ride */

/*

-(void)webViewDidStartLoad:(UIWebView *)theWebView {
    return [super webViewDidStartLoad:theWebView];
}

-(void)webView:(UIWebView*)theWebView didFailLoadWithError:(NSError *)error {
    return [super webView:theWebView didFailLoadWithError:error];
}

-(BOOL)webView:(UIWebView*)theWebView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType {
    return [super webView:theWebView shouldStartLoadWithRequest:request navigationType:navigationType];
}
*/

@end

@implementation MainCommandDelegate

/* To override the methods, uncomment the line in the init function(s) in MainViewController.m */

#pragma mark CDVCommandDelegate implementation

-(id)getCommandInstance:(NSString *)className {
    return [super getCommandInstance:className];
}

/*
   NOTE: this will only inspect execute calls coming explicitly from native plugins,
   not the commandQueue (from JavaScript). To see execute calls from JavaScript, see
   MainCommandQueue below
*/
-(BOOL)execute:(CDVInvokedUrlCommand *)command {
    return [super execute:command];
}

-(NSString *)pathForResource:(NSString *)resourcepath {
    return [super pathForResource:resourcepath];
}

@end

@implementation MainCommandQueue

/* To override, uncomment the line in the init function(s)
   in MainViewController.m
 */
-(BOOL)execute:(CDVInvokedUrlCommand *)command {
    return [super execute:command];
}

@end
