#import "CoreDataPlugin.h"
#import "AppDelegate.h"
#import "MainViewController.h"
#import <Cordova/CDVViewController.h>

#import "RNEncryptor.h"
#import "RNDecryptor.h"

#import "OfflineVeteranJSON.h"
#import "ResponseInstance.h"
#import "AssessmentInstance.h"


@implementation CoreDataPlugin

+(NSArray*)assessmentQueryKeys {
	return @[@"isDecisionSupport", @"userId", @"veteranLocalID", @"veteranHomesId"];
}

+(NSArray*)assessmentsQueryKeys {
	return @[@"isDecisionSupport", @"userId"];
}

+(NSArray*)assessmentAttributes {
	return [NSArray arrayWithObjects:@"assessmentDate", @"stationNumber", @"submitDate", @"finished", @"assessmentFormVersion", @"veteranHomesId", @"veteranFirstName", @"veteranLastName", @"responses", @"currentSectionIndex", @"isDecisionSupport", @"reassessment", @"userId", @"veteranSSN", @"veteranLocalID", nil];
}

-(BOOL)checkDictForStrings:(NSDictionary *)dict {
    NSEnumerator *enumerator = [dict keyEnumerator];
    id key;
	while (key = [enumerator nextObject]) {
		id value = [dict objectForKey:key];
        if(value==nil || ![value isKindOfClass:[NSString class]]) {
            NSLog(@"%s:%d\n ERROR dictionary value for key %@ was nil or not a string", __func__, __LINE__, key);
            return NO;
        }
    };
    return YES;
}

-(NSArray*)stringAttributes {
	return [NSArray arrayWithObjects:@"assessmentDate", @"assessmentFormVersion", @"stationNumber", @"submitDate",
            @"userId", @"veteranHomesId", @"veteranFirstName", @"veteranLastName", @"veteranSSN", @"veteranLocalID",nil];
}

-(NSError*)checkAttributes:(NSMutableDictionary *)instance {
    NSArray* names = [self stringAttributes];
    
	for (NSString *name in names) {
        NSString * value = [instance objectForKey:name];
        if( value != nil && ![value isKindOfClass:[NSString class]] ) {
            NSLog(@"%s:%d\n ERROR checkAttributes value (%@) name (%@) was not a string", __func__, __LINE__, value, name);
            return [NSError errorWithDomain:@"mHOMES" code:200 userInfo:nil];
        }
	}
    return nil;
}

-(BOOL)checkDictForExpectedKeys:(NSDictionary *)dict keys:(NSArray*)keys {
	for (NSString* key in keys) {
		NSString * value = [dict objectForKey:key];
        if(![value isKindOfClass:[NSString class]]) {
            NSLog(@"%s:%d\n ERROR dictionary value (%@) for key (%@) was not a string", __func__, __LINE__, value, key);
            return NO;
        }
    };
    return YES;
}

-(void)saveOfflineJSON:(CDVInvokedUrlCommand *)command {
	CDVPluginResult *pluginResult = nil;
	NSLog(@"%s:%d\n Saving JSON", __func__, __LINE__);
	BOOL shouldCopyVeteranLocalID = NO;

	NSDictionary *jsonObjectDictionary = [command.arguments objectAtIndex:0];
	NSMutableDictionary *queryParameters;
	NSMutableDictionary *jsonObject = [[NSMutableDictionary alloc] init];
    
    NSString *retainLocalId = nil;
    
    //OfflineVeteranJSON *saveVet = [NSEntityDescription insertNewObjectForEntityForName:@"OfflineVeteranJSON" inManagedObjectContext:[self getContext]];

	NSError *error = nil;
    
    if (![self checkDictForExpectedKeys:jsonObjectDictionary
                keys:@[@"veteranHomesId",@"veteranLocalID"]]) {
        NSLog(@"%s:%d\n saveOfflineJSON had error", __func__, __LINE__);
        error = [NSError errorWithDomain:@"mHOMES" code:200 userInfo:nil];
    }
    
    if (error == nil) {
        if ([jsonObjectDictionary[@"veteranHomesId"] isEqualToString:@"-1"] &&
            [jsonObjectDictionary[@"veteranLocalID"] isEqualToString:@"-1"] == NO) {
            
            queryParameters = [NSMutableDictionary dictionaryWithObjects:
                               @[[jsonObjectDictionary valueForKey:@"userId"],
                                 [jsonObjectDictionary valueForKey:@"veteranLocalID"]]
                                                                 forKeys:
                               @[@"userId",
                                 @"veteranLocalID"]];
            
            [jsonObject setValue:[jsonObjectDictionary valueForKey:@"veteranLocalID"] forKey:@"veteranLocalID"];
            retainLocalId = jsonObject[@"veteranLocalID"];
            
        } else if ([jsonObjectDictionary[@"veteranLocalID"] isEqualToString:@"-1"] &&
                   [jsonObjectDictionary[@"veteranHomesId"] isEqualToString:@"-1"] == NO) {
            NSString* localID = [self generateRandomInt];
            shouldCopyVeteranLocalID = YES;
            
            queryParameters = [NSMutableDictionary dictionaryWithObjects:
                               @[[jsonObjectDictionary valueForKey:@"userId"],
                                 [jsonObjectDictionary valueForKey:@"veteranHomesId"]]
                                                                 forKeys:
                               @[@"userId",
                                 @"veteranHomesId"]];
            
            [jsonObject setValue:localID forKey:@"veteranLocalID"];
            retainLocalId = localID;
            
        } else if ([jsonObjectDictionary[@"veteranLocalID"] isEqualToString:@"-1"] &&
                   [jsonObjectDictionary[@"veteranHomesId"] isEqualToString:@"-1"]) {
            
            NSString* localID = [self generateRandomInt];
            queryParameters = [NSMutableDictionary dictionaryWithObjects:
                               @[[jsonObjectDictionary valueForKey:@"userId"],
                                 localID]
                                                                 forKeys:
                               @[@"userId",
                                 @"veteranLocalID"]];
            //need to set to maintainable object
            [jsonObject setValue:localID forKey:@"veteranLocalID"];
            retainLocalId = localID;
            
        } else {
            queryParameters = [NSMutableDictionary dictionaryWithObjects:
                               @[[jsonObjectDictionary valueForKey:@"userId"],
                                 [jsonObjectDictionary valueForKey:@"veteranLocalID"]]
                                                                 forKeys:
                               @[@"userId",
                                 @"veteranLocalID"]];
            
            [jsonObject setValue:[jsonObjectDictionary valueForKey:@"veteranLocalID"] forKey:@"veteranLocalID"];
            retainLocalId = jsonObject[@"veteranLocalID"];
        }
        
        NSError *serializeError;
        NSMutableDictionary *jsonStringDictionary = [[NSMutableDictionary alloc] init];
        NSArray* keyArray = @[@"veteranJSON", @"assessmentJSON"];
        
        for (NSString* key in keyArray) {
            NSData *jsonData = [NSJSONSerialization dataWithJSONObject:[jsonObjectDictionary objectForKey:key]
                                                               options:NSJSONWritingPrettyPrinted
                                                                 error:&serializeError];
            
            if (serializeError != nil) {
                break;
            } else {
                NSString* jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
                [jsonStringDictionary setObject:jsonString forKey:key];
            }
        }
        
        NSManagedObject *duplicateOfflineJSONObject = [self findOfflineJSONObjectWithQueryParameters:queryParameters];
        
        if (duplicateOfflineJSONObject != nil) {
            
            if (shouldCopyVeteranLocalID) {
                [jsonObject setValue:retainLocalId forKey:@"veteranLocalID"];
            }
            NSError *error2 = [self deleteEntity:duplicateOfflineJSONObject];
            NSLog(@"%s:%d\n Was there an error? %@", __func__, __LINE__, [error2 localizedDescription]);
        }
        
        
        [jsonObject setValue:[jsonStringDictionary objectForKey:@"veteranJSON"] forKey:@"veteranJSON"];
        [jsonObject setValue:[jsonStringDictionary objectForKey:@"assessmentJSON"] forKey:@"assessmentJSON"];
        [jsonObject setValue:[jsonObjectDictionary valueForKey:@"veteranHomesId"] forKey:@"veteranHomesId"];
        [jsonObject setValue:[jsonObjectDictionary valueForKey:@"veteranSSN"] forKey:@"veteranSSN"];
        
        jsonObject = [[self encryptDictionaryData:jsonObject withUserId:[jsonObjectDictionary valueForKey:@"userId"]] mutableCopy];
        [jsonObject setValue:[jsonObjectDictionary valueForKey:@"userId"] forKey:@"userId"];
        [jsonObject setValue:retainLocalId forKey:@"veteranLocalID"];
        
        NSManagedObjectContext *managedObjectContext = [self getContext];
        NSEntityDescription *entity = [NSEntityDescription entityForName:@"OfflineVeteranJSON" inManagedObjectContext:managedObjectContext];
        NSFetchRequest *request = [[NSFetchRequest alloc] init];
        [request setEntity:entity];
        [request setFetchLimit:1];
        [request setPredicate:[NSPredicate predicateWithFormat:@"veteranLocalID == %@", jsonObject[@"veteranLocalID"]]];
        
        NSUInteger count = [[self getContext] countForFetchRequest:request
                                                             error:&error];
        /*if(count == 0){
            saveVet.veteranJSON = jsonObject[@"veteranJSON"];
            saveVet.assessmentJSON = jsonObject[@"assessmentJSON"];
            saveVet.veteranHomesId = jsonObject[@"veteranHomesId"];
            saveVet.userId = jsonObject[@"userId"];
            saveVet.veteranSSN = jsonObject[@"veteranSSN"];
            saveVet.veteranLocalID = retainLocalId;
            NSLog(@"Encrypted save");
            NSLog(@"%@", saveVet);
        }
        else{
            error = [self saveEntity:@"OfflineVeteranJSON"
                             instance:jsonObject
                           attributes:@[@"veteranJSON", @"assessmentJSON", @"veteranHomesId", @"userId", @"veteranSSN", @"veteranLocalID"]];
        }*/
        
        if(count == 0){
            error = [self saveEntity:@"OfflineVeteranJSON"
                        instance:jsonObject
                      attributes:@[@"veteranJSON", @"assessmentJSON", @"veteranHomesId", @"userId", @"veteranSSN", @"veteranLocalID"]];
        }
        
        
    }
    

	if (error != nil) {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:error.localizedDescription];

	} else {
		NSManagedObject *returnedOfflineJSON = [self findOfflineJSONObjectWithQueryParameters:queryParameters];

		if ([retainLocalId intValue] != -1) {
			pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString:[returnedOfflineJSON valueForKey:@"veteranLocalID"]];
		} else {
			pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"Error Saving Offline JSON Objects"];
		}
	}

	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];

}

-(void)deleteOfflineJSON:(CDVInvokedUrlCommand *)command {
	CDVPluginResult *pluginResult = nil;

	NSLog(@"%s:%d\n deleting offline JSON args:\n%@", __func__, __LINE__, command.arguments);
	NSMutableDictionary *queryDictionary = [command.arguments objectAtIndex:0];

    NSError* error = [self deleteEntity: [self findOfflineJSONObjectWithQueryParameters:queryDictionary]];
    
	if (error != nil) {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:error.localizedDescription];

	} else {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString:@"Offline Veteran Data Successfully Deleted"];
	}

	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

-(void)fetchOfflineJSON:(CDVInvokedUrlCommand *)command {
	CDVPluginResult *pluginResult = nil;

	NSLog(@"%s:%d\n fetching offline JSON with query:\n%@", __func__, __LINE__, command.arguments);

	NSMutableDictionary *queryDictionary = [command.arguments objectAtIndex:0];

	NSManagedObject *offlineJSONObject = [self findOfflineJSONObjectWithQueryParameters:queryDictionary];

	if (offlineJSONObject) {
		NSArray *keys = [[[offlineJSONObject entity] attributesByName] allKeys];
		NSDictionary *dict = [offlineJSONObject dictionaryWithValuesForKeys:keys];

		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:dict];
	} else {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString:@""];
	}

	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

-(NSManagedObject*)findOfflineJSONObjectWithQueryParameters:(NSMutableDictionary *)queries {
	NSLog(@"%s:%d\n Fetching single offline JSON object with queries %@ ", __func__, __LINE__, queries);

	NSPredicate* predicate;
    
    if (![self checkDictForExpectedKeys:queries
                keys:@[@"userId"]]) {
        NSLog(@"%s:%d\n findOfflineJSONObjectWithQueryParameters had error", __func__, __LINE__);
        return nil;
    }
    
    //queries = [[self encryptDictionaryData:queries withUserId:[queries valueForKey:@"userId"]] mutableCopy];

	if (queries[@"veteranLocalID"] && [queries[@"veteranLocalID"] isEqualToString:@"-1"] == NO) {
		predicate = [NSPredicate predicateWithFormat:@"veteranLocalID = %@ AND userId = %@", queries[@"veteranLocalID"], queries[@"userId"]];

	} else if (queries[@"veteranHomesId"] &&
               [queries[@"veteranHomesId"] isEqualToString:@"-1"] == NO) {
		predicate = [NSPredicate predicateWithFormat:@"veteranHomesId = %@ AND userId = %@", queries[@"veteranHomesId"], queries[@"userId"]];

	} else {
		predicate =[NSPredicate predicateWithFormat:@"userId = %@", queries[@"userId"]];
	}

    NSArray *objects = [self fetchEntities:@"OfflineVeteranJSON" predicate:predicate];
    
	if ([objects count] == 0) {
		NSLog(@"%s:%d\n No matches found", __func__, __LINE__);
        return nil;
	} else {
		NSLog(@"%s:%d\n %d Matches were Found", __func__, __LINE__, [objects count]);
		return objects[0];
	}
}

-(void)fetchAllOfflineJSON:(CDVInvokedUrlCommand *)command {
	CDVPluginResult *pluginResult = nil;
	NSDictionary *fetchOfflineJSONQuery = [command.arguments objectAtIndex:0];

	NSPredicate* predicate =[NSPredicate predicateWithFormat:@"userId = %@", fetchOfflineJSONQuery[@"userId"]];
    
	NSMutableArray* offlineJSONResults =
    [self arrayToJSON: [self fetchEntities:@"OfflineVeteranJSON" predicate:predicate]];


    NSMutableDictionary *jsonObject = [[NSMutableDictionary alloc] init];
    for(int i = 0; i < [offlineJSONResults count]; i++){
        [jsonObject setValue:[offlineJSONResults[i] objectForKey:@"veteranJSON"] forKey:@"veteranJSON"];
        [jsonObject setValue:[offlineJSONResults[i] objectForKey:@"assessmentJSON"] forKey:@"assessmentJSON"];
        [jsonObject setValue:[offlineJSONResults[i] valueForKey:@"veteranHomesId"] forKey:@"veteranHomesId"];
        [jsonObject setValue:[offlineJSONResults[i] valueForKey:@"userId"] forKey:@"userId"];
        [jsonObject setValue:[offlineJSONResults[i] valueForKey:@"veteranSSN"] forKey:@"veteranSSN"];
        jsonObject = [[self decryptDictionaryData:jsonObject withUserId:fetchOfflineJSONQuery[@"userId"]] mutableCopy];
        [offlineJSONResults replaceObjectAtIndex:i withObject:jsonObject];
    }
    
    
    
    
	if (offlineJSONResults == nil) {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"No Offline JSON Objects found"];

	} else {
        pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:offlineJSONResults];
	}

	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    
    
    //Lets remove all the Null items from the database, shall we?
    NSManagedObjectContext *managedObjectContext = [self getContext];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"OfflineVeteranJSON" inManagedObjectContext:managedObjectContext];
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:entity];
    [request setPredicate:[NSPredicate predicateWithFormat:@"userId == %@", NULL]];
    
    NSError *error;
    NSUInteger count = [[self getContext] countForFetchRequest:request
                                                         error:&error];
    if(count > 0){
        NSArray *deleteThese = [managedObjectContext executeFetchRequest:request error:&error];\
        for(int i = 0; i < count; i++){
            [self deleteEntity: deleteThese[i]];
        }
        
    }
    

}

-(void)saveAssessmentInstance:(CDVInvokedUrlCommand *)command {
    NSLog(@"%s:%d\n SaveAssessmentInstance", __func__, __LINE__);
    
    CDVPluginResult *pluginResult = nil;
    NSManagedObjectContext *context = [self getContext];
    NSMutableOrderedSet *responses = [[NSMutableOrderedSet alloc] init];
    NSDictionary *assessmentInstance = [[command.arguments objectAtIndex:0] mutableCopy];
    
    for (NSObject* response in [assessmentInstance valueForKey:@"responses"]) {
        NSManagedObject *responseInstance = [NSEntityDescription insertNewObjectForEntityForName:@"ResponseInstance"
                                                                          inManagedObjectContext:context];
        [responseInstance setValue:[response valueForKey:@"QuestionID"] forKey:@"questionId"];
        [responseInstance setValue:[response valueForKey:@"ResponseValue"] forKey:@"responseValue"];
        [responseInstance setValue:[response valueForKey:@"PrevResponseValue"] forKey:@"prevResponseValue"];
        
        [responses addObject:responseInstance];
    }
    
    BOOL isDSForm = [[assessmentInstance valueForKey:@"isDecisionSupport"] boolValue];
    BOOL isReassessment = [[assessmentInstance valueForKey:@"reassessment"] boolValue];
    
    if (isDSForm && [responses count] > 26) {
        NSLog(@"%s:%d\n DSFORM HAS TOO MANY QUESTIONS!", __func__, __LINE__);
    }
    NSMutableDictionary *toEncrypt = [[NSMutableDictionary alloc] init];
    toEncrypt[@"stationNumber"] = [assessmentInstance[@"stationNumber"] mutableCopy];
    toEncrypt[@"veteranFirstName"] = [assessmentInstance[@"veteranFirstName"] mutableCopy];
    toEncrypt[@"veteranHomesId"] = [assessmentInstance[@"veteranHomesId"] mutableCopy];
    toEncrypt[@"veteranLastName"] = [assessmentInstance[@"veteranLastName"] mutableCopy];
    toEncrypt[@"veteranSSN"] = [assessmentInstance[@"veteranSSN"] mutableCopy];
    toEncrypt = [[self encryptDictionaryData:toEncrypt withUserId:assessmentInstance[@"userId"]] mutableCopy];
    [assessmentInstance setValue:toEncrypt[@"stationNumber"] forKey:@"stationNumber"];
    [assessmentInstance setValue:toEncrypt[@"veteranFirstName"] forKey:@"veteranFirstName"];
    [assessmentInstance setValue:toEncrypt[@"veteranHomesId"] forKey:@"veteranHomesId"];
    [assessmentInstance setValue:toEncrypt[@"veteranLastName"] forKey:@"veteranLastName"];
    [assessmentInstance setValue:toEncrypt[@"veteranSSN"] forKey:@"veteranSSN"];
    
    NSMutableDictionary *instance = [[NSMutableDictionary alloc] init];
    [instance setValuesForKeysWithDictionary:assessmentInstance];
    
    [instance setValue:[[NSOrderedSet alloc] initWithOrderedSet:responses] forKey:@"responses"];
    [instance setValue:[NSNumber numberWithBool:isDSForm] forKey:@"isDecisionSupport"];
    [instance setValue:[NSNumber numberWithBool:isReassessment] forKey:@"reassessment"];
    
    static NSArray *attributes;
    if (attributes == nil) {
        attributes = [CoreDataPlugin assessmentAttributes];
    }
    
    NSError *error;
    NSArray *queryKeys = [CoreDataPlugin assessmentQueryKeys];
    NSDictionary *queries = [NSDictionary
                             dictionaryWithObjects: [instance objectsForKeys:queryKeys notFoundMarker:@""]
                             forKeys: queryKeys];
    NSManagedObject *duplicateAssessment = [self findAssessmentInstanceWithQueryParameters:queries];
    
    if (duplicateAssessment != nil) {
        if ([[duplicateAssessment valueForKey:@"isDecisionSupport"] boolValue] != isDSForm) {
            NSLog(@"\n\n\n\n\n\n\n\nBAD\n\n\n\n\n\n\n\nBAD\n\n\n\n\n\n\n\nBAD\n\n\n\n\n\n\n\nBAD\n\n\n\n\n\n\n\nBAD\n\n\n\n\n\n\n\nBAD\n\n\n\n\n\n\n\nBAD");
            NSLog(@"%s:%d\n Trying to overwrite DSForm=%@ with !DSForm", __func__, __LINE__, [duplicateAssessment valueForKey:@"isDecisionSupport"]);
        } else {
            NSError *error2 = [self deleteEntity:duplicateAssessment];
            NSLog(@"%s:%d\n Was there an error? %@", __func__, __LINE__, [error2 localizedDescription]);
        }
    }

    error = [self saveEntity:@"AssessmentInstance" instance:instance attributes:attributes];
    
    if (error != nil) {
        pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:error.localizedDescription];
        
    } else {
        NSManagedObject *returnedAssessment = [self findAssessmentInstanceWithQueryParameters:queries];
        
        if (returnedAssessment != nil) {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString:@"Assessment successfully saved"];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"No Assessment Found"];
        }
    }
    
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];}

-(void)deleteAssessmentInstance:(CDVInvokedUrlCommand *)command {
	CDVPluginResult *pluginResult = nil;

NSLog(@"%s:%d\n deleteAssessmentInstance args:\n%@", __func__, __LINE__, command.arguments);
	NSDictionary *assessmentInstance = [command.arguments objectAtIndex:0];

	NSError *error;
	NSMutableDictionary *queryInstance = [[NSMutableDictionary alloc] initWithDictionary:assessmentInstance];


	NSArray *queryKeys = [CoreDataPlugin assessmentQueryKeys];
	NSDictionary *queries = [NSDictionary
                             dictionaryWithObjects:[queryInstance objectsForKeys:queryKeys notFoundMarker:@""]
                             forKeys:queryKeys];

	NSManagedObject *asssessmentToDelete = [self findAssessmentInstanceWithQueryParameters:queries];

	if (asssessmentToDelete != nil) {
        NSError *error = [self deleteEntity:asssessmentToDelete];
		NSLog(@"%s:%d\n Was there an error? %@", __func__, __LINE__, [error localizedDescription]);
	}

	if (error != nil) {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:error.localizedDescription];

	} else {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString:@"Assessment successfully deleted"];
	}

	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}


-(void)fetchAssessmentInstances:(CDVInvokedUrlCommand*)command {
	CDVPluginResult *pluginResult = nil;
	NSDictionary *assessmentInstance = [command.arguments objectAtIndex:0];

	NSMutableDictionary *queries = [[NSMutableDictionary alloc] initWithDictionary:assessmentInstance];
    NSPredicate* predicate = [NSPredicate predicateWithFormat:@"isDecisionSupport = %@ AND userId = %@",
                              queries[@"isDecisionSupport"], queries[@"userId"]];
    
    NSMutableArray* assessmentResults = [[self fetchEntities:@"AssessmentInstance" predicate:predicate] mutableCopy];
	if ([assessmentResults count] == 0) {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"No Assessments found"];

	} else {
        NSMutableDictionary *toDecrypt = [[NSMutableDictionary alloc] init];
         for (int i = 0; i < [assessmentResults count]; i++){
         //store in decrypt dictionary
         toDecrypt[@"stationNumber"] = [assessmentResults[i] valueForKey:@"stationNumber"];
         toDecrypt[@"veteranFirstName"] = [assessmentResults[i] valueForKey:@"veteranFirstName"];
         toDecrypt[@"veteranHomesId"] = [assessmentResults[i] valueForKey:@"veteranHomesId"];
         toDecrypt[@"veteranLastName"] = [assessmentResults[i] valueForKey:@"veteranLastName"];
         toDecrypt[@"veteranSSN"] = [assessmentResults[i] valueForKey:@"veteranSSN"];
         toDecrypt = [[self decryptDictionaryData:toDecrypt withUserId:queries[@"userId"]] mutableCopy];
         //get assessmentResults index for each key
         
         //store back into assessmentResults
         [[assessmentResults objectAtIndex:i] setValue:toDecrypt[@"stationNumber"] forKey:@"stationNumber"];
         [[assessmentResults objectAtIndex:i] setValue:toDecrypt[@"veteranFirstName"] forKey:@"veteranFirstName"];
         [[assessmentResults objectAtIndex:i] setValue:toDecrypt[@"veteranHomesId"] forKey:@"veteranHomesId"];
         [[assessmentResults objectAtIndex:i] setValue:toDecrypt[@"veteranLastName"] forKey:@"veteranLastName"];
         [[assessmentResults objectAtIndex:i] setValue:toDecrypt[@"veteranSSN"] forKey:@"veteranSSN"];
         }
		NSMutableArray *jsArray = [[NSMutableArray alloc] init];
		for (int i=0; i<[assessmentResults count]; i++) {
			[jsArray addObject:[self convertAssessmentInstanceToObject:[assessmentResults objectAtIndex:i]]];
		}
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:jsArray];
	}

	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

-(void)fetchOneAssessmentInstance:(CDVInvokedUrlCommand *)command {
	CDVPluginResult *pluginResult = nil;
	NSDictionary *assessmentInstance = [command.arguments objectAtIndex:0];

	NSMutableDictionary *queryObject = [[NSMutableDictionary alloc] initWithDictionary:assessmentInstance];

	NSManagedObject* assessmentResult = [self findAssessmentInstanceWithQueryParameters:queryObject];
	NSLog(@"%s:%d\nfetching one, query object %@", __func__, __LINE__, [queryObject description]);

	NSLog(@"%s:%d\n fetch one assessment result %@", __func__, __LINE__, [assessmentResult valueForKey:@"veteranLocalID"]);
	NSLog(@"%s:%d\n fetch one assessment result date %@", __func__, __LINE__, [assessmentResult valueForKey:@"assessmentDate"]);

	if (assessmentResult == nil) {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"No Assessment found"];

	} else {
		NSMutableDictionary *jsAssessment = [self convertAssessmentInstanceToObject:assessmentResult];
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:jsAssessment];
	}

	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

-(NSMutableDictionary*)convertAssessmentInstanceToObject:(NSManagedObject*)managedObjectAssessment {

	NSArray *keys = [[[managedObjectAssessment entity] attributesByName] allKeys];
	NSMutableDictionary *dict = [[managedObjectAssessment dictionaryWithValuesForKeys:keys] mutableCopy];

	NSMutableArray *responsesArray = [[NSMutableArray alloc] init];
	for (NSManagedObject *managedObjectResponse in [[managedObjectAssessment valueForKey:@"responses"] allObjects]) {
		[responsesArray addObject:[self convertResponseInstanceToObject:managedObjectResponse]];
	}

	[dict setObject:responsesArray forKey:@"responses"];

	return dict;
}

-(NSMutableDictionary*)convertResponseInstanceToObject:(NSManagedObject*)managedObjectResponse {
	NSArray *keys = [[[managedObjectResponse entity] attributesByName] allKeys];
	NSMutableDictionary *dict = [[managedObjectResponse dictionaryWithValuesForKeys:keys] mutableCopy];

	return dict;
}

-(NSManagedObject*)findAssessmentInstanceWithQueryParameters:(NSDictionary *)queries {
	NSLog(@"%s:%d\n Fetching single assessment with queries %@ ", __func__, __LINE__, queries);

	NSPredicate* predicate;
    if (![self checkDictForExpectedKeys:queries
                keys:@[@"veteranHomesId",@"veteranLocalID"]]) {
        NSLog(@"%s:%d\n findAssessmentInstanceWithQueryParameters had error", __func__, __LINE__);
        return nil;
    }
    
    if([queries[@"veteranLocalID"] isKindOfClass:[NSString class]])
    {
        if (queries[@"veteranLocalID"] &&
            [queries[@"veteranLocalID"] isEqualToString:@"-1"] == NO) {
            predicate =[NSPredicate predicateWithFormat:@"veteranLocalID = %@ AND userId = %@ AND isDecisionSupport = %@", queries[@"veteranLocalID"], queries[@"userId"], queries[@"isDecisionSupport"]];
            
        } else if (queries[@"veteranHomesId"] &&
                   [queries[@"veteranHomesId"] isEqualToString:@"-1"] == NO) {
            predicate =[NSPredicate predicateWithFormat:@"veteranHomesId = %@ AND userId = %@ AND isDecisionSupport = %@", queries[@"veteranHomesId"], queries[@"userId"], queries[@"isDecisionSupport"]];
            
        } else {
            predicate =[NSPredicate predicateWithFormat:@"userId = %@ AND isDecisionSupport = %@", queries[@"userId"], queries[@"isDecisionSupport"]];
        }
    } else {
        
    }


    NSArray *objects = [self fetchEntities:@"AssessmentInstance" predicate:predicate];
    
	NSLog(@"%s:%d\n %d assessments found!", __func__, __LINE__, [objects count]);

	if ([objects count] == 0) {
		NSLog(@"%s:%d\n No matches found", __func__, __LINE__);
        return nil;
	} else {
        NSManagedObject *match = nil;
		match = objects[0];
		NSLog(@"%s:%d\n match single: SN:%@ AD:%@ SD:%@ id:%@ veteranLocalID:%@", __func__, __LINE__,
			[match valueForKey:@"stationNumber"],
			[match valueForKey:@"assessmentDate"],
			[match valueForKey:@"submitDate"],
			[match objectID],
			[match valueForKey:@"veteranLocalID"]
			);
        return match;
	}
}

-(void)saveMetric:(CDVInvokedUrlCommand *)command {
	NSLog(@"%s:%d\n Saving Metric", __func__, __LINE__);
    
	NSMutableDictionary *jsonObject = [[NSMutableDictionary alloc] init];
    
    NSString* localID = [self generateRandomInt];
	[jsonObject setValue:localID forKey:@"metricID"];
    
    NSMutableDictionary *jsonStringDictionary = [self serializeJSON:@[@"metricJSON"] dict:[command.arguments objectAtIndex:0]];
	[jsonObject setValue:[jsonStringDictionary objectForKey:@"metricJSON"] forKey:@"metricJSON"];
    
	NSError *error = [self saveEntity:@"MetricJSON" instance:jsonObject attributes:@[@"metricJSON", @"metricID"]];
    
	CDVPluginResult *pluginResult = nil;
	if (error == nil) {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString:localID];
	} else {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:error.localizedDescription];
	}
    
	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

-(void)deleteMetric:(CDVInvokedUrlCommand *)command {
	NSLog(@"%s:%d\n deleting metric args:\n%@", __func__, __LINE__, command.arguments);
    
    NSError *error = [self deleteEntity: [self metricWithQueryParameters: [command.arguments objectAtIndex:0]]];
    
	CDVPluginResult *pluginResult = nil;
	if (error != nil) {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:error.localizedDescription];
        
	} else {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString:@"Metric Successfully Deleted"];
	}
    
	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    
}

-(void)fetchMetrics:(CDVInvokedUrlCommand *)command {
	NSLog(@"%s:%d\n fetchMetrics", __func__, __LINE__);
    
	NSMutableArray* metricResults = nil;
    metricResults = [self arrayToJSON: [self fetchEntities:@"MetricJSON" predicate:nil]];
    
	CDVPluginResult *pluginResult = nil;
	if (metricResults == nil) {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"No Metric Objects found"];
        
	} else {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:metricResults];
	}
    
	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

-(void)fetchParm:(CDVInvokedUrlCommand *)command {
	NSLog(@"%s:%d\n fetchParm started", __func__, __LINE__);
    CDVPluginResult* pluginResult = [CDVPluginResult resultWithStatus:
            CDVCommandStatus_OK messageAsDictionary:[self getPlist]];
	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
	NSLog(@"%s:%d\n fetchParm finished", __func__, __LINE__);
}

-(NSManagedObject*)metricWithQueryParameters:(NSDictionary *)queries {
	NSLog(@"%s:%d\n Fetching metric with queries %@ ", __func__, __LINE__, queries);
    
	NSPredicate* predicate = [NSPredicate predicateWithFormat:@"metricID = %@", queries[@"id"]];
	NSArray* arr = [self fetchEntities:@"MetricJSON" predicate:predicate];
    
    if (arr == nil) {
        return nil;
    } else if ([arr count] == 0) {
        return nil;
    } else {
        return arr[0];
    }
}

-(NSError *)saveEntity:(NSString *)entityName arguments:(NSArray *)arguments attributes:(NSArray *)attributes {
//	NSLog(@"\n\n saveEntity with name %@ and arguments %@ and attributes %@", entityName, arguments, attributes);
    NSLog(@"%s:%d\n saveEntity with arguments and name %@", __func__, __LINE__, entityName);
    NSManagedObjectContext *context = [self getContext];

	NSManagedObject *newEntity = [NSEntityDescription insertNewObjectForEntityForName:entityName
															inManagedObjectContext:context];
    
    NSError* check = [self checkAttributes:[arguments objectAtIndex:0]];
    if (check != nil) {
        return check;
    }

	for (NSString *attribute in attributes) {
        NSString * value = [[arguments objectAtIndex:0] objectForKey:attribute];
		[newEntity setValue:value forKey:attribute];
	}

	NSError *error;
	[context save:&error];

	return error;
}

-(NSError *)saveEntity:(NSString *)entityName instance:(NSMutableDictionary *)instance attributes:(NSArray *)attributes {
    NSLog(@"%s:%d\n saveEntity with instance and name %@", __func__, __LINE__, entityName);
//	NSLog(@"\n\n saveEntity with name %@ and instance %@ and attributes %@", entityName, instance, attributes);
    NSManagedObjectContext *context = [self getContext];

	NSManagedObject *newEntity = [NSEntityDescription insertNewObjectForEntityForName:entityName
															inManagedObjectContext:context];
    
    NSError* check = [self checkAttributes:instance];
    if (check != nil) {
        return check;
    }
    
	for (NSString *attribute in attributes) {
		[newEntity setValue:[instance objectForKey:attribute] forKey:attribute];
	}

	NSError *error;
	[context save:&error];

	return error;
}

-(NSArray *)fetchEntities: (NSString *)entityName predicate:(NSPredicate*) predicate {
    
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSManagedObjectContext *context = [self getContext];
    NSEntityDescription *entity = [NSEntityDescription entityForName:entityName inManagedObjectContext:context];
    [fetchRequest setEntity:entity];
    [fetchRequest setReturnsObjectsAsFaults:NO];
    if (predicate != nil) {
        [fetchRequest setPredicate:predicate];
    }
    NSError *error;
    NSArray *result = [context executeFetchRequest:fetchRequest error:&error];
    if (error) {
        NSLog(@"Unable to execute fetch request.");
        NSLog(@"%@, %@", error, error.localizedDescription);
        
    } else {
        NSLog(@"Encrypted Results shown here");
        NSLog(@"%@", result);
    }
    return result;
}

-(NSError *)deleteEntity:(NSManagedObject *)objectToDelete {
	NSLog(@"%s:%d\n deleteEntity", __func__, __LINE__);
//	NSLog(@"\n\n deleteEntity with objectToDelete %@", objectToDelete);
    
	if (objectToDelete != nil) {
        NSLog(@"%s:%d\n Deleting Object %@", __func__, __LINE__,objectToDelete);
        NSManagedObjectContext *context = [self getContext];
        NSError *error;
		[context deleteObject:objectToDelete];
		[context save:&error];
		NSLog(@"%s:%d\n Was there an error? %@", __func__, __LINE__, [error localizedDescription]);
        return error;
	} else {
        NSLog(@"%s:%d\n Attempted to delete a null Object", __func__, __LINE__);
        return nil;
    }
}

-(NSMutableArray *) arrayToJSON:(NSArray *)managedObjects {
	NSMutableArray *dictionaryResults = [[NSMutableArray alloc] initWithCapacity:0];
    
	if (managedObjects == nil) {
		NSLog(@"%s:%d\n arrayToJSON managedObjects was nil", __func__, __LINE__);
	} else if ([managedObjects count] == 0) {
		NSLog(@"%s:%d\n arrayToJSON managedObjects had zero entries", __func__, __LINE__);
	} else {
		NSLog(@"%s:%d\n arrayToJSON %d entities found!", __func__, __LINE__, [managedObjects count]);
		NSLog(@"%s:%d\n arrayToJSON entities=%@", __func__, __LINE__, managedObjects);
		for (NSManagedObject* managedObject in managedObjects) {
			NSArray *attributes = [[managedObject.entity attributesByName] allKeys];
			NSDictionary *dict = [managedObject dictionaryWithValuesForKeys:attributes];
			[dictionaryResults addObject:dict];
		}
	}
	return dictionaryResults;
}

-(NSMutableDictionary *)serializeJSON:(NSArray *)keyArray dict:(NSDictionary *)jsonObjectDictionary {
	NSMutableDictionary *jsonStringDictionary = [[NSMutableDictionary alloc] init];
	NSError *serializeError;

	for (NSString* key in keyArray) {
		NSData *jsonData = [NSJSONSerialization
                            dataWithJSONObject:[jsonObjectDictionary objectForKey:key]
                            options:NSJSONWritingPrettyPrinted
                            error:&serializeError];

		if (serializeError != nil) {
			break;
		} else {
			NSString* jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
			[jsonStringDictionary setObject:jsonString forKey:key];
		}
	}
    return jsonStringDictionary;
}

-(NSData *)serializeJSON:(NSDictionary *)jsonObjectDictionary {
	NSError *serializeError;
    
		NSData *jsonData = [NSJSONSerialization
                            dataWithJSONObject:jsonObjectDictionary
                            options:NSJSONWritingPrettyPrinted
                            error:&serializeError];
    return jsonData;
}

-(NSString*) generateRandomInt {
	NSUInteger r = arc4random_uniform(1000000);
	return [NSString stringWithFormat:@"%d", r];
}

-(NSManagedObjectContext*) getContext {
    AppDelegate *appDelegate = [[UIApplication sharedApplication] delegate];
    return [appDelegate managedObjectContext];
}

-(NSDictionary *) getPlist {
    CDVViewController* viewController = [self getViewController];
    
    NSDictionary* plist = [viewController getPlist:@"Info"];
//    NSLog(@"getPList %@",plist);
    
    return plist;
}

-(CDVViewController*) getViewController {
    AppDelegate *appDelegate = [[UIApplication sharedApplication] delegate];
    return [appDelegate viewController];
}

-(NSString *)getKey:(NSString *)userId{
    NSString        *secretKey;
    NSString        *serviceName;
    NSString        *accountName;
    NSData          *secret;
    NSDictionary    *keychainQuery;
    OSStatus        status;
    NSData          *returnedData;
    CFTypeRef       dataTypeRef;
    NSString        *cryptoKey;
    
    
    serviceName = @"homesCryptoKey";
    accountName = userId;
    
    //
    // Get the data from the keychain
    //
    keychainQuery = @{(id)(__bridge id)kSecClass: (__bridge id)kSecClassGenericPassword,
                      (id)(__bridge id)kSecAttrService: serviceName,
                      (id)(__bridge id)kSecAttrAccount: accountName,
                      (id)(__bridge id)kSecReturnData: @YES,
                      };
    
    
    returnedData = NULL;
    dataTypeRef = (__bridge CFTypeRef)returnedData;
    
    status = SecItemCopyMatching((__bridge CFDictionaryRef)keychainQuery, &dataTypeRef);
    
    if(status == errSecItemNotFound) {
        //
        // Generate a unique secretKey for this user.
        //
        secretKey = [[NSString alloc]  initWithString:serviceName];
        secretKey = [secretKey stringByAppendingString:accountName];
        secretKey = [secretKey stringByAppendingFormat:@"%u", arc4random()];
        secret = [secretKey dataUsingEncoding:NSUTF8StringEncoding];
        
        //
        // Add the crypto key to the keychain.
        //
        keychainQuery = @{
                          (__bridge id)kSecClass: (__bridge id)kSecClassGenericPassword,
                          (__bridge id)kSecAttrService: serviceName,
                          (__bridge id)kSecAttrAccount: accountName,
                          (__bridge id)kSecValueData: secret,
                          };
        
        status = SecItemAdd((__bridge CFDictionaryRef)keychainQuery, NULL);
        
        NSAssert(status == errSecSuccess, @"Could not add crypto key to the keychain.");
        
        return secretKey;
    }
    else {
        if (status == errSecSuccess) {
            //
            // The call was successful, but we should make sure a crypto key was returned.
            //
            NSAssert(dataTypeRef != nil, @"No crypto key was returned from the keychain.");
            
            //
            // Now we can set the cryptoKey variable and return it.
            //
            returnedData = (__bridge NSData *)dataTypeRef;
            
            cryptoKey = [NSString stringWithUTF8String:[returnedData bytes]];
        }
        
        return cryptoKey;
    }
}

-(NSDictionary *)encryptDictionaryData:(NSDictionary *)dict withUserId:(NSString *)userId {
    NSString  *jsonKey;
    NSData    *encryptedResult;
    NSString  *key;
    NSError   *error = nil;
    NSMutableDictionary     *dictionaryToReturn;

    dictionaryToReturn = [[NSMutableDictionary alloc] initWithDictionary:dict];
    
    key = [self getKey:userId];
    
    //
    // Get values from jsonToProcess encrypt them and replace with encrypted values.
    //
    for(jsonKey in dictionaryToReturn.allKeys) {
        
        NSString  *value = [dict valueForKey:jsonKey];

        if(![value isEqualToString:@"userId"] && ![value isEqualToString:@"veteranLocalID"] && ![value isEqualToString:@"isDecisionSupport"]) {

            if([value isKindOfClass:[NSNumber class]]) {
                if([(NSNumber *)value boolValue]) {
                    value = @"true";
                }
                else {
                    value = @"false";
                }
            }
            
            encryptedResult = [RNEncryptor  encryptData: [value dataUsingEncoding:NSUTF8StringEncoding]
                                           withSettings:kRNCryptorAES256Settings
                                               password:key
                                                  error:&error];
            
            NSAssert(error == nil, @"Could not encrypt the data." );
            
            [dictionaryToReturn setValue:[encryptedResult base64EncodedStringWithOptions:0] forKey:jsonKey];
        }
    }

    return dictionaryToReturn;
}

-(NSMutableDictionary *)decryptDictionaryData:(NSMutableDictionary *)dict withUserId:(NSString *)userId {
    NSString  *jsonKey;
    NSData    *decryptedResult;
    NSString  *key;
    NSError   *error = nil;
    
    
    key = [self getKey:userId];

    
    //
    // Get values from jsonToProcess encrypt them and replace with encrypted values.
    //
    for(jsonKey in dict.allKeys) {
        
        NSString  *value = [dict valueForKey:jsonKey];
        
        if(![value isEqualToString:@"userId"] && ![value isEqualToString:@"veteranLocalID"] && ![value isEqualToString:@"isDecisionSupport"]) {

            decryptedResult = [RNDecryptor  decryptData:[[NSData alloc] initWithBase64EncodedString:value options:0]
                                           withSettings:kRNCryptorAES256Settings
                                               password:key
                                                  error:&error];
            
            NSAssert(error == nil, @"Could not decrypt the data." );
            
            NSString  *tmpStr = [[NSString alloc] initWithData:decryptedResult encoding:NSASCIIStringEncoding];
            
            //
            // Look for "true" or "false" and convert to a boolean value.
            //
            if([tmpStr isEqualToString: @"true"]) {
                [dict setValue:[[NSNumber alloc] initWithBool:YES] forKey:jsonKey];
            }
            else {
                if([tmpStr isEqualToString: @"false"]) {
                    [dict setValue:[[NSNumber alloc] initWithBool:NO] forKey:jsonKey];
                }
                else {
                    [dict setValue:[[NSString alloc] initWithData:decryptedResult encoding:NSASCIIStringEncoding] forKey:jsonKey];
                }
            }
        }
    }

    return dict;
}

-(NSDictionary *)getDictionaaryFromManagedObject:(NSManagedObject *)obj {

    NSArray *keys = [[[obj entity] attributesByName] allKeys];
    NSDictionary *dict = [obj dictionaryWithValuesForKeys:keys];
    
    return dict;
}

-(void)setDictionaaryInManagedObject:(NSManagedObject *)obj withDictionary:(NSDictionary *)dict {
    
    [obj setValuesForKeysWithDictionary:dict];
}

@end
