#import "KeyboardNotificationPlugin.h"
#import "AppDelegate.h"
#import "MainViewController.h"
#import <Cordova/CDVViewController.h>

//iOS7 statusbar lays on top of content, hence 0px height
const int kSTATUS_BAR_HEIGHT = 0;
BOOL prevIsLandscape = NO;

@implementation KeyboardNotificationPlugin

//Keyboard notifications.
//  register event handlers
- (void)keyboardShow:(CDVInvokedUrlCommand*)command {
    [[NSNotificationCenter defaultCenter] removeObserver:self
        name:UIKeyboardDidShowNotification
        object:nil];
    self.keyboardShowcallbackId = command.callbackId;
    [[NSNotificationCenter defaultCenter] addObserver:self
        selector:@selector(keyBoardShowHandler:)
        name:UIKeyboardDidShowNotification
        object:nil];
}

- (void)keyboardHide:(CDVInvokedUrlCommand*)command {
    [[NSNotificationCenter defaultCenter] removeObserver:self
        name:UIKeyboardDidHideNotification
        object:nil];
    self.keyboardHidecallbackId = command.callbackId;
    [[NSNotificationCenter defaultCenter] addObserver:self
        selector:@selector(keyBoardHideHandler:)
        name:UIKeyboardDidHideNotification
        object:nil];
}

// event handlers
- (void)keyBoardShowHandler:(NSNotification*)notification  {
    NSLog(@"%s:%d\n keyBoardShowHandler", __func__, __LINE__);
    NSValue* keyboardValue = (NSValue*)[notification.userInfo objectForKey:UIKeyboardFrameEndUserInfoKey];
    int height = [self resizeWithKeyboard:keyboardValue];
    NSLog(@"%s:%d\n show keyboard returning height %i", __func__, __LINE__, height);
    
    if (self.keyboardShowcallbackId) {
        CDVPluginResult* result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsInt:height];

        [result setKeepCallbackAsBool:YES];
        [self.commandDelegate sendPluginResult:result callbackId:self.keyboardShowcallbackId];
    }
}

- (void)keyBoardHideHandler:(NSNotification*)notification {
    NSLog(@"%s:%d\n keyBoardHideHandler", __func__, __LINE__);
    int height = [self resizeWithoutKeyboard];
    NSLog(@"%s:%d\n hide keyboard returning height %i", __func__, __LINE__, height);
    
    if (self.keyboardHidecallbackId) {
        CDVPluginResult* result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsInt:height];
        [result setKeepCallbackAsBool:YES];

        [self.commandDelegate sendPluginResult:result callbackId:self.keyboardHidecallbackId];
    }
}

-(int)resizeWithKeyboard:(NSValue*)keyboardValue {
    CGRect keyboardRect = keyboardValue.CGRectValue;
    CGFloat keyboardWidth = keyboardRect.size.width;
    CGFloat keyboardHeight = keyboardRect.size.height;
    
    return [self resize:(int)keyboardWidth keyboardHeight:(int)keyboardHeight];
}

-(int)resizeWithoutKeyboard {
    return [self resize:0 keyboardHeight:0];
}

-(int)resize:(int)keyboardWidth keyboardHeight:(int)keyboardHeight {
    CGRect appBounds = [[UIScreen mainScreen] bounds];
    
    int height = 0;
    if ([self isLandscape]) {
        NSLog(@"%s:%d\n orientation landscape", __func__, __LINE__);
        height = (int)(appBounds.size.width);
        height = height - keyboardWidth;
        [self resizeViewport:height keyboard: keyboardWidth];
    } else {
        NSLog(@"%s:%d\n orientation portrait", __func__, __LINE__);
        height = (int)(appBounds.size.height);
        height = height - keyboardHeight;
        [self resizeViewport:height keyboard: keyboardHeight];
    }
    return height - kSTATUS_BAR_HEIGHT;
}

- (void)resizeViewport: (int)size keyboard:(int)keyboard {
    NSLog(@"%s:%d\n resizeViewport size%i keyboard=%i", __func__, __LINE__,size,keyboard);
    
    CDVViewController* viewController = [self getViewController];
    CGRect resizedFrame = [viewController.webView frame];

    NSLog(@"%s:%d\n Before calculations:", __func__, __LINE__);
    [self report];

    resizedFrame.size.height = size;
    [viewController.webView  setFrame:resizedFrame];
    
    NSLog(@"%s:%d\n After calculations:", __func__, __LINE__);
    [self report];
}

- (void)screenHeight:(CDVInvokedUrlCommand*)command {
    
    CGRect screen = [UIScreen mainScreen].bounds;
    CGFloat height;
    
    NSLog(@"%s:%d\n screenHeight calling isLandscape", __func__, __LINE__);
    if ([self isLandscape]) {
        height = screen.size.width;
    } else {
        height = screen.size.height;
    }
    
    int heightInt = (int)height;
    
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsInt:heightInt];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    
    NSLog(@"%s:%d\n screenHeight:", __func__, __LINE__);
    [self report];
}

- (void)init:(CDVInvokedUrlCommand*)command {
    int height = [self resizeWithoutKeyboard];
    
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsInt:height];
    [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    
    NSLog(@"%s:%d\n init:", __func__, __LINE__);
    [self report];
}

- (BOOL)isLandscape {
    // this one uses the accelerometer to determine orientation which fails if the device is lying flat
//    UIDeviceOrientation orientation = [[UIDevice currentDevice] orientation];
    UIInterfaceOrientation orientation = [UIApplication sharedApplication].statusBarOrientation;

//    BOOL generates = [[UIDevice currentDevice] isGeneratingDeviceOrientationNotifications];
//    NSLog(@"generates = %s", generates ? "generates notifications" : "does NOT generate notifications");
    
    if (UIDeviceOrientationIsPortrait(orientation)) {
        NSLog(@"orientation = %s", prevIsLandscape ? "landscape" : "portrait");
        prevIsLandscape = NO;
        return NO;
    } else if (UIDeviceOrientationIsLandscape(orientation)) {
        NSLog(@"orientation = %s", prevIsLandscape ? "landscape" : "portrait");
        prevIsLandscape = YES;
        return YES;
    } else {
        // this never happens now that we use status bar orientation
        NSLog(@"could not determine orientation, returning previous value = %s", prevIsLandscape ? "landscape" : "portrait");
        return prevIsLandscape;
    }
}

- (void) report {
    CDVViewController* viewController = [self getViewController];
    CGPoint center = [viewController.view center];  ;
    
    CGRect viewBounds = [viewController.view  bounds];
    CGRect viewFrame = [viewController.view  frame];
    CGRect webViewBounds = [viewController.webView  bounds];
    CGRect webViewFrame = [viewController.webView  frame];
    CGRect appFrame = [[UIScreen mainScreen] applicationFrame];
    CGRect appBounds = [[UIScreen mainScreen] bounds];
        
    NSLog(@"center           %@", NSStringFromCGPoint(center));
    
    NSLog(@"app     frame    %@", NSStringFromCGRect(appFrame));
    NSLog(@"app     bounds   %@", NSStringFromCGRect(appBounds));
    
    NSLog(@"view    frame    %@", NSStringFromCGRect(viewFrame));
    NSLog(@"view    bounds   %@", NSStringFromCGRect(viewBounds));
    
    NSLog(@"webView frame    %@", NSStringFromCGRect(webViewFrame));
    NSLog(@"webView bounds   %@", NSStringFromCGRect(webViewBounds));
    
}

-(CDVViewController*) getViewController {
    AppDelegate *appDelegate = [[UIApplication sharedApplication] delegate];
    return [appDelegate viewController];
}

@end
