# GETTING STARTED
# -----------------
# This documentation is intended to show you how to get started with a
# simple Selenium-Webdriver test.  This test is written with RSpec but the
# webdriver commands (everything called after @driver) will work with any
# testing framework.
#
# INSTALLING RVM
# --------------
# We're assuming you've got rvm installed, but if not, from a terminal
# run the following line (removing the ""'s):
#
# "\curl -L https://get.rvm.io | bash -s stable --ruby"
#
# INSTALLING GEMS
# ---------------
# Then, change to the example directory:
#   "cd appium-location/sample-code/examples/ruby"
#
# and install the required gems with bundler by doing:
#   "bundle install"
#
# RUNNING THE TESTS
# -----------------
# To actually run the tests, make sure appium is running in another terminal
# window, then from the same window you used for the above commands, type
#   "rspec simple_test.rb"
#
# It will take a while, but once it's done you should get nothing but a line telling you "1 example, 0 failures".

require 'rspec'
require 'selenium-webdriver'

#APP_PATH = '/Users/borsts/Documents/Code/appium/sample-code/apps/TestApp/build/release-iphonesimulator/TestApp.app'
#APP_PATH = '/Users/borsts/Library/Developer/Xcode/DerivedData/mHOMES-dbvwsrvdsnsrzreuwpoqkhxfkplv/Build/Products/   ug-iphonesimulator/mHOMES.app'
#APP_PATH = ENV['APP_PATH']


APP_PATH = File.expand_path('../build/mHOMES.app', File.dirname(__FILE__))

print APP_PATH

def absolute_app_path
    file = File.join("", APP_PATH)
    raise "App doesn't exist #{file}" unless File.exist? file
    file
end

capabilities = {
  'device' => 'iPad Simulator',
  'browserName' => 'iOS',
  'platform' => 'Mac',
  'version' => '7.0',
  'app' => absolute_app_path
}

server_url = "http://127.0.0.1:4723/wd/hub"

describe "HOMES app" do
  before :all do
    @driver = Selenium::WebDriver.for(:remote, :desired_capabilities => capabilities, :url => server_url)
    @driver.manage.timeouts.implicit_wait = 10 # seconds

    webview = @driver.window_handles.last
    @driver.switch_to.window(webview)

   end

  after :all do
	@driver.execute_script("mobile: leaveWebView")

    @driver.quit
  end

  ssn = rand(999999999).to_s.rjust(9, '0')

#   it "should accept the EULA" do
#     @driver.find_element(:id, 'accept-btn').click
# 	sleep(1)
#
#   end
#
  it "should log in to HOMES" do

	title = @driver.find_elements(:tag_name => 'h1')
	title[1].text.should eql("Homeless Mobile Intervention")
	title[2].text.should eql("Homes Login")

    @driver.find_element(:id, "username").send_keys "MDM\\homescasemanager"
    @driver.find_element(:id, "password").send_keys "Agilexadmin99$"

    @driver.find_element(:id, 'loginButton').click
    @driver.find_element(:id, 'search-field')

  end
# 
#   it "should accept the SUD" do
# 	wait = Selenium::WebDriver::Wait.new :timeout => 10
# 	wait.until { @driver.find_element(:id, "accept-btn").displayed? }
# 
#     @driver.find_element(:id, 'sud')
#     @driver.find_element(:id, 'accept-btn').click
# 	sleep(1)
# 
#   end

  it "should be able to add a Veteran" do
	sleep(3)
  	@driver.find_element(:id, "search-field").send_keys ssn + "\n"
#   	@driver.find_element(:id, "search-field").send_keys "\n"
  	@driver.find_element(:id, 'add-btn').click

    genderListbox = @driver.find_element(:id, "gender")
	all_options = genderListbox.find_elements(:tag_name, "option")
	all_options.each do |option|
	  ["Other", "Male", "Female"].include?option.attribute("value")
    end

  	@driver.find_element(:id, "lastName").send_keys "ZZHOMES"

  	dateTime = Time.new.strftime("%Y-%m-%d@%H:%M:%S")
  	@driver.find_element(:id, "firstName").send_keys "AUTO" + dateTime
  	@driver.find_element(:id, "middleInitial").send_keys "X"

  	@driver.find_element(:id, "sSN").send_keys ssn
    @driver.find_element(:name, 'ssnConfirm').click

# 	@driver.execute_script("mobile: leaveWebView")
# 
# 	@driver.find_element(:xpath, "//window[1]/scrollview[1]/webview[1]/text[@name='Date of Birth *']").click
# 
# 	@driver.find_element(:xpath, "//window[1]/popover[1]/picker[1]/pickerwheel[1]").send_keys "January"
# 	@driver.find_element(:xpath, "//window[1]/popover[1]/picker[1]/pickerwheel[2]").send_keys "1"
# 	@driver.find_element(:xpath, "//window[1]/popover[1]/picker[1]/pickerwheel[3]").send_keys "1990"
# 
#     webview = @driver.window_handles.last
#     @driver.switch_to.window(webview)

    @driver.find_element(:id, 'dateOfBirth').click
	@driver.find_element(:id, 'date').click
	@driver.find_element(:xpath, '//b[@class=\'month\']/ul/li/button[@data-value=\'0\']').click
	@driver.find_element(:xpath, '//b[@class=\'day\']/ul/li/button[@data-value=\'1\']').click
	@driver.find_element(:xpath, '//b[@class=\'year\']/ul/li/button[@data-value=\'1990\']').click
	@driver.find_element(:id, 'set-btn').click
	@driver.find_element(:id, 'apply-btn').click

    gender = Selenium::WebDriver::Support::Select.new(genderListbox)
    gender.select_by(:text, "Female")

  	@driver.find_element(:id, 'add-btn').click

	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:id, 'ok-btn').displayed? }
  	@driver.find_element(:id, 'ok-btn').click

  end

  def check_visibility(elements, visibility)
    elements.each { |element| @driver.find_element(:id, element).displayed?.should eql(visibility) }
  end

  def check_contents(elements, responses)
    elements.each do |element|
	  responses.each do |response|
        xpath = "//ul[@id=\"" + element + "\"]/li/div[1]/div[1]/a[text()=\"" + response + "\"]"
        @driver.find_element(:xpath, xpath)
      end
    end
  end

  def check_contents_numeric(elements, from, to)
    elements.each do |element|
	  for response in from..to do
        xpath = "//ul[@id=\"" + element + "\"]/li/div[1]/div[1]/a[text()=\"" + response.to_s + "\"]"
        @driver.find_element(:xpath, xpath)
      end
    end
  end

  def check_options_numeric(elements, from, to)
    elements.each do |element|
	  for response in from..to do
        xpath = "//ul[@id=\"" + element + "\"]/li/div[1]/div[1]/a[text()=\"" + response.to_s + "\"]"
        @driver.find_element(:xpath, xpath)
      end
    end
  end

  def check_listbox_numeric(elements, from, to)
    elements.each do |element|
	  option = Selenium::WebDriver::Support::Select.new(@driver.find_element(:xpath => "//select[@id=\"" + element + "\"]"))
	  for response in from..to 
	  	print response.to_s
	  	option.select_by(:text, response.to_s)
	  end
    end
  end

  def clear_and_sendkeys(question_id, value)
  	sleep(1)
  	@driver.find_element(:id, question_id).clear
	@driver.find_element(:id, question_id).send_keys value
  end

  def select_from_listbox(question_id, value)
#   	@driver.find_element(:id, question_id + '-button').click
	option = Selenium::WebDriver::Support::Select.new(@driver.find_element(:xpath => "//select[@id=\"" + question_id + "\"]"))
	option.select_by(:text, value);
	sleep(1)
  end

  def select_from_multiselect(question_id, value)
#   	@driver.find_element(:id, question_id + '-button').click
	@driver.find_element(:xpath, "//ul[@id=\"" + question_id + "-menu\"]/li/div[1]/div[1]/a[text()=\"" + value + "\"]").click
# 	@driver.find_element(:xpath, "//select[@id='" + question_id + "']/option[text()='" + value + "']").click
	sleep(1)
  end

  def click_with_retry(button_id, max_tries, element)
    tries = 0
    begin
	  sleep(1)
      tries = tries + 1
      @driver.find_element(:id, button_id).click
      stop_condition = (tries == max_tries ? true : false)
      stop_condition = (@driver.find_element(:id, element).displayed?.eql? true ? true: stop_condition)
    rescue Exception=>e
      stop_condition = true
    end until stop_condition == true
    sleep(1)
  end

  it "should be able to save and resume an assessment" do
	sleep(1)
	click_with_retry("search-btn", 25, "search-field")
  	@driver.find_element(:id, "search-field").send_keys ssn + "\n"
# 	@driver.find_element(:class, "line-item").click
	@driver.find_element(:xpath, "//div[@id='homesListView']/div[@data-role='content']/ul/li[@data-index='0']/div/div/div/div").click
	@driver.find_element(:xpath, "//div[@data-role='footer']").click

	sleep(1)
	click_with_retry("assessment-btn", 10, "questionForm")
# 	@driver.find_element(:id, 'assessment-btn').click

	sleep(1)
	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:class, "title").displayed? }

	@driver.find_element(:class => "title").text.should eql("PRE-ENGAGEMENT SCREENING")

	select_from_listbox("ddlHSA_Q5", "Yes")
	sleep(2)
	select_from_listbox("ddlHSA_Q6", "Yes")
	sleep(2)
	click_with_retry("next-btn", 30, "ddlHSA_Q7")
#	@driver.find_element(:id, 'next-btn').click

	@driver.find_element(:class => "title").text.should eql("ASSESSMENT INTERVIEW")

	select_from_listbox("ddlHSA_Q7", "Native Hawaiian or Other Pacific Islander")
	select_from_listbox("ddlHSA_Q8", "Hispanic/Latino")
	select_from_listbox("ddlHSA_Q9", "Separated")

	sleep(1)
	click_with_retry("contact-btn", 5, "edit-btn")
	sleep(1)
	click_with_retry("search-btn", 25, "search-field")

	sleep(1)
# 	@driver.find_element(:id, 'search-btn').click
  	@driver.find_element(:id, "search-field").send_keys "666001001\n"
# 	@driver.find_element(:class, "line-item").click
	@driver.find_element(:xpath, "//div[@id='homesListView']/div[@data-role='content']/ul/li[@data-index='0']/div/div/div/div").click
	@driver.find_element(:xpath, "//div[@data-role='footer']").click

	sleep(1)
	click_with_retry("assessment-btn", 25, "questionForm")
# 	@driver.find_element(:id, 'assessment-btn').click
	sleep(1)
	@driver.find_element(:id, 'ok-btn').click

	sleep(1)
# 	@driver.find_element(:class => "title").text.should eql("PRE-ENGAGEMENT SCREENING")

  	@driver.find_element(:id, 'saved-data').click
  	@driver.find_element(:xpath, '//ul[@id="assessmentsList"]/li/div/div[1]/div[2]').click

# 	sleep(1)
# 	click_with_retry("contact-btn", 5, "edit-btn")
# 	sleep(1)
# 	click_with_retry("search-btn", 25, "fullSSNTab")
#   	@driver.find_element(:id, 'fullSSNTab').click
#   	@driver.find_element(:id, "search-field").send_keys ssn + "\n"
# 	@driver.find_element(:class, "line-item").click
#
# 	sleep(1)
# 	click_with_retry("assessment-btn", 25, "questionForm")

	sleep(1)
	@driver.find_element(:class => "title").text.should eql("ASSESSMENT INTERVIEW")

	@driver.find_element(:xpath, "//a[@id='ddlHSA_Q7-button']/span[1]/span[1]/span[text()='Native Hawaiian or Other Pacific Islander']")
	@driver.find_element(:xpath, "//a[@id='ddlHSA_Q8-button']/span[1]/span[1]/span[text()='Hispanic/Latino']")
	@driver.find_element(:xpath, "//a[@id='ddlHSA_Q9-button']/span[1]/span[1]/span[text()='Separated']")
	@driver.find_element(:id, 'prev-btn').click
  end

  it "should be able to fill out the pre-engagement section of an assessment" do
	select_from_listbox("ddlHSA_Q5", "No")
	select_from_listbox("ddlHSA_Q6", "No")

# 	sleep(1)
# 	click_with_retry("search-btn", 25, "fullSSNTab")
# # 	@driver.find_element(:id, 'search-btn').click
#
#
#   	@driver.find_element(:id, 'fullSSNTab').click
#   	@driver.find_element(:id, "search-field").send_keys ssn + "\n"
# 	@driver.find_element(:class, "line-item").click
#
# 	wait = Selenium::WebDriver::Wait.new :timeout => 10
# 	wait.until { @driver.find_element(:id, "assessment-btn").displayed? }
#
# # 	assessment_btn = @driver.find_element(:id, 'assessment-btn')
# # 	10.times { assessment_btn.click }
# 	click_with_retry("assessment-btn", 15, "questionForm")
# # 	sleep(1)
# # 	click_with_retry("assessment-btn", 10, "questionForm")

	@driver.find_element(:class => "title").text.should eql("PRE-ENGAGEMENT SCREENING")
	@driver.find_element(:class => "sectionCount").text.should eql("Section 1/8")

	check_visibility(["ddlHSA_Q5a-button", "ddlHSA_Q5b-button", "ddlHSA_Q5c-button", "ddlHSA_Q5d-button", "ddlHSA_Q5e-button", "ddlHSA_Q5f-button", "ddlHSA_Q5g-button"], true)
	check_contents(["ddlHSA_Q5a-menu", "ddlHSA_Q5b-menu", "ddlHSA_Q5c-menu", "ddlHSA_Q5d-menu", "ddlHSA_Q5e-menu", "ddlHSA_Q5f-menu", "ddlHSA_Q5g-menu"], ["No", "Yes", "Veteran declined to answer", "Interviewer omitted item"])
	check_contents(["ddlHSA_Q5-menu", "ddlHSA_Q6-menu", "ddlHSA_Q6b-menu"], ["No", "Yes"])

	select_from_listbox("ddlHSA_Q5", "No")
	check_visibility(["ddlHSA_Q5a-button", "ddlHSA_Q5b-button", "ddlHSA_Q5c-button", "ddlHSA_Q5d-button", "ddlHSA_Q5e-button", "ddlHSA_Q5f-button", "ddlHSA_Q5g-button"], true)

	select_from_listbox("ddlHSA_Q5", "Yes")
	check_visibility(["ddlHSA_Q5a-button", "ddlHSA_Q5b-button", "ddlHSA_Q5c-button", "ddlHSA_Q5d-button", "ddlHSA_Q5e-button", "ddlHSA_Q5f-button", "ddlHSA_Q5g-button"], false)
	check_visibility(["ddlHSA_Q6a-button", "ddlHSA_Q6b-button", "ddlHSA_Q6d-button"], true)
	check_visibility(["ddlHSA_Q6c-button", "txtHSA_Q6c_other"], false)

	select_from_listbox("ddlHSA_Q6", "Yes")
	check_visibility(["ddlHSA_Q6a-button", "ddlHSA_Q6b-button", "ddlHSA_Q6c-button", "txtHSA_Q6c_other", "ddlHSA_Q6d-button"], false)

	select_from_listbox("ddlHSA_Q6", "No")
	check_visibility(["ddlHSA_Q6a-button", "ddlHSA_Q6b-button", "ddlHSA_Q6d-button"], true)
	check_visibility(["ddlHSA_Q6c-button", "txtHSA_Q6c_other"], false)

	select_from_listbox("ddlHSA_Q6b", "Yes")
	check_visibility(["ddlHSA_Q6a-button", "ddlHSA_Q6b-button", "ddlHSA_Q6c-button", "ddlHSA_Q6d-button"], true)
	check_visibility(["txtHSA_Q6c_other"], false)

	select_from_multiselect("ddlHSA_Q6c", "Other")
	select_from_multiselect("ddlHSA_Q6c", "Non-VA housing")
	select_from_multiselect("ddlHSA_Q6c", "Done")

	check_visibility(["txtHSA_Q6c_other"], true)
# 	Add check for selected items here
# 	@driver.find_element(:xpath, "//a[@id='ddlHSA_Q6c-button']/span[1]/span[1]/span[text()='Non-VA housing, Other']")

	@driver.find_element(:id, 'next-btn').click

	@driver.find_element(:xpath, "//div[@class='ui-field-contain ui-body ui-br flagged']/div/div/label[@class='error']").text.should eql("Please provide the reason that the Assessment form will not be completed.")

	select_from_listbox("ddlHSA_Q6a", "Veteran will not consent to interview")
	@driver.find_element(:id, 'txtHSA_Q6c_other').send_keys "Food"
	select_from_listbox("ddlHSA_Q6d", "No")
	select_from_listbox("ddlHSA_Q6d", "No")
	sleep(1)

	@driver.find_element(:id, 'next-btn').click

	sleep(1)
	@driver.find_element(:class => "title").text.should eql("INTERVIEWER INFORMATION")
	@driver.find_element(:class => "sectionCount").text.should eql("Section 8/8")

	@driver.find_element(:id, 'prev-btn').click
	sleep(1)
	@driver.find_element(:class => "title").text.should eql("PRE-ENGAGEMENT SCREENING")

	select_from_listbox("ddlHSA_Q6", "Yes")

	@driver.find_element(:id, 'next-btn').click

  end

  it "should be able to fill out the Interview section of an assessment" do
	@driver.find_element(:class => "title").text.should eql("ASSESSMENT INTERVIEW")
	@driver.find_element(:class => "sectionCount").text.should eql("Section 2/8")

	check_visibility(["ddlHSA_Q7-button", "ddlHSA_Q8-button", "ddlHSA_Q9-button", "txtHSA_Q10-button", "txtHSA_Q10a-button", "txtHSA_Q11-button"], true)
    check_contents(["ddlHSA_Q7-menu"], ["American Indian or Alaskan", "Asian", "Black or African American", "Native Hawaiian or Other Pacific Islander", "White", "Don’t know", "Veteran declined to answer", "Interviewer omitted item"])
    check_contents(["ddlHSA_Q8-menu"], ["Non-Hispanic/Non-Latino", "Hispanic/Latino", "Don’t know", "Veteran declined to answer", "Interviewer omitted item"])
    check_contents(["ddlHSA_Q9-menu"], ["Married", "Remarried", "Widowed", "Separated", "Divorced", "Never married", "Committed relationship/partnered", "Veteran declined to answer", "Interviewer omitted item"])

	select_from_listbox("ddlHSA_Q7", "Native Hawaiian or Other Pacific Islander")
	select_from_listbox("ddlHSA_Q8", "Hispanic/Latino")
	select_from_listbox("ddlHSA_Q9", "Separated")

# 	clear_and_sendkeys("txtHSA_Q10", "N")
# 	check_listbox_numeric(["txtHSA_Q10", "txtHSA_Q10a"], 0, 20)
	select_from_listbox("txtHSA_Q10", "N")
	sleep(1)
	check_visibility(["txtHSA_Q10a-button"], true)
# 	@driver.find_element(:id, 'txtHSA_Q10a').text.should eql("N")

# 	clear_and_sendkeys("txtHSA_Q10", "0")
	select_from_listbox("txtHSA_Q10", "0")
	sleep(1)
	check_visibility(["txtHSA_Q10a-button"], false)

	select_from_listbox("txtHSA_Q10", "4")
	sleep(1)
	check_visibility(["txtHSA_Q10a-button"], true)
# 	check_listbox_numeric(["txtHSA_Q10a"], 0, 4)
	select_from_listbox("txtHSA_Q10a", "2")

# 	clear_and_sendkeys("txtHSA_Q11", "21")
# 	@driver.find_element(:id, 'next-btn').click
#
# 	@driver.find_element(:xpath, "//div[@class='ui-field-contain ui-body ui-br flagged']/div/div/label[@class='error']").text.should eql("Years of formal education must be between 0-20 or code N if Veteran refused to answer")

# 	clear_and_sendkeys("txtHSA_Q11", "20")
# 	check_listbox_numeric(["txtHSA_Q11"], 0, 20)
	select_from_listbox("txtHSA_Q11", "16")
	sleep(1)

	@driver.find_element(:id, 'next-btn').click

  end

  it "should be able to fill out the Military History section of an assessment" do
	@driver.find_element(:class => "title").text.should eql("MILITARY HISTORY")
	@driver.find_element(:class => "sectionCount").text.should eql("Section 3/8")

	check_visibility(["txtHSA_Q12a", "txtHSA_Q12b", "ddlHSA_Q13-button", "ddlHSA_Q14-button", "ddlHSA_Q15-button", "txtHSA_Q16", "ddlHSA_Q17-button", "ddlHSA_Q19-button"], true)
	check_visibility(["ddlHSA_Q18a-button", "ddlHSA_Q18b-button", "ddlHSA_Q18c-button", "ddlHSA_Q18d-button", "ddlHSA_Q18e-button", "ddlHSA_Q18f-button", "ddlHSA_Q18g-button", "ddlHSA_Q18h-button", ], true)

    check_contents(["ddlHSA_Q13-menu"], ["Army", "Navy", "Marines", "Air Force", "Coast Guard", "Veteran declined to answer", "Interviewer omitted item"])
    check_contents(["ddlHSA_Q14-menu"], ["Active Duty (Regular)", "National Guard", "Reserves", "Veteran declined to answer", "Interviewer Omitted Item"])
    check_contents(["ddlHSA_Q15-menu"], ["Enlisted", "Warrant Officer", "Commissioned Officer", "Veteran declined to answer", "Interviewer Omitted Item"])
    check_contents(["ddlHSA_Q17-menu"], ["No", "Active duty in military", "Active in Reserves", "Active in National Guard", "Veteran declined to answer", "Interviewer Omitted Item"])
    ["a", "b", "c", "d", "e", "f", "g", "h"].each { |element| check_contents(["ddlHSA_Q18" + element + "-menu"], ["No", "Yes", "Veteran declined to answer", "Interviewer omitted item"]) }
    check_contents(["ddlHSA_Q19-menu"], ["No", "Yes", "Veteran declined to answer", "Interviewer omitted item"])

# 	@driver.find_element(:id, 'txtHSA_Q12a').send_keys "A"
# 	@driver.find_element(:id, 'next-btn').click
# 	@driver.find_element(:xpath, "//div[@class='ui-field-contain ui-body ui-br flagged']/div/div/label[@class='error']").text.should eql("Please enter a valid year.")

# 	@driver.find_element(:id, 'txtHSA_Q12a').send_keys "1990"
# 	clear_and_sendkeys("txtHSA_Q12a", "1990")
# 	check_listbox_numeric(["txtHSA_Q12a"], 2007, 2014)
	select_from_listbox("txtHSA_Q12a", "2010")

# 	@driver.find_element(:id, 'txtHSA_Q12b').send_keys "A"
# 	@driver.find_element(:id, 'next-btn').click
# 	@driver.find_element(:xpath, "//div[@class='ui-field-contain ui-body ui-br flagged']/div/div/label[@class='error']").text.should eql("Please enter a valid year.")

	sleep(1)
# 	click_with_retry("assessment-btn", 10, "questionForm")
# 	@driver.find_element(:id, 'txtHSA_Q12b').send_keys "1994"
# 	clear_and_sendkeys("txtHSA_Q12b", "1994")
# 	check_listbox_numeric(["txtHSA_Q12b"], 2010, 2014)
	select_from_listbox("txtHSA_Q12b", "2012")

	sleep(1)
	click_with_retry("assessment-btn", 10, "questionForm")
	sleep(1)
	select_from_listbox("ddlHSA_Q13", "Army")
	select_from_listbox("ddlHSA_Q14", "National Guard")
	select_from_listbox("ddlHSA_Q15", "Enlisted")

	sleep(1)
# 	@driver.find_element(:id, 'txtHSA_Q16').send_keys "10"
# 	check_listbox_numeric(["txtHSA_Q16"], 1, 9)
	select_from_listbox("txtHSA_Q16", "8")
# 	@driver.find_element(:id, 'next-btn').click
	sleep(1)
# 	@driver.find_element(:xpath, "//div[@class='flagged']/div/div/label[@class='error']").text.should eql("Item 15 is Enlisted, E-rating must be between 1-9 or N if unknown or Veteran declined to answer.")

	select_from_listbox("ddlHSA_Q15", "Warrant Officer")
	sleep(1)
# 	clear_and_sendkeys("txtHSA_Q16", "6")
# 	check_listbox_numeric(["txtHSA_Q16"], 1, 5)
	select_from_listbox("txtHSA_Q16", "4")
# 	@driver.find_element(:id, 'next-btn').click
# 	@driver.find_element(:xpath, "//div[@class='ui-field-contain ui-body ui-br flagged']/div/div/label[@class='error']").text.should eql("Item 15 is Warrant Officer, W-rating must be between 1-5 or N if unknown or Veteran declined to answer.")

	select_from_listbox("ddlHSA_Q15", "Commissioned Officer")
	sleep(1)
# 	clear_and_sendkeys("txtHSA_Q16", "11")
# 	check_listbox_numeric(["txtHSA_Q16"], 1, 10)
	select_from_listbox("txtHSA_Q16", "9")
# 	@driver.find_element(:id, 'next-btn').click
# 	@driver.find_element(:xpath, "//div[@class='ui-field-contain ui-body ui-br flagged']/div/div/label[@class='error']").text.should eql("Item 15 is Commissioned Officer, C-rating must be between 1-10 or N if unknown or Veteran declined to answer.")

# 	sleep(1)
# 	click_with_retry("assessment-btn", 10, "questionForm")
# 	clear_and_sendkeys("txtHSA_Q16", "10")

# 	sleep(1)
# 	click_with_retry("assessment-btn", 10, "questionForm")
	select_from_listbox("ddlHSA_Q17", "Active in National Guard")

	["a", "b", "c", "d", "e", "f", "g", "h"].each { |element| @driver.find_element(:xpath, "//a[@id='ddlHSA_Q18" + element + "-button']/span[1]/span[1]/span[text()='No']") }

	select_from_listbox("ddlHSA_Q18a", "Yes")
	select_from_listbox("ddlHSA_Q19", "Yes")

	@driver.find_element(:id, 'next-btn').click

  end

  it "should be able to fill out the Living Situation section of an assessment" do
	@driver.find_element(:class => "title").text.should eql("LIVING SITUATION")
	@driver.find_element(:class => "sectionCount").text.should eql("Section 4/8")

	check_visibility(["ddlHSA_Q20-button", "txtHSA_Q21-button", "ddlHSA_Q23-button", "ddlHSA_Q23a-button", "ddlHSA_Q24-button", "ddlHSA_Q26-button", "ddlHSA_Q27-button"], true)
	check_visibility(["ddlHSA_Q25-button"], false)
    ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t"].each { |element| check_visibility(["txtHSA_Q20" + element], true) }
	check_visibility(["txtHSA_Q22", "txtHSA_Q23b", "txtHSA_Q23c", "txtHSA_Q23d"], true)

    check_contents(["ddlHSA_Q20-menu"], ["Veteran responded", "Veteran declined to answer", "Interviewer omitted item"])
    check_contents(["txtHSA_Q21-menu"], ["Housing owned by Veteran, no ongoing housing subsidy", "Housing owned by Veteran, with ongoing housing subsidy", "Housing rented by Veteran, no ongoing housing subsidy", "Housing rented by Veteran with HUD-VASH voucher", "Housing rented by Veteran with non-HUD-VASH housing subsidy", "Permanent housing for formerly homeless persons (such as SHP, S+C, or SRO MOD Rehab)", "Staying or living in family member’s room, apartment or house", "Staying or living in a friend’s room, apartment or house", "GPD transitional housing", "Non-VA transitional housing for homeless persons", "Safe Haven (special transitional supportive housing or drop-in supportive service center for homeless SMI individuals)", "VA MH RRTP [all types: DCHV, CWT/TR, SA RRTP, PTSD RRTP, General RRTP]", "VA contracted residential treatment programs (ATU-HWH or HCHV contract)", "Non-VA residential treatment program", "Non-psychiatric hospital (acute care)", "Psychiatric hospital (acute care)", "Hotel or motel paid for without emergency shelter voucher", "Emergency shelter, including hotel or motel paid for with emergency shelter voucher", "Prison, jail", "Place not meant for habitation (outdoors, automobile, truck, boat)", "Veteran declined to answer", "Interviewer omitted item"])
    check_contents(["ddlHSA_Q23-menu"], ["No", "Yes", "Veteran declined to answer", "Interviewer omitted item"])
    check_contents(["ddlHSA_Q23a-menu"], ["No", "Yes"])
    check_contents(["ddlHSA_Q24-menu"], ["Literally homeless", "Imminent risk of losing housing", "Unstably housed/at risk of losing housing", "Stably housed", "Don’t know", "Veteran declined to answer", "Interviewer omitted item"])
    check_contents(["ddlHSA_Q25-menu"], ["At least one night but less than one month", "At least one month but less than 6 months", "At least 6 months but less than 1 year", "At least one year but less than 2 years", "Two years or more", "Unknown", "Veteran declined to answer", "Interviewer omitted item"])
    check_contents(["ddlHSA_Q26-menu"], ["0", "1", "2", "3", "4", "5 or more", "Veteran declined to answer", "Interviewer omitted item"])
    check_contents(["ddlHSA_Q27-menu"], ["None", "Less than 1 month", "Between 1 month and 1 year", "More than 1 year", "Veteran declined to answer", "Interviewer omitted item"])

# 	@driver.find_element(:id, 'txtHSA_Q20').text.should eql("Veteran responded")
# 	["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t"].each { |element| @driver.find_element(:id, 'txtHSA_Q20' + element).text.should eql("0") }

	select_from_listbox("ddlHSA_Q20", "Veteran responded")
# 	@driver.find_element(:id, 'ddlHSA_Q20-button').click
# 	@driver.find_element(:xpath, "//ul[@id='ddlHSA_Q20-menu']/li/div[1]/div[1]/a[text()='']").click
    ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t"].each { |element| check_visibility(["txtHSA_Q20" + element], true) }
	check_visibility(["txtHSA_Q21-button", "txtHSA_Q22", "ddlHSA_Q23-button", "ddlHSA_Q23a-button", "txtHSA_Q23b", "txtHSA_Q23c", "txtHSA_Q23d"], true)

# 	clear_and_sendkeys("txtHSA_Q20a", "31")
# 	click_with_retry("assessment-btn", 10, "questionForm")

# 	@driver.find_element(:id, 'next-btn').click
# 	@driver.find_element(:xpath, "//div[@class='ui-field-contain ui-body ui-br flagged']/div/div/label[@class='error']").text.should eql("Item 20 (categories a-t) currently add up to 31 but must add up to 30.")

# 	wait = Selenium::WebDriver::Wait.new :timeout => 10
# 	wait.until { @driver.find_element(:id, 'txtHSA_Q20a').displayed? }
# 	check_visibility(["txtHSA_Q20a"], true)
# # 	check_listbox_numeric(["txtHSA_Q20a"], 0, 30)
# 	sleep(3)
# 	select_from_listbox("txtHSA_Q20a", "15")
# # 	clear_and_sendkeys("txtHSA_Q20a", "10")
# # 	click_with_retry("assessment-btn", 10, "questionForm")
# 	sleep(2)
# 
# 	wait = Selenium::WebDriver::Wait.new :timeout => 10
# 	wait.until { @driver.find_element(:id, 'txtHSA_Q20b').displayed? }
# 	check_visibility(["txtHSA_Q20b"], true)
# # 	check_listbox_numeric(["txtHSA_Q20b"], 0, 20)
# 	sleep(2)
# 	select_from_listbox("txtHSA_Q20b", "15")
# # 	clear_and_sendkeys("txtHSA_Q20b", "10")
# # 	click_with_retry("assessment-btn", 10, "questionForm")
# 	sleep(2)
# 
# 	wait = Selenium::WebDriver::Wait.new :timeout => 10
# 	wait.until { @driver.find_element(:id, 'txtHSA_Q20c').displayed? }
# 	check_visibility(["txtHSA_Q20c"], true)
# # 	check_listbox_numeric(["txtHSA_Q20c"], 0, 10)
# 	sleep(2)
# 	select_from_listbox("txtHSA_Q20c", "10")
# # 	clear_and_sendkeys("txtHSA_Q20c", "11")
# # 	click_with_retry("assessment-btn", 10, "questionForm")

# 	@driver.find_element(:id, 'next-btn').click
# 	@driver.find_element(:xpath, "//div[@class='ui-field-contain ui-body ui-br flagged']/div/div/label[@class='error']").text.should eql("Item 20 (categories a-t) currently add up to 31 but must add up to 30.")

# 	click_with_retry("assessment-btn", 10, "questionForm")
# 	clear_and_sendkeys("txtHSA_Q20o", "10")
# 	click_with_retry("assessment-btn", 10, "questionForm")

	option = Selenium::WebDriver::Support::Select.new(@driver.find_element(:xpath => "//select[@id='txtHSA_Q21']"))
	option.select_by(:text, "GPD transitional housing");
# 	option.select_by(:text, "Housing owned by Veteran, no ongoing housing subsidy");

# 	@driver.find_element(:id, 'next-btn').click
# 	@driver.find_element(:xpath, "//div[@class='ui-field-contain ui-body ui-br flagged']/div/div/label[@class='error']").text.should eql("Item 21=i; there must be at least one day in item 20(i).")

# 	@driver.find_element(:id, 'txtHSA_Q22').send_keys "A"
#
# 	@driver.find_element(:id, 'next-btn').click
# 	@driver.find_element(:xpath, "//div[@class='ui-field-contain ui-body ui-br flagged']/div/div/label[@class='error']").text.should eql("Zip code must be between 00001 - 99999.")

	sleep(1)
	@driver.find_element(:id, 'txtHSA_Q22').send_keys "99999"
# 	clear_and_sendkeys("txtHSA_Q22", "99999")
	click_with_retry("assessment-btn", 10, "questionForm")

	select_from_listbox("ddlHSA_Q23", "Yes")
	check_visibility(["ddlHSA_Q23a-button", "txtHSA_Q23b", "txtHSA_Q23c", "txtHSA_Q23d"], true)

# 	sleep(2)
# 	wait = Selenium::WebDriver::Wait.new :timeout => 10
# 	wait.until { @driver.find_element(:id, 'txtHSA_Q23b').displayed? }
# # 	check_listbox_numeric(["txtHSA_Q23b"], 0, 20)
# 	select_from_listbox("txtHSA_Q23b", "4")
# 
# 	sleep(2)
# 	wait = Selenium::WebDriver::Wait.new :timeout => 10
# 	wait.until { @driver.find_element(:id, 'txtHSA_Q23c').displayed? }
# # 	check_listbox_numeric(["txtHSA_Q23c"], 0, 20)
# 	select_from_listbox("txtHSA_Q23c", "3")
# 
# 	sleep(2)
# 	wait = Selenium::WebDriver::Wait.new :timeout => 10
# 	wait.until { @driver.find_element(:id, 'txtHSA_Q23d').displayed? }
# # 	check_listbox_numeric(["txtHSA_Q23d"], 0, 20)
# 	select_from_listbox("txtHSA_Q23d", "2")

	sleep(2)
	select_from_listbox("ddlHSA_Q23", "No")
	check_visibility(["ddlHSA_Q23a-button", "txtHSA_Q23b", "txtHSA_Q23c", "txtHSA_Q23d"], false)

	select_from_listbox("ddlHSA_Q20", "Veteran declined to answer")
    ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t"].each { |element| check_visibility(["txtHSA_Q20" + element], false) }
	check_visibility(["txtHSA_Q21-button", "txtHSA_Q22", "ddlHSA_Q23-button", "ddlHSA_Q23a-button", "txtHSA_Q23b", "txtHSA_Q23c", "txtHSA_Q23d"], false)

	select_from_listbox("ddlHSA_Q24", "Literally homeless")
	check_visibility(["ddlHSA_Q25-button"], true)

	select_from_listbox("ddlHSA_Q24", "Stably housed")
	check_visibility(["ddlHSA_Q25-button"], false)

	select_from_listbox("ddlHSA_Q26", "3")
	select_from_listbox("ddlHSA_Q27", "Less than 1 month")

	@driver.find_element(:id, 'next-btn').click

  end

  it "should be able to fill out the Employment and Income section of an assessment" do
	@driver.find_element(:class => "title").text.should eql("EMPLOYMENT AND INCOME")
	@driver.find_element(:class => "sectionCount").text.should eql("Section 5/8")

	check_contents(["ddlHSA_Q28-menu"], ["Full time (40 hrs/wk)", "Full time (irregular)", "Part time (regular hours)", "Part time (irregular day work)", "VA CWT or other vocational training program", "Student", "Military Service", "Retired / disability", "Unemployed", "Controlled environment (e.g., hospital, prison)", "Veteran declined to answer", "Interviewer omitted item"])
	check_contents(["ddlHSA_Q30-menu", "ddlHSA_Q31-menu", "ddlHSA_Q32-menu", "ddlHSA_Q33-menu"], ["No", "Yes", "Veteran declined to answer", "Interviewer omitted item"])

	check_visibility(["ddlHSA_Q28-menu", "txtHSA_Q29", "ddlHSA_Q30-menu", "ddlHSA_Q31-menu", "ddlHSA_Q32-menu", "ddlHSA_Q33-menu"], true)
    ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p"].each { |element| check_visibility(["txtHSA_Q30" + element], false) }
	check_visibility(["ddlHSA_Q31_1", "ddlHSA_Q32_1", "txtHSA_Q32j_other"], false)

	select_from_listbox("ddlHSA_Q28", "Student")
# 	check_listbox_numeric(["txtHSA_Q29"], 0, 30)
	select_from_listbox("txtHSA_Q29", "10")
# 	@driver.find_element(:id, 'txtHSA_Q29').send_keys "10"
# 	click_with_retry("assessment-btn", 10, "questionForm")
	sleep(2)
	
	select_from_listbox("ddlHSA_Q30", "No")
    ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p"].each { |element| check_visibility(["txtHSA_Q30" + element], false) }

	select_from_listbox("ddlHSA_Q30", "Yes")
    ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p"].each { |element| check_visibility(["txtHSA_Q30" + element], true) }

	@driver.find_element(:id, 'txtHSA_Q30a').clear
	@driver.find_element(:id, 'txtHSA_Q30a').send_keys "5000"
	click_with_retry("assessment-btn", 10, "questionForm")

	select_from_listbox("ddlHSA_Q31", "No")
	check_visibility(["ddlHSA_Q31_1"], false)

	select_from_listbox("ddlHSA_Q31", "Yes")
	check_visibility(["ddlHSA_Q31_1"], true)

	option = Selenium::WebDriver::Support::Select.new(@driver.find_element(:xpath => "//select[@id='ddlHSA_Q31_1']"))
	option.select_by(:text, "Other")

	select_from_listbox("ddlHSA_Q32", "No")
	check_visibility(["ddlHSA_Q32_1", "txtHSA_Q32j_other"], false)

	select_from_listbox("ddlHSA_Q32", "Yes")
	check_visibility(["ddlHSA_Q32_1"], true)
	check_visibility(["txtHSA_Q32j_other"], false)

	option = Selenium::WebDriver::Support::Select.new(@driver.find_element(:xpath => "//select[@id='ddlHSA_Q32_1']"))
# 	option.select_by(:text, "other (specify)")
	option.select_by(:text, "alimony");
# 	check_visibility(["txtHSA_Q32j_other"], true)
# 	@driver.find_element(:id, 'txtHSA_Q32j_other').send_keys "efgh"

# 	wait = Selenium::WebDriver::Wait.new :timeout => 10
# 	wait.until { @driver.find_element(:id, 'blah').displayed? }
#
# 	click_with_retry("assessment-btn", 10, "questionForm")

	select_from_listbox("ddlHSA_Q33", "No")

	@driver.find_element(:id, 'next-btn').click

  end

  it "should be able to fill out the Clinical Status section of an assessment" do
	@driver.find_element(:class => "title").text.should eql("CLINICAL STATUS")
	@driver.find_element(:class => "sectionCount").text.should eql("Section 6/8")

	check_visibility(["ddlHSA_Q34-menu", "ddlHSA_Q35-menu", "ddlHSA_Q37-menu", "ddlHSA_Q41-menu", "ddlHSA_Q42-menu", "ddlHSA_Q43-menu"], true)
    ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j"].each { |element| check_visibility(["ddlHSA_Q36" + element], true) }
	check_visibility(["txtHSA_Q38", "txtHSA_Q39", "txtHSA_Q40"], true)

    check_contents(["ddlHSA_Q34-menu"], ["Excellent", "Very Good", "Good", "Fair", "Poor", "Veteran declined to answer", "Interviewer omitted item"])
    check_contents(["ddlHSA_Q35-menu"], ["Excellent", "Very Good", "Good", "Fair", "Poor", "Veteran declined to answer", "Interviewer omitted item"])
    ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j"].each { |element| check_contents(["ddlHSA_Q36" + element + "-menu"], ["No", "Yes", "Veteran declined to answer", "Interviewer omitted item"]) }
	check_contents(["ddlHSA_Q37-menu", "ddlHSA_Q42-menu", "ddlHSA_Q43-menu"], ["No", "Yes", "Veteran declined to answer", "Interviewer omitted item"])
    check_contents(["ddlHSA_Q41-menu"], ["Not at all", "Slightly", "Moderately", "Considerably", "Extremely", "Veteran declined to answer", "Interviewer omitted item"])

	select_from_listbox("ddlHSA_Q34", "Excellent")
	select_from_listbox("ddlHSA_Q35", "Very Good")
	["a", "b", "c", "d", "e", "f", "g", "h", "i", "j"].each { |element| select_from_listbox("ddlHSA_Q36" + element, "No") }
	select_from_listbox("ddlHSA_Q37", "No")
	sleep(1)

# 	check_listbox_numeric(["txtHSA_Q38"], 0, 30)
	select_from_listbox("txtHSA_Q38", "10")
	sleep(1)

# 	check_listbox_numeric(["txtHSA_Q39"], 0, 30)
	select_from_listbox("txtHSA_Q39", "5")
	sleep(1)

# 	check_listbox_numeric(["txtHSA_Q40"], 0, 30)
	select_from_listbox("txtHSA_Q40", "2")
	sleep(1)

# 	@driver.find_element(:id, 'txtHSA_Q38').send_keys "5"
# 	click_with_retry("assessment-btn", 10, "questionForm")
# 	@driver.find_element(:id, 'txtHSA_Q39').send_keys "2"
# 	click_with_retry("assessment-btn", 10, "questionForm")
# 	@driver.find_element(:id, 'txtHSA_Q40').send_keys "1"
# 	click_with_retry("assessment-btn", 10, "questionForm")
	select_from_listbox("ddlHSA_Q41", "Moderately")
	select_from_listbox("ddlHSA_Q42", "No")
	select_from_listbox("ddlHSA_Q43", "Yes")

	@driver.find_element(:id, 'next-btn').click

  end

  it "should be able to fill out the Clinical Impressions section of an assessment" do
	@driver.find_element(:class => "title").text.should eql("CLINICAL IMPRESSIONS")
	@driver.find_element(:class => "sectionCount").text.should eql("Section 7/8")

    check_visibility(["ddlHSA_Q44-menu", "ddlHSA_Q45a-menu", "ddlHSA_Q46a-menu", "ddlHSA_Q47a-menu", "ddlHSA_Q48a-menu", "ddlHSA_Q49a-menu", "ddlHSA_Q50-menu", "ddlHSA_Q51-menu"], true)
    check_visibility(["ddlHSA_Q45b-menu", "ddlHSA_Q46b-menu", "ddlHSA_Q47b-menu", "ddlHSA_Q48b-menu", "ddlHSA_Q49b-menu"], true)

    check_contents(["ddlHSA_Q44-menu"], ["Alcohol use disorder", "Drug use disorder", "Gambling problem or pathological gambling", "Schizophrenia", "Other psychotic disorder", "Bipolar disorder", "Military related PTSD", "Non-Military related PTSD", "Anxiety disorder", "Affective disorder (including depression)", "Adjustment disorder", "Nicotine dependence", "Organic brain syndrome", "Personality disorder", "Other psychiatric disorder"])
    check_contents(["ddlHSA_Q45a-menu", "ddlHSA_Q46a-menu", "ddlHSA_Q47a-menu", "ddlHSA_Q48a-menu", "ddlHSA_Q49a-menu", "ddlHSA_Q50-menu", "ddlHSA_Q51-menu"], ["No", "Yes"])
    check_contents(["ddlHSA_Q45b-menu", "ddlHSA_Q46b-menu", "ddlHSA_Q47b-menu", "ddlHSA_Q48b-menu", "ddlHSA_Q49b-menu"], ["No", "Yes", "Don't know"])

	select_from_listbox("ddlHSA_Q45a", "No")
	select_from_listbox("ddlHSA_Q45b", "Don't know")
	select_from_listbox("ddlHSA_Q46a", "Yes")
	select_from_listbox("ddlHSA_Q46b", "No")
	select_from_listbox("ddlHSA_Q47a", "No")
	select_from_listbox("ddlHSA_Q47b", "Don't know")
	select_from_listbox("ddlHSA_Q48a", "Yes")
	select_from_listbox("ddlHSA_Q48b", "No")
	select_from_listbox("ddlHSA_Q49a", "No")
	select_from_listbox("ddlHSA_Q49b", "Don't know")
	select_from_listbox("ddlHSA_Q50", "Yes")
	select_from_listbox("ddlHSA_Q51", "No")

	@driver.find_element(:id, 'next-btn').click

  end

  it "should be able to fill out the Interviewer Information section of an assessment" do
	@driver.find_element(:class => "title").text.should eql("INTERVIEWER INFORMATION")
	@driver.find_element(:class => "sectionCount").text.should eql("Section 8/8")

	check_visibility(["ddlHSA_Q74-button", "ddlHSA_Q75-button"], true)
	check_visibility(["txtHSA_Q74_Other", "txtHSA_Q75_Other"], false)

    check_contents(["ddlHSA_Q74-menu"], ["HUD-VA Supported Housing (HUD-VASH)", "Healthcare for Homeless Veterans (HCHV)", "Grant and Per Diem (GPD)", "VA MH RRTP [Includes all types - DCHV, CWT/TR, SA RRTP; PTSD RRTP; General RRTP]", "Healthcare for Re-entry Veterans (HCRV)", "Veterans Justice Outreach (VJO)", "Other VA affiliation"])
    check_contents(["ddlHSA_Q75-menu"], ["Street outreach initiated by VA staff", "Justice System outreach initiated by VA staff", "Other community outreach by VA staff", "Contacted at Stand Down", "Referral from VA MH RRTP [Includes all types - DCHV, CWT/TR, SA RRTP; PTSD RRTP; General RRTP]", "Referral from VA mental health outpatient unit", "Referral from VA substance abuse outpatient unit", "Referral from VA medical outpatient unit", "Referral from VA Emergency Room", "Referral from VA inpatient unit", "Referral from Vet Center", "Referral from VBA", "Referral from VA Homeless Veterans Hotline (1-877-424-3838)", "Street outreach by non-VA staff", "Referral by shelter staff or other community homeless services provider", "Referral from VA Grant and Per Diem", "Referral from Non-VA Emergency Room", "Referral from Non-VA Community Mental Health Center or clinic", "Referral from other Federal Agency (HUD, Dept. of Labor, HHS)", "Referred by jail or prison staff", "Referred by law enforcement official", "Referred by Court (judge or District Attorney)", "Referred by an attorney (e.g., public defender or defense attorney)", "Referred by probation/parole officer", "Referred by family member", "Self referred", "Other (please specify)", "Interviewer omitted item"])

	select_from_listbox("ddlHSA_Q74", "Other VA affiliation")
	option = Selenium::WebDriver::Support::Select.new(@driver.find_element(:xpath => "//select[@id='ddlHSA_Q75']"))
	option.select_by(:text, "Other (please specify)");
	sleep(1)
	check_visibility(["txtHSA_Q74_Other", "txtHSA_Q75_Other"], true)
	@driver.find_element(:id, 'txtHSA_Q74_Other').send_keys "ABC"
	click_with_retry("assessment-btn", 10, "questionForm")
	@driver.find_element(:id, 'txtHSA_Q75_Other').send_keys "DEF"
	click_with_retry("assessment-btn", 10, "questionForm")

	@driver.find_element(:id, 'submit-btn').click

	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:id, 'ok-btn').displayed? }
  	@driver.find_element(:id, 'ok-btn').click

	@driver.find_element(:id, 'homesSummary')

  end

end
