# GETTING STARTED
# -----------------
# This documentation is intended to show you how to get started with a
# simple Selenium-Webdriver test.  This test is written with RSpec but the
# webdriver commands (everything called after @driver) will work with any
# testing framework.
#
# INSTALLING RVM
# --------------
# We're assuming you've got rvm installed, but if not, from a terminal
# run the following line (removing the ""'s):
#
# "\curl -L https://get.rvm.io | bash -s stable --ruby"
#
# INSTALLING GEMS
# ---------------
# Then, change to the example directory:
#   "cd appium-location/sample-code/examples/ruby"
#
# and install the required gems with bundler by doing:
#   "bundle install"
#
# RUNNING THE TESTS
# -----------------
# To actually run the tests, make sure appium is running in another terminal
# window, then from the same window you used for the above commands, type
#   "rspec simple_test.rb"
#
# It will take a while, but once it's done you should get nothing but a line telling you "1 example, 0 failures".

require 'rspec'
require 'selenium-webdriver'

#APP_PATH = '/Users/borsts/Documents/Code/appium/sample-code/apps/TestApp/build/release-iphonesimulator/TestApp.app'
#APP_PATH = '/Users/borsts/Library/Developer/Xcode/DerivedData/mHOMES-dbvwsrvdsnsrzreuwpoqkhxfkplv/Build/Products/   ug-iphonesimulator/mHOMES.app'
#APP_PATH = ENV['APP_PATH']
APP_PATH = File.expand_path('../build/mHOMES.app', File.dirname(__FILE__))

def absolute_app_path
    file = File.join("", APP_PATH)
    raise "App doesn't exist #{file}" unless File.exist? file
    file
end

capabilities = {
  'device' => 'iPad Simulator',
  'browserName' => 'iOS',
  'platform' => 'Mac',
  'version' => '7.0',
  'app' => absolute_app_path
}

server_url = "http://127.0.0.1:4723/wd/hub"

describe "HOMES app" do
  before :all do
    @driver = Selenium::WebDriver.for(:remote, :desired_capabilities => capabilities, :url => server_url)
    @driver.manage.timeouts.implicit_wait = 10 # seconds

    webview = @driver.window_handles.last
    @driver.switch_to.window(webview)

   end

  after :all do
	@driver.execute_script("mobile: leaveWebView")

    @driver.quit
  end

  it "should be able to log in" do

	title = @driver.find_elements(:tag_name => 'h1')
	title[1].text.eql? "Homeless Mobile Intervention"
	title[2].text.eql? "Homes Login"

    @driver.find_element(:id, "username").send_keys "mdm\\homescasemanager"
    @driver.find_element(:id, "password").send_keys "Agilexadmin99$"

    @driver.find_element(:id, 'loginButton').click
    @driver.find_element(:id, 'search-field')

  end

  ssn = rand(999999999).to_s.rjust(9, '0')

  it "should be able to add a Veteran" do
	sleep(1)
  	@driver.find_element(:id, "search-field").send_keys ssn + "\n"
  	@driver.find_element(:id, 'add-btn').click

    genderListbox = @driver.find_element(:id, "gender")
	all_options = genderListbox.find_elements(:tag_name, "option")
	all_options.each do |option|
	  ["Other", "Male", "Female"].include?option.attribute("value")
    end

  	@driver.find_element(:id, "lastName").send_keys "ZZHOMES"

  	dateTime = Time.new.strftime("%Y-%m-%d@%H:%M:%S")
  	@driver.find_element(:id, "firstName").send_keys "AUTO" + dateTime
  	@driver.find_element(:id, "middleInitial").send_keys "X"

  	@driver.find_element(:id, "sSN").send_keys ssn
    @driver.find_element(:name, 'ssnConfirm').click

    @driver.find_element(:id, 'dateOfBirth').click
	@driver.find_element(:id, 'date').click
	@driver.find_element(:xpath, '//b[@class=\'month\']/ul/li/button[@data-value=\'0\']').click
	@driver.find_element(:xpath, '//b[@class=\'day\']/ul/li/button[@data-value=\'1\']').click
	@driver.find_element(:xpath, '//b[@class=\'year\']/ul/li/button[@data-value=\'1990\']').click
	@driver.find_element(:id, 'set-btn').click
	@driver.find_element(:id, 'apply-btn').click

    gender = Selenium::WebDriver::Support::Select.new(genderListbox)
    gender.select_by(:text, "Female")

  	@driver.find_element(:id, 'add-btn').click

	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:id, 'ok-btn').displayed? }
  	@driver.find_element(:id, 'ok-btn').click

  end

#   it "should accept the SUD" do
# 	wait = Selenium::WebDriver::Wait.new :timeout => 10
# 	wait.until { @driver.find_element(:id, "accept-btn").displayed? }
# 
#     @driver.find_element(:id, 'sud')
#     @driver.find_element(:id, 'accept-btn').click
# 	sleep(1)
# 	
#   end
# 
  def check_visibility(elements, visibility)
    elements.each { |element| @driver.find_element(:id, element).displayed?.should eql(visibility) }
  end

  def check_contents(elements, responses)
    elements.each do |element|
	  responses.each do |response|
        xpath = "//ul[@id='" + element + "']/li/div[1]/div[1]/a[text()='" + response + "']"
        @driver.find_element(:xpath, xpath)
      end
    end
  end

  def select_from_listbox(question_id, value)
  	@driver.find_element(:id, question_id + '-button').click
  	sleep(1)
	option = Selenium::WebDriver::Support::Select.new(@driver.find_element(:xpath => "//select[@id=\"" + question_id + "\"]"))
	option.select_by(:text, value);
  end

  def select_from_multiselect(question_id, value)
  	@driver.find_element(:id, question_id + '-button').click
	@driver.find_element(:xpath, "//ul[@id=\"" + question_id + "-menu\"]/li/div[1]/div[1]/a[text()=\"" + value + "\"]").click
# 	@driver.find_element(:xpath, "//select[@id='" + question_id + "']/option[text()='" + value + "']").click
  end

  def click_with_retry(button_id, max_tries, element)
    tries = 0
    begin
	  sleep(1)
      tries = tries + 1
      @driver.find_element(:id, button_id).click
      stop_condition = (tries == max_tries ? true : false)
      stop_condition = (@driver.find_element(:id, element).displayed?.eql? true ? true: stop_condition)
    rescue Exception=>e
      stop_condition = true
    end until stop_condition == true
    sleep(1)
  end

  it "should be able to complete the Homeless Decision Support form" do
	click_with_retry("search-btn", 25, "search-field")
	sleep(1)
	@driver.find_element(:id, "search-field").send_keys ssn + "\n"
# 	@driver.find_element(:class, "line-item").click
	@driver.find_element(:xpath, "//div[@id='homesListView']/div[@data-role='content']/ul/li[@data-index='0']/div/div/div/div").click
	@driver.find_element(:xpath, "//div[@data-role='footer']").click

	@driver.find_element(:id, 'homesSummary')
# 	@driver.find_element(:id, 'decision-btn').click
	click_with_retry("decision-btn", 25, "title")

	sleep(1)
	@driver.find_element(:class => "title").text.eql? "Assessment Questionnaire"

	check_visibility(["ddlDssQ1-button", "ddlDssQ2-button", "ddlDssQ3-button", "ddlDssQ4-button", "ddlDssQ6-button", "ddlDssQ9-button", "ddlDssQ10-button", "ddlDssQ11-button", "ddlDssQ12-button", "ddlDssQ13-button", "ddlDssQ14-button", "ddlDssQ15-button", "ddlDssQ16-button", "ddlDssQ17-button", "ddlDssQ18-button"], true)
	check_visibility(["ddlDssQ5-button", "ddlDssQ7-button", "ddlDssQ8-button"], false)
 	check_contents(["ddlDssQ1-menu", "ddlDssQ7-menu", "ddlDssQ9-menu", "ddlDssQ12-menu", "ddlDssQ13-menu", "ddlDssQ18-menu"], ["No", "Yes"])

# 	select_from_listbox("ddlDssQ1", "Yes")
# 	sleep(1)
# 
# # 	@driver.find_element(:id, 'submit-btn').click
# # 	sleep(1)
# # 	@driver.find_element(:xpath, "//div[@class='ui-field-contain ui-body ui-br flagged']/div/div/label[@class='error']").text.should eql("Please answer item 2")
# 
# 	select_from_listbox("ddlDssQ2", "a. Single adult")
# 
# # 	@driver.find_element(:id, 'ddlDssQ3-button').click
# 	select_from_listbox("ddlDssQ3", "a. Anywhere outside, on the streets or other location not fit for people to live in like a car, abandoned building, bus/train/subway station or airport")
# 
# # 	wait = Selenium::WebDriver::Wait.new :timeout => 10
# # 	wait.until { @driver.find_element(:class, "blah").displayed? }
# 
# # 	click_with_retry("decision-btn", 25, "title")
# 	sleep(3)
# 	check_visibility(["ddlDssQ5-button"], true)
# 	check_visibility(["ddlDssQ8-button"], false)
# 
# 	select_from_listbox("ddlDssQ5", "e. 91 to 180")
# 
# 	select_from_listbox("ddlDssQ3", "f. Stayed with family or friends in a room, apartment, or house")
# 	check_visibility(["ddlDssQ5-button"], false)
# 	check_visibility(["ddlDssQ8-button"], true)
# 
# 	select_from_listbox("ddlDssQ4", "f. Stayed with family or friends in a room, apartment, or house")
# 
# 	select_from_listbox("ddlDssQ6", "b. 1 time")
# 	check_visibility(["ddlDssQ7-button"], true)

	select_from_listbox("ddlDssQ1", "Yes")
	select_from_listbox("ddlDssQ2", "b. 2 or more adults with no dependent children")
	select_from_listbox("ddlDssQ3", "d. In non-transitional temporary living accommodations")
	select_from_listbox("ddlDssQ4", "c. In a facility, such a hospital, rehab center, drug treatment center, prison/jail or nursing home ")
	select_from_listbox("ddlDssQ6", "c. 2 times")
	select_from_listbox("ddlDssQ9", "Yes")
	select_from_listbox("ddlDssQ11", "b. $626 to $1,250")
	select_from_listbox("ddlDssQ12", "Yes")
	select_from_listbox("ddlDssQ13", "Yes")
	select_from_listbox("ddlDssQ14", "b. 1 to 3 times")
	select_from_listbox("ddlDssQ15", "b. Immediate problems are being adequately addressed. The Vet has access to primary care provider and accesses care when necessary (annual checkup)")
	select_from_listbox("ddlDssQ16", "b. History of substance abuse/dependence; no current indication of dependence or abuse or need for treatment")
	select_from_listbox("ddlDssQ17", "a. Basic needs are being met and Veteran has no trouble in daily functioning")
	select_from_listbox("ddlDssQ18", "No")
	select_from_listbox("ddlDssQ7", "Yes")
	select_from_multiselect("ddlDssQ10", "a. Employed - Full Time")
	select_from_multiselect("ddlDssQ10", "d. Student")

	click_with_retry("submit-btn", 25, "Transitional-fieldSet")
# 	@driver.find_element(:id, 'submit-btn').click
	sleep(1)

    @driver.find_element(:id, 'Transitional-fieldSet')
	@driver.find_element(:xpath, "//fieldset[@id='Transitional-fieldSet']/legend").text.should eql("Transitional Housing")
	@driver.find_element(:xpath, "//fieldset[@id='Transitional-fieldSet']/p[1]").text.should eql("Ranking: 300")

	@driver.find_element(:id, 'back-btn').click
    sleep(1)

	select_from_listbox("ddlDssQ17", "c. Basic needs are not being met and Veteran's well-being is severely impacted; services are needed to assist Veteran with daily functioning and on-going assistance for maintaining and monitoring stability are required")
	@driver.find_element(:id, 'submit-btn').click
    sleep(1)

    @driver.find_element(:id, 'ShortTerm-fieldSet')
	@driver.find_element(:xpath, "//fieldset[@id='ShortTerm-fieldSet']/legend").text.should eql("Short Term Treatment")
	@driver.find_element(:xpath, "//fieldset[@id='ShortTerm-fieldSet']/p[1]").text.should eql("Ranking: 1000")

	@driver.find_element(:id, 'back-btn').click
	sleep(1)

	select_from_listbox("ddlDssQ17", "b. Basic needs are not being met and Veteran is unable to or has difficulty identifying resources to help; daily functioning is poor")
	@driver.find_element(:id, 'submit-btn').click
	sleep(1)

    @driver.find_element(:id, 'Transitional-fieldSet')
	@driver.find_element(:xpath, "//fieldset[@id='Transitional-fieldSet']/legend").text.should eql("Transitional Housing")
	@driver.find_element(:xpath, "//fieldset[@id='Transitional-fieldSet']/p[1]").text.should eql("Ranking: 300")

    @driver.find_element(:id, 'Permanent-fieldSet')
	@driver.find_element(:xpath, "//fieldset[@id='Permanent-fieldSet']/legend").text.should eql("Permanent Supportive Housing")
	@driver.find_element(:xpath, "//fieldset[@id='Permanent-fieldSet']/p[1]").text.should eql("Ranking: 300")

	@driver.find_element(:id, 'submit-btn').click
    sleep(1)

	@driver.find_element(:class, "error").text.should eql("You must accept or reject")
	@driver.find_elements(:class, 'accept-btn')[0].click
	@driver.find_elements(:class, 'accept-btn')[0].enabled?.should eql(false)

	@driver.find_elements(:class, 'reject-btn')[1].click
	@driver.find_elements(:class, 'reject-btn')[1].enabled?.should eql(false)
	rejectOptions = @driver.find_elements(:tag_name, "option");
 	rejectOptions[1].text.should eql("Veteran declined")
 	rejectOptions[2].text.should eql("Program not available")
 	rejectOptions[3].text.should eql("Best Fit")
 	rejectOptions[4].text.should eql("Clinician identified another program")
  	rejectOptions[5].text.should eql("No room")

 	rejectOptions[2].click
	@driver.find_element(:id, 'Permanent-rejectText').send_keys "This is a reject reason!"
 	
	@driver.find_element(:id, 'submit-btn').click
	@driver.find_element(:id, 'ok-btn').click
	sleep(5)

  end

end
