# GETTING STARTED
# -----------------
# This documentation is intended to show you how to get started with a
# simple Selenium-Webdriver test.  This test is written with RSpec but the
# webdriver commands (everything called after @driver) will work with any
# testing framework.
#
# INSTALLING RVM
# --------------
# We're assuming you've got rvm installed, but if not, from a terminal
# run the following line (removing the ""'s):
#
# "\curl -L https://get.rvm.io | bash -s stable --ruby"
#
# INSTALLING GEMS
# ---------------
# Then, change to the example directory:
#   "cd appium-location/sample-code/examples/ruby"
#
# and install the required gems with bundler by doing:
#   "bundle install"
#
# RUNNING THE TESTS
# -----------------
# To actually run the tests, make sure appium is running in another terminal
# window, then from the same window you used for the above commands, type
#   "rspec simple_test.rb"
#
# It will take a while, but once it's done you should get nothing but a line telling you "1 example, 0 failures".

require 'rspec'
require 'selenium-webdriver'

#APP_PATH = '/Users/borsts/Documents/Code/appium/sample-code/apps/TestApp/build/release-iphonesimulator/TestApp.app'
#APP_PATH = '/Users/borsts/Library/Developer/Xcode/DerivedData/mHOMES-dbvwsrvdsnsrzreuwpoqkhxfkplv/Build/Products/   ug-iphonesimulator/mHOMES.app'
#APP_PATH = ENV['APP_PATH']
APP_PATH = File.expand_path('../build/mHOMES.app', File.dirname(__FILE__))


def absolute_app_path
    file = File.join("", APP_PATH)
    raise "App doesn't exist #{file}" unless File.exist? file
    file
end

capabilities = {
  'device' => 'iPad Simulator',
  'browserName' => 'iOS',
  'platform' => 'Mac',
  'version' => '7.0',
  'app' => absolute_app_path
}

server_url = "http://127.0.0.1:4723/wd/hub"

describe "HOMES app" do
  before :all do
    @driver = Selenium::WebDriver.for(:remote, :desired_capabilities => capabilities, :url => server_url)
    @driver.manage.timeouts.implicit_wait = 10 # seconds

    webview = @driver.window_handles.last
    @driver.switch_to.window(webview)

   end

  after :all do
	@driver.execute_script("mobile: leaveWebView")

    @driver.quit
  end

  it "should log in to HOMES" do

	title = @driver.find_elements(:tag_name => 'h1')
	title[0].text.eql? "Homeless Mobile Intervention"
	title[1].text.eql? "Homes Login"

    @driver.find_element(:id, "username").send_keys "mdm\\homescasemanager"
    @driver.find_element(:id, "password").send_keys "Agilexadmin99$"

    @driver.find_element(:id, 'loginButton').click
    @driver.find_element(:id, 'search-field')

  end
# 
#   it "should accept the SUD" do
# 	wait = Selenium::WebDriver::Wait.new :timeout => 10
# 	wait.until { @driver.find_element(:id, "accept-btn").displayed? }
# 
#     @driver.find_element(:id, 'sud')
#     @driver.find_element(:id, 'accept-btn').click
# 	sleep(1)
# 
#   end

  def click_with_retry(button_id, max_tries, element)
    tries = 0
    begin
	  sleep(1)
      tries = tries + 1
      @driver.find_element(:id, button_id).click
      stop_condition = (tries == max_tries ? true : false)
      stop_condition = (@driver.find_element(:id, element).displayed?.eql? true ? true: stop_condition)
    rescue Exception=>e
      stop_condition = true
    end until stop_condition == true
    sleep(1)
  end

  ssn = rand(999999999).to_s.rjust(9, '0')

  it "should be able to add a Veteran" do
	click_with_retry("search-btn", 25, "search-field")
  	@driver.find_element(:id, "search-field").send_keys ssn + "\n"
  	@driver.find_element(:id, 'add-btn').click
	sleep(1)
	
    genderListbox = @driver.find_element(:id, "gender")
	all_options = genderListbox.find_elements(:tag_name, "option")
	all_options.each do |option|
	  ["Other", "Male", "Female"].include?option.attribute("value")
    end

  	@driver.find_element(:id, "lastName").send_keys "ZZHOMES"

  	dateTime = Time.new.strftime("%Y-%m-%d@%H:%M:%S")
  	@driver.find_element(:id, "firstName").send_keys "AUTO" + dateTime
  	@driver.find_element(:id, "middleInitial").send_keys "X"

  	@driver.find_element(:id, "sSN").send_keys "00000"
  	@driver.find_element(:id, 'add-btn').click
  	sleep(1)
    dialogText = @driver.find_elements(:class, 'text_line')
    dialogText[2].text.should eql('Please enter nine digit SSN with no dashes')
  	@driver.find_element(:id, 'ok-btn').click

  	@driver.find_element(:id, "sSN").clear
  	@driver.find_element(:id, "sSN").send_keys "0000000000"
  	@driver.find_element(:id, 'add-btn').click
  	sleep(1)
    dialogText = @driver.find_elements(:class, 'text_line')
    dialogText[2].text.should eql('Please enter nine digit SSN with no dashes')
  	@driver.find_element(:id, 'ok-btn').click

  	@driver.find_element(:id, "sSN").clear
  	@driver.find_element(:id, "sSN").send_keys ssn
    @driver.find_element(:name, 'ssnConfirm').click

# 	@driver.execute_script("mobile: leaveWebView")
# 
# 	@driver.find_element(:xpath, "//window[1]/scrollview[1]/webview[1]/text[@name='Date of Birth *']").click
# 
# 	@driver.find_element(:xpath, "//window[1]/popover[1]/picker[1]/pickerwheel[1]").send_keys "January"
# 	@driver.find_element(:xpath, "//window[1]/popover[1]/picker[1]/pickerwheel[2]").send_keys "1"
# 	@driver.find_element(:xpath, "//window[1]/popover[1]/picker[1]/pickerwheel[3]").send_keys "1990"
# 
#     webview = @driver.window_handles.last
#     @driver.switch_to.window(webview)

    @driver.find_element(:id, 'dateOfBirth').click
	@driver.find_element(:id, 'date').click
	@driver.find_element(:xpath, '//b[@class=\'month\']/ul/li/button[@data-value=\'0\']').click
	@driver.find_element(:xpath, '//b[@class=\'day\']/ul/li/button[@data-value=\'1\']').click
	@driver.find_element(:xpath, '//b[@class=\'year\']/ul/li/button[@data-value=\'1990\']').click
	@driver.find_element(:id, 'set-btn').click
	@driver.find_element(:id, 'apply-btn').click

    gender = Selenium::WebDriver::Support::Select.new(genderListbox)
    gender.select_by(:text, "Female")

  	@driver.find_element(:id, 'add-btn').click

	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:id, 'ok-btn').displayed? }
  	@driver.find_element(:id, 'ok-btn').click

  end

  it "should be able to edit a Veteran" do
	click_with_retry("search-btn", 25, "search-field")
	sleep(1)
  	@driver.find_element(:id, "search-field").send_keys ssn + "\n"
	@driver.find_element(:xpath, "//div[@id='homesListView']/div[@data-role='content']/ul/li[@data-index='0']/div/div/div/div").click
	@driver.find_element(:xpath, "//div[@data-role='footer']").click

# 	HOMESIDField = @driver.find_element(:id => "homesId")

  	@driver.find_element(:id, 'edit-btn').click
    @driver.find_element(:id, "middleInitial").clear
    @driver.find_element(:id, "middleInitial").send_keys "X"
    genderListbox = @driver.find_element(:id, "gender")
	gender = Selenium::WebDriver::Support::Select.new(genderListbox)
    gender.select_by(:text, "Female")

    @driver.find_element(:id, "address1").clear
    @driver.find_element(:id, "address2").clear
    @driver.find_element(:id, "address3").clear
    @driver.find_element(:id, "city").clear
    @driver.find_element(:id, "zipCode").clear
    @driver.find_element(:id, "telephonePrimary").clear
    @driver.find_element(:id, "telephoneSecondary").clear
    @driver.find_element(:id, "email").clear
    @driver.find_element(:id, "pOCName").clear
    @driver.find_element(:id, "pOCPhone").clear
    @driver.find_element(:id, "middleInitial").clear

    @driver.find_element(:id, "address1").send_keys "abc"
    @driver.find_element(:id, "address2").send_keys "def"
    @driver.find_element(:id, "address3").send_keys "ghi"
    @driver.find_element(:id, "city").send_keys "jkl"
    @driver.find_element(:id, "zipCode").send_keys "11111"
    @driver.find_element(:id, "telephonePrimary").send_keys "pqr"
    @driver.find_element(:id, "telephoneSecondary").send_keys "st"
    @driver.find_element(:id, "email").send_keys "uv"
    @driver.find_element(:id, "pOCName").send_keys "wx"
    @driver.find_element(:id, "pOCPhone").send_keys "yz"
  	@driver.find_element(:id, 'add-btn').click

	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:id, 'ok-btn').displayed? }
  	@driver.find_element(:id, 'ok-btn').click

	click_with_retry("search-btn", 25, "search-field")
  	@driver.find_element(:id, "search-field").send_keys ssn + "\n"
	@driver.find_element(:xpath, "//div[@id='homesListView']/div[@data-role='content']/ul/li[@data-index='0']/div/div/div/div").click
	@driver.find_element(:xpath, "//div[@data-role='footer']").click

	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:id, 'edit-btn').displayed? }
  	@driver.find_element(:id, 'edit-btn').click

    @driver.find_element(:id, "address1").clear
    @driver.find_element(:id, "address2").clear
    @driver.find_element(:id, "address3").clear
    @driver.find_element(:id, "city").clear
    @driver.find_element(:id, "zipCode").clear
    @driver.find_element(:id, "telephonePrimary").clear
    @driver.find_element(:id, "telephoneSecondary").clear
    @driver.find_element(:id, "email").clear
    @driver.find_element(:id, "pOCName").clear
    @driver.find_element(:id, "pOCPhone").clear
    @driver.find_element(:id, "middleInitial").clear

  	@driver.find_element(:id, 'add-btn').click

# 	wait = Selenium::WebDriver::Wait.new :timeout => 10
# 	wait.until { @driver.find_element(:id, 'ok-btn').displayed? }
#   	@driver.find_element(:id, 'ok-btn').click

  end

end