# GETTING STARTED
# -----------------
# This documentation is intended to show you how to get started with a
# simple Selenium-Webdriver test.  This test is written with RSpec but the
# webdriver commands (everything called after @driver) will work with any
# testing framework.
#
# INSTALLING RVM
# --------------
# We're assuming you've got rvm installed, but if not, from a terminal
# run the following line (removing the ""'s):
#
# "\curl -L https://get.rvm.io | bash -s stable --ruby"
#
# INSTALLING GEMS
# ---------------
# Then, change to the example directory:
#   "cd appium-location/sample-code/examples/ruby"
#
# and install the required gems with bundler by doing:
#   "bundle install"
#
# RUNNING THE TESTS
# -----------------
# To actually run the tests, make sure appium is running in another terminal
# window, then from the same window you used for the above commands, type
#   "rspec simple_test.rb"
#
# It will take a while, but once it's done you should get nothing but a line telling you "1 example, 0 failures".

require 'rspec'
require 'selenium-webdriver'

#APP_PATH = '/Users/borsts/Documents/Code/appium/sample-code/apps/TestApp/build/release-iphonesimulator/TestApp.app'
#APP_PATH = '/Users/borsts/Library/Developer/Xcode/DerivedData/mHOMES-dbvwsrvdsnsrzreuwpoqkhxfkplv/Build/Products/   ug-iphonesimulator/mHOMES.app'
#APP_PATH = ENV['APP_PATH']
APP_PATH = File.expand_path('../build/mHOMES.app', File.dirname(__FILE__))

def absolute_app_path
    file = File.join("", APP_PATH)
    raise "App doesn't exist #{file}" unless File.exist? file
    file
end

capabilities = {
  'device' => 'iPad Simulator',
  'browserName' => 'iOS',
  'platform' => 'Mac',
  'version' => '7.0',
  'app' => absolute_app_path
}

server_url = "http://127.0.0.1:4723/wd/hub"

describe "HOMES app" do
  before :all do
    @driver = Selenium::WebDriver.for(:remote, :desired_capabilities => capabilities, :url => server_url)
    @driver.manage.timeouts.implicit_wait = 10 # seconds

    webview = @driver.window_handles.last
    @driver.switch_to.window(webview)
	
   end

  after :all do
	@driver.execute_script("mobile: leaveWebView")
	
    @driver.quit
  end

  def click_native_button(button_id)
  	@driver.execute_script("mobile: leaveWebView")

	@driver.find_element(:xpath, "//window[1]/scrollview[1]/webview[1]/text[@name='Date of Birth *']").click

    webview = @driver.window_handles.last
    @driver.switch_to.window(webview)
  end
  
  def click_with_retry(button_id, max_tries, selector, element)
    tries = 0
    begin
	  sleep(1)
      tries = tries + 1
      @driver.find_element(:id, button_id).click
      stop_condition = (tries == max_tries ? true : false)
      stop_condition = (@driver.find_element(selector, element).displayed?.eql? true ? true: stop_condition)
    rescue Exception=>e
      stop_condition = true
    end until stop_condition == true
    sleep(1)
  end


  it "should display a message when username or password are incorrect" do

	title = @driver.find_elements(:tag_name => 'h1')
	title[1].text.should eql("Homeless Mobile Intervention")
	title[2].text.should eql("Homes Login")
	
    @driver.find_element(:id, "username").send_keys "mdm\\homescasemanager"
    @driver.find_element(:id, "password").send_keys "Agilexadmin99#"
	
    @driver.find_element(:id, 'loginButton').click
	sleep(5)
    dialogText = @driver.find_elements(:class, 'text_line')
    dialogText[0].text.should eql('Invalid Login Attempt')
    dialogText[1].text.should eql('Incorrect user name or password')
    dialogText[3].text.should eql('Please try again')
    @driver.find_element(:id, 'ok-btn').click
	
    @driver.find_element(:id, "username").clear
    @driver.find_element(:id, "username").send_keys "mdm\\homecasemanager"
    @driver.find_element(:id, "password").clear
    @driver.find_element(:id, "password").send_keys "Agilexadmin99$"
	
    @driver.find_element(:id, 'loginButton').click
	sleep(5)
    dialogText = @driver.find_elements(:class, 'text_line')
    dialogText[0].text.should eql('Invalid Login Attempt')
    dialogText[1].text.should eql('Incorrect user name or password')
    dialogText[3].text.should eql('Please try again')
    @driver.find_element(:id, 'ok-btn').click
	
  end

  it "should log in to HOMES" do

	title = @driver.find_elements(:tag_name => 'h1')
	title[0].text.should eql("Homeless Mobile Intervention")
	title[1].text.should eql("Homes Login")
	
    @driver.find_element(:id, "username").clear
    @driver.find_element(:id, "username").send_keys "mdm\\homescasemanager"
    @driver.find_element(:id, "password").clear
    @driver.find_element(:id, "password").send_keys "Agilexadmin99$"
	
    @driver.find_element(:id, 'loginButton').click
#     @driver.find_element(:id, 'sud')

  end
# 
#   it "should accept the SUD" do
# 	wait = Selenium::WebDriver::Wait.new :timeout => 10
# 	wait.until { @driver.find_element(:id, "accept-btn").displayed? }
# 
#     @driver.find_element(:id, 'sud')
#     @driver.find_element(:id, 'accept-btn').click
# 	sleep(1)
# 	
#   end

  it "should be on the Search HOMES screen" do
	wait = Selenium::WebDriver::Wait.new :timeout => 10

	footerLabels = @driver.find_elements(:class => "footerLabel")
# 	footerLabels[0].text.should eql("Name:")
	footerLabels[0].text.should eql("Staff:")
	footerLabels[1].text.should eql("Roles:")
	footerLabels[2].text.should eql("Site:")
	
	footerData = @driver.find_elements(:class => "footerData")
	footerData[0].text.should eql("CaseManager, Homes")
	footerData[1].text.should eql("Admin")
	footerData[2].text.should eql("688")
	
  end

  it "should have an About window" do
	click_with_retry("about-btn", 25, ":class", "about")
	
	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:class, "about").displayed? }

	aboutTitles = @driver.find_elements(:class => "title")
	aboutTitles[0].text.should eql("About the HMLSmobile Application")

  end

  it "should be able to search by SSN" do
  	sleep(1)
  	@driver.find_element(:id, "search-field").send_keys "666001001\n"

	title = @driver.find_elements(:tag_name => 'h1')
	title[0].text.should eql("Homeless Mobile Intervention")
# 	title[1].text.should eql("List of found Veterans")
	
	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:class => "ui-last-child").displayed? }

	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:class => "ui-first-child").displayed? }

	sleep(1)
	veteranFields = @driver.find_elements(:class => "data")
	veteranFields2 = @driver.find_elements(:class => "data-light")

	veteranFields[0].text.should eql("ZZHOMES, ONE J")
	veteranFields2[0].text.should eql("01/01/1985 (29 yo) M")
	veteranFields2[1].text.should eql("666-00-1001")

  	@driver.find_element(:id, "search-field").clear
	sleep(1)

  end

  it "should be able to search by last initial and last 4 of SSN" do
  	sleep(1)
  	@driver.find_element(:id, "search-field").send_keys "Z1002\n"

	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:class => "ui-last-child").displayed? }

	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:class => "ui-first-child").displayed? }

	sleep(1)
	veteranFields = @driver.find_elements(:class => "data")
	veteranFields2 = @driver.find_elements(:class => "data-light")

	veteranFields[0].text.should eql("ZZHOMES, TWO Y")
	veteranFields2[0].text.should eql("01/09/1970 (44 yo) F")
	veteranFields2[1].text.should eql("666-00-1002")

  	@driver.find_element(:id, "search-field").clear
	sleep(1)
  	  	
  end

  it "should be able to search by last name" do
  	sleep(1)
  	@driver.find_element(:id, "search-field").send_keys "ZZHOMES\n"

	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:class => "ui-last-child").displayed? }

	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:class => "ui-first-child").displayed? }

	sleep(1)
	veteranFields = @driver.find_elements(:class => "data")
	nameFieldNumber = 1
	veteranFields.each do |data|
	  if nameFieldNumber % 3 == 0 then veteranFields[nameFieldNumber].text.upcase.should include("ZZHOMES") end
	end
	
  	@driver.find_element(:id, "search-field").clear 
	sleep(1)

  end

  it "should be able to search by HOMES ID" do
  	sleep(1)
  	@driver.find_element(:id, "search-field").send_keys "3\n"

	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:class => "ui-last-child").displayed? }

	wait = Selenium::WebDriver::Wait.new :timeout => 10
	wait.until { @driver.find_element(:class => "ui-first-child").displayed? }

	sleep(2)
	veteranFields = @driver.find_elements(:class => "data")
	veteranFields2 = @driver.find_elements(:class => "data-light")

	veteranFields[0].text.should eql("ZZHOMES, THREE")
	veteranFields2[0].text.should eql("01/09/1970 (44 yo) M")
	veteranFields2[1].text.should eql("666-00-1003")

  	@driver.find_element(:id, "search-field").clear  	
	sleep(1)

  end

end