define([],
	function() {
		var app = {
			title:		'Homeless',
			id:			'homeless',
			patient:	'patient'
		};

		var utilities = {
			init: function() {
				this.test_toCharEntity();
				return this;
			},
			// http://www.freeformatter.com/html-entities.html
			// ; # & * % : \ / ? ~ ! @ $ ^ _ - + = ` | ( ) { } [ ] < > " ' , .
			// ;#&*%:\/?~!@$^_-+=`|(){}[]<>"',.
			specialChrsMap: [
				[';', '&#59;'],  ['#', '&#35;'],  ['&', '&#38;'],
				['*', '&#42;'],  ['%', '&#37;'],  [':', '&#58;'],  ['\\', '&#92;'],
				['/', '&#47;'],  ['?', '&#63;'],  ['~', '&#126;'], ['!', '&#33;'],  ['@', '&#64;'],
				['$', '&#36;'],  ['^', '&#94;'],  ['_', '&#95;'],  ['-', '&#45;'],  ['+', '&#43;'],
				['=', '&#61;'],  ['`', '&#96;'],  ['|', '&#124;'], ['(', '&#40;'],  [')', '&#41;'],
				['{', '&#123;'], ['}', '&#125;'], ['[', '&#91;'],  [']', '&#93;'],  ['<', '&#60;'],
				['>', '&#62;'],  ['"', '&#34;'],  ['\'', '&#39;'], [',', '&#44;'],  ['.', '&#46;']
			],

			replSingle: function(str,map) {
				for (var i=0; i<map.length; i++) {
					var pair = map[i];
					if (str===pair[0]) {
						return pair[1];
						break;
					}
				};
				return str;
			},

			replEachChar: function(str,map) {
				var retVal = '';
				for (var i=0; i<str.length; i++) {
					retVal = retVal + this.replSingle(str[i],map);
				}
				return retVal;
			},

			readOnlyRoles: ['Read Only', 'Inactivate Records', 'National Read Only'],

			isReadonly: function(roles) {
				for (var i=0; i<roles.length; i++) {
					for (var j=0; j<this.readOnlyRoles.length; j++) {
						if (roles[i]==this.readOnlyRoles[j]) {
							return true;
						}
					}
				}
				return false;
			},

			toCharEntity: function(s) {
//				console.log('original passwd\n' + s);
				var replaced = this.replEachChar(s,this.specialChrsMap);
//				console.log('replaced\n' + replaced);
				return replaced;
			},

			test_toCharEntity: function() {
				var pass = true;
				var tests = [
					['abc', 'abc'],
					['@#$', '&#64;&#35;&#36;'],
					['~!@#$%^&*_',		'&#126;&#33;&#64;&#35;&#36;&#37;&#94;&#38;&#42;&#95;'],
					['-+=`|(){}[]:;',	'&#45;&#43;&#61;&#96;&#124;&#40;&#41;&#123;&#125;&#91;&#93;&#58;&#59;'],
					['"\'<>,.?/\\',		'&#34;&#39;&#60;&#62;&#44;&#46;&#63;&#47;&#92;'],
					['abc@#$%def',		'abc&#64;&#35;&#36;&#37;def'],
					['0$0$',			'0&#36;0&#36;'],
					['&#36;',			'&#38;&#35;36&#59;'],
					['',				''],
				];

				for (var i=0; i<tests.length; i++) {
					var test = tests[i];
					var encoded = this.toCharEntity(test[0]);
					if (encoded !== test[1]) {
						console.log('encoded=' + encoded + ', expected=' + test[1]);
						pass = false;
					}
				}
				console.log('test_toCharEntity pass=' + pass);
			},

			errorUndefined: function(mess) {
				console.log('undefined setup: ' + mess);
				utilities.alert('undefined setup: ' + mess);
			},
			errorNoSoapResponse: function(mess) {
				console.log('noSoapResponse: ' + mess);
				utilities.alert('noSoapResponse: ' + mess);
			},
			timer: function() {
				this.startT;
				this.markT;

				this.getTime = function() {
					return (new Date()).getTime();
				}
				this.startWaiting = function() {
					this.startT = this.getTime();
				};
				this.markTime = function(msg) {
					this.markT = this.getTime();
				};
				this.writeTimeSinceMark = function(msg) {
					var tmp = this.getTime();
					console.log("TIMING " + msg + (tmp-this.markT));
					this.markT = tmp
				};
				this.writeTimeSinceStart = function(msg) {
					console.log("TIMING " + msg + (this.getTime()-this.startT));
				};
			},
			sync: function(interval,maxWait) {
				this.maxMills = 10000;
				this.interval = 100;
				if (interval) this.interval = interval;
				if (maxWait) this.maxMills = maxWait;
				this.mills = 0;
				this.finished = true;
				this.callOver = true;
				this.successData;
				this.errorData;
				this.success;
				this.ping;

				this.reset = function() {
					this.mills = 0;
					this.finished = true;
					this.callOver = true;
				};
				this.startWaiting = function(ping) {
					this.mills = 0;
					this.finished = false;
					this.callOver = false;
					this.successData = null;
					this.errorData = null;
					this.ping = ping;
				};
				this.iterate = function(online) {
					this.mills += this.interval;
					return this.mills < this.maxMills && (this.ping || online);
				};
				this.iterateLonger = function() {
					this.mills += this.interval;
					return this.mills < 2*this.maxMills;
				};

				this.destroyIcon = function(callback) {
					var extra = 500 - this.mills;
					if (extra > 0) {
						console.log('keeping loader icon up and extra ' + extra + ' mills');
						setTimeout(function() {
							$.mobile.loading('hide');
						}, extra);
					} else {
						$.mobile.loading('hide');
					}
					this.callback(callback);
				};
				this.callback = function(callback) {
					if (callback) {
						var obj = {success: this.success, successData: this.successData, errorData: this.errorData};
						callback(obj);
					}
				};
			},
			alert: function(message) {
				if (state.testMode) {
					alert(message);
				} else {
					console.log(message);
				}
			},
			validateSearchOptions: function(options) {
				var homesId = options.homesId;
				homesId = utilities.intToStr(homesId);
				options.homesId = homesId;
				return options;
			},
			intToStr: function(obj) {
				if (typeof obj === 'string') {
					var num = parseInt(obj);
					if (isNaN(num))
						num = 0;
					return num;
				}
				if (typeof obj === 'undefined') {
					return 0;
				}
				if (typeof obj === 'object') {
					return 0;
				}
				if (typeof obj === 'number') {
					return obj;
				}
				return 0;
			}
		}.init();

		var state = {
			init: function() {
				return this;
			},
			appName:			'',
			currentVersion:		'',
			yearsOfHistory:		10,
			contentHeight:		0,
			isKeyboardShown:	false,
			isLoaderShown:		false,
			showBackButton:		true,
			vetIndex:			null,
			homesEndpoint:		'',
			metricEndpoint:		'',
			doingSoapTest:		false,
			locked:				false,

			scrollOptions: function(func) {
				return {axis: 'y', offset: -100, easing: 'linear', duration: 'fast', onAfter: func};
			},
			// get default soap options
			soapDefaults: function(call) {
				if (call === 'ping') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'GetLastError',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/GetLastError',
						enableLogging: false
					};
				} else if (call === 'homesUserLogin') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'LogIn',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/LogIn',
					};
				} else if (call === 'lastAssessment') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'GetVeteranLastestAssessmentForm',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/GetVeteranLastestAssessmentForm',
					};
				} else if (call === 'allAssessment') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'GetAssessmentForms',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/GetAssessmentForms',
					};
				} else if (call === 'allDS') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'GetDecisionSuportForms',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/GetDecisionSuportForms',
					};
				} else if (call === 'assessment') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'GetAssessmentForm',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/GetAssessmentForm',
					};
				} else if (call === 'ds') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'GetDecisionSuportForm',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/GetDecisionSuportForm',
					};
				} else if (call === 'addHomesVeteran') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'AddOrEditVeteran',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/AddOrEditVeteran',
					};
				} else if (call === 'updateVeteran') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'AddOrEditVeteran',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/AddOrEditVeteran',
					};
				} else if (call === 'SubmitAssessmentForm') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'SubmitAssessmentForm',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/SubmitAssessmentForm',
					};
				} else if (call === 'SubmitDecisionSupportForm') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'SubmitDecisionSupportForm',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/SubmitDecisionSupportForm',
					};
				} else if (call === 'searchVeteranName') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'GetVeteranListByName',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/GetVeteranListByName',
					};
				} else if (call === 'searchVeteranSSN') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'GetVeteranListBySSN',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/GetVeteranListBySSN',
					};
				} else if (call === 'searchVeteranLastFour') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'GetVeteranListByLast4',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/GetVeteranListByLast4',
					};
				} else if (call === 'searchVeteranID') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'GetVeteran',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/GetVeteran',
					};
				} else if (call === 'homesEpisodeCheck') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'GetOpenEpisodeAssessment',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/GetOpenEpisodeAssessment',
					};
				} else if (call === 'sud') {
					return {
						url: this.homesEndpoint + '?op=',
						method: 'SubmitStaffSUDDate',
						namespaceQualifier: 'homs',
						namespaceURL: 'http://tempuri.org/',
						soap12: false,
						SOAPServer: this.currentHomesIP,
						SOAPAction: 'http://tempuri.org/SubmitStaffSUDDate',
					};
				} else {
					errorUndefined('need to define VA backend options for ' + call);
				}
			},

			// testing setup
			testMode: false,
			setTestMode: function() {
				this.testMode = true;
			},
			isTestMode: function() {
				return this.testMode;
			},

			//online indicators
			vaOnline: true,
			setVAOnline: function(ans) {
				this.vaOnline = ans;
			},
			isVAOnline: function() {
				return this.vaOnline;
			},

			homesOnline: true,
			setHomesOnline:	function(ans) {
				console.log('ONLINE ' + ans);
				if (!ans) {
					this.haveBeenOffline = true;
				}
				this.homesOnline = ans;
			},
			isHomesOnline: function() {
				return this.homesOnline;
			},
			isHomesOffline: function() {
				return !this.homesOnline;
			},
			checkForLockout: function(errorData) {
				var lockout = false;
				if (typeof errorData != 'undefined' && errorData) {
					console.log('checkForLockout httpText=' + errorData.httpText);
					if (errorData.httpText && errorData.httpText.indexOf('lock') >= 0) {
						lockout = true;
					}
				}
				this.locked = lockout;
				return this.isLockout();
			},
			isLockout: function() {
				return this.locked;
			},
			workingOffline: false,
			setWorkingOffline: function(ans) {
				this.workingOffline = ans;
			},
			isWorkingOffline: function() {
				return this.workingOffline;
			},
			completeForm:		false
		}.init(); // end of state

		var CONSTANTS = {
			PATIENT_SEARCH_TITLE: 'Veteran Search',
			INTERNET_RETRY_MILLS: 30000,
			DEFAULT_LEFT_ACTION: {
				id: '',
				href: '#',
				name: '',
				theme: 'd',
				icon: 'arrow-l',
				noText: false
			},
			DEFAULT_RIGHT_ACTION: {
				id: '',
				href: '#',
				name: '',
				theme: 'd',
				icon: 'add',
				noText: false,
				dropdown: false
			},
			DS_FORM:				'Decision Support',
			ASSESSMENT_FORM:		'Assessment'
		}; // end of CONSTANTS

		var metaData = {
			homesLogin: {
				title: 'Homes Login'
			},
			vistaLogin: {
			},
			main: {
				headerTitle:		'Homeless Mobile Intervention',
				searchLabel:		'Search',
				editLabel:			'Contact Info',
				historyLabel:		'History',
				assessLabel:		'Assess',
				openEpisodeAssessLabel: 'Assess*',
				reassessLabel:		'Re-Assess',
				decisionLabel:		'Decision Support',
				referralLabel:		'Referral',
				savedDataBtn:		'Show Saved Data',
				ssnLabel:			'SSN:',
				homesIDLabel:		'Homes ID:',
				footerStaffLabel:	'Staff:',
				footerRoleLabel:	'Roles:',
				footerSiteLabel:	'Site:'
			},
			config: {
				title:				'Configuration',
				yearsLabel:			'Back Years in History',
				submitBtn:			'Submit'
			},
			homesList: {
				title: 'List of found Veterans',
				backBtn: 'Back',
				addBtn: 'Add New Veteran',
				showAdd: false
			},
			history: {
			},
			homesVeteranSummary: {
				backBtn: 'Back to Search Results',
				legend: 'Homeless Intervention',
				title: 'Veteran Summary',
				homesId: 'Homes ID',
				lastName: 'Last Name',
				firstName: 'First Name',
				middleInitial: 'Middle Initial',
				dob: 'Date of Birth',
				ssn: 'SSN',
				gender: 'Gender',
				addressLine1: 'Address 1 (c/o)',
				addressLine2: 'Address 2 (ex. 123 Street)',
				addressLine3: 'Address 3 (ex. Apt 101)',
				city: 'City',
				state: 'State',
				zip: 'ZIP',
				primaryPhone: 'Phone (Primary)',
				secondaryPhone: 'Phone (Secondary)',
				email: 'Email',
				pOCName: 'POC Name',
				pOCPhone: 'POC Phone',
				pOCRelationship: 'POC Relationship',
				pOCRelationshipOther: 'POC Relationship Other',
				editLabel: 'Edit'
			},
			homesSavedData: {
				title:			'Saved Local Data',
				vetsTitle:		'Saved Veterans',
				assessTitle:	'Saved Assessments',
				dsTitle:		'Saved Decision Support Forms'
			},
			homesAssessment: {
				backBtn:			'Back',
				prevBtn:			'Previous Section',
				nextBtn:			'Next Section',
				showNextPrev:		true,
				submitBtn:			'Submit Form',
			},
			hds: {
				backBtn:			'Back',
				showNextPrev:		false,
				submitBtn:			'Recommendations',
			},
			recommend: {
				backBtn:			'Back To Form',
				submitBtn:			'Submit',
			},
			form: {
			},
			eula: {
				key:				'homeless_eula'
			},
			sud: {
				key:				'homeless_sud'
			},
			dob: {
				title:				'Date of Birth'
			},
			assessDate: {
				title:				'Date of Assessment'
			},
			dsDate: {
				title:				'Date of Assessment'
			}
		}; // end of metaData

		var messages = {
			confirmVeteranDeletion: {
				title:				'Confirmation',
				text:				['Are you sure you want to delete this Veteran data locally?' ],
				messageClass:		'msg-confirmation',
				okButtonText:		'Yes, delete it',
				cancelButtonText:	'No, do not delete it'
			},
			confirmAssessmentDeletion: {
				title:				'Confirmation',
				text:				['Are you sure you want to delete this Assessment Form locally?' ],
				messageClass:		'msg-confirmation',
				okButtonText:		'Yes, delete it',
				cancelButtonText:	'No, do not delete it'

			},
			confirmDSFormDeletion: {
				title:				'Confirmation',
				text:				['Are you sure you want to delete this Decision Support Form locally?' ],
				messageClass:		'msg-confirmation',
				okButtonText:		'Yes, delete it',
				cancelButtonText:	'No, do not delete it'
			},
			successfullyAddedVetOnline: {
				title:				'Information',
				text:				['Veteran was successfully added to HOMES' ],
				messageClass:		'msg-info',
				okButtonText:		'OK'
			},
			errorAddingVetOnline: {
				title:				'Error',
				text:				['Veteran was not successfully added to HOMES',
									'Do you want to save the veteran information locally on the device so you can keep working?' ],
				messageClass:		'msg-error',
				cancelButtonText:	'Do not save',
				okButtonText: 		'Save information locally'
			},
			errorAddingVet: {
				title:				'Error',
				text:				['Veteran was not successfully added to HOMES' ],
				messageClass:		'msg-error',
				okButtonText: 		'Ok'
			},
			successfullyAddedVetOffline: {
				title:				'Information',
				text:				['Veteran was successfully saved locally to your device' ],
				messageClass:		'msg-info',
				okButtonText:		'OK'
			},
			successfullyEditedVetOffline: {
				title:				'Information',
				text:				['Veteran edits were successfully saved locally to your device' ],
				messageClass:		'msg-info',
				okButtonText:		'OK'
			},
			errorAddingVetOffline: {
				title:				'Error',
				text:				['Veteran was not successfully saved locally to your device' ],
				messageClass:		'msg-error',
				okButtonText:		'OK'
			},
			errorEditingVetOffline: {
				title:				'Error',
				text:				['Veteran edits were not successfully saved locally to your device' ],
				messageClass:		'msg-error',
				okButtonText:		'OK'
			},
			successfullyEditedVetOnline: {
				title:				'Information',
				text:				['Veteran was successfully edited in HOMES' ],
				messageClass:		'msg-info',
				okButtonText:		'OK'
			},
			veteranSummaryValidation: {
				title:				'Error',
				text:				['Please fill in the required fields' ],
				messageClass:		'msg-error',
				okButtonText:		'OK'
			},
			veteranSummarySSNValidation: {
				title:				'Error',
				text:				['SSN input is invalid',
									'Please make sure the SSN is nine digits without dashes and ',
									'tap the checkbox to confirm the SSN is correct' ],
				messageClass:		'msg-error',
				okButtonText:		'OK'
			},
			veteranSummaryZipCodeValidation: {
				title:				'Error',
				text:				['Zip Code input is invalid',
									'Please make sure the Zip Code is 5 digits' ],
				messageClass:		'msg-error',
				okButtonText:		'OK'
			},
			homesOffline: {
				title:				'Information',
				text:				['HOMES Offline',
									'You are unable to connect to the HOMES DB',
									'Would you like to wait 30 seconds and attempt to connect again or work offline?' ],
				messageClass:		'msg-info',
				okButtonText:		'Work Offline',
				cancelButtonText:	'Try Again'
			},
			badHomesVetsSearch: {
				title:				'Error',
				text:				['Invalid Search',
									'There were errors with your search',
									'Please try again' ],
				messageClass:		'msg-error',
				okButtonText:		'Ok'
			},
			historyYearsValidation: {
				title:				'Error',
				text:				['Years of history input is invalid',
									'Please make sure the years entry is an integer greater than 0' ],
				messageClass:		'msg-error',
				okButtonText:		'OK'
			},
			configSubmitSuccess: {
				title:				'Information',
				text:				['Configuration changes successfully completed'],
				messageClass:		'msg-info',
				okButtonText:		'OK'
			},
			webCallFailed: {
				title:				'Error',
				text:				['Call to HOMES failed',
									'Please try again' ],
				messageClass:		'msg-error',
				okButtonText:		'Ok'
			},
			incomplete: {
				title:				'Error',
				text:				['Incomplete Form',
									'Please ensure all requested fields are filled out.'],
				messageClass:		'msg-error',
				okButtonText:		'Ok'
			},
			saveLocalAssessment: {
				title:				'Error',
				text:				['While Saving the Assessment'],
				messageClass:		'msg-error',
				okButtonText:		'Ok'
			},
			successfullySubmittedAssessment: {
				title:				'Information',
				text:				['Assessment Successfully Submitted'],
				messageClass:		'msg-info',
				okButtonText:		'Ok'
			},
			successfullySubmittedDSForm: {
				title:				'Information',
				text:				['Decision Support Form Successfully Submitted'],
				messageClass:		'msg-info',
				okButtonText:		'Ok'
			},
			submitMatchFoundSummary: {
				title:				'Confirmation',
				text:				[''],
				messageClass:		'msg-confirmation',
				okButtonText:		'Overwrite',
				cancelButtonText:	'Cancel'
			},
			submitMatchFoundForm: {
				title:				'Confirmation',
				text:				[''],
				messageClass:		'msg-confirmation',
				okButtonText:		'Overwrite',
				cancelButtonText:	'Cancel'
			},
			manualFormSubmitReminder: {
				title:				'Information',
				text:				['You will need to manually submit any Assessments or Decision Support Forms ',
									'stored on this device after verifying and submitting the Veteran Summary data.'],
				messageClass:		'msg-info',
				okButtonText:		'Ok'
			},
			errorSubmittingAssessment: {
				title:				'Error',
				text:				['Error Submitting Assessment'],
				messageClass:		'msg-error',
				okButtonText:		'Ok'
			},
			errorSubmittingDSForm: {
				title:				'Error',
				text:				['Error Submitting Decision Support Form'],
				messageClass:		'msg-error',
				okButtonText:		'Ok'
			},
			searchValidationError: {
				title:				'Error',
				text:				['Improper Search Query'],
				messageClass:		'msg-error',
				okButtonText:		'Ok'
			},
			invalidLoginAttempt: {
				title:				'Error',
				text:				['Invalid Login Attempt',
									'Incorrect user name or password'],
				messageClass:		'msg-error',
				okButtonText:		'Ok'
			},
			lockoutdLogin: {
				title:				'Error',
				text:				['Account Locked',
									'Your account has been locked.',
									'Please try again later.'],
				messageClass:		'msg-error',
				okButtonText:		'Ok'
			},
			readonlyLoginAttempt: {
				title:				'Error',
				text:				['You do not have sufficient permissions to use this application.',
									'Please contact your HOMES administrator if you have questions.'],
				messageClass:		'msg-error',
				okButtonText:		'Ok'
			},
			offline: {
				title:				'Information',
				text:				['Connection to Homes was lost. Some data may not be available offline.'],
				messageClass:		'msg-info',
				okButtonText:		'Ok'
			},
			openEpisodeInProgressInVISN: {
				title:				'Information',
				text:				['A Reassessment is not indicated at this time'],
				messageClass:		'msg-info',
				okButtonText:		'Ok'
			},
			openEpisodeInProgressOutsideVISN: {
				title:				'Information',
				text:				['A Reassessment is not indicated at this time.'],
				messageClass:		'msg-info',
				okButtonText:		'Ok'
			},
			mustAgree:	{
				title:				'Confirmation',
				text:				['Must Agree',
									'You must agree to this disclaimer to use the Homeless Application'],
				messageClass:		'msg-confirmation',
				cancelButtonText:	'Leave App',
				okButtonText:		'Go Back'
			},
			confirmOverwriteLocalChanges: {
				title:				'Confirmation',
				text:				['You have data saved locally for this Veteran but not yet submitted to HOMES.',
									'If you continue, you will overwrite your local changes for this Veteran.',
									'If you do not wish to overwrite your local changes, tap Show Saved Data,',
									' select the Veteran and submit your changes.'],
				messageClass:		'msg-confirmation',
				cancelButtonText:	'Do Not Overwrite',
				okButtonText:		'Overwrite my Local Data'
			},
			confirmAbout: {
				title:				'Confirmation',
				text:				['If you open the About Section now, you will lose ',
									'any changes made to the Current Veteran.'],
				messageClass:		'msg-confirmation',
				cancelButtonText:	'Cancel',
				okButtonText:		'Open About'
			},
			homesError: {
				title:				'Error',
				text:				['HOMES Returned an Error' ],
				messageClass:		'msg-error',
				okButtonText:		'Ok'
			},
			cantSubmitAssessment: {
				title:				'Information',
				text:				['You are currently not connected to HOMES.',
									'Your Assessment Form will be saved locally to your device.',
									'You will be able to submit the form once data connectivity is reestablished.'],
				messageClass:		'msg-info',
				okButtonText:		'Ok'
			},
			cantSubmitDS: {
				title:				'Information',
				text:				['You are currently not connected to HOMES.',
									'Your Decision Support Form will be saved locally to your device.',
									'You will be able to submit the form once data connectivity is reestablished.'],
				messageClass:		'msg-info',
				okButtonText:		'Ok'
			},
			dssQ5h: {
				title:				'Notice',
				text:				['Chronic homelessness is defined as 365 or more days of homelessness.',
									'Please conform to that definition when possible during program placement decision ',
									'but use your discretion when considering "borderline" cases.'],
				messageClass:		'msg-info',
				okButtonText:		'Ok'
			},
		}; // end of messages

		return {
			getUtilities: function() {
				return utilities;
			},
			assembleListItems: function() {
				return listItems;
			},
			defineResourceTitles: function() {
				return app;
			},
			getConstants: function() {
				return CONSTANTS;
			},
			getMetaData: function() {
				return metaData;
			},
			getMessages: function() {
				return messages;
			},
			getState: function() {
				return state;
			},
			setParms: function(parms) {
				state.appName = parms.CFBundleDisplayName;
				state.metricClient = parms.metricClient;
				state.metricSecret = parms.metricSecret;
				state.homesEndpoint = parms.homesEndpoint;
				state.metricEndpoint = parms.metricEndpoint;
				state.currentVersion = parms.CFBundleVersion;
			}
		};
	} // end of big function
);
