define(
	[
	'Global', 'App', 'Response'
	],
	function(Global, App, Response) {
		'use strict';
		var singleInstance, collection,
		Responses = Backbone.Collection.extend({
			model:		Response,
			initialize: function(params) {
				collection = this;
			},
			setDefaults_HOMES: function() {
				var letters;
				letters = 'abcdefgh';
				for (var i in letters) {
					collection.setOrCreate('ddlHSA_Q18' + letters[i], '0', null);
				}
				collection.setOrCreate('ddlHSA_Q20', '0', null);
				letters = 'abcdefghijklmnopqrst';
				for (var i in letters) {
					collection.setOrCreate('txtHSA_Q20' + letters[i], '0', null);
				}
				letters = 'abcdefghijklmnop';
				for (var i in letters) {
					collection.setOrCreate('txtHSA_Q30' + letters[i], '0', null);
				}
				letters = 'abcdefghij';
				for (var i in letters) {
					collection.setOrCreate('ddlHSA_Q36' + letters[i], '0', null);
				}
				// HMLS-807 dont pre-populate the date
				console.log('setting txtAssessmentDate blank');
				collection.setOrCreate('txtAssessmentDate','',null)
			},
			setDefaults_HDS: function() {
				// HMLS-807 dont pre-populate the date
				console.log('setting txtAssessmentDate blank');
				collection.setOrCreate('txtAssessmentDate','',null)
			},
			getSectionResponses: function(section) {
				return this.models.filter(collection, function(response) {
					return reponse.get('section') === section;
				})
			},
			acceptAnswer: function(vaId) {
				var response = collection.get(vaId);
				if (response) {
					response.set('response',response.get('prev'));
					response.set('prev',null);
				}
			},
			removeAnswer: function(vaId) {
				var response = collection.get(vaId);
				if (response) {
					collection.remove(response,{silent: true});
				}
			},
			setOrCreate: function(vaId,val,prev) {
				if (!vaId) {
					// what?
					return;
				}
				var response = collection.get(vaId);
				if (response) {
					response.set('response',val);
					response.set('prev',prev);
				} else {
					response = new Response({vaId: vaId, response: val, prev: prev});
					collection.push(response);
				}
			},
			setOrCreateNoPrevChange: function(vaId,val) {
				if (!vaId) {
					// what?
					return;
				}
				var response = collection.get(vaId);
				if (response) {
					response.set('response',val);
				} else {
					response = new Response({vaId: vaId, response: val});
					collection.push(response);
				}
			},
			setFieldNoPrevChange: function(valId,val) {
				if (val) {
					collection.setOrCreateNoPrevChange(valId,val)
				} else {
					var response = collection.get(valId);
					if (response && !response.get('prev')) {
						collection.remove(response,{silent: true});
					}
				}
			},
			getAnswer:	function(vaId) {
				var response = collection.get(vaId);
				if (response) {
					return response.get('response');
				}
				return null;
			},
			getPrev:	function(vaId) {
				var response = collection.get(vaId);
				if (response) {
					return response.get('prev');
				}
				return null;
			},
			erasePrev:	function(vaId) {
				var response = collection.get(vaId);
				if (response) {
					response.set('prev',null);
				}
			},
			eraseResponse:	function(vaId) {
				var response = collection.get(vaId);
				if (response) {
					response.set('response',null);
				}
			},
			fromVAforAssessment: function(vaId, answer) {
				var ans = answer;

				if (vaId==='txtHSA_Q21' || vaId==='ddlHSA_Q31_1' || vaId==='ddlHSA_Q32_1' || vaId==='ddlHSA_Q44') {
					ans = collection.toLower(ans);
				}
				if (ans==='NaN') ans = ' ';  //single space, this happened on ddlHSA_Q20 on the blank answer

				var prev = '';
				if (vaId==='txtAssessmentDate') {
					// HMLS-807 dont pre-populate the date
					ans = '';
				} else if (vaId.indexOf('Total') < 0) {
					prev = ans;
				}

				collection.setOrCreate(vaId,ans,prev);
			},
			fromVAforHistory: function(vaId, answer) {
				var arrayPattern = /(.*Q(?:6c_|31|32|44|10_))(\w*)/i;
				var baseToVaId = {
					ddlHSA_Q6c_:	'ddlHSA_Q6c',
					ddlHSA_Q31:		'ddlHSA_Q31_1',
					ddlHSA_Q32:		'ddlHSA_Q32_1',
					ddlHSA_Q44:		'ddlHSA_Q44',
					ddlDssQ10_:		'ddlDssQ10'
				}
				var ans = answer;

				if (ans==='NaN') ans = ' ';  //single space, this happened on ddlHSA_Q20 on the blank answer

				if (vaId==='txtAssesmentDate') { // used to have a misspelling
					vaId = 'txtAssessmentDate';
				}

				if (vaId==='txtAssessmentDate') {
					ans = formatDateNoTime(ans);
				}

				// turn multiselects into arrays Q44a,Q44b.. --> Q44 as an array
				// taken from formRules.fromLocalStorage()
				var result = arrayPattern.exec(vaId);
				if (result && result.length > 1 && baseToVaId[result[1]] && result[2].length > 0 && result[2].indexOf('ther') < 0) {
					vaId = baseToVaId[result[1]];
					ans = collection.getAnswer(vaId);
					if (!ans || !$.isArray(ans)) {
						ans = [];
					}
					if (answer==='1') {
						ans.push(result[2]);
					}
				}
				collection.setOrCreate(vaId,ans,null);
			},
			toLower: function(answer) {
				if ($.isArray(answer)) {
					for (var i in answer) {
						if (typeof answer[i] === 'string' && answer[i].length==1) {
							answer[i] = answer[i].toLowerCase();
						}
					}
				} else {
					if (typeof answer === 'string' && answer.length==1) {
						answer = answer.toLowerCase();
					}
				}
				return answer;
			}
		});

		Responses.instance = function (params) {
			if (typeof singleInstance === "undefined") {
				console.log('new Responses params=' + JSON.stringify(params));
				singleInstance = new Responses(params);
			}
            return singleInstance;
        };

        Responses.newInstance = function() {
        	Responses.clearInstance();
        	return Responses.instance();
        };

        Responses.clearInstance = function() {
            singleInstance = undefined;
        };

        return Responses;
	}
);
