define(['Responses', 'Response'], function(Responses, Response) {
	var getAnswer = function(name) {
		return Responses.instance().getAnswer(name);
	};

	var getPrev = function(name) {
		return Responses.instance().getPrev(name);
	};

	var is = function(name, val) {
		var answer = Responses.instance().getAnswer(name);
		if (answer == null) {
			return val == null;
		} else {
			if ($.isArray(answer)) {
				for (var i = 0; i < answer.length; i++) {
					if (answer[i] === val)
						return true;
				}
				return false;
			} else {
				return answer == val;
			}
		}
	};

	var isYes = function(name) {
		return is(name, '1');
	};

	var isNo = function(name) {
		return is(name, '0');
	};

	var isAnswered = function(name) {
		return !isIn(name, ['Choose options', '', null]);
	};

	var isInRange = function(name,min,max) {
		if (isInt(name)) {
			var answer = getAnswer(name);
			return min <= answer && answer <= max;
		} else {
			return false;
		}
	};

	var isInt = function(name) {
		var answer = getAnswer(name);
		return isInteger(answer);
	};

	var isIn = function(name,vals) {
		var answer = getAnswer(name);
		for (var i in vals) {
			if (answer===vals[i]) return true;
		}
		return false;
	};

	var vaIdPattern		= /.*_Q(\d*)(\w*|$)/i;
	var hdsPattern		= /ddlDssQ(\d*)/i;
	var idPattern		= /.*\.(\d*)(\w*|$)/i;
	var answerPattern	= /([^-]*).*/i;
	var yesPattern		= /.*\.yes$/i;

	var displayLogic = {
		isSubQuestion: function(question) {
			var result = vaIdPattern.exec(question.vaId);
			if (result && result.length > 1) {
				return result[2].length > 0;
			} else {
				return false;
			}
		},
		buildPossibleAnswers: function(questionHash,answerHash) {
			var possibleAnswers = {};
			for (var section in questionHash) {
				var questions = questionHash[section];
				for (var i in questions) {
					var question = questions[i];
					possibleAnswers[question.vaId] = this.getPossibleAnswers(answerHash,question.answerBase)
				}
			}
			return possibleAnswers;
		},
		getPossibleAnswers:	function(answersHash,baseName) {
			var possibleAnswers = [];
			var result = answerPattern.exec(baseName);
			if (result && result.length > 0 && answersHash[result[1]]) {
				possibleAnswers = answersHash[result[1]];
			}
			return possibleAnswers;
		},
		isQuestionDisplayed: function(question) {
			if (question.vaId === '') {
				return this.isQuestionDisplayedBaseOnId(question.id);
			} else {
				return this.isQuestionDisplayedBaseOnVaId(question.vaId);
			}
		},
		isQuestionDisplayedBaseOnId: function(id) {
			var result = idPattern.exec(id);
			if (result && result.length > 0) {
				var category = result[1];
				var subCategory = result[2];
				var fullId = category + subCategory;
				if (isNo('ddlHSA_Q6')) {
					if (category!='74' && category!='75' && category!='5' && category!='6') return false;
				}
				if (fullId === '5') {
					return isNo('ddlHSA_Q5');
				}
			}
			return true;
		},
		isQuestionDisplayedBaseOnVaId:	function(vaId) {
			var result = vaIdPattern.exec(vaId);
			if (result && result.length > 0) {
				var category = result[1];
				var subCategory = result[2];
				var fullId = category + subCategory;
				if (isNo('ddlHSA_Q6')) {
					if (category!='74' && category!='75' && category!='5' && category!='6') return false;
				}
				if (category === '5') {
					if (subCategory.length > 0 && 'abcdefg'.indexOf(subCategory) >= 0) {
					return this.cat5Rules.defaultRule(this);
					}
				} else if (category === '6') {
					if (this.cat6Rules[fullId]) {
						return this.cat6Rules[fullId](this);
					}
				} else if (category==='10') {
					if (this.cat10Rules[fullId]) {
						return this.cat10Rules[fullId](this);
					}
				} else if (category==='16') {
		               return !isIn('ddlHSA_Q15', ['98', '99']);
				} else if (category==='20') {
					if (subCategory.length > 0 && 'abcdefghijklmnopqrst'.indexOf(subCategory) >= 0) {
						return this.cat20Rules.defaultRule(this);
					}
				} else if (category==='23') {
					if (!this.cat20Rules.defaultRule(this)) {
						return false;
					} else if (subCategory.length > 0 && 'abcd'.indexOf(subCategory) >= 0) {
						return this.cat23Rules.defaultRule(this);
					}
					return true;
				} else if (category==='25') {
					return is('ddlHSA_Q24', 1);
				} else if (category==='30') {
					if (subCategory.length > 0 && 'abcdefghijklmnop'.indexOf(subCategory) >= 0) {
		                return isYes('ddlHSA_Q30');
					}
				} else if (category==='31') {
					if (subCategory==='_1') {
		                return isYes('ddlHSA_Q31');
					}
				} else if (category==='32') {
					if (subCategory==='_1') {
		                return isYes('ddlHSA_Q32');
					}
					if (subCategory==='j_other') {
		                return isYes('ddlHSA_Q32') && is('ddlHSA_Q32_1', 'j');
					}
				} else if (category==='36') {
					if (subCategory==='j_other') {
		                return isYes('ddlHSA_Q36j');
					}
				} else if (category==='45') {
					if (subCategory==='b') {
		                return isYes('ddlHSA_Q45a');
					}
				} else if (category==='46') {
					if (subCategory==='b') {
		                return isYes('ddlHSA_Q46a');
					}
				} else if (category==='47') {
					if (subCategory==='b') {
		                return isYes('ddlHSA_Q47a');
					}
				} else if (category==='48') {
					if (subCategory==='b') {
		                return isYes('ddlHSA_Q48a');
					}
				} else if (category==='49') {
					if (subCategory==='b') {
		                return isYes('ddlHSA_Q49a');
					}
				} else if (category==='74') {
					if (subCategory==='_Other') {
		                return is('ddlHSA_Q74', '7');
					}
				} else if (category==='75') {
					if (subCategory==='_Other') {
		                return is('ddlHSA_Q75', '27');
					}
				} else {
					if (this.displayRules[fullId]) {
						return this.displayRules[fullId](this);
					}
				}
				return true;
			} else {
				if (isNo('ddlHSA_Q6')) return false;

				if (vaId==='txtHSA_Q20_Total') {
					return this.cat20Rules.defaultRule(this);
				} else if (vaId==='txtHSA_Q30_Total') {
	                return isYes('ddlHSA_Q30');
				} else if (vaId==='ddlDssQ5') {
	                return isIn('ddlDssQ3', ['0', '1']);
				} else if (vaId==='ddlDssQ7') {
	                return isIn('ddlDssQ6', ['1', '2', '3', '4']);
				} else if (vaId==='ddlDssQ8') {
	                return isIn('ddlDssQ3', ['5', '6', '7']);
				} else {
					return true;
				}
			}
		},
		cat5Rules: {
			defaultRule: function(parent) {
                return isNo('ddlHSA_Q5');
            },
		},
		cat6Rules: {
			defaultRule:
				function(parent) {
					return isNo('ddlHSA_Q6');
				},
			// key is id of a question, the function gives answers whether the question should be displayed
			'6a':
				function(parent) {
					return this.defaultRule(parent);
				},
			'6b':
				function(parent) {
					return this.defaultRule(parent);
				},
			'6c':
				function(parent) {
					return isNo('ddlHSA_Q6') &&
							isYes('ddlHSA_Q6b');
				},
			'6c_other':
				function(parent) {
					return isNo('ddlHSA_Q6') &&
							isYes('ddlHSA_Q6b') &&
							is('ddlHSA_Q6c', '8');
				},
			'6d':
				function(parent) {
					return this.defaultRule(parent);
				}
		},
		cat10Rules: {
			'10a':
				function(parent) {
					return !isIn('txtHSA_Q10', ['0', 'N']);
				}
		},
		cat20Rules: {
			defaultRule: function(parent) {
                return !isIn('ddlHSA_Q20', ['98', '99']);
            }
		},
		cat23Rules: {
			defaultRule: function(parent) {
                return !isIn('ddlHSA_Q23', ['98', '99', '0']);
            }
		},
		displayRules: {
			defaultRule: function(parent) {
                return !isIn('ddlHSA_Q20',  ['98', '99']);
            },
            '21':	function(parent) {
            	return parent.cat20Rules.defaultRule(parent);
            },
            '22':	function(parent) {
            	return parent.cat20Rules.defaultRule(parent);
            }
		}
	}; // end of displayLogic


	var validationLogic = {
		validate: function(ids,vet) {
			var errs = [];
			for (var i in ids) {
				if (this.validateRules[ids[i]]) {
					var err = this.validateRules[ids[i]](vet,this);
					if (err) {
						if ($.isArray(err)) {
							for (var i in err) {
								errs.push(err[i]);
							}
						} else {
							errs.push(err);
						}
                    }
				}
				ans = this.validateRules.listRule(ids[i]);
				if (ans) {
					errs.push(ans);
				}
				ans = this.validateRules.hdsRule(ids[i]);
				if (ans) {
					errs.push(ans);
				}
				ans = this.validateRules.prevRule(ids[i]);
				if (ans) {
					errs.push(ans);
				}
			}
			return errs;
		},
		getSum:	function(vaId) {
			if (vaId==='Q20') {
				var sum = 0;
				var letters = 'abcdefghijklmnopqrst';
				for (var i in letters) {
					var val = getAnswer('txtHSA_Q20' + letters[i]);
					if (isNumber(val)) {
						sum += parseInt(val);
					}
				}
				return sum;
			} else if (vaId==='Q30') {
				var sum = 0;
				var letters = 'abcdefghijklmnop';
				for (var i in letters) {
					var vaId = 'txtHSA_Q30' + letters[i];
					if (isInRange(vaId,1,99999)) {
						sum += parseInt(getAnswer(vaId));
					}
				}
				return sum;
			}
			return 0;
		},
		validateRules:	{
			standardMissingRule:	function(vaId) {
				if (!isAnswered(vaId)) {
					var result = vaIdPattern.exec(vaId);
					if (result && result.length > 0) {
						var category = result[1];
						var subCategory = result[2];
						var fullId = category + subCategory;
						return {msgType: 'incomplete',
								msgText: 'Please answer item ' + fullId,
								elementId: vaId};
					}
				}
			},
			standardPrevRule:	function(vaId) {
				if (!isBlank(getPrev(vaId))) {
					var idToDisplay = vaId;
					var result = vaIdPattern.exec(vaId);
					if (result && result.length > 0) {
						var idToDisplay = result[1] + result[2];
					}
					return {msgType: 'incomplete',
							msgText: 'Please accept or answer item ' + idToDisplay,
							elementId: vaId};
				}
			},
			standardMissingRule_HDS:	function(vaId) {
				if (!isAnswered(vaId)) {
					var result = hdsPattern.exec(vaId);
					if (result && result.length > 0) {
						var category = result[1];
						return {msgType: 'incomplete',
								msgText: 'Please answer item ' + category,
								elementId: vaId};
					}
				}
			},
			rangeOrNCheck:	function(vaId,min,max,msg) {
				if (!is(vaId, 'N') && !isInRange(vaId, min, max)) {
					return {msgType: 'incomplete', msgText: msg, elementId: vaId};
				}
			},
			listRule:	function(id) {
				var list = ['8', '9', '13', '14', '15', '17', '19', '27', '28', '33', '34', '35', '37', '41', '42',
				            '43', '45a', '45b', '46a', '46b', '47a', '47b', '48a', '48b', '49a', '49b', '50', '51', '74', '75'];
				for (var i in list) {
					var vaId = 'ddlHSA_Q' + list[i];
					if (vaId===id) {
						return this.standardMissingRule(vaId);
					}
				}
			},
			hdsRule:	function(id) {
				var list = ['1', '2', '3', '4', '6', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18'];
				for (var i in list) {
					var vaId = 'ddlDssQ' + list[i];
					if (vaId===id) {
						return this.standardMissingRule_HDS(vaId);
					}
				}
			},
			prevRule:	function(id) {
				return this.standardPrevRule(id);
			},
			'ddlDssQ5':	function(vet,parent) {
				if (!isAnswered('ddlDssQ5') && isIn('ddlDssQ3', ['0', '1'])) {
					return this.standardMissingRule_HDS('ddlDssQ5');
				}
			},
			'ddlDssQ7':	function(vet,parent) {
				if (!isAnswered('ddlDssQ7') && isIn('ddlDssQ6', ['1', '2', '3', '4'])) {
					return this.standardMissingRule_HDS('ddlDssQ7');
				}
			},
			'ddlDssQ8':	function(vet,parent) {
				if (!isAnswered('ddlDssQ8') && isIn('ddlDssQ3', ['5', '6', '7'])) {
					return this.standardMissingRule_HDS('ddlDssQ8');
				}
			},
			'ddlHSA_Q5':	function(vet,parent) {
				if (!isAnswered('ddlHSA_Q5')) {
					return {msgType: 'incomplete',
							msgText: 'Please indicate whether Pre-screening may be skipped.',
							elementId: 'ddlHSA_Q5'};
				}
//				var msgs = [];
				var ids = [];
				if (isNo('ddlHSA_Q5')) {
					var letters = 'abcdefg';
					for (var i in letters) {
						if (!isAnswered('ddlHSA_Q5' + letters[i])) {
//							msgs.push('Please answer item 5' + letters[i] + '.');
							ids.push({msgType: 'incomplete',
								msgText: 'Please answer item 5' + letters[i] + '.',
								elementId: 'ddlHSA_Q5' + letters[i]});
						}
					}
//					if (msgs.length > 0) {
//						var msg = '';
//						for (var i = 0; i < msgs.length; i++) {
//							msg += '<br>' + msgs[i];
//						}
//						return {msgType: 'incomplete', msgText: msg, elementId: 'ddlHSA_Q5'};
//					}
				}
				return ids;
			},
			'ddlHSA_Q6':	function(vet,parent) {
				if (!isAnswered('ddlHSA_Q6')) {
					return {msgType: 'incomplete',
							msgText: 'Please indicate whether the assessment interview will be completed.',
							elementId: 'ddlHSA_Q6'};
				}
			},
			'ddlHSA_Q6a':	function(vet,parent) {
				if (isNo('ddlHSA_Q6')) {
					if (!isAnswered('ddlHSA_Q6a')) {
						return {msgType: 'incomplete',
                                msgText: 'Please provide the reason that the Assessment form will not be completed.',
                                elementId: 'ddlHSA_Q6a'};
					}
				}
				return null;
			},
			'ddlHSA_Q6b':	function(vet,parent) {
				if (isNo('ddlHSA_Q6')) {
					if (!isAnswered('ddlHSA_Q6b')) {
						return {msgType: 'incomplete',
                                msgText: 'Please identify if immediate Non-VA homeless services are required.',
                                elementId: 'ddlHSA_Q6b'};
					}
				}
				return null;
			},
			'ddlHSA_Q6c':	function(vet,parent) {
				if (isNo('ddlHSA_Q6') && isYes('ddlHSA_Q6b')) {
					if (!isAnswered('ddlHSA_Q6c')) {
						return {msgType: 'incomplete',
                                msgText: 'Please specify which immediate Non-VA homeless services are required.',
                                elementId: 'ddlHSA_Q6c'};
					}
				}
				return null;
			},
			'txtHSA_Q6c_other':	function(vet,parent) {
				if (isNo('ddlHSA_Q6') &&
						isYes('ddlHSA_Q6b') &&
						is('ddlHSA_Q6c', '8')) {
					if (!isAnswered('txtHSA_Q6c_other')) {
						return {msgType: 'incomplete',
                                msgText: 'Please specify other immediate Non-VA homeless services required.',
                                elementId: 'txtHSA_Q6c_other'};
					}
				}
				return null;
			},
			'ddlHSA_Q6d':	function(vet,parent) {
				if (isNo('ddlHSA_Q6')) {
					if (!isAnswered('ddlHSA_Q6d')) {
						return {msgType: 'incomplete',
								msgText: 'Please answer item 6d.',
								elementId: 'ddlHSA_Q6d'};
					}
				}
				return null;
			},
			'ddlHSA_Q7':	function(vet,parent) {
				if (!isAnswered('ddlHSA_Q7')) {
					return {msgType: 'incomplete', msgText: 'Please answer item 7.',
						elementId: 'ddlHSA_Q7'};
				}
				return null;
			},
			'txtHSA_Q10':	function(vet,parent) {
				if (!isAnswered('txtHSA_Q10')) {
					return {msgType: 'incomplete', msgText: 'Please answer item 10.',
						elementId: 'txtHSA_Q10'};
				}
				return this.rangeOrNCheck('txtHSA_Q10', 0, 99,
						'Number of children must be between 0-99 or code N if Veteran refused to answer.');
			},
			'txtHSA_Q10a':	function(vet,parent) {
				ans10 = getAnswer('txtHSA_Q10');
				ans10a = getAnswer('txtHSA_Q10a');
				if (isNumber(ans10)) {
					if (ans10 > 0) {
						if (ans10a == 'N') {
						} else if (!isNumber(ans10a) || parseInt(ans10a) < 0 || (parseInt(ans10a) > parseInt(ans10))) {
							return {msgType: 'incomplete', msgText: 'Please answer item 10a with a number between 0 and ' + ans10 + '.',
								elementId: 'txtHSA_Q10a'};
						}
					}
				}
				return null;
			},
			'txtHSA_Q11':	function(vet,parent) {
				if (!isAnswered('txtHSA_Q11')) {
					return {msgType: 'incomplete',
							msgText: 'Please answer item 11.',
							elementId: 'txtHSA_Q11'};
				}
				return this.rangeOrNCheck('txtHSA_Q11', 0, 20,
					'Years of formal education must be between 0-20 or code N if Veteran refused to answer');
			},
			'txtHSA_Q12a':	function(vet,parent) {
				var vaId = 'txtHSA_Q12a';
				if (!isAnswered(vaId)) {
					return {msgType: 'incomplete',
							msgText: 'Please specify the year the Veteran entered military service.',
							elementId: vaId};
				} else if (!is(vaId, 'N') && !isValidYear(getAnswer(vaId))) {
					return {msgType: 'incomplete',
							msgText: 'Please enter a valid year.',
							elementId: vaId};
				}
				return null;
			},
			'txtHSA_Q12b':	function(vet,parent) {
				var vaId = 'txtHSA_Q12b';
				if (!isAnswered(vaId)) {
					return {msgType: 'incomplete',
							msgText: 'Please specify the year the Veteran separated from military service.',
							elementId: vaId};
				} else if (!is(vaId, 'N') && !isValidYear(getAnswer(vaId))) {
					return {msgType: 'incomplete',
							msgText: 'Please enter a valid year.',
							elementId: vaId};
				} else {
					var yearA = getAnswer('txtHSA_Q12a');
					var yearB = getAnswer('txtHSA_Q12b');
					if (yearA!=='N' && yearB!=='N' && yearB < yearA) {
						return {msgType: 'incomplete',
								msgText: 'Date of separation cannot be less than year of entry.',
								elementId: vaId};
					}
				}
				return null;
			},
			'txtHSA_Q16':	function(vet,parent) {
				if (isIn('ddlHSA_Q15', ['1', '2', '3'])) {
					var ans = this.standardMissingRule('txtHSA_Q16');
					if (ans) {
						return ans;
					}
				}
				if (is('ddlHSA_Q15', '1')) {
					var ans = this.rangeOrNCheck('txtHSA_Q16', 1, 9,
							'Item 15 is Enlisted, E-rating must be between 1-9 or N if unknown or Veteran declined to answer.');
					if (ans) return ans;
				} else if (is('ddlHSA_Q15', '2')) {
					var ans = this.rangeOrNCheck('txtHSA_Q16', 1, 5,
							'Item 15 is Warrant Officer, W-rating must be between 1-5 or N if unknown or Veteran declined to answer.');
					if (ans) return ans;
				} else if (is('ddlHSA_Q15', '3')) {
					var ans = this.rangeOrNCheck('txtHSA_Q16', 1, 10,
							'Item 15 is Commissioned Officer, C-rating must be between 1-10 or N if unknown or Veteran declined to answer.');
					if (ans) return ans;
				}
			},
			'ddlHSA_Q20':	function(vet,parent) {
				if (!isIn('ddlHSA_Q20',  ['98', '99'])) {
					var sum = parent.getSum('Q20');
					if (sum != 30) {
						return {msgType: 'incomplete',
							msgText: 'Item 20 (categories a-t) currently add up to ' + sum + ' but must add up to 30.',
							elementId: 'ddlHSA_Q20'};
					}
				}
			},
			'txtHSA_Q21':	function(vet,parent) {
				if (!isIn('ddlHSA_Q20',  ['98', '99'])) {
					var ans = this.standardMissingRule('txtHSA_Q21');
					if (ans) return ans;
					var letters = 'abcdefghijklmnopqrst';
					for (var i in letters) {
						if (is('txtHSA_Q21', letters[i]) &&
							!isInRange('txtHSA_Q20' + letters[i], 1, 30)) {
							return {msgType: 'incomplete',
									msgText: 'Item 21=' + letters[i] + '; there must be at least one day in item 20(' + letters[i] + ').',
									elementId: 'txtHSA_Q21'};
						}
					}
				}
			},
			'txtHSA_Q22':	function(vet,parent) {
				if (!isIn('ddlHSA_Q20',  ['98', '99'])) {
					if (!isAnswered('txtHSA_Q22')) {
						return {msgType: 'incomplete',
								msgText: 'Please enter zip code or enter N if unknown.',
								elementId: 'txtHSA_Q22'};
					} else if (!is('txtHSA_Q22', 'N') && !isValidUSZip(getAnswer('txtHSA_Q22'))) {
						return {msgType: 'incomplete',
							msgText: 'Zip code must be between <a href="#" class="not-link" x-apple-data-detectors="false">'
								+ '00001 - 99999.' + '</a>',
							elementId: 'txtHSA_Q22'};
					}
				}
			},
			'ddlHSA_Q23':	function(vet,parent) {
				if (!isIn('ddlHSA_Q20',  ['98', '99'])) {
					var ans = this.standardMissingRule('ddlHSA_Q23');
					if (ans) return ans;
					if (is('ddlHSA_Q23', '1')) {
						vals = [];
						ans = this.standardMissingRule('ddlHSA_Q23a');
						if (ans) {
							vals.push(ans);
						}
						var letters = 'bcd';
						for (var i in letters) {
							var vaId = 'txtHSA_Q23' + letters[i];
							ans = this.standardMissingRule(vaId);
							if (ans) {
								vals.push(ans);
							} else {
								ans = this.rangeOrNCheck(vaId, 0, 99,
								'Number of children must be between 0-99 or code N if Veteran refused to answer.');
								if (ans) {
									vals.push(ans);
								}
							}
						}
						return vals;
					}
				}
			},
			'ddlHSA_Q24':	function(vet,parent) {
				var ans = this.standardMissingRule('ddlHSA_Q24');
				if (ans) {
					return ans
				}
				if (!isIn('ddlHSA_Q20',  ['98', '99'])) {
					if (is('ddlHSA_Q24', 1)) {
						var letters = 'abcdefs';
						for (var i in letters) {
							if (is('txtHSA_Q21', letters[i])) {
								return {msgType: 'incomplete',
										msgText: 'item 21 indicates Veteran is housed or incarcerated; housing stability cannot be \"literally homeless\".',
										elementId: 'ddlHSA_Q24'};
							}
						}
					} else {
						var letters = 'rt';
						for (var i in letters) {
							if (is('txtHSA_Q21', letters[i])) {
								return {msgType: 'incomplete',
										msgText: 'Item 21 indicates Veteran is living in a shelter or place not meant for habitation; housing stability must be \"literally homeless\".',
										elementId: 'ddlHSA_Q24'};
							}
						}
					}
				}
			},
			'ddlHSA_Q25':	function(vet,parent) {
				if (is('ddlHSA_Q24', 1)) {
					return this.standardMissingRule('ddlHSA_Q25');
				}
			},
			'ddlHSA_Q26':	function(vet,parent) {
				var ans = this.standardMissingRule('ddlHSA_Q26');
				if (ans) {
					return ans;
				} else if (is('ddlHSA_Q24', 1) && is('ddlHSA_Q26', 0)) {
					return {msgType: 'incomplete',
							msgText: 'Item 24 indicates Veteran is literally homeless; item 26 cannot be 0.  Please remember to count the current episode of homelessness',
							elementId: 'ddlHSA_Q26'};
				}
			},
			'txtHSA_Q29':	function(vet,parent) {
				var ans = this.standardMissingRule('txtHSA_Q29');
				if (ans) return ans;
				return this.rangeOrNCheck('txtHSA_Q29', 0, 30,
						'Days worked for pay must be between 0-30; code N if unknown.');
			},
			'ddlHSA_Q30':	function(vet,parent) {
				var ans = this.standardMissingRule('ddlHSA_Q30');
				if (ans) {
					return ans
				}
				// check whole dollar amounts
				if (isYes('ddlHSA_Q30')) {
					var sum = 0;
					var letters = 'abcdefghijklmnop';
					for (var i in letters) {
						var vaId = 'txtHSA_Q30' + letters[i];
						if (isAnswered(vaId)) {
							if (!isInRange(vaId,0,99999)) {
								return {msgType: 'incomplete',
										msgText: 'Amount recorded must be between 0-99,999. Only whole dollar amounts are allowed.',
										elementId: vaId};
							}
							sum += parseInt(getAnswer(vaId));
						}
					}
					if (sum===0) {
						return {msgType: 'incomplete',
								msgText: 'Item 30 indicates that the Veteran received money in the past 30 days. The sum of Items 30a-p must be greater than $0.',
								elementId: 'ddlHSA_Q30'};
					}
				}
			},
			'ddlHSA_Q31':	function(vet,parent) {
				var ans = this.standardMissingRule('ddlHSA_Q31');
				if (ans) {
					return ans
				}
				if (isYes('ddlHSA_Q31')) {
					var count = 0;
					var letters = 'abcdefghijklm';
					for (var i in letters) {
						if (is('ddlHSA_Q31_1', letters[i])) {
							count++;
						}
					}
					if (count == 0) {
						return {msgType: 'incomplete',
								msgText: 'Item 31 indicates that Veteran is receiving non-cash benefits, at least one item must be checked in 31a-31m',
								elementId: 'ddlHSA_Q31'};
					}
				}
			},
			'ddlHSA_Q32':	function(vet,parent) {
				var ans = this.standardMissingRule('ddlHSA_Q32');
				if (ans) {
					return ans
				}
				if (isYes('ddlHSA_Q32')) {
					var count = 0;
					var letters = 'abcdefghij';
					for (var i in letters) {
						if (is('ddlHSA_Q32_1', letters[i])) {
							count++;
						}
					}
					if (count == 0) {
						return {msgType: 'incomplete',
								msgText: 'Item 32 indicates that Veteran has outstanding debts, at least one item must be checked in 32a-32j',
								elementId: 'ddlHSA_Q32_1'};
					}
					if (is('ddlHSA_Q32_1', 'j')) {
						var ans = this.standardMissingRule('txtHSA_Q32j_other');
						if (ans) {
							return ans
						}
					}
				}
			},
			'ddlHSA_Q36j':	function(vet,parent) {
				if (isYes('ddlHSA_Q36j') && !isAnswered('txtHSA_Q36j_other')) {
					return {msgType: 'incomplete',
							msgText: 'Please specify other medical condition you have.',
							elementId: 'txtHSA_Q36j_other'};
				}
			},
			'txtHSA_Q38':	function(vet,parent) {
				var ans = this.standardMissingRule('txtHSA_Q38');
				if (ans) return ans;
				return this.rangeOrNCheck('txtHSA_Q38', 0, 30, 'Days used alcohol must be between 0 - 30; code N if unknown');
			},
			'txtHSA_Q39':	function(vet,parent) {
				var ans = this.standardMissingRule('txtHSA_Q39');
				if (ans) return ans;
				ans = this.rangeOrNCheck('txtHSA_Q39', 0, 30, 'Days must be between 0 - 30; code N if unknown');
				if (ans) return ans;
				if (parseInt(getAnswer('txtHSA_Q39')) > parseInt(getAnswer('txtHSA_Q38'))) {
					return {msgType: 'incomplete',
							msgText: 'Item 39 cannot be greater than item 38.',
							elementId: 'txtHSA_Q39'};
				}
			},
			'txtHSA_Q40':	function(vet,parent) {
				var ans = this.standardMissingRule('txtHSA_Q40');
				if (ans) return ans;
				return this.rangeOrNCheck('txtHSA_Q40', 0, 30, 'Days used drugs must be between 0 - 30; code N if unknown');
			},
			'txtHSA_Q74_Other':	function(vet,parent) {
				if (is('ddlHSA_Q74', '7') && !isAnswered('txtHSA_Q74_Other')) {
					return {msgType: 'incomplete',
							msgText: 'Please specify other VA affiliation',
							elementId: 'txtHSA_Q74_Other'};
				}
			},
			'txtHSA_Q75_Other':	function(vet,parent) {
				if (is('ddlHSA_Q75', '27') && !isAnswered('txtHSA_Q75_Other')) {
					return {msgType: 'incomplete',
							msgText: 'Please specify other referral type',
							elementId: 'txtHSA_Q75_Other'};
				}
			},
			'txtAssessmentDate':	function(vet,parent) {
				var userAnswer = getAnswer('txtAssessmentDate');
				if (!userAnswer) {
					return {msgType: 'dateBlankError',
							msgText: 'Please specify the date this form was completed.',
							elementId: 'txtAssessmentDate'};
				}
				if (!validDate(userAnswer)) {
					return {msgType: 'dateFormatError',
							msgText: 'Date this form was completed must be in the format mm/dd/yyyy.',
							elementId: 'txtAssessmentDate'};
				}
				if (!validDateRange(userAnswer)) {
					return {msgType: 'dateRangeError',
							msgText: 'Date must be within the past year and not later than today.',
							elementId: 'txtAssessmentDate'};
				}
			}
		}, // end of validateRules()
		getWarning:	function(vaId,vet) {
			if (this.warningRules[vaId]) {
				var warning = this.warningRules[vaId](vet,this);
				if (warning) {
					return warning.msgText;
                }
			}
			return '';
		},
		warningRules:	{
			'txtHSA_Q12a':	function(vet,parent) {
//				if (isAnswered('ddlHSA_Q13')) { not sure why this was here, it the validity checks doc does not mention it
					var dateOfBirth = new Date(vet.dateOfBirth);
					if (isNumber(getAnswer('txtHSA_Q12a')) && isNumber(dateOfBirth.getYear())) {
						var yearEntry = getAnswer('txtHSA_Q12a');
						var yearDOB = dateOfBirth.getFullYear();
						if (yearEntry < yearDOB + 18) {
							return {msgText: 'Warning: Response indicates Veteran was less than 18 at the time of entry to military service. ' +
									         'Please verify that both the DOB and entry date are accurate.'};
						}
					}
//				}
				return null;
			},
			'txtHSA_Q12b':	function(vet,parent) {
//				if (isAnswered('ddlHSA_Q13')) {
					var dateOfBirth = new Date(vet.dateOfBirth);
					if (isNumber(getAnswer('txtHSA_Q12b')) && isNumber(dateOfBirth.getYear())) {
						var yearLeave = getAnswer('txtHSA_Q12b');
						var yearDOB = dateOfBirth.getFullYear();
						if (yearLeave < yearDOB + 18) {
							return {msgText: 'Warning: Response indicates Veteran was less than 18 at the time of separation from military service. ' +
									         'Please verify that both the DOB and separation date are accurate.'};
						}
					}
//				}
				return null;
			},
			'ddlHSA_Q44':	function(vet,parent) {
				// this prevents the warning coming up when you first enter the page
				if (isAnswered('ddlHSA_Q45a')) {
					var count = 0;
					var letters = 'abcdefghijklmno';
					for (var i in letters) {
						if (is('ddlHSA_Q44', letters[i])) {
							count++;
						}
					}
					if (count == 0) {
						return {msgText: 'You have indicated that the Veteran does not have any treatment concerns, if this is correct then please continue'};
					}
				}
				return null;
			},
		},
	}; // end of validationLogic

	var soapLogic = {
		toLocalStorage: function() {
			return this.toSoap(true);
		},
		toVAStorage: function() {
			return this.toSoap(false);
		},
		alertToVAStorage: function(alerts) {
			var arr = [];
			for (var i in alerts) {
				arr.push(this.getAlertObj(alerts[i]));
			}
			return arr;
		},
		recommendToVAStorage: function(recommendations) {
			var arr = [];
			for (var key in recommendations) {
				arr.push(this.getRecommendObj(recommendations[key]));
			}
			return arr;
		},
		toSoap: function(forLocal) {
			var arr = [];
			var caller = this;
			Responses.instance().forEach(function(model) {
				var vaId = model.get('vaId');
				if (vaId) {
					var answer = model.get('response');
					var prev = model.get('prev');
					if (answer==='Choose options' && !prev) {
						//skip unanswered questions
					} else {
						if (caller.toSoapRules[vaId]) {
							var map = {};
							caller.toSoapRules[vaId](answer,map);
							var mapPrev = {};
							caller.toSoapRules[vaId](prev,mapPrev);
							for (var key in map) {
		                        var responseObject = caller.getResponseObj(key, map[key], mapPrev[key], forLocal);
		                        arr.push(responseObject);
							}
						} else {
	                        var responseObject = caller.getResponseObj(vaId, answer, prev, forLocal);
	                        arr.push(responseObject);
						}
					}
				}
			});
			return arr;
		},
		fromLocalStorage: function(arr) {
			var arrayPattern = /(.*Q(?:6c_|31|32|44|10_))(\w*)/i;
			var baseToVaId = {
				ddlHSA_Q6c_:	'ddlHSA_Q6c',
				ddlHSA_Q31:		'ddlHSA_Q31_1',
				ddlHSA_Q32:		'ddlHSA_Q32_1',
				ddlHSA_Q44:		'ddlHSA_Q44',
				ddlDssQ10_:		'ddlDssQ10',
			}
			for (var i in arr) {
//				console.log('fromLocalStorage value=' + JSON.stringify(arr[i]));
				var vaId = arr[i].questionId;
				if (!vaId) continue;
				var answer = arr[i].responseValue;
				var prev = arr[i].prevResponseValue;
				var result = arrayPattern.exec(vaId);
				if (result && result.length > 1 && baseToVaId[result[1]] && result[2].length > 0 && result[2].indexOf('ther') < 0) {
					vaId = baseToVaId[result[1]];

					answer = getAnswer(vaId);
					if (!answer || !$.isArray(answer)) {
						answer = [];
					}
					if (arr[i].responseValue==='1') {
						answer.push(result[2]);
					}

					prev = getPrev(vaId);
					if (!prev || !$.isArray(prev)) {
						prev = [];
					}
					if (arr[i].prevResponseValue==='1') {
						prev.push(result[2]);
					}
				}
				if (vaId==='txtAssessmentDate') {
					answer = formatDateYYtoYYYY(answer);
					prev = formatDateYYtoYYYY(prev);
				}
				Responses.instance().setOrCreate(vaId,answer,prev);
			}
		},
		getResponseObj: function (questionId, answerValue, prevValue, forLocal) {
			var nullToString = function(obj) {
				if (typeof obj == 'undefined' || obj==null) {
					return ''
				} else {
					return obj.toString();
				}
			};
			var getResponseLocal = function (questionId, answerValue, prevValue) {
				return {
					QuestionID:			questionId,
					ResponseValue:		nullToString(answerValue),
					PrevResponseValue:	nullToString(prevValue)
				};
			};
			var getResponseVA = function (questionId, answerValue) {
				return {
					QuestionID:			questionId,
					ResponseValue:		nullToString(answerValue)
				};
			};
			if (forLocal) {
				return getResponseLocal(questionId, answerValue, prevValue);
//				var x = getResponseLocal(questionId, answerValue, prevValue);
//				console.log('questionId=' + questionId + ', answerValue type=' + (typeof answerValue) + ', answerValue=' + answerValue + ', ResponseValue=' + x.ResponseValue);
//				return x;
			} else {
				return getResponseVA(questionId, answerValue);
			}
		},
		getAlertObj: function(text) {
			var getAlertVA = function (text) {
				return {
					AlertMessage:		text
				};
			};
			return getAlertVA(text);
		},
		getRecommendObj: function(recommendation) {
			var getRecommendVA = function (recommendation) {
				return {
					ProgramType:			recommendation.title,
					TotalScore:				recommendation.score,
					Status:					recommendation.accepted ? 'accepted' : 'rejected',
					RejectedReasonTypes:	{
												string:	recommendation.rejectReason ? recommendation.rejectReason : []
											},
					RejectedReason:			recommendation.rejectText ? recommendation.rejectText : ''
				};
			};
			return getRecommendVA(recommendation);
		},
		toSoapRules: {
			defaultRule: function(baseId,answer,map,letters,separator) {
				for (var i in letters) {
					map[baseId + separator + letters[i]] = 0;
				}
				if ($.isArray(answer)) {
					for (var i = 0; i < answer.length; i++) {
						map[baseId + separator + answer[i]] = 1;
					}
				}
			},
			'ddlHSA_Q6c': function(answer,map) {
				this.defaultRule('ddlHSA_Q6c',answer,map,'12345678','_')
			},
			'ddlHSA_Q31_1': function(answer,map) {
				this.defaultRule('ddlHSA_Q31',answer,map,'abcdefghijklm','')
			},
			'ddlHSA_Q32_1': function(answer,map) {
				this.defaultRule('ddlHSA_Q32',answer,map,'abcdefghij','')
			},
			'ddlHSA_Q44': function(answer,map) {
				this.defaultRule('ddlHSA_Q44',answer,map,'abcdefghijklmno','')
			},
			'ddlDssQ10': function(answer,map) {
				this.defaultRule('ddlDssQ10',answer,map,'01234567','_')
			}
		}
	};

	return {
		getDisplayLogic: function() {
			return displayLogic;
		},
		getSoapLogic: function() {
			return soapLogic;
		},
		getValidationLogic: function() {
			return validationLogic;
		}
	}
});
