define(
	['jQuery', 'App'],
	function($, App) {
		//Global should not have any dependencies
		var utilities = {},
			headerHeight,
			footerHeight;

		utilities.flattenLinks = function(object) {
			var links = object.get('link');
			if (typeof links !== 'undefined') {
				size = links.length;
				for (var i = 0; i < size; i++) {
					if (links[i].title) {
						object.set(links[i].title,links[i]);
					} else {
						object.set('self',links[i]);
					}
				}
				object.unset('link');
			}
		};

		utilities.checkConnection = function() {
			if (navigator.connection) {
			    var networkState = navigator.connection.type;

			    var states = {};
			    states[Connection.UNKNOWN]  = 'Unknown connection';
			    states[Connection.ETHERNET] = 'Ethernet connection';
			    states[Connection.WIFI]     = 'WiFi connection';
			    states[Connection.CELL_2G]  = 'Cell 2G connection';
			    states[Connection.CELL_3G]  = 'Cell 3G connection';
			    states[Connection.CELL_4G]  = 'Cell 4G connection';
			    states[Connection.CELL]     = 'Cell generic connection';
			    states[Connection.NONE]     = 'No network connection';

			    console.log('Connection type: ' + states[networkState]);
			}
		}

		utilities.isChrome = function() {
			var test = navigator.userAgent;
			return test && test.indexOf('Chrome') >= 0;
		};

		utilities.globalInit = function(app) {
			console.log('keyboard global init');

			// this sets up an event handler for the resize event
			// but keyboard up/down events are enough
			//$(window).resize(utilities.resizeWithWindow);

			$('body').on('pageshow',function(event, ui) {
				var prevPage = $(ui.prevPage);
				if (prevPage.hasClass('ui-dialog')) {
					console.log('KEYBOARD pageshow removing prevPage');
					prevPage.remove();
				}
			});

			var years = utilities.readFromStorage('homes_yearsOfHistory');
			if (isInteger(years) && years > 0) {
				App.getState().yearsOfHistory = years;
			} else {
				App.getState().yearsOfHistory = 10;
			}
		};

		utilities.slidePrimary = function(selector) {
			var $moveContent = $(selector || '#content-primary');
			if ($moveContent.hasClass('closed')) {
				$moveContent.removeAttr('style').toggleClass('closed');
			} else {
				$moveContent.animate({
					left: '0'
				}, 300, function() {
					$moveContent.addClass('closed');
				});
			}
		};

		utilities.getRestServer = function() {
			if (utilities.isChrome()) {
//				return "..";
				return "https://hastafftest.agilexhealth.com";
			} else {
				return "https://hastafftest.agilexhealth.com";
			}
//            return "http://localhost:8080";
		}

		utilities.getRedirect = function() {
			if (utilities.isChrome()) {
//				return "http://localhost:8080/homeless";
//				return utilities.getLocalReferencePathWithHash();
				return "";
			} else {
				return "mHomes://?where=not_used";  // + utilities.getLocalReferencePathWithHash();
			}
		}

//		utilities.resizePanes = function(customHeight) {
//			console.log('KEYBOARD utilities.resizePanes customHeight=' + customHeight);
//			if (typeof headerHeight === 'undefined' && $('#main .ui-header').height() !== null) {
//				headerHeight = customHeight || 42;
//				footerHeight = 86;
//			}
//			//header and footer have lost relevence in soc
//			$('#main .scrollable').height(App.getState().contentHeight - headerHeight - footerHeight);
//			utilities.eventAggregator.trigger('footer');
//		};

		utilities.deleteCookie = function(cookieName) {
			document.cookie = encodeURIComponent(cookieName) + "=deleted; expires=" + new Date(0).toUTCString();
		};

		utilities.findMessage = function(errMsg) {
			var lines = errMsg.split('\n');

			for (var i = 0; i < lines.length; i++) {
				if (lines[i].indexOf('Message:') >= 0) {
					return lines[i];
				}
			}
			return errMsg;
		};

		utilities.hasToken = function() {
			var storedToken = window.sessionStorage['token'];
			return storedToken && storedToken.length > 0;
		}

		utilities.parseTokenAndRegisterOauthHandler = function() {
			$(document).ajaxError(function(e, jqxhr, settings, exception) {
				console.log('ajax error status=' + jqxhr.status);
				if (jqxhr.status == 200 || jqxhr.status == 400 || jqxhr.status == 404) {
				} else if (jqxhr.status == 401) {
					if (jqxhr.responseText.indexOf('invalid_token') > 0) {
						utilities.cleanUpSession();
						$.ajax(settings);
					}
				} else if (jqxhr.status == 0 || jqxhr.status == 522 || jqxhr.status == 403) {
					console.log('ONLINE ajaxError status=' + jqxhr.status);
					App.getState().setHomesOnline(false);
				} else if (jqxhr.status == 500) {
				} else {
					App.getUtilities().alert('unknown error at url: ' + settings.url + '\nstatus code: ' + jqxhr.status + ' \nerror:\n' + utilities.findMessage(jqxhr.responseText));
				}
			});

			$(document).ajaxSuccess(function(event, xhr, settings) {
				console.log('ajax success status=' + xhr.status);
			});
		};
		utilities.parseTokenAndRegisterOauthHandler();

		utilities.create = function(page) {
			try {
				page.page("destroy");
			} catch (err) {
				// if page has not been initialized yet, destroy is not necessary
			}
			page.page();
		};

		utilities.renderDetail = function($container, template, data, Model) {
			var $sectionContainer = $('#main').find(":jqmData(role='content')");
			if (typeof data !== 'undefined' && data != null) {
				if (data instanceof Model) {
					model = data;
				} else {
					model = new Model(data);
				}
				$container.empty().append(template(model.toJSON()));
				$sectionContainer.empty();
				$sectionContainer.append($container);
			}
		};

		utilities.clone = function(obj) {
			if (!obj) {
				return null;
			} else if (typeof obj ==='number' || typeof obj === 'string') {
				var x = obj;
				return x;
			} else if ($.isArray(obj)) {
				var arr = [];
				for (var i in obj) {
					arr.push(utilities.clone(obj[i]));
				}
				return arr;
			} else {
				var newObj = {}
				for (name in obj) {
					newObj[name] = utilities.clone(obj[name]);
				}
				return newObj;
			}
		};
		utilities.writeToStorage = function(key, value, param) {
			var formatter = JSON;
			if (param && param.formatter) {
				formatter = param.formatter;
			}
			console.log('writeToStorage key=' + key + ', value=' + formatter.stringify(value));
			if (!value) {
				value = null;
			}

			try {
				window.localStorage[key] = formatter.stringify(value);
			} catch (e) {
				if (e == QUOTA_EXCEEDED_ERR) {
					var message = 'Local-Storage Quota exceeded!';
					if (window.console && window.console.log) {
						console.log(message);
					} else {
						alert(message);
					}
				}
			}
		};

		utilities.readFromStorage = function(key, param) {
			var formatter = JSON;
			if (param && param.formatter) {
				formatter = param.formatter;
			}
			var value = null;
			if (typeof window.localStorage != 'undefined' && window.localStorage[key]) {
				value = formatter.parse(window.localStorage[key] || null);
			}
			return value;
		};

		utilities.eulaOk = function(key,modifiedDate) {
			var value = utilities.readFromStorage(key);
			return value !== null && new Date(modifiedDate).getTime() <= value.dateAccepted;
		};

		utilities.sudOk = function(sudDate) {
			if (sudDate) {
				var sudDate = new Date(sudDate);
				if (sudDate == 'Invalid Date') {
					return false;
				} else {
					return true;
//					var today = new Date();
//					return sudDate < today;
				}
			} else {
				return false;
			}
		};

		utilities.reportError = function(msg,errorData) {
			var text = [msg];
			if (errorData && errorData.content) {
				if (errorData.content.indexOf('do not have permission to view') >= 0) {
					text = text.concat('Your session has become invalid, one of these may have occurred');
					text = text.concat('1) Your session timed out due to inactivity');
					text = text.concat('2) Someone else has signed on with your credentials');
					text = text.concat('3) HOMES server is not responding');
				} else {
					var fault = '';
					try {
						var parser = new DOMParser();
						var doc = parser.parseFromString(errorData.content,"text/xml");
						if (doc) {
							var snippet = doc.getElementsByTagName('faultstring');
							if (snippet && snippet.length > 0) {
								snippet = snippet[0].childNodes;
								if (snippet && snippet.length > 0) {
									fault = snippet[0].nodeValue;
								}
							}
						}
					} catch(e) {
					}
					if (fault && typeof fault === 'string') {
						text = text.concat(fault.split('--->'));
					}
				}
			}
			utilities.eventAggregator.trigger('message', 'homesError', text);
		}

		utilities.sudOkCookie = function(key) {
			var value = utilities.readFromStorage(key);
			return value !== null;
		};

		utilities.timeStamp = 0;
		utilities.scrollToId = '';
		utilities.showFooter = true;

		utilities.alterFooter = function(mills) {
			setTimeout(function() {
				if (utilities.showFooter) {
					if ($('#footer').hasClass('ui-fixed-hidden')) {
						console.log('KEYBOARD showing footer');
						$('#footer').removeClass('ui-fixed-hidden');
					};
				} else {
					if (!$('#footer').hasClass('ui-fixed-hidden')) {
						console.log('KEYBOARD hide footer');
						$('#footer').addClass('ui-fixed-hidden');
					};
				}
			},mills);
		};

		utilities.setupKeyEvents = function() {
			$("[type='text']").on('tap', function(e) {
				console.log('KEYBOARD tap type text');
				var $target = $(e.currentTarget);
				var id = $target.attr('id');
				utilities.scrollToId = id;
				utilities.scrollDelayed(null);
			});
		};

		utilities.resizeUseSaved = function() {
			if (App.getState().isLoaderShown) {
//				console.log('loader should be up');
				$.mobile.loading('show');
//			} else {
//				$.mobile.loading('hide');
			}

			// hide or show footer
			if (App.getState().isKeyboardShown) {
				$('#footer').attr('style','display:none');
				$('#footer').height(0);
				utilities.showFooter = false;
			} else {
				$('#footer').removeAttr('style');
				utilities.showFooter = true;
			}
			utilities.alterFooter(100);
			utilities.alterFooter(700);

			setTimeout(function() {
				var height = App.getState().contentHeight - $('#header').height();

				//removed 5/19/14
//				if (App.getState().isKeyboardShown) {
//				} else {
//					height = height - $('#footer').height();
//				}

				console.log('KEYBOARD resizeUseSaved height=' + height
					+ ', App contentHeight=' + App.getState().contentHeight
					+ ', App header=' + $('#header').height()
					+ ', App footer=' + $('#footer').height()
					+ ', id=' + utilities.scrollToId
					+ ', App isKeyboardShown=' + App.getState().isKeyboardShown
				);
				if (isNaN(height)) {
					console.log('KEYBOARD made no change');
				} else {
					if (height > 0) {
						$('#main-content').height(height);
					}
					utilities.scrollDelayed(height);
				}
			},100)
		};

		$("*").on('pagehide', function(event) {
			if (event.timeStamp != utilities.timeStamp) {
				console.log('KEYBOARD page hide event called');
				utilities.timeStamp = event.timeStamp;
				utilities.resizeUseSaved();
			}
		});

		utilities.putToTop = function() {
			setTimeout(function() {
				$('.scrollable').scrollTo(10,0);
				console.log('KEYBOARD putToTop called after 200');
				utilities.scrollToId = null;
			}, 200);
			setTimeout(function() {
				$('.scrollable').scrollTo(0,0);
				console.log('KEYBOARD putToTop called after 400');
				utilities.scrollToId = null;
			}, 400);
		}

		utilities.setNextFocus = function(id) {
			console.log('KEYBOARD setNextFocus id' + id);
			utilities.scrollToId = id;
		}

		utilities.scrollDelayed = function(height) {
			setTimeout(function() {
				utilities.scroll(utilities.scrollToId);
				if (height > 0) {
					$('#main-content').height(height);
				}
			}, 1000);
		};

		utilities.scroll = function(moveToId) {
			if (moveToId) {
				utilities.scrollToId = moveToId;
				if ($('#' + moveToId + '[type="text"]').length>0) {
					console.log('KEYBOARD scroll to ' + moveToId + '-fieldcontain');
					$('#' + moveToId).focus();
					$('#main .scrollable').scrollTo(
						$('#' + moveToId + '-fieldcontain'),
						App.getState().scrollOptions(function() {
							elem = $('#' + moveToId);
							var curVal = elem.val();
							elem.val(curVal + ' ');
							elem.val(curVal);
							elem.focus();
						})
					);
				} else {
					console.log('KEYBOARD focus on ' + moveToId);
					$('#' + moveToId).focus();
				}
			}
		};

		utilities.eventAggregator = _.extend({}, Backbone.Events);
		utilities.eventAggregator.invoke = function(name) {
			utilities.eventAggregator.trigger(name,name);
		}
		utilities.eventAggregator.on("logout", function() {
			utilities.cleanUpSession();
		});

//		alert('finished loading global')
		return {
			getUtilities: function() {
				return utilities;
			}
		};
	}
);
