define(
	['Global', 'App'],
	function(Global, App) {
		var currentVet;
		var HomesVet = Backbone.Model.extend({
			defaults: {
				// The first letter of each attribute has to be lowercase in order to be used in CoreData
				// these are the required fields, when we pull from HOMES sometimes these are not there
				firstName:					'',
				middleInitial:				'',
				lastName:					'',
				sSN:						'',
				dateOfBirth:				''
			},
			initialize: function() {
				model = this;
				model.changeKeysToLowerCase();
			},
			isLocal: function() {
				var homesID = model.get('veteranID');
				if (homesID) {
					return homesID < 0;
				} else {
					return true;
				}
			},
			changeKeysToLowerCase: function() {
				var capitalizedKeys = [];

				for (var keyName in model.attributes) {
					var firstLetter = keyName.charAt(0);
					if (model.attributes[keyName]) {
						model.set(lowercaseFirstLetter(keyName), model.attributes[keyName]);
					} else {
						model.set(lowercaseFirstLetter(keyName), '');
					}
					if (firstLetter === firstLetter.toUpperCase() && firstLetter !== firstLetter.toLowerCase()) {
						capitalizedKeys.push(keyName);
					}
				}
				for (var i=0; i<capitalizedKeys.length; i++) {
					delete model.attributes[capitalizedKeys[i]];
				}
			},
			updateEpisode: function(data) {
				console.log('updateEpisode data=' + JSON.stringify(data));
				var isOpen = false;
				if (data.length > 0) {
					isOpen = true;
					this.set('homesEpisodeVISN', data);
				}
				this.set('isHomesEpisodeOpen', isOpen);
			}
		});

		HomesVet.getCurrentVet = function() {
			if (currentVet) {
				var veteranLocalID = currentVet.get('veteranLocalID');
				if ( (typeof veteranLocalID !== 'undefined') && isNaN(veteranLocalID) ) {
					console.log('ERROR! getCurrentVet has bad veteranLocalID=' + currentVet.get('veteranLocalID'));
				}
			}
			return currentVet;
		};

		HomesVet.setCurrentVet = function(vet) {
			if (vet) {
				var veteranLocalID = vet.get('veteranLocalID');
				if ( (typeof veteranLocalID !== 'undefined') && isNaN(veteranLocalID) ) {
					console.log('ERROR! setCurrentVet has bad veteranLocalID=' + veteranLocalID);
				}
			}
			currentVet = vet;
		}

		HomesVet.getHomesID = function() {
			if (currentVet) {
				return currentVet.get('veteranID');
			} else {
				return null;
			}
		}

		HomesVet.isVetLocal = function() {
			var homesID = currentVet.get('veteranID');
			if (homesID) {
				return homesID < 0;
			} else {
				return true;
			}
		}

		return HomesVet;
	}
);
