define(
	['Global', 'App', 'MetricLib'],
	function (Global, App, MetricLib) {
		'use strict';
		var model;
		var nop = function() {};
		var Metric = Backbone.Model.extend({
			defaults: {
				"startDate":	'',
				"event":		'',
				"appName":		App.getState().appName,
				"userId":		'',
				"objectType":	"ServiceTiming"
			},
			initialize: function () {
				model = this;
				model.url = App.getState().metricEndpoint;
			},
			toModel: function(storedMetrics) {
				console.log('found ' + storedMetrics.length + ' stored metrics');
				var metrics = {};
				if ($.isArray(storedMetrics)) {
					for (var i=0; i<storedMetrics.length; i++) {
						var metric = new Metric($.parseJSON(storedMetrics[i].metricJSON));
						var metricID = storedMetrics[i].metricID;
						console.log('toModel metric ' + metricID);
						metrics[metricID] = metric;
					}
				}
				return metrics;
			},
			pushAndDeleteMetrics: function(metrics) { // metrics that were in local storage
				var ok = true;
				for (var metricID in metrics) {
					if (!ok) break;
					MetricLib.saveMetric(metrics[metricID],
						function(model, response, options) {
							MetricLib.deleteFromCordova(metricID);
						},
						function(xhr, response, error) {
							ok = false;
						}
					);
				}
			},
			pushLocalMetrics: function() {
				var storedMetrics = [];
				var callback = function(obj) {
					console.log('fetchFromCordova obj.successData=' + obj.successData);
					var metrics = model.toModel(obj.successData);
					model.pushAndDeleteMetrics(metrics);
				};
				MetricLib.fetchFromCordova(callback,nop);
			},
			push: function(user) {
				console.log ('METRIC pushing model url=' + model.url);
				model.set('userId',user.get('StaffID'));
				var arr = [];
				arr.push({
					"key":		"VISN",
					"value":	user ? user.get('VISN') : ''
				});
				arr.push({
					"key":		"APP_VERSION",
					"value":	App.getState().currentVersion
				});
				model.set('tag',arr);
				model.set('appName',App.getState().appName);
				model.set('startDate',formatDateAndTime(new Date()));
				var loginObj = App.getState().soapDefaults('homesUserLogin');
				model.set('event',loginObj.url + loginObj.method);

				MetricLib.saveMetric(model,model.pushLocalMetrics,
					function(xhr, response, error) {
						MetricLib.saveToCordova(model, nop, nop);
					}
				);
			}
		});
		return Metric;
	}
);
