define(
	['Global', 'App', 'Templates', 'FormRules', 'HomesVet', 'Responses'],
	function (Global, App, Templates, FormRules, HomesVet, Responses) {
		'use strict';
		var QuestionsData = Backbone.Model.extend({
			defaults: {
			},
			initialize: function (options) {
				if (options.form == 'HOMES') {
					this.set('sectionHash', Templates.sectionHash_HOMES);
					this.set('questionHash',Templates.questionHash_HOMES);
					this.set('answerHash',  Templates.answerHash_HOMES);
				} else if (options.form == 'HDS') {
					this.set('sectionHash', Templates.sectionHash_HDS);
					this.set('questionHash',Templates.questionHash_HDS);
					this.set('answerHash',  Templates.answerHash_HDS);
				}
				this.set('section',  {});
				this.set('questions', {});
			},
			getSectionIds:	function(questions) {
				var list = typeof questions == 'undefined' ? this.get('questions') : questions;
				var ids = ['txtAssessmentDate'];
				for (var i in list) {
					var question = list[i];
					if (FormRules.getDisplayLogic().isQuestionDisplayed(question)) {
						ids.push(question.vaId);
					}
				}
				return ids;
			},
			chooseSection: function() {
				var sectionHash = this.get('sectionHash');
				var questionHash = this.get('questionHash');
				var section,questions;

				sectionHash.reset();
				var errs = null;
				do {
					section = sectionHash.currentSection();
					questions = questionHash[section.id];
					var idsToValidate = this.getSectionIds(questions);
					errs = FormRules.getValidationLogic().validate(idsToValidate, HomesVet.getCurrentVet().toJSON());
					if (errs && errs.length > 0) {
						console.log('chooseSection errs=' + JSON.stringify(errs));
					} else {
						console.log('chooseSection section ' + section.id + ' was ok');
						section.done = true;
						section = sectionHash.next();
					}
				} while (!(errs && errs.length > 0) && sectionHash.hasNext())
				if (section.done) {
					App.getState().completeForm = true;
				}
				this.set('section', sectionHash.currentSection());
				this.set('questions',questionHash[section.id]);
			},
			calculateSectionCount: function() {
				var start = this.get('section').index;
				var stop = this.get('sectionHash').maxIndex;
				return 'Section ' + start + '/' + stop;
			},
			prev: function(e) {
				var ids = null;
				var section,questions;
				var sectionHash = this.get('sectionHash');
				var questionHash = this.get('questionHash');
				do {
					section = sectionHash.prev();
					questions = questionHash[section.id];
					ids = this.getSectionIds(questions);
				} while (sectionHash.hasPrev() && ids.length < 2);  // always have the assessment date
				this.set('section',section);
				this.set('questions',questions);
			},
			next: function(e) {
				var section,questions;
				var ids = null;
				var sectionHash = this.get('sectionHash');
				var questionHash = this.get('questionHash');

				this.get('section').done = true;
				do {
					section = sectionHash.next();
					questions = questionHash[section.id];
					ids = this.getSectionIds(questions);
				} while (sectionHash.hasNext() && ids.length < 2);  // always have the assessment date
				this.set('section',section);
				this.set('questions',questions);
			},
			isReassess: function() {
				var questions = this.get('questions');
				for (var i in questions) {
					var response = Responses.instance().get(questions[i].vaId);
					if (response) {
						if (response.get('prev')) {
							return true;
						}
					}
				}
				return false;
			},
			removeUnusedAnswers: function(keepList) {
				var questionHash = this.get('questionHash');
				// remove unanswered questions
				for (var id in questionHash) {
					var sectionQuestions = questionHash[id];
					for (var i in sectionQuestions) {
						var question = sectionQuestions[i];
						if (!FormRules.getDisplayLogic().isQuestionDisplayed(question)) {
							Responses.instance().removeAnswer(question.vaId);
						}
					}
				}

				// remove responses that don't exist in the excel spreadsheet, except ones on the keep list
				for (var index in Responses.instance().models) {
					var vaId = Responses.instance().models[index].id;
					var found = false;
					for (var i in keepList) {
						if (vaId == keepList[i]) {
							found = true;
							break;
						}
					}
					if (found) {
						continue;
					}
					found = false;
					for (var id in questionHash) {
						var sectionQuestions = questionHash[id];
						for (var i in sectionQuestions) {
							var question = sectionQuestions[i];
							if (question.vaId==vaId) {
								found = true;
								break;
							}
						}
						if (found) {
							break;
						}
					}
					if (!found) {
						console.log('removing response with vaId=' + vaId + ' since it is not in the excel');
						Responses.instance().removeAnswer(vaId);
					}
				}
			},
			buildPossibleAnswers: function() {
//				var possibleAnswers = [];
				var questionHash = this.get('questionHash');
				var answerHash = this.get('answerHash');
				return FormRules.getDisplayLogic().buildPossibleAnswers(questionHash,answerHash);
//				for (var section in questionHash) {
//					var questions = questionHash[section];
//					for (var i in questions) {
//						var question = questions[i];
//						possibleAnswers[question.vaId] = FormRules.getDisplayLogic().getPossibleAnswers(answerHash,question.answerBase)
//					}
//				}
//				return possibleAnswers;
			},
		});
		return QuestionsData;
	}
);
