define(
	[
	'Global', 'App', 'CordovaLib', 'EulaView', 'SudView',
	'DateView', 'SideBar', 'HomesVet',
	'MainView', 'HomesSoap',
	'HomesUser', 'HomesListView', 'HistoryView',
	'HomesLoginView',
	'MessageView', 'ErrorView',
	'BlankView', 'HomesVeteranSummaryView', 'HomesSavedDataView',
	'HomesAssessmentView', 'HDSView',
	'RecommendView',
	'FormView', 'ConfigView', 'AboutView',
	'HomesVet'
	],
	function(Global, App, CordovaLib, EulaView, SudView,
			DateView, SideBar, HomesVet,
			MainView, HomesSoap,
			HomesUser, HomesListView, HistoryView,
			HomesLoginView,
			MessageView, ErrorView,
			BlankView, HomesVeteranSummaryView, HomesSavedData,
			HomesAssessmentView, HDSView,
			RecommendView,
			FormView, ConfigView, AboutView,
			HomesVet) {
		var initialLoad = true;
		var nop = function() {};
		var router;
		var AppRouter = Backbone.Router.extend({
			initialize: function() {
				router = this;
				router.parmFromCordova(function(obj) {
					App.setParms(obj.successData);
					$.mobile.autoInitializePage = false;
					router.title = App.defineResourceTitles().title;
					router.location = window.location.hash;
					router.metaData = App.getMetaData();
					var eulaHTML = new EulaView().template.call();
					router.eulaModifiedDate = $('#modified-date', eulaHTML).text();
					Global.getUtilities().globalInit({});
					router.ping();
				},
				function(obj) {
//					alert('could not get initial parameters from IOS');
				});
			},
			routes: {
			},
			openPage: function(page,model) {
				var id = page.$el.attr('id')
				console.log('openPage ' + id);
				$.mobile.loading('show');
				if (initialLoad) {
					var mainView = new MainView();
					$('body').append(mainView.render(router.metaData['main']));

					var app = {};
					Global.getUtilities().globalInit(app);
					initialLoad = false;
				} else {
					MainView.refresh(id);
				}
				router.alterButtons();
				router.routePage(page, model);
			},
			routePage: function(page, model) {
				var role = $(page.el).eq(0).attr('data-role');
				console.log('routePage id=' + page.id + ', role=' + role);
				if (role === 'dialog' || role === 'popup') {
					$.mobile.loading('show');
					page.render(model);
					$.mobile.changePage("#" + page.id, {transition: 'pop', allowSamePageTransition: false});
				} else {
					page.render(model);
					$.mobile.changePage("#main", {allowSamePageTransition: true, showLoadMsg: true});
				}
			},
			ping: function() {
				console.log('pinging homes');
				HomesSoap.ping(router.pingCallback);
			},
			pingCallback: function(obj) {
				console.log('pingCallback success=' + obj.success +
						', successData=' + JSON.stringify(obj.successData) +
						', errorData=' + JSON.stringify(obj.errorData));
				if (obj.success || !HomesUser.instance().isLoggedIn()) {
					HomesUser.instance().logout();
					HomesVet.setCurrentVet(null);
					MainView.refresh();
					Global.getUtilities().eventAggregator.invoke('homesLogin');
				} else {
					MainView.refresh();
				}
			},
			alterButtons: function() {
				if (HomesUser.instance().isLoggedIn()) {
					$('#main-content').addClass('double');
					var vet = HomesVet.getCurrentVet();
					if (vet) {
						$('#contact-btn').removeClass('ui-disabled');
						$('#history-btn').removeClass('ui-disabled');
						$('#assessment-btn').removeClass('ui-disabled');
						$('#decision-btn').removeClass('ui-disabled');
						$('#referral-btn').removeClass('ui-disabled');
					} else {
						$('#contact-btn').addClass('ui-disabled');
						$('#history-btn').addClass('ui-disabled');
						$('#assessment-btn').addClass('ui-disabled');
						$('#decision-btn').addClass('ui-disabled');
						$('#referral-btn').addClass('ui-disabled');
					}
				} else {
					$('#main-content').removeClass('double');
				}
			},
			parmFromCordova: function(success,fail) {
				CordovaLib.fetchParm(function(obj) {
					if (obj.success) {
						console.log('METRIC parmFromCordova succeeded');
						success(obj);
					} else {
						console.log('METRIC parmFromCordova failed');
						fail(obj);
					}
				});
			},
		}.initialize());

		var eventAggregator = Global.getUtilities().eventAggregator;

		// add routes to eventAggregator
		_.map(router.routes, function(method, key) {
			eventAggregator.on(key, function() {
				router.navigate('#'+key, {trigger: true});
			});
		});

		var events = [
				{ name:	'homesLogin',			view:	HomesLoginView,				button: null},
				{ name:	'blank',				view:	BlankView,					button: null},
				{ name:	'homesList',			view:	HomesListView,				button: null},
				{ name:	'history',				view:	HistoryView,				button: '#history-btn'},
				{ name:	'homesSavedData',		view:	HomesSavedData,				button: null},
				{ name:	'homesAssessment',		view:	HomesAssessmentView,		button: '#assessment-btn'},
				{ name:	'hds',					view:	HDSView,					button: '#decision-btn'},
				{ name:	'config',				view:	ConfigView,					button: '#config-btn'},
				{ name:	'about',				view:	AboutView,					button: '#about-btn'},
				{ name:	'recommend',			view:	RecommendView,				button: null},
				{ name:	'homesVeteranSummary',	view:	HomesVeteranSummaryView,	button: '#contact-btn'},
				{ name:	'eula',					view:	EulaView,					button: null},
				{ name:	'sud',					view:	SudView,					button: null},
				{ name:	'form',					view:	FormView,					button: null}
		];

		for (var i in events) {
			var event = events[i];
			eventAggregator.on(event.name, function(name) {
				var event;
				for (var i in events) {
					if (events[i].name === name) {
						event = events[i];
						break;
					}
				}
				if (event) {
					console.log('eventAggregator name=' + event.name);
					var view = new event.view(router.metaData[event.name]);
					App.getState().currentView = event.name;
					router.openPage(view, {});
					$.mobile.loading('show');
					SideBar.changeBackground(event.button);
					// this will cause the resizeUseSaved method to kill the spinner
					// usually when the page actually shows, the spinner is removed
					App.getState().isLoaderShown = false;
					setTimeout(function() {
						$.mobile.loading('hide');
					},900);
				} else {
					console.log('event ' + name + ' was not in the events list');
				}
			});
		}

		eventAggregator.on('connect', function(data) {
			router.ping();
		});

		eventAggregator.on('message', function(metaId,errMsg,caller) {
			var view = new MessageView();
			var message = App.getMessages()[metaId];
			if (message) {
				var data = {};
				for (var attrname in message) {
					data[attrname] = Global.getUtilities().clone(message[attrname]);
				}
				if (!data.text) {
					data.text = [];
				}
				if (errMsg) {
					data.text.push('linebreak');
					if (Array.isArray(errMsg)) {
						for (var i = 0; i < errMsg.length; i++) {
							data.text.push(errMsg[i]);
						}
					} else {
						data.text.push(errMsg);
					}
				}
				if (typeof caller !== 'undefined' && caller) {
					data.caller = caller;
				}
				view.render(data).showModal({
					fadeInDuration:					500,
					backgroundClickClosesModal:		false,
					bodyOverflowHidden:				false
				});
				$('#messageBox').keypress(function(event) {
					console.log('key pressed K' + event.keyCode);
					});
			} else {
				console.log('missing metaId ' + metaId);
			}
		});

		$(document).ajaxError(function (e, xhr, options) {
			eventAggregator.trigger('app:ajaxError', xhr.status);
		});

		Backbone.history.start();
	}
);
