define(
	['Global', 'App', 'HomesUser', 'HomesVet', 'FormRules', 'Responses', 'Forms', 'HomesAssessment'],
	function(Global, App, HomesUser, HomesVet, FormRules, Responses, Forms, HomesAssessment) {
		var sync, currentMethod, method, library, callNumber=0;
		var cordovaLib = {
			init:	function() {
				sync = new (App.getUtilities().sync)(20);
				sync.success = false;
//				timer = new (App.getUtilities().timer)();
				currentMethod = 'init';
				library = this;
				return this;
			},
			prep: function() {
				sync.startWaiting(false);
				sync.success = false;
			},
			storeVet: function(callback) {
				var name = 'storeVet' + callNumber++;
				library.waitForLastCall(name, function () {
					library.prep();
					library.storeVetCall();
					library.wait(name, callback);
				});
			},
			storeVetCall: function() {
				var vet = HomesVet.getCurrentVet();
                if (!vet.get('veteranID')) {
                    vet.set('veteranID','-1');
                }

                if (!vet.get('veteranLocalID')) {
                    vet.set('veteranLocalID', '-1');
                }
                console.log('STORING VETERAN LOCAL ID ' + vet.get('veteranLocalID'));

				var data = {
					veteranJSON:      vet,
					assessmentJSON:   Forms.instance(),
					veteranHomesId:   vet.get('veteranID'),
                    veteranSSN:       vet.get('sSN'),
                    veteranLocalID:   vet.get('veteranLocalID'),
					userId:           HomesUser.instance().get('username')
				};
//                console.log('store vet call saving: ' + JSON.stringify(data));

//				timer.markTime();
				cordova.exec(
                    function(response) {
                        console.log('STORED VETERAN LOCAL ID ' + response);

                        if (isNaN(response)) {
                            //shutting the alert for now until I find a solution
                            alert('ERROR! CoreData responses are being mixed!');
                        }
                        library.standardSuccess(response);
                    },
                    function(errorValue) {
                        Global.getUtilities().eventAggregator.trigger('message', 'errorAddingVetOffline', errorValue);
                        library.standardError(errorValue);
                    },
                    "CoreDataStorage", "saveOfflineJSON", [data]
				);
//				timer.writeTimeSinceMark('storeVetCall exec time ');
			},
            fetchOfflineJSON: function(queryParameters, callback) {
				library.waitForLastCall('fetchOfflineJSON', function () {
					library.prep();
					library.fetchOfflineJSONCall(queryParameters);
					library.wait('fetchOfflineJSON', callback);
				});
			},
			fetchOfflineJSONCall: function (queryParameters) {
                queryParameters.userId = HomesUser.instance().get('username');

				cordova.exec(library.standardSuccess, library.standardError,
					"CoreDataStorage", "fetchOfflineJSON", [queryParameters]
				);
			},
            fetchAllOfflineJSON: function(callback) {
				library.waitForLastCall('fetchAllOfflineJSON', function () {
					library.prep();
					library.fetchAllOfflineJSONCall();
					library.wait('fetchAllOfflineJSON',callback);
				});
			},
			fetchAllOfflineJSONCall: function () {
                var offlineJSONQueries = {
                        userId:    HomesUser.instance().get('username')
                    };

				cordova.exec(library.standardSuccess, library.standardError,
					"CoreDataStorage", "fetchAllOfflineJSON", [offlineJSONQueries]
				);
			},
            deleteOfflineJSON: function(queryParameters, callback) {
                library.waitForLastCall('deleteOfflineJSON', function () {
					library.prep();
					library.deleteOfflineJSONCall(queryParameters);
					library.wait('deleteOfflineJSON', callback);
				});
            },
            deleteOfflineJSONCall: function(queryParameters) {
                queryParameters.userId = HomesUser.instance().get('username');

				cordova.exec(library.standardSuccess, library.standardError,
					"CoreDataStorage", "deleteOfflineJSON", [queryParameters]);
            },
			storeMetric: function(metric,callback) {
				library.waitForLastCall('storeMetric', function () {
					library.prep();
					library.storeMetricCall(metric);
					library.wait('storeMetric', callback);
				});
			},
			storeMetricCall: function(metric) {
                console.log('storing metric date=' + metric.get('startDate'));

				var data = {
					metricJSON:      metric
				};
				cordova.exec(
                    function(response) {
                        console.log('METRIC stored localId ' + response);
                        library.standardSuccess(response);
                    },
                    function(errorValue) {
                        library.standardError(errorValue);
                    },
                    "CoreDataStorage", "saveMetric", [data]);
			},
			fetchMetrics: function(callback) {
				library.waitForLastCall('fetchMetrics', function () {
					library.prep();
					library.fetchMetricsCall();
					library.wait('fetchMetrics', callback);
				});
			},
			fetchMetricsCall: function() {
                console.log('fetching metrics');
				cordova.exec(
					library.standardSuccess,library.standardError,
					"CoreDataStorage", "fetchMetrics", []);
			},
			fetchParm: function(callback) {
				library.waitForLastCall('fetchParm', function () {
					library.prep();
					library.fetchParmCall();
					library.wait('fetchParm', callback);
				});
			},
			fetchParmCall: function() {
                console.log('fetching parms');
				cordova.exec(
					library.standardSuccess,library.standardError,
					"CoreDataStorage", "fetchParm", []);
			},
			deleteMetric: function(metricID,callback) {
				library.waitForLastCall('deleteMetric', function () {
					library.prep();
					library.deleteMetricCall(metricID);
					library.wait('deleteMetric', callback);
				});
			},
			deleteMetricCall: function(metricID) {
                console.log('deleting metric id=' + metricID);
				cordova.exec(
					library.standardSuccess,library.standardError,
					"CoreDataStorage", "deleteMetric", [{id : metricID}]);
			},
			fetchAssessment: function(queryParameters,callback) {
				var name = 'fetchAssessment' + callNumber++;
				library.waitForLastCall(name, function () {
					library.prep();
					library.fetchAssessmentCall(queryParameters);
					library.wait(name, callback);
				});
			},
			fetchAssessmentCall: function(queryParameters) {
                queryParameters.isDecisionSupport = 0;
                queryParameters.userId = HomesUser.instance().get('username');

				console.log('fetchAssessment queryObject=' + JSON.stringify(queryParameters));
				cordova.exec(
					library.standardSuccess, library.standardError,
					"CoreDataStorage", "fetchOneAssessmentInstance", [queryParameters]);
			},
			saveAssessmentLocally: function() {
				library.waitForLastCall('saveAssessmentLocally', function () {
					library.prep();
					var vet = HomesVet.getCurrentVet();

					if (vet) {
                        if (!vet.get('veteranID')) {
                            vet.set('veteranID','-1');
                        }

                        if (!vet.get('veteranLocalID')) {
                            vet.set('veteranLocalID', '-1');
                        }
						library.saveAssessmentLocallyCall(vet);
						library.wait('saveAssessmentLocally', null);
					} else {
						Global.getUtilities().eventAggregator.trigger('message', 'saveLocalAssessment', 'No Vet was selected');
						return;
					}
				});
			},
			saveAssessmentLocallyCall: function(vet) {
				var coreDataAssessment = {
						veteranHomesId:         vet.get('veteranID'),
						veteranFirstName:       vet.get('firstName'),
						veteranLastName:        vet.get('lastName'),
                        veteranSSN:             vet.get('sSN'),
                        veteranLocalID:         vet.get('veteranLocalID'),
						assessmentDate:         Responses.instance().getAnswer('txtAssessmentDate'),
						stationNumber:          HomesUser.instance().get('SiteCode'),
						finished:               App.getState().completeForm,
						responses:              FormRules.getSoapLogic().toLocalStorage(),
						isDecisionSupport:      0,
						reassessment:           HomesAssessment.instance().get('reassessment'),
						userId:                 HomesUser.instance().get('username')
					};
//					timer.markTime();
//					console.log('saveAssessmentLocally core=' + JSON.stringify(coreDataAssessment));
//					timer.writeTimeSinceMark('saveAssessmentLocally core time ');
					cordova.exec(library.standardSuccess, library.standardError,
								"CoreDataStorage", "saveAssessmentInstance", [coreDataAssessment]);
//					timer.writeTimeSinceMark('saveAssessmentLocally exec time ');
			},
			saveDSFormLocally: function() {
				library.waitForLastCall('saveDSFormLocally', function () {
					library.prep();
					var vet = HomesVet.getCurrentVet();

					if (vet) {
                        if (!vet.get('veteranID')) {
                            vet.set('veteranID','-1');
                        }

                        if (!vet.get('veteranLocalID')) {
                            vet.set('veteranLocalID', '-1');
                        }

						library.saveDSFormLocallyCall(vet);
						library.wait('saveDSFormLocally', null);
					} else {
						Global.getUtilities().eventAggregator.trigger('message', 'saveLocalAssessment', 'No Vet was selected');
						return;
					}
				});
			},
			saveDSFormLocallyCall: function(vet) {
				var coreDataAssessment = {
					veteranHomesId:		vet.get('veteranID'),
					veteranFirstName:	vet.get('firstName'),
					veteranLastName:	vet.get('lastName'),
                    veteranSSN:         vet.get('sSN'),
                    veteranLocalID:     vet.get('veteranLocalID'),
					assessmentDate:		Responses.instance().getAnswer('txtAssessmentDate'),
					stationNumber:		HomesUser.instance().get('SiteCode'),
					finished:			App.getState().completeForm,
					responses:			FormRules.getSoapLogic().toLocalStorage(),
					isDecisionSupport:	1,
					reassessment:		0,
					userId:             HomesUser.instance().get('username')
				};

				console.log('saveDSFormLocally core=' + JSON.stringify(coreDataAssessment));
				cordova.exec(library.standardSuccess, library.standardError,
						"CoreDataStorage", "saveAssessmentInstance", [coreDataAssessment]);
			},
			fetchAllAssessments: function(callback) {
				library.waitForLastCall('fetchAllAssessments', function () {
					library.prep();
					library.fetchAllFormsCall(0);
					library.wait('fetchAllAssessments', callback);
				});
			},
			fetchAllDSForms: function(callback) {
				library.waitForLastCall('fetchAllDSForms', function () {
					library.prep();
					library.fetchAllFormsCall(1);
					library.wait('fetchAllDSForms', callback);
				});
			},
			fetchAllFormsCall: function(dsForm) {
				var queryObject = {
					"isDecisionSupport":     dsForm,
					"userId":                HomesUser.instance().get('username')
				};
				cordova.exec(library.standardSuccess, library.standardError,
						"CoreDataStorage", "fetchAssessmentInstances", [queryObject]);
			},
			deleteAssessment: function(queryParameters, callback) {
				library.waitForLastCall('deleteAssessment', function () {
					library.prep();
					library.deleteFormCall(0, queryParameters);
					library.wait('deleteAssessment', callback);
				});
			},
			deleteDSForm: function(queryParameters, callback) {
				library.waitForLastCall('deleteDSForm', function () {
					library.prep();
					library.deleteFormCall(1, queryParameters);
					library.wait('deleteDSForm', callback);
				});
			},
			deleteFormCall: function(dsForm, queryParameters) {
                queryParameters.userId = HomesUser.instance().get('username');
                queryParameters.isDecisionSupport = dsForm;

				cordova.exec(library.standardSuccess, library.standardError,
					"CoreDataStorage", "deleteAssessmentInstance", [queryParameters]);
			},
            fetchDSForm: function(queryParameters,callback) {
				library.waitForLastCall('fetchDSForm', function () {
					library.prep();
					library.fetchDSFormCall(queryParameters);
					library.wait('fetchDSForm', callback);
				});
			},
			fetchDSFormCall: function(queryParameters, callback) {
                queryParameters.isDecisionSupport = 1;
                queryParameters.userId = HomesUser.instance().get('username');

				console.log('fetchDSForm queryObject=' + JSON.stringify(queryParameters));
				cordova.exec(library.standardSuccess, library.standardError,
						"CoreDataStorage", "fetchOneAssessmentInstance", [queryParameters]);
			},
			standardSuccess: function(data) {
				console.log("cordova standardSuccess");
				sync.finished = true;
				sync.successData = data;
				sync.success = true;
			},
			standardError: function(data) {
				console.log("cordova standardError");
				sync.finished = true;
				sync.errorData = data;
				sync.success = false;
			},
			wait: function(method, callback) {
				console.log('WAIT calling ' + method);
				$.mobile.loading('show');
				if (sync.finished) {
					console.log('WAIT ' + method + ' finished');
					sync.callOver = true;
					sync.destroyIcon(callback);
				} else {
					if (sync.iterate(true)) {
						console.log('cordova going to wait again for ' + method);
						setTimeout(function() {
							library.wait(method,callback);
						}, sync.interval);
					} else {
						sync.reset();
						console.log('cordova gave up waiting');
						sync.destroyIcon(callback);
					}
				}
			},
			waitForLastCall: function(nextMethod, callback) {
				console.log('WAIT waitForLastCall want to call ' + nextMethod);
				if (sync.callOver) {
					console.log('previous cordova call ' + currentMethod + ' had finished');
					currentMethod = nextMethod;
					if (callback) {
						console.log('WAIT waitForLastCall calling ' + currentMethod);
						callback();
					}
				} else {
					console.log('cordova call ' + currentMethod + ' had NOT finished');
					if (sync.iterateLonger()) {
						console.log('going to wait for last cordova to finish again ' + nextMethod);
						setTimeout(function() {
							library.waitForLastCall(nextMethod,callback);
						}, sync.interval);
					} else {
						sync.reset();
						console.log('ERROR gave up waiting for last cordova to finish');
						currentMethod = nextMethod;
						if (callback) {
							callback();
						}
					}
				}
			}
		}.init();

		return cordovaLib;
	}
);
