define(
	['App', 'Global', 'HomesSoap', 'CordovaLib',
	'HomesUser', 'HomesAssessment', 'MainView', 'Forms', 'HomesVet'],
	function(App, Global, HomesSoap, CordovaLib,
			HomesUser, HomesAssessment, MainView, Forms, HomesVet) {
		var sync, success, errorData, successData, vaData, library;
		var homesLib = {
			init:	function() {
				library = this;
				return this;
			},
			readAssessment: function(callback) {
				var readAssessmentCallback = function(bool) {
					if (callback) {
						callback(bool);
					}
				};
				var vet = HomesVet.getCurrentVet();

				if (vet) {
					var data = vet.toJSON();
					var queryParameters;

					if (data.veteranID && data.veteranLocalID) {
						queryParameters = {
							'veteranHomesId': data.veteranID,
							'veteranLocalID': data.veteranLocalID
						};
					} else if (data.veteranID) {
						queryParameters =  {'veteranHomesId': data.veteranID};
					} else {
						queryParameters =  {'veteranLocalID': data.veteranLocalID};
					}

					CordovaLib.fetchAssessment(queryParameters, function(obj) {
						if (obj.success) {
							var assessment = obj.successData;
							console.log('found assessment in local storage');
							HomesAssessment.instance().updateFromLocal(assessment,false);
							console.log('reassessment=' + HomesAssessment.instance().get('reassessment'));
							readAssessmentCallback(true);
						} else {
							if (App.getState().isHomesOnline()) {
								console.log('calling getLastAssessment' + JSON.stringify(data));
								HomesSoap.getLastAssessment(data, function(obj) {
									if (obj.success && obj.successData.FormID != '0') {
										console.log('found assessment in DB');
										HomesAssessment.instance().updateFromVA(obj.successData);
										CordovaLib.saveAssessmentLocally();
									} else {
										console.log('did not find any assessment creating new one');
										HomesAssessment.instance().createNew(false);
										MainView.refresh();
									}
									readAssessmentCallback(true);
								});
							} else {
								HomesAssessment.instance().createNew(false);
								readAssessmentCallback(true);
							}
						}
					});
				} else {
					console.log('vet does not exist');
					readAssessmentCallback(false);
				}
			},
			readDecision: function(callback) {
				library.readDecisionCallback = callback;
				var vet = HomesVet.getCurrentVet();
				console.log('vet ' + JSON.stringify(vet));
				if (vet) {
					var data = vet.toJSON();

					var queryParameters;

					if (data.veteranID && data.veteranLocalID) {
						queryParameters = {
							'veteranHomesId': data.veteranID,
							'veteranLocalID': data.veteranLocalID
						};
					} else if (data.veteranID) {
						queryParameters =  {'veteranHomesId': data.veteranID};
					} else {
						queryParameters =  {'veteranLocalID': data.veteranLocalID};
					}

					CordovaLib.fetchDSForm(queryParameters, function(obj) {
						if (obj.success) {
							var dsForm = obj.successData;
							console.log('found dsForm in local storage');
							HomesAssessment.instance().updateFromLocal(dsForm,true);
							console.log('reassessment=' + HomesAssessment.instance().get('reassessment'));
						} else {
							console.log('did not find any dsForm creating new one');
							HomesAssessment.instance().createNew(true);
						}
						if (library.readDecisionCallback) {
							library.readDecisionCallback(true);
						}
					});
				} else {
					console.log('vet does not exist');
					if (library.readDecisionCallback) {
						library.readDecisionCallback(false);
					}
				}
			},
			sendAssessment: function(answers, callback) {
				library.sendAssessmentCallback = callback;
				var currentDateString = formatDateLongForm(new Date());
				var currentVeteran = HomesVet.getCurrentVet();

				if (currentVeteran.get('veteranID') && currentVeteran.get('veteranID') != '-1') {
					var params = {
						SessionID:		HomesUser.instance().get('SessionID'),
						UserLogin:		HomesUser.instance().get('username'),
						Assessment: {
							VeteranID:		currentVeteran.get('veteranID'),
							SubmitDate:		currentDateString,
							SiteID:			HomesUser.instance().get('SiteCode'),
							Responses: {
								Response: answers
							}
						}
					};
					HomesSoap.saveAssessment(params,library.sendAssessmentCallback);

				} else {  //Local veteran needs to be submitted to HOMES first
					var options = {
						sessionId:		HomesUser.instance().get('SessionID'),
						loginId:		HomesUser.instance().get('LoginId'),
						username:		HomesUser.instance().get('username'),
						searchQuery:    currentVeteran.get('sSN'),
						searchType:     SEARCH_TYPE_SSN
					};

					HomesSoap.getVets(options, function(obj) {
						if (obj.success) {
							var data = obj.successData;
							console.log('data check vet=' + JSON.stringify(data));

							if (data) { // Vet already saved in database with same SSN
								var homesVeteran = new HomesVet(data);
								var alertString = ['A matching SSN was found in the HOMES database.',
								'To overwrite the Veteran data saved on your device with the',
								'record found in HOMES: Tap “Overwrite”.'];
//								'To update the HOMES record with the data on your local device: Tap “Verify & Update” (You will be redirected to the Veteran Summary screen).'];

								var callbackMessage = {success: false, matchFound: true, dialogText: alertString, homesVeteranMatch: homesVeteran};
								if (library.sendAssessmentCallback) {
									library.sendAssessmentCallback(callbackMessage);
								}
							} else { // No vet in HOMES with this SSN, need to submit
								var vet = HomesVet.getCurrentVet();

								var currentDateTime = new Date();
								vet.attributes.username       = HomesUser.instance().get('username');
								vet.attributes.lastActionDate = currentDateTime;
								vet.attributes.creationDate = currentDateTime;
								vet.attributes.sessionId = HomesUser.instance().get('SessionID');

								HomesSoap.addVet(vet.attributes, function(obj) {
									if (obj.success) {
										var data = obj.successData;
										var vet = new HomesVet(data);
										//Need former LocalID
										var veteranLocalID = HomesVet.getCurrentVet().get('veteranLocalID');
										vet.set('veteranLocalID', veteranLocalID);
										HomesVet.setCurrentVet(vet);

										MainView.refresh();
										// a recursive call?
										library.sendAssessment(answers, library.sendAssessmentCallback);
									} else {
										Global.getUtilities().reportError('Could not save the Veteran ',obj.errorData);
										MainView.refresh();
										if (library.sendAssessmentCallback) {
											library.sendAssessmentCallback();
										}
									}
								});
							}
						} else {
							var error = obj.errorData;
							console.log('error check vet=' + JSON.stringify(error));
							MainView.refresh();
							if (library.sendAssessmentCallback) {
								library.sendAssessmentCallback();
							}
						}
					});
				}
			},
			sendDS: function(answers, alerts, recommendations, callback) {
				library.sendDSCallback = callback;
				var currentDateString = formatDateLongForm(new Date());
				var currentVeteran = HomesVet.getCurrentVet();
				if (currentVeteran.get('veteranID') && currentVeteran.get('veteranID') != '-1') {
					var params = {
						SessionID:		HomesUser.instance().get('SessionID'),
						UserLogin:		HomesUser.instance().get('username'),
						DecisionSupport: {
							VeteranID:		currentVeteran.get('veteranID'),
							SubmitDate:		currentDateString,
							SiteID:			HomesUser.instance().get('SiteCode'),
							Version:		formatDate(new Date()),
							Responses: {
								Response: answers
							},
							AlertDecisions: {
								AlertDecision: alerts
							},
							Recommendations: {
								Recommendation: recommendations
							}
						}
					};
					console.log('sendDS ' + JSON.stringify(params))
					HomesSoap.saveDSForm(params,library.sendDSCallback);
				} else {  //Local veteran needs to be submitted to HOMES first
					var options = {
						sessionId:		HomesUser.instance().get('SessionID'),
						loginId:		HomesUser.instance().get('LoginId'),
						username:		HomesUser.instance().get('username'),
						searchQuery:    currentVeteran.get('sSN'),
						searchType:     SEARCH_TYPE_SSN
					};

					HomesSoap.getVets(options, function(obj) {
						if (obj.success) {
								var data = obj.successData;
								console.log('data check vet=' + JSON.stringify(data));
								if (data) { // Vet already saved in database with same SSN
									var homesVeteran = new HomesVet(data);
									var alertString = ['A matching SSN was found in the HOMES database.',
									'To overwrite the Veteran data saved on your device with the',
									'record found in HOMES: Tap “Overwrite”.'];
//									'To update the HOMES record with the data on your local device: Tap “Verify & Update” (You will be redirected to the Veteran Summary screen).'];

									var callbackMessage = {success: false, matchFound: true, dialogText: alertString, homesVeteranMatch: homesVeteran};
									if (library.sendDSCallback) {
										library.sendDSCallback(callbackMessage);
									}
								} else { // No vet in HOMES with this SSN, need to submit
									var vet = HomesVet.getCurrentVet();
									var currentDateTime = new Date();
									vet.attributes.username       = HomesUser.instance().get('username');
									vet.attributes.lastActionDate = currentDateTime;
									vet.attributes.creationDate = currentDateTime;
									vet.attributes.sessionId = HomesUser.instance().get('SessionID');

									HomesSoap.addVet(vet.attributes, function(obj) {
										if (obj.success) {
											var data = obj.successData;
											var vet = new HomesVet(data);
											//Need former LocalID
											var veteranLocalID = HomesVet.getCurrentVet().get('veteranLocalID');
											vet.set('veteranLocalID', veteranLocalID);
											HomesVet.setCurrentVet(vet);

											MainView.refresh();
											library.sendDS(answers, alerts, recommendations, library.sendDSCallback);
										} else {
											Global.getUtilities().reportError('Could not save the Veteran',obj.errorData);
											MainView.refresh();
											if (library.sendDSCallback) {
												library.sendDSCallback(callbackMessage);
											}
										}
									});
								}
							} else {
								var error = obj.errorData;
								console.log('error check vet=' + JSON.stringify(error));
								MainView.refresh();
								if (library.sendDSCallback) {
									library.sendDSCallback();
								}
							}
					});
				}
			},
			loadForms: function(callback) {
				var loadFormsCallback = function() {
					if (callback) {
						callback();
					}
				};
				Forms.clearInstance();
				if (HomesVet.isVetLocal()) {
					loadFormsCallback();
					return;
				}

				var homesID = HomesVet.getHomesID();
				if (App.getState().isHomesOnline()) {
					HomesSoap.getAssessments(homesID,function(obj) {
						console.log('getAssessments SYNC success=' + obj.success);
						if (obj.success) {
							var data = obj.successData;
//							library.reportOn(data);
							if (data && data.VeteranAssessment) {
								if ($.isArray(data.VeteranAssessment)) {
									for (var i in data.VeteranAssessment) {
										data.VeteranAssessment[i].FormType = App.getConstants().ASSESSMENT_FORM;
										data.VeteranAssessment[i].assessmentDate = data.VeteranAssessment[i].AssessmentDate;
										data.VeteranAssessment[i].submitDate = data.VeteranAssessment[i].SubmitDate;
									}
								} else {
									data.VeteranAssessment.FormType = App.getConstants().ASSESSMENT_FORM;
									data.VeteranAssessment.assessmentDate = data.VeteranAssessment.AssessmentDate;
									data.VeteranAssessment.submitDate = data.VeteranAssessment.SubmitDate;
								}
								Forms.instance().addTo(data.VeteranAssessment);
							}
						} else {
							console.log('getAssessments failure=' + JSON.stringify(obj.errorData));
							if (App.getState().isHomesOffline()) {
								MainView.refresh();
								Global.getUtilities().eventAggregator.trigger('message', 'offline');
							} else {
								MainView.refresh();
								Global.getUtilities().reportError('HOMES error',obj.errorData);
								if (HomesVet.getCurrentVet()) {
									var json = HomesVet.getCurrentVet().get('assessmentJSON');
									if (json) {
										console.log('getAssessments had failure but adding assessment' + JSON.stringify(json));
										Forms.instance().addTo(json);
									}
								}
							}
						}
					});

					HomesSoap.getDSList(homesID,function(obj) {
						console.log('getDSList SYNC success=' + obj.success);
						if (obj.success) {
							var data = obj.successData;
							if (data && data.VeteranDecisionSupport) {
								if ($.isArray(data.VeteranDecisionSupport)) {
									for (var i in data.VeteranDecisionSupport) {
										data.VeteranDecisionSupport[i].FormType = App.getConstants().DS_FORM;
										data.VeteranDecisionSupport[i].assessmentDate = data.VeteranDecisionSupport[i].AssessmentDate;
										data.VeteranDecisionSupport[i].submitDate = data.VeteranDecisionSupport[i].SubmitDate;
									}
								} else {
									data.VeteranDecisionSupport.FormType = App.getConstants().DS_FORM;
									data.VeteranDecisionSupport.assessmentDate = data.VeteranDecisionSupport.AssessmentDate;
									data.VeteranDecisionSupport.submitDate = data.VeteranDecisionSupport.SubmitDate;
								}
								Forms.instance().addTo(data.VeteranDecisionSupport);
							}
						} else {
							if (App.getState().isHomesOffline()) {
								MainView.refresh();
								Global.getUtilities().eventAggregator.trigger('message', 'offline');
							} else {
								console.log('getDSList failure=' + JSON.stringify(obj.errorData));
								MainView.refresh();
								Global.getUtilities().reportError('HOMES error',obj.errorData);
							}
						}
						loadFormsCallback();
					});
				} else {
					loadFormsCallback();
				}
			},
			sendSUD: function(callback) {
				HomesUser.instance().set('SUDAcceptedDate',formatDateLongForm(new Date()));
				var params =
				{
					_UserLogin:		HomesUser.instance().get('username'),
					_SessionID:		HomesUser.instance().get('SessionID'),
					_User:			{
						Authenticated:		HomesUser.instance().get('Authenticated'),
						StaffName:			HomesUser.instance().get('StaffName'),
						StaffID:			HomesUser.instance().get('StaffID'),
						Facility:			HomesUser.instance().get('Facility'),
						Roles:				HomesUser.instance().get('RoleObj'),
						StaffEmail:			HomesUser.instance().get('StaffEmail'),
						SUDAcceptedDate:	formatDateLongForm(new Date())
					}
				};
				HomesSoap.sud(params,callback);
			},
			loadVetInfo: function(callback) {
				var loadVetInfoCallback = function() {
					console.log('loadVetInfo SYNC');
					if (callback) {
						callback();
					}
				}

				var loadFormsCallback = function() {
					console.log('loadFormsCallback');
					console.log('calling storVet');
					CordovaLib.storeVet(function(obj) {
						console.log('storeVet finished');
						if (obj.success) {
							console.log('storeVet data=' + JSON.stringify(obj.successData));
							if (isNaN(obj.successData)) {
								console.log('BAD LOCAL VETERAN ID! Configure view load vet info');
							}
							HomesVet.getCurrentVet().set('veteranLocalID', obj.successData);
						}
						library.readAssessment(loadVetInfoCallback); // will be called with a bool we can ignore
					});
				};

				if (HomesVet.getCurrentVet()) {
					library.loadForms(loadFormsCallback);
				} else {
					loadVetInfoCallback();
				}
			}
		}.init();

		return homesLib;
	}
);
