define(
	['App', 'HomesUser', 'HomesVet'],
	function(App, HomesUser, HomesVet) {
		var sync;
		var vaData, library, currentMethod;
		var soapCalls = {
			init:	function() {
				library = this;
				sync = new (App.getUtilities().sync)(100,10000);
				sync.success = false;
				return this;
			},
			prep: function(ping) {
				if (ping) {
					sync.startWaiting(true);
				} else {
					sync.startWaiting(false);
				}
				sync.success = false;
			},
			ping: function(callback) {
				library.waitForLastSoap('ping', function () {
					console.log('ONLINE ping is assuming online until proven otherwise');
					App.getState().setHomesOnline(true); // online until proven offline
					library.prep(true);
					library.pingSoapCall();
					library.wait(callback);
				});
			},
			pingSoapCall: function() {
				$.soap($.extend({}, App.getState().soapDefaults('ping'), {
					params: {
					},
					success: function (data) {
						var successData = data.toJSON().Body.GetLastErrorResponse.GetLastErrorResult;
						library.standardSuccess(successData);
					},
					error: function (data) {
						library.standardError(data);
					}
				}));
			},
			getLastAssessment: function(params,callback) {
				library.waitForLastSoap('getLastAssessment', function () {
					// I think wait() handles homes going offline ok but I'll leave this as is
					if (App.getState().isHomesOnline()) {
						library.prep();
						library.getLastAssessmentSoapCall(params);
						library.wait(callback);
					} else {
						sync.callback(callback);
					}
				});
			},
			getLastAssessmentSoapCall: function(params) {
				$.soap($.extend({}, App.getState().soapDefaults('lastAssessment'), {
					params: {
						UserLogin:		HomesUser.instance().get('username'),
						SessionID:		HomesUser.instance().get('SessionID'),
						VeteranID:		params.veteranID
					},
					success: function (data) {
						var successData = data.toJSON().Body.GetVeteranLastestAssessmentFormResponse.GetVeteranLastestAssessmentFormResult;
						library.convertVaId_FromVA(successData);
						library.standardSuccess(successData);
					},
					error: function (data) {
						library.standardError(data);
					}
				}));
			},
			login: function(model,callback) {
				library.waitForLastSoap('login', function () {
					library.prep();
					library.loginSoapCall(model);
					library.wait(callback);
				});
			},
			loginSoapCall: function(model) {
				$.soap($.extend({}, App.getState().soapDefaults('homesUserLogin'), {
					params: {
						UserLogin:		model.get('username'),
//						UserPassword:	encodeURIComponent(model.get('password'))
						UserPassword:	App.getUtilities().toCharEntity( model.get('password') )
					},
					success: function (data) {
						var successData = data.toJSON().Body.LogInResponse.LogInResult;
						library.standardSuccess(successData);
					},
					error: function (data) {
						library.standardError(data);
//						var msg =
//							'content:    ' + data.content +
//							'httpText:   ' + data.httpText +
//							' httpCode:  ' + data.httpCode +
//							' status:    ' + data.status;
//						alert(msg);
					}
				}));
			},
			saveAssessment: function(params,callback) {
				library.waitForLastSoap('saveAssessment', function () {
					library.prep();
					library.saveAssessmentSoapCall(params);
					library.wait(callback);
				});
			},
			saveAssessmentSoapCall: function(params) {
				$.soap($.extend({}, App.getState().soapDefaults('SubmitAssessmentForm'), {
					params: params,
					success: function (data) {
						var successData = data.toJSON().Body.SubmitAssessmentFormResponse.SubmitAssessmentFormResult;
						library.standardSuccess(successData);
					},
					error: function (data) {
						library.standardError(data);
					}
				}));
			},
			saveDSForm: function(params,callback) {
				library.waitForLastSoap('saveDSForm', function () {
					library.prep();
					library.saveDSFormSoapCall(params);
					library.wait(callback);
				});
			},
			saveDSFormSoapCall: function(params) {
				$.soap($.extend({}, App.getState().soapDefaults('SubmitDecisionSupportForm'), {
					params: params,
					success: function (data) {
						var successData = data.toJSON().Body.SubmitDecisionSupportFormResponse.SubmitDecisionSupportFormResult;
						library.standardSuccess(successData);
					},
					error: function (data) {
						library.standardError(data);
					}
				}));
			},
			getVets: function(params,callback) {
				var soapParams = {
					UserLogin:  params.username,
					SessionID:  params.sessionId
				};
				if (params.searchType === SEARCH_TYPE_SSN) {
					soapParams.SSN = params.searchQuery;
				} else if (params.searchType === SEARCH_TYPE_NAME) {
					soapParams.Name = params.searchQuery;
				} else if (params.searchType === SEARCH_TYPE_LAST4) {
					soapParams.Last4 = params.searchQuery;
				} else if (params.searchType === SEARCH_TYPE_ID) {
					soapParams.VeteranID = params.searchQuery;
				}
				library.waitForLastSoap('getVets', function () {
					library.prep();
					library.getVetsSoapCall(soapParams,params.searchType);
					library.wait(callback);
				});
			},
			getVetsSoapCall: function(params,searchType) {
				$.soap($.extend({}, App.getState().soapDefaults(searchType), {
					params:	params,
					success: function (data) {
						var successData;
						if (searchType === SEARCH_TYPE_SSN) {
							successData = data.toJSON().Body.GetVeteranListBySSNResponse.GetVeteranListBySSNResult.Veteran;
						} else if (searchType === SEARCH_TYPE_NAME) {
							successData = data.toJSON().Body.GetVeteranListByNameResponse.GetVeteranListByNameResult.Veteran;
						} else if (searchType === SEARCH_TYPE_LAST4) {
							successData = data.toJSON().Body.GetVeteranListByLast4Response.GetVeteranListByLast4Result.Veteran;
						} else if (searchType === SEARCH_TYPE_ID) {
							successData = data.toJSON().Body.GetVeteranResponse.GetVeteranResult;
						}
						library.standardSuccess(successData);
					},
					error: function (data) {
						library.standardError(data);
					}
				}));
			},
			getAssessments: function(homesID,callback) {
				library.waitForLastSoap('getAssessments', function () {
					if (App.getState().isHomesOnline()) {
						library.prep();
						library.getAssessmentsSoapCall(homesID);
						library.wait(callback);
					} else {
						console.log('SOAP homes went offline');
						sync.callback(callback);
					}
				});
			},
			getAssessmentsSoapCall: function(homesID) {
				$.soap($.extend({}, App.getState().soapDefaults('allAssessment'), {
					params: {
						UserLogin:					HomesUser.instance().get('username'),
						SessionID:					HomesUser.instance().get('SessionID'),
						VeteranID:					homesID,
						YearsOfAssessmentHistory:	App.getState().yearsOfHistory
					},
					success: function (data) {
						var successData = data.toJSON().Body.GetAssessmentFormsResponse.GetAssessmentFormsResult;
						if (!successData) successData = {};
						successData.fromCall = 'getAssessments';
						library.standardSuccess(successData);
					},
					error: function (data) {
						library.standardError(data);
					}
				}));
			},
			getDSList: function(homesID,callback) {
				library.waitForLastSoap('getDSList', function () {
					if (App.getState().isHomesOnline()) {
						library.prep();
						library.getDSListSoapCall(homesID);
						library.wait(callback);
					} else {
						console.log('SOAP homes went offline');
						sync.callback(callback);
					}
				});
			},
			getDSListSoapCall: function(homesID) {
				$.soap($.extend({}, App.getState().soapDefaults('allDS'), {
					params: {
						UserLogin:					HomesUser.instance().get('username'),
						SessionID:					HomesUser.instance().get('SessionID'),
						VeteranID:					homesID,
						YearsOfAssessmentHistory:	10
					},
					success: function (data) {
						var successData = data.toJSON().Body.GetDecisionSuportFormsResponse.GetDecisionSuportFormsResult;
						if (!successData) successData = {};
						successData.fromCall = 'getDSList';
						library.standardSuccess(successData);
					},
					error: function (data) {
						library.standardError(data);
					}
				}));
			},
			getAssessment: function(data,callback) {
				library.waitForLastSoap('getAssessment', function () {
					library.prep();
					library.getAssessmentSoapCall(data);
					library.wait(callback);
				});
			},
			getAssessmentSoapCall: function(form) {
				$.soap($.extend({}, App.getState().soapDefaults('assessment'), {
					params: {
						UserLogin:					HomesUser.instance().get('username'),
						SessionID:					HomesUser.instance().get('SessionID'),
						FormID:						form.get('FormID')
					},
					success: function (data) {
						var successData = data.toJSON().Body.GetAssessmentFormResponse.GetAssessmentFormResult;
						library.convertVaId_FromVA(successData);
						library.standardSuccess(successData);
					},
					error: function (data) {
						library.standardError(data);
					}
				}));
			},
			convertVaId_FromVA: function(data) {
				var Responses = null;
				if (data.Responses) {
					Responses = data.Responses;
				}
				if (Responses) {
					for (var i in Responses.Response) {
						var vaId = Responses.Response[i].QuestionID;
						if (vaId==='txtAssessmentDate') {
							console.log('fromVA removing vaId=' + vaId);
							Responses.Response[i].QuestionID = 'ignore';
						}
					}
					for (var i in Responses.Response) {
						var vaId = Responses.Response[i].QuestionID;
						if (vaId==='txtAssesmentDate') { // missing the s
							console.log('fromVA renaming vaId=' + vaId);
							Responses.Response[i].QuestionID = 'txtAssessmentDate';
						}
					}
				}
			},
			getDS: function(data,callback) {
				library.waitForLastSoap('getDS', function () {
					library.prep();
					library.getDSSoapCall(data);
					library.wait(callback);
				});
			},
			getDSSoapCall: function(form) {
				$.soap($.extend({}, App.getState().soapDefaults('ds'), {
					params: {
						UserLogin:					HomesUser.instance().get('username'),
						SessionID:					HomesUser.instance().get('SessionID'),
						FormID:						form.get('FormID')
					},
					success: function (data) {
						var successData = data.toJSON().Body.GetDecisionSuportFormResponse.GetDecisionSuportFormResult;
						library.standardSuccess(successData);
					},
					error: function (data) {
						library.standardError(data);
					}
				}));
			},
			addVet: function(params,callback) {
				var options = {
					sessionId:		HomesUser.instance().get('SessionID'),
					loginId:		HomesUser.instance().get('LoginId'),
					username:		HomesUser.instance().get('username'),
					searchQuery:    params.sSN,
					searchType:     SEARCH_TYPE_SSN
				};

				library.getVets(options, function(obj) {
					if (obj.success) {
						var data = obj.successData;
//						console.log('data check vet=' + JSON.stringify(data));

						if (data) { // Vet already saved in database with same SSN
							var homesVeteran = new HomesVet(data);

							var alertString = ['A matching SSN was found in the HOMES database.',
							'To overwrite the Veteran data saved on your device with the',
							'record found in HOMES: Tap “Overwrite”.'];
//							'To update the HOMES record with the data on your local device: Tap “Submit” to update the HOMES record.'];

							var callbackMessage = {success: false, matchFound: true, dialogText: alertString, homesVeteranMatch: homesVeteran};

							callback(callbackMessage);
						} else { // No vet in HOMES with this SSN, need to submit
							library.waitForLastSoap('addVet', function () {
								library.prep();
								library.addVetSoapCall(params);
								library.wait(callback);
							});
						}
					} else {
						var error = obj.errorData;
						console.log('SOAP error check vet=' + JSON.stringify(error));
						callback(obj);
					}
				});
			},
			addVetSoapCall: function(params) {
				$.soap($.extend({}, App.getState().soapDefaults('addHomesVeteran'), {
					params: {
						userLogin:      params.username,
						sessionID:      params.sessionId,
						veteran: {
							FirstName:              params.firstName,
							LastName:               params.lastName,
							MiddleInitial:          params.middleInitial,
							SSN:                    params.sSN,
							SSNType:                1,
							DateOfBirth:            params.dateOfBirth,
							Gender:                 params.gender,
							Address1:               params.address1,
							Address2:               params.address2,
							Address3:               params.address3,
							City:                   params.city,
							State:                  params.state,
							ZipCode:                params.zipCode,
							TelephonePrimary:       params.telephonePrimary,
							TelephoneSecondary:     params.telephoneSecondary,
							Email:                  params.email,
							POCName:                params.pOCName,
							POCPhone:               params.pOCPhone,
							POCRelationship:        params.pOCRelationship,
							POCRelationshipOther:   params.pOCRelationshipOther,
							LastActionDate:         ISOdate(params.lastActionDate),
							CreationDate:           ISOdate(params.creationDate)
						}
					},
					success: function (data) {
						var successData = data.toJSON().Body.AddOrEditVeteranResponse.AddOrEditVeteranResult;
						successData.hasUnsubmittedChanges = false;
						library.standardSuccess(successData);
					},
					error: function (data) {
						library.standardError(data);
					}
				}));
			},
			updateVet: function(params,callback) {
				library.waitForLastSoap('updateVet', function () {
					library.prep();
					library.updateVetSoapCall(params);
					library.wait(callback);
				});
			},
			updateVetSoapCall: function(params) {
				$.soap($.extend({}, App.getState().soapDefaults('updateVeteran'), {
					params: {
						userLogin:      params.username,
						sessionID:      params.sessionId,
						veteran: {
							VeteranID:              params.veteranID,
							FirstName:              params.firstName,
							LastName:               params.lastName,
							MiddleInitial:          params.middleInitial,
							SSN:                    params.sSN,
							SSNType:                1,
							DateOfBirth:            params.dateOfBirth,
							Gender:                 params.gender,
							Address1:               params.address1,
							Address2:               params.address2,
							Address3:               params.address3,
							City:                   params.city,
							State:                  params.state,
							ZipCode:                params.zipCode,
							TelephonePrimary:       params.telephonePrimary,
							TelephoneSecondary:     params.telephoneSecondary,
							Email:                  params.email,
							POCName:                params.pOCName,
							POCPhone:               params.pOCPhone,
							POCRelationship:        params.pOCRelationship,
							POCRelationshipOther:   params.pOCRelationshipOther,
							LastActionDate:         ISOdate(params.lastActionDate),
							CreationDate:           ISOdate(params.creationDate)
						}
					},
					success: function (data) {
						var successData = data.toJSON().Body.AddOrEditVeteranResponse.AddOrEditVeteranResult;
						successData.hasUnsubmittedChanges = false;
						library.standardSuccess(successData);
					},
					error: function (data) {
						library.standardError(data);
					}
				}));
			},
			checkVeteranHomesEpisodeStatus: function(callback) {
				library.waitForLastSoap('checkVeteranHomesEpisodeStatus', function () {
					library.prep();
					library.checkVeteranHomesEpisodeStatusCall();
					library.wait(callback);
				});
			},
			checkVeteranHomesEpisodeStatusCall: function() {
				$.soap($.extend({}, App.getState().soapDefaults('homesEpisodeCheck'), {
					params: {
						UserLogin:					HomesUser.instance().get('username'),
						SessionID:					HomesUser.instance().get('SessionID'),
						VeteranID:                  HomesVet.getCurrentVet().get('veteranID')
					},
					success: function (data) {
						var successData = data.toJSON().Body.GetOpenEpisodeAssessmentResponse;
						if (successData.GetOpenEpisodeAssessmentResult) {
							successData = successData.GetOpenEpisodeAssessmentResult.VISN;
						} else {
							successData = '';
						}
						HomesVet.getCurrentVet().updateEpisode(successData);
						library.standardSuccess(successData);
					},
					error: function (data) {
						library.standardError(data);
					}
				}));
			},
			sud: function(params,callback) {
				library.waitForLastSoap('sud', function () {
					library.prep();
					library.sudCall(params);
					library.wait(callback);
				});
			},
			sudCall: function(params) {
				$.soap($.extend({}, App.getState().soapDefaults('sud'), {
					params: params,
					success: function (data) {
						var successData = data.toJSON().Body.SubmitStaffSUDDateResponse.SubmitStaffSUDDateResult;
						library.standardSuccess(successData);
					},
					error: function (data) {
						library.standardError(data);
					}
				}));
			},
			standardSuccess: function(data) {
				removeKey('xsi:nil', data);
				sync.finished = true;
				sync.successData = data;
				sync.success = true;
			},
			standardError: function(data) {
				console.log("SOAP Error: " + data);
				sync.finished = true;
				sync.errorData = data;
				sync.success = false;
			},
			wait: function(callback) {
				$.mobile.loading('show');
				console.log('SOAP showing icon'+ ', ' + (new Date()).getSeconds());
				if (sync.finished) {
					// do not turn online true here
					// global ajax may have gotten a 0 or 522 and set the online flag false
					console.log('SOAP done waiting, calling callback');
					sync.callOver = true;
					sync.destroyIcon(callback);
				} else {
					if (sync.iterate(App.getState().isHomesOnline())) {
//						console.log('SOAP  going to wait again');
						setTimeout(function() {
							library.wait(callback);
						}, sync.interval);
					} else {
						console.log('ONLINE wait timeout');
						App.getState().setHomesOnline(false);
						sync.reset();
						App.getUtilities().errorNoSoapResponse(sync.mills + ' mills');
						sync.destroyIcon(callback);
					}
				}
			},
			waitForLastSoap: function(nextMethod, callback) {
				if (sync.callOver) {
					console.log('SOAP previous soap call ' + currentMethod + ' finished');
					currentMethod = nextMethod;
					if (callback) {
						callback();
					}
				} else {
					if (sync.iterateLonger()) {
						console.log('SOAP going to wait for ' + currentMethod + ' to finish again');
						setTimeout(function() {
							library.waitForLastSoap(nextMethod, callback);
						}, sync.interval);
					} else {
						sync.reset();
						console.log('ONLINE waitForLastSoap timeout');
						App.getState().setHomesOnline(false);
						console.log('SOAP gave up waiting for' + currentMethod + ' to finish');
						if (callback) {
							callback();
						}
					}
				}
			}
		}.init();

		return soapCalls;
	}
);
