define(
	['App', 'Global'],
	function(App,Global) {
		var library;
		var JsonFormatter = {
			stringify: function (cipherParams) {
				// create json object with ciphertext
				var jsonObj = {
					ct: cipherParams.ciphertext.toString(CryptoJS.enc.Base64)
				};

				// optionally add iv and salt
				if (cipherParams.iv) {
					jsonObj.iv = cipherParams.iv.toString();
				}
				if (cipherParams.salt) {
					jsonObj.s = cipherParams.salt.toString();
				}

				// stringify json object
				return JSON.stringify(jsonObj);
			},

			parse: function (jsonStr) {
				// parse json string
				var jsonObj = JSON.parse(jsonStr);

				// extract ciphertext from json object, and create cipher params object
				var cipherParams = CryptoJS.lib.CipherParams.create({
					ciphertext: CryptoJS.enc.Base64.parse(jsonObj.ct)
				});

				// optionally extract iv and salt
				if (jsonObj.iv) {
					cipherParams.iv = CryptoJS.enc.Hex.parse(jsonObj.iv)
				}
				if (jsonObj.s) {
					cipherParams.salt = CryptoJS.enc.Hex.parse(jsonObj.s)
				}

				return cipherParams;
			}
		};
		var Cred = function(username,password,data) {
			this.key = CryptoJS.SHA3(username + '@#$skke', { outputLength: 224 })
			this.value = CryptoJS.SHA3(password + '@#$skke', { outputLength: 224 });
			this.encrypted = JsonFormatter.stringify(CryptoJS.AES.encrypt(data, password, { format: JsonFormatter }));

			this.date = formatDate(new Date());
			// cookies won't store functions, so this method will be missing when returned from storage
			this.matchValue = function(cred) {
				if (cred.value.words && this.value.words &&
					cred.value.words.length===this.value.words.length) {
					for (var i in cred.value.words) {
//						console.log('value wordA=' + cred.value.words[i] + ', wordB=' + this.value.words[i])
						if (cred.value.words[i] !== this.value.words[i]) return false;
						return true;
					}
				} else {
					return false;
				}
			};
			this.matchKey = function(cred) {
				if (cred.key.words && this.key.words &&
					cred.key.words.length===this.key.words.length) {
					for (var i in cred.key.words) {
//						console.log('key wordA=' + cred.value.words[i] + ', wordB=' + this.value.words[i])
						if (cred.key.words[i] !== this.key.words[i]) return false;
					}
					return true;
				} else {
					return false;
				}
			};
			this.match = function(cred) {
				return this.matchValue(cred) && this.matchKey(cred);
			};
			this.getData = function(cred,password) {
				var encrypted = JsonFormatter.parse(cred.encrypted);
				var decrypted = CryptoJS.AES.decrypt(encrypted, password, { format: JsonFormatter });
				return decrypted.toString(CryptoJS.enc.Utf8);
			};
			this.writeOut = function() {
				console.log('date=' + this.date);
				for (var i in this.key.words) {
					console.log('key word=' + this.key.words[i])
				}
				for (var i in this.value.words) {
					console.log('value word=' + this.value.words[i])
				}
			};
		};
		var localCreds = {
			init: function() {
				library = this;
				return this;
			},
			eraseOldCreds: function(days) {
				var creds = Global.getUtilities().readFromStorage('creds');
				var change = false;
				if (creds) {
					do {
						var found = false;
						for (var i in creds) {
							if (getDayAge(creds[i].date) > days) {
								creds.splice(i,1);
								found = true;
								change = true;
								break;
							}
						}
					} while(found);
				}
				if (change) {
					Global.getUtilities().writeToStorage('creds',creds);
				}
			},
			storeToLocal: function(cred) {
				var creds = Global.getUtilities().readFromStorage('creds');
				if (!creds) {
					creds = [];
				}
				for (var i in creds) {
					if (cred.matchKey(creds[i])) {
						creds[i] = cred;
						Global.getUtilities().writeToStorage('creds',creds);
						console.log('replaced cred with key=' + creds[i].key);
						return;
					}
				}
				creds.push(cred);
				Global.getUtilities().writeToStorage('creds',creds);
				console.log('added cred with key=' + cred.key);
			},
			storeCred: function(username,password,data) {
				library.eraseOldCreds(60);
				library.storeToLocal(new Cred(username,password,data));
			},
			getCred: function(username,password,data) {
				return new Cred(username,password,data);
			},
			isCredInStorage: function(username,password) {
				var cred = new Cred(username,password);
				var creds = Global.getUtilities().readFromStorage('creds');
				if (creds) {
					for (var i in creds) {
						console.log('checking cred for match');
						if (cred.match(creds[i])) {
							return cred.getData(creds[i],password);
						}
					}
					return null;
				} else {
					console.log('threre were no creds in storage');
					return null;
				}
			},
		}.init();

		return {
			getLocalStorage: function() {
				return localCreds;
			}
		};
	}
);
