var formatDate = function (dateStr) {
	'use strict';
	var date = new Date(dateStr);
	if (dateStr && !isNaN(date.getTime())) {
		var month = date.getMonth() + 1,
			day = date.getDate(),
			year = date.getFullYear();

		day = day < 10 ? '0' + day : day;
		month = month < 10 ? '0' + month : month;

		return (month + "/" + day + "/" + year);
	} else {
		return '';
	}
};

var formatDateAndTime = function (date) {
	'use strict';
	if (isNaN(date.getTime())) {
		return '';
	} else {
		var month = date.getMonth() + 1,
		day = date.getDate(),
		year = date.getFullYear(),
		hour = date.getHours(),
		min = date.getMinutes(),
		sec = date.getSeconds();
		return (month + '/' + day + '/' + year + ' ' + hour + ':' + min + ':' + sec);
	}
};

var stripTime = function(dateStr) {
	if (typeof dateStr === 'string') {
		var result = /([\d\-\/]+).*/.exec(dateStr);
		if (result) {
			return result[1];
		}
	}
	return dateStr;
}

var isoPattern = /(\d{4})\-(\d{2})\-(\d{2}).*/;
var formatDateNoTime = function(dateStr) {
	// check for the ISO format, convert to American format so the GMT hour adjustment won't happen
	// new Date(2000-01-01) is Dec 31 1999 but new Date(01-01-2000) is Jan 1 2000
	var result = isoPattern.exec(dateStr);
	if (result) {
		return result[2] + '/' + result[3] + '/' + result[1];
	} else {
		return formatDate(stripTime(dateStr));
	}
}

//to yyyy-mm-dd whether str is in yyyy-mm-dd or mm/dd/yyyy
var ISOdate = function(str) {
	if (str) {
		return new Date(stripTime(str)).toISOString().substring(0, 10)
	} else {
		return '';
	}
};

// yyyy-mm-dd to mm/dd/yyyy
var ISOtoDateStr = function(iso) { // iso format is yyyy-mm-dd
	var year = iso.substring(0, 4);
	var month = iso.substring(5, 7);
	var day = iso.substring(8, 10);
	return month + '/' + day + '/' + year;

	return month + "&#47;" + day + "&#47;" + year;
};

var mmddyyPattern = /(\d{2})\/(\d{2})\/(\d{2}$)/;
// mm/dd/yy to mm/dd/20yy
var formatDateYYtoYYYY = function(date) {
	var result = mmddyyPattern.exec(date);
	if (result) {
		return result[1] + '/' + result[2] + '/20' + result[3];
	} else {
		return date;
	}
};

var formatDateMMDDYYYY = function (dateStr) {
	'use strict';
	var date = new Date(stripTime(dateStr));
	if (dateStr && !isNaN(date.getTime())) {
		var month = date.getMonth() + 1,
			day = date.getDate(),
			year = date.getFullYear();
		//year = year % 100;

		day = day < 10 ? '0' + day : day;
		month = month < 10 ? '0' + month : month;

		return (month + "/" + day + "/" + year);
	} else {
		return '';
	}
};

var formatDateLongForm = function (date) {
	var month = date.getMonth() + 1;
	var day = date.getDate();
	var year = date.getFullYear();

	day = day < 10 ? '0' + day : day;
	day = day;
	month = month < 10 ? '0' + month : month;

	var hours = date.getHours();
	var minutes = date.getMinutes();
	var seconds = date.getSeconds();
	var milli = date.getMilliseconds();

	hours = hours < 10 ? '0' + hours : hours;
	minutes = minutes < 10 ? '0' + minutes : minutes;
	seconds = seconds < 10 ? '0' + seconds : seconds;

	return year + "-" + month + "-" + day + 'T' + hours + ":" + minutes + ":" + seconds;
}

var produceDate = function (duration, date) {
	'use strict';
	var nonNullDate = date ? new Date(date) : new Date();
	return subtractFromDate(0, 0, duration, 0, 0, 0, 0, nonNullDate);
};

var subtractFromDate = function (month, day, year, hours, minutes, seconds, milliseconds, date) {
	'use strict';
	return formatDate(subtractFromDateUnformatted(month, day, year, hours, minutes, seconds, milliseconds, date));
};

var produceDateFromMonths = function (durationInMonths, date) {
	'use strict';
	var nonNullDate = date ? new Date(date) : new Date();
	return subtractFromDate(durationInMonths, 0, 0, 0, 0, 0, 0, nonNullDate);
};

var produceDateFromDays = function (durationInDays, date) {
	'use strict';
	var nonNullDate = date ? new Date(date) : new Date();
	return subtractFromDate(0, durationInDays, 0, 0, 0, 0, 0, nonNullDate);
};

var subtractFromDateUnformatted = function (month, day, year, hours, minutes, seconds, milliseconds, date) {
	'use strict';
	month = date.getMonth() - parseInt(month);
	day = date.getDate() - parseInt(day);
	year = date.getFullYear() - parseInt(year);
	hours = date.getHours() - parseInt(hours);
	minutes = date.getMinutes() - parseInt(minutes);
	seconds = date.getSeconds() - parseInt(seconds);
	milliseconds = date.getMilliseconds() - parseInt(milliseconds);

	return new Date(year, month, day, hours, minutes, seconds, milliseconds);
};

var getDayAge = function(dateStr) {
	var date = new Date(dateStr);
	date.setHours(0);
	date.setMinutes(0);
	date.setSeconds(0);

	var today = new Date();
	today.setHours(0);
	today.setMinutes(0);
	today.setSeconds(0);

	var milliDay = 1000 * 60 * 60 * 24; // a day in milliseconds;
	return Math.floor((today - date) / milliDay);
};

var getYearAge = function(dateStr) {
	return Math.floor(getDayAge(dateStr) / 365.25 );
};

var addCustomDates = function (dateFrom, dateTo, callbackFn) {
	'use strict';
	$('#add-custom').before('<li data-startDate="'+ dateFrom +'" data-endDate="'+ dateTo +'">'+ dateFrom + ' - ' + dateTo + '<div class="listing-bg"></div></li>')

	if ($.isFunction(callbackFn)) {
		callbackFn.call();
	}
};

var indexOf = function (array, searchElement) {
	'use strict';
	var size = array.length,
		i = 0;
	for (i; i < size; i++) {
		if (array[i] === searchElement) {
			return i;
		}
	}
	return -1;
};

var formatTime = function (date) {
	'use strict';
	var d = new Date(date);
	if (d=='Invalid Date') {
		return '';
	}
	var hh = d.getHours(),
		m = d.getMinutes(),
		s = d.getSeconds(),
		dd = "AM",
		h = hh;

	if (h >= 12) {
		h = hh - 12;
		dd = "PM";
	}

	h = h === 0 ? 12 : h;
	m = m < 10 ? "0" + m : m;
	s = s < 10 ? "0" + s : s;

	/* if you want 2 digit hours:
	h = h<10?"0"+h:h; */

	return h + ":" + m + " " + dd;
};

var formatDateToDays = function (date, shortenWeekday) {
	'use strict';
	var d = new Date(date);
	if (d=='Invalid Date') {
		return '';
	}

	var weekday = shortenWeekday ? ["Sun", "Mon", "Tue", "Wed", "Thur", "Fri", "Sat"] : ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
		month = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];

	return "<span><strong>" + weekday[d.getDay()] + "</strong></span>" + " <span>" + month[d.getMonth()] + ' ' + d.getDate() + ', ' + d.getFullYear() + '</span>';
};

var writeLocationsToStorage = function (key, value) {
	'use strict';
	if (value) {
		try {
			if (typeof(window.localStorage) !== 'undefined') {
				var currentArray = readLocationsFromStorage(key),
					currentArrayLength = currentArray.length,
					i = 0;

				if (currentArrayLength > 0) {
					for (i; i < currentArrayLength; i++) {
						if (!currentArray[i] || currentArray[i].name === value.name) {
							currentArray.splice(i, 1);
						}
					}
					if (currentArray.length >= 3) {
						currentArray.pop();
					}
				}
				currentArray.unshift(value);
				window.localStorage[key] = JSON.stringify(currentArray);
			}
		} catch (e) {
//            if (e == QUOTA_EXCEEDED_ERR) {
//                alert('Quota exceeded!');
//            }
		}
	}
};

var readLocationsFromStorage = function (key) {
	'use strict';
	var value = null;
	if (typeof(window.localStorage) !== 'undefined') {
		value = JSON.parse(window.localStorage[key] || null);
	}

	if (value) {
		var currentArrayLength = value.length;
		for (var i = 0; i < currentArrayLength; i ++) {
			if (!value[i]) {
				value.splice(i, 1);
			}
		}
	}

	return value ? value : [];
};

var getUriByTitle = function (atomLinks, title) {
	'use strict';
	var uri = null;
	$.each(atomLinks, function (index, atomLink) {
		if (atomLink.rel === 'related' && atomLink.title === title){
			uri = atomLink.href;
			return;
		}
	});

	return uri;
};

var flattenLinks = function (object) {
	'use strict';
	var links = object.get('link'),
		size = links.length,
		i = 0;
	for (i; i<size; i++) {
		object.set(links[i].title,links[i]);
	}
	object.unset('link');
};

var capitalizeFirstLetter = function (string) {
	'use strict';
	return string.charAt(0).toUpperCase() + string.slice(1);
};

var lowercaseFirstLetter = function(string) {
	'use strict';
	return string.charAt(0).toLowerCase() + string.slice(1);
};

// First letter of each word in string
var titleCase = function (str) {
	'use strict';
	return str.replace(/\w\S*/g, function (txt) {
		return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();
	});
};

var isWithinDateRange = function(startDate, endDate, checkDate) {
	if (typeof startDate === 'undefined' || typeof endDate === 'undefined') {
		return true;
	} else {
		var sDate = new Date(startDate);
		var eDate = new Date(endDate);
		var cDate = new Date(checkDate);
		if (sDate=='Invalid Date' || eDate=='Invalid Date' || cDate=='Invalid Date') {
			return '';
		}
		return cDate >= sDate && cDate <= eDate;
	}
};

var getAlertLevel = function(daysOnList) {
	if (daysOnList >= 38) {
		return 'C';
	} else if (daysOnList >= 37) {
		return 'B';
	} else if (daysOnList >= 35) {
		return 'A';
	} else if (daysOnList >= 8) {
		return 'C';
	} else if (daysOnList >= 7) {
		return 'B';
	} else if (daysOnList >= 5) {
		return 'A';
	} else {
		return '';
	}
	return '';
};

var validDate = function(date) {
	var regex = /\d{2}\/\d{2}\/\d{4}/;
	var date = date.trim();
	return regex.test(date);
};

var validDateRange = function(dateStr) {
	var days = getDayAge(dateStr);
	var date = new Date(dateStr);
	if (isNaN(days)) {
		return false;
	} else {
		return days >= 0 && days <= 365;
	}
};

// from yyyy-mm-dd to mm/dd/yyyy
var standardDateFromIsoDate = function(isoDateString) {

	var year = isoDateString.substring(0, 4);
	var month = isoDateString.substring(5, 7);
	var day = isoDateString.substring(8, 10);

	return month + "&#47;" + day + "&#47;" + year;
}

var isNumber = function(n) {
	return !isNaN(parseFloat(n)) && isFinite(n);
};

var isInteger = function(n) {
	return /^\d*$/.exec(n) ? true : false;
};

var isValidYear = function(str) {
	return isNumber(str) && str <= ((new Date()).getYear() + 1900);
};

var isValidUSZip = function(sZip) {
	if (sZip=='00000') {
		return false;
	}
	return /^\d{5}(-\d{4})?$/.test(sZip);
};

var isValidUSZipNoExtension = function(sZip) {
	if (sZip=='00000') {
		return false;
	}
	return /^\d{5}$/.test(sZip);
};

var isBlank = function(val) {
	if (val===0) return false;		// number 0 is not blank
	if (!val) return true;			// includes null, undefined and ''
	if ($.isArray(val)) {
		return val.length==0;
	}
	return false;
}

var displaySSN = function(ssn) {
	return ssn.substring(0,3) + '-' + ssn.substring(3,5) + '-' + ssn.substring(5,9);
}

var genderLetter = function(gender) {
	if (gender.toLowerCase()==='female') {
		return 'F';
	} else if (gender.toLowerCase()==='male') {
		return 'M';
	} else {
		return gender;
	}
}

var valNotBlank = function(val,def) {
	if (val.length==0) { return def; }
	return val;
}

var removeKey = function(keyToRemove,obj) {
	if (typeof obj === 'string' || typeof obj === 'number') {
		return;
	} else {
		for (var key in obj) {
			if (obj[key][keyToRemove]) {
				obj[key] = '';
			} else {
				removeKey(keyToRemove,obj[key]);
			}
		}
	}
}

var last4Pattern = /^[a-z,A-Z]\d{4}$/;
var ssnPattern = /^\d{9}$/;
var idPattern = /^\d{1,8}$/;
var namePattern = /^[a-z,A-Z]+$/;

var SEARCH_TYPE_SSN		= 'searchVeteranSSN';
var SEARCH_TYPE_LAST4	= 'searchVeteranLastFour';
var SEARCH_TYPE_NAME	= 'searchVeteranName';
var SEARCH_TYPE_ID		= 'searchVeteranID';
var SEARCH_TYPE_UNKNOWN = 'searchVeteranUnknown';

var homesSearchType = function(query) {
	if (last4Pattern.exec(query)) {
		return SEARCH_TYPE_LAST4;
	} else if (ssnPattern.exec(query)) {
		return SEARCH_TYPE_SSN;
	} else if (idPattern.exec(query)) {
		return SEARCH_TYPE_ID;
	} else if (namePattern.exec(query)) {
		return SEARCH_TYPE_NAME;
	} else {
		return SEARCH_TYPE_UNKNOWN;
	}
}

var spliceArray = function(arr,condition) {
	if (!arr || !condition) {
		return;
	}
	var more = true;
	do {
		var more = false;
		indexToRemove = -1;
		for (var i=0; i<arr.length; i++) {
			if (condition(arr[i])) {
				indexToRemove = i;
				break;
			}
		}
		if (indexToRemove >= 0) {
			arr.splice(indexToRemove,1);
			more = true;
		}
	} while (more);
}

var different = function(a,b) {
	if ($.isArray(a) && $.isArray(b)) {
		if (a.length != b.length) return true;
		for (var i in a) {
			if (a[i] != b[i]) return true;
		}
		return false;
	}
	if (!$.isArray(a) && !$.isArray(b)) {
		return a != b;
	}
	return true;
}

var convertHTML = function(text) {
	var str = text
		.replace(/<ul>/g,		' startList ')
		.replace(/<\/ul>/g,		' endList ')
		.replace(/<li>/g,		' startItem ')
		.replace(/<\/li>/g,		' endItem ')
		.replace(/<br>/g,		' break ')
		.replace(/<b>/g,		' setBold ')
		.replace(/<\/b>/g,		' unsetBold ')

	return str.split(' ');
}

var compareVetByName = function(a, b) {
//	console.log('compareVetByName ')
	var lastNameA = a.veteranJSON['lastName'].toLowerCase(),
	lastNameB = b.veteranJSON['lastName'].toLowerCase();

	if(lastNameA < lastNameB) return -1;
	if(lastNameA > lastNameB) return 1;

	var firstNameA = a.veteranJSON['firstName'].toLowerCase(),
	firstNameB = b.veteranJSON['firstName'].toLowerCase();
	if(firstNameA < firstNameB) return -1;
	if(firstNameA > firstNameB) return 1;
	return 0;
}

var compareVetByDate = function(a, b) {
//	console.log('compareVetByDate ')
	var creationDateA = new Date(a.veteranJSON['creationDate']),
	creationDateB = new Date(b.veteranJSON['creationDate']);

//	console.log('creationDateA ' + creationDateA + ', creationDateB ' + creationDateB);
	if(creationDateA < creationDateB) return 1;
	if(creationDateA > creationDateB) return -1;

	return 0;
}

//var moveScreen = function(amount) {
//	console.log('KEYBOARD moveScreen ' + amount);
//	$('#header').offset({top: amount});
//	$('#main-content').offset({top: amount + $('#header').height()});
//}

//var getScreenOffset = function(offsets,type,id) {
//	if (id) {
//		var offset = offsets[id];
//		if (offset) {
//			var val = offset[type];
//			return val ? val : 0;
//		}
//	}
//	return 0;
//}

