define(
	[ 'Global', 'App', 'HomesLib', 'HomesSoap',
	'Text!../html/templates/panes/homes/config.html'
	],
	function(Global, App, HomesLib, HomesSoap,
			template) {
		'use strict';
		var view,soapResult,soapCount,parms;
		var ConfigView = Backbone.View.extend({
			id:				'config',
			tagName:		'div',
			attributes : {
				'class':	'config'
			},
			initialize : function(metaData) {
				view = this;
				view.template = _.template(template);
				view.metaData = metaData ? metaData : {};
				Global.getUtilities().checkConnection();
			},
			render : function() {
				view.$el.empty();
				view.metaData.yearsOfHistory = App.getState().yearsOfHistory;
				view.metaData.secBetweenNetworkPings = '0';
				view.$el.append(view.template(view.metaData))
				var $container = $('#main').find(":jqmData(role='content')");
				$container.empty().append(view.$el).trigger('create');
				if (App.getState().doingSoapTest) {
					view.disableSoapTest();
				}
				return view;
			},
			events: {
				'click #submit-btn':		'submit',
				'click #soap-test-btn':		'soap'
			},
			submit:	function(e) {
				e.preventDefault();
				e.stopPropagation();
				$.each($('#configForm input').serializeArray(), function(_, kv) {
					if (kv.name==='years') {
						var years = kv.value;
						if (isInteger(years) && years > 0) {
							App.getState().yearsOfHistory = years;
							Global.getUtilities().writeToStorage('homes_yearsOfHistory',years);
							HomesLib.loadVetInfo(view.loadVetCallback);
						} else {
							Global.getUtilities().eventAggregator.trigger('message', 'historyYearsValidation', years + ' is invalid');
						}
					}
				});
			},
			loadVetCallback: function() {
				console.log('loadVetCallback');
				Global.getUtilities().eventAggregator.trigger('message', 'configSubmitSuccess');
			},
			soap: function(e) {
				e.preventDefault();
				e.stopPropagation();
				soapResult = '';
				soapCount = 0;
				$('#soap-results').text(soapResult);
				parms = {};
				$.each($('#soapTestForm input').serializeArray(), function(_, kv) {
					parms[kv.name] = kv.value;
				});
				App.getState().doingSoapTest = true;
				view.disableSoapTest();
				view.soapTest(parms);
			},
			disableSoapTest: function() {
				$('#secBetweenNetworkPings').attr('disabled','true');
				$('#soap-test-btn').addClass('hidden');
			},
			enableSoapTest: function() {
				$('#secBetweenNetworkPings').removeAttr('disabled');
				$('#soap-test-btn').removeClass('hidden');
			},
			soapTest: function(parms) {
				console.log('pinging homes');
				HomesSoap.ping(view.soapTestCallback);
			},
			soapTestCallback: function(obj) {
				console.log('pingCallback success=' + obj.success +
						', successData=' + JSON.stringify(obj.successData) +
						', errorData=' + JSON.stringify(obj.errorData));
				setTimeout(function() {
					soapCount++;
					if (obj.success) {
						soapResult += 'S #' + soapCount + ' ';
					} else {
						soapResult += 'F #' + soapCount;
						if (obj.errorData != 'undefined' && obj.errorData) {
							soapResult += ' (';
							if (obj.errorData.httpCode) {
								soapResult += 'httpCode=' + obj.errorData.httpCode;
							}
							if (obj.errorData.status) {
								soapResult += ' status=' + obj.errorData.status;
							}
							soapResult += ')';
						}
						soapResult += ' ';
					}
					if (soapCount < 30) {
						view.soapTest();
					} else {
						soapResult += 'Test Complete';
						App.getState().doingSoapTest = false;
						view.enableSoapTest();
					}
					$('#soap-results').text(soapResult);
					$('#secBetweenNetworkPings').val(parms['secBetweenNetworkPings']);
				},parms['secBetweenNetworkPings']*1000);
			},
		});
		return ConfigView;
	});
