define(
	[ 'Global', 'App', 'CordovaLib', 'HomesLib', 'HomesVet',
	'Templates', 'FormRules', 'DateView',
	'Responses', 'Response', 'SideBar', 'MainView',
	'QuestionsData', 'FormData',
	'Text!../html/templates/panes/homes/assessmentList.html',
	'Text!../html/templates/panes/homes/assessmentListEmpty.html',
	'Text!../html/templates/panes/homes/assessmentListItem.html'
	],
	function(Global, App, CordovaLib, HomesLib, HomesVet,
			Templates, FormRules, DateView,
			Responses, Response, SideBar, MainView,
			QuestionsData, FormData,
			template, templateEmpty, templateItem) {
		'use strict';
		var view;
		var HDSView = Backbone.View.extend({
			id:				'hds-assessment',
			tagName:		'div',
			attributes : {
				'class':	'homes-assessment'
			},
			initialize : function(metaData) {
				view = this;

				view.questionsData = new QuestionsData({form: 'HDS'});
				view.formData = new FormData({
					possibleAnswers:	null,
					timeStamp:			0
				});
				view.formData.set('possibleAnswers', view.questionsData.buildPossibleAnswers());

				view.template = _.template(template);
				view.templateEmpty = _.template(templateEmpty);
				view.templateItem = _.template(templateItem);
				view.metaData = metaData ? metaData : {};
			},
			render : function() {
				HomesLib.readDecision(function(success) {
					if (success) {
						view.questionsData.chooseSection();
						view.$el.empty();
						view.metaData.title = view.questionsData.get('section').title;
						view.metaData.assessmentDate = formatDateNoTime(Responses.instance().getAnswer('txtAssessmentDate'));
						view.$el.append(view.template(view.metaData))
						var $container = $('#main').find(":jqmData(role='content')");
						$container.empty().append(view.$el).trigger('create');
						view.buildList();
						SideBar.modifyButtons();
					} else {
						view.$el.empty();
						view.metaData.title = 'Cannot Edit/Create Decision Support Form';
						view.$el.append(view.templateEmpty(view.metaData))
						var $container = $('#main').find(":jqmData(role='content')");
						$container.empty().append(view.$el).trigger('create');
						SideBar.modifyButtons();
					}
				});
				return view;
			},
			events: {
				'tap a':							'showQuestion',
				'tap #submit-btn':					'submit',
				'click #submit-btn':				'submit',
				'blur input[type="text"]':			'blur',
				'click .accept-btn':				'accept',
				"tap #txtAssessmentDate":			'pickDate',
			},
			showQuestion: function(e) {
				setTimeout(function() {
					$('div.ui-popup-active').each(function() {
						var id = $(this).attr('id');
						var result = /(.*?)-listbox-popup$/.exec(id);
						if (result) {
							id = result[1] + '-question';
							var newText = $('#' + id).html();
							if (newText.length > 0) {
								$(this).find('.ui-title').html(newText);
							}
						}
					});
				}, 500);
			},
			pickDate : function(e) {
				e.preventDefault();
				e.stopPropagation();

				var applyDate = function (date) {
					console.log('applyDate date=' + date);
					if (typeof date === 'string') {
						$('#txtAssessmentDate-fieldcontain .ui-btn-text').html(formatDateNoTime(date));
						$('#txtAssessmentDate').val(formatDateNoTime(date));
						view.removeError('txtAssessmentDate');
						view.updateAnswers();
						view.buildList();
					}
				};

				new DateView($.extend(App.getMetaData()['dsDate'], {
					'date':			formatDateNoTime(Responses.instance().getAnswer('txtAssessmentDate')),
					'onApply':		applyDate,
					'yearsBack':	1,
					'caller':		$(e.currentTarget)
				})).render();
			},
			accept:	function(e) {
				e.preventDefault();
				e.stopPropagation();
				var $target = $(e.currentTarget);
				var vaId = $target.attr('data-vaid');
				Responses.instance().acceptAnswer(vaId);
				CordovaLib.saveDSFormLocally();
				view.buildList(vaId);
			},
			blur:	function(e) {
				var $target = $(e.currentTarget);
				var vaId = $target.attr('name');
				console.log('blur ' + vaId);
				Responses.instance().erasePrev(vaId);
				view.updateAnswers();
				var warning = FormRules.getValidationLogic().getWarning(vaId,HomesVet.getCurrentVet().toJSON());
				if (warning) {
					$target.parents(":jqmData(role='fieldcontain')").find('.warning').html(warning);
				}
			},
			selectWork: function(vaId) {
				console.log('selectWork');
				Responses.instance().erasePrev(vaId);
				view.updateAnswers();
				view.buildList(vaId);
			},
			handleSelect: function(e) {
				var id = $(e.currentTarget).attr('id');
				var result = /((?:ddlDss).*?)-.*/.exec(id);
				if (result) {
					var vaId = result[1];
					view.selectWork(vaId);
					if (vaId==='ddlDssQ5' && $('#ddlDssQ5').val()=='7') {  // 7 is h
						Global.getUtilities().eventAggregator.trigger('message', 'dssQ5h');
					}
				}
			},
			pagehide: function(e) {
//				console.log('pagehide on currentTarget=' + $(e.currentTarget).attr('id')
//						+ ', target=' + $(e.target).attr('id')
//						+ ', timestamp=' + e.timeStamp);
				var id = $(e.target).attr('id');
				var result = /((?:ddlDss).*?)-dialog$/.exec(id);
				if (result) {
					if (e.timeStamp != view.formData.get('timeStamp')) {  // we get a bunch of of these with the same timeStamp
						view.formData.set('timeStamp',e.timeStamp)
						view.selectWork(result[1]);
					}
				}
			},
			buildList:	function(moveToId) {
				view.$el.find('ul').empty();

				var questions = view.questionsData.get('questions');
				for (var i in questions) {
					var question,userAnswer,classes,warning;
					question = questions[i];
					userAnswer = Responses.instance().getAnswer(question.vaId)

					classes = '';
					if (FormRules.getDisplayLogic().isSubQuestion(question)) {
						classes += ' indent';
					}
					if (!FormRules.getDisplayLogic().isQuestionDisplayed(question)) {
						classes += ' hidden';
					}
					warning = FormRules.getValidationLogic().getWarning(question.vaId,HomesVet.getCurrentVet().toJSON());
					var parenthetical = '';
					if (question.vaId=='ddlDssQ1') {
						parenthetical = '(Note: A "safe place" to stay includes a shelter, a hotel/motel, staying with friends, in an facility such a hospital, rehab center, drug treatment center, prison/jail or nursing home, in non-transitional temporary living accommodations or their own apartment or house.)';
					}
					if (question.vaId=='ddlDssQ2') {
						parenthetical = '(Note: If adult is pregnant count as family (choices c/e))';
					}
					if (question.vaId=='ddlDssQ5') {
						parenthetical = '(Note: Days Homeless does not include nights staying with family and/or friends.)';
					}
					view.$el.find('ul').append(view.templateItem({
						question:		question.text,
						vaId:			question.vaId,
						id:				question.id,
						type:			question.type,
						answer:			userAnswer,
						prev:			'',
						display:		'',
						warning:		warning,
						classes:		classes,
						parenthetical:	parenthetical
					}));
				}
				view.setAnswers();
				view.$el.find('.title').html(view.questionsData.get('section').title);
				view.setDisable();
				view.$el.find('ul').trigger('create');
				view.handleMultipleSelect();
				$('select').selectmenu('refresh');
				if (moveToId) {
					moveToId = view.findNextQuestion(moveToId);
					console.log('scroll to ' + moveToId);
					Global.getUtilities().scroll(moveToId);
				}
				$( "div[id$='-listbox-popup']" ).on('popupafterclose', function(event) {
					view.handleSelect(event);
				});
				view.$el.on('pagehide', function(event) {
					view.pagehide(event);
				});
			},
			findNextQuestion: function(vaId) {
				var questions = view.questionsData.get('questions');
				for (var i in questions) {
					var id = questions[i].vaId;
					if (vaId==='txtAssessmentDate') {
						return id;
					} else if (id===vaId) {
						var $collection = $('#' + id + '-fieldcontain')
							.parent('li')
							.nextAll('li:not(.hidden)')
							.find(':jqmData(role="fieldcontain")');
						if ($collection.length > 0) {
							var result =  /(.*)-.*/.exec($($collection[0]).attr('id'));
							if (result) {
								return result[1];
							}
						}
						return vaId;
					}
				}
				return vaId;
			},
			handleMultipleSelect:	function() {
				$('select[multiple]').siblings('a').find('span.ui-btn-text').html('Select All That Apply');
			},
			setAnswers: function() {
				var possibleAnswers = view.formData.get('possibleAnswers');
				var $formSelects = view.$el.find('select');
				$formSelects.each(function () {
					var $this = $(this);
					var selections = possibleAnswers[$this.attr('name')];
					if (selections) {
						for(var i=0; i<selections.length; i++) {
							$this
								.append($('<option>', { value : selections[i].value })
								.text(selections[i].text));
						}
					}
					var userAnswer = Responses.instance().getAnswer($this.attr('name'));
					if ($.isArray(userAnswer)) {
						for(var i=0; i<userAnswer.length; i++) {
							$this.find('option[value="' + userAnswer[i] + '"]').attr("selected", "selected");
						}
					} else {
						$this.find('option[value="' + userAnswer + '"]').attr("selected", "selected");
					}
				});
				var $inputs = view.$el.find('input');
				$inputs.each(function () {
					var $this = $(this);
					var userAnswer = Responses.instance().getAnswer($this.attr('name'));
					// HMLS-807 says don't prepopulate the assessment date
//					if ($this.attr('name')==='txtAssessmentDate' && !userAnswer) {
//						userAnswer = formatDate(new Date());
//						Responses.instance().setOrCreate('txtAssessmentDate',userAnswer,null)
//					}
					$this.val(userAnswer);
				});
			},
			compareAnswers: function(a,b) {
				if (typeof a === 'string') {
					a = a.toLowerCase();
				}
				if (typeof b === 'string') {
					b = b.toLowerCase();
				}
				return a == b;
			},
			setDisable: function() {
				if (view.questionsData.get('sectionHash').hasPrev()) {
					view.$el.find('#prev-btn').removeClass('ui-disabled');
				} else {
					view.$el.find('#prev-btn').addClass('ui-disabled');
				}

				if (view.questionsData.get('sectionHash').hasNext()) {
					view.$el.find('#next-btn').removeClass('ui-disabled');
					view.$el.find('#submit-btn').addClass('ui-disabled');
				} else {
					view.$el.find('#next-btn').addClass('ui-disabled');
					view.$el.find('#submit-btn').removeClass('ui-disabled');
				}
			},
			submit:	function(e) {
				e.preventDefault();
				e.stopPropagation();

				view.updateAnswers();
				var valid = this.validate();
				if (valid) {
					// backend needs the misspelled date, so create it here
					var newFormatDate = formatDateMMDDYYYY(Responses.instance().getAnswer('txtAssessmentDate'));
					Responses.instance().setOrCreate('txtAssesmentDate',newFormatDate,null); // notice the missing 's'
					view.questionsData.get('section').done = true;
					App.getState().completeForm = true;
					var keepList = ['txtAssesmentDate'];
					view.questionsData.removeUnusedAnswers(keepList);
					Global.getUtilities().eventAggregator.invoke('recommend');
                    if (App.getState().isHomesOffline()) {
                        Global.getUtilities().eventAggregator.trigger('message', 'cantSubmitDS');
                    }
                    MainView.refresh();
                }
			},
			parse: function(response) {
				return response;
			},
			removeError: function(vaId) {
				var field = $('#' + vaId).closest(':jqmData(role="fieldcontain")');
				field.removeClass('flagged');
				field.find('.error').html('');
			},
			addError: function(vaId,msg) {
				var field = $('#' + vaId).closest(':jqmData(role="fieldcontain")');
				field.addClass('flagged');
				field.find('.error').html(msg);
			},
			updateAnswers: function() {
				Responses.instance().setFieldNoPrevChange('txtAssessmentDate', $('#txtAssessmentDate').val());
				$.each($('#questionForm input').serializeArray(), function(_, kv) {
					if (kv.value === 'n') kv.value = 'N';
					Responses.instance().setFieldNoPrevChange(kv.name, kv.value)
				});
				$('#questionForm select').each(function() {
					Responses.instance().setFieldNoPrevChange($(this).attr('name'), $(this).val());
//					view.removeDone($(this).attr('name'));
				});
				CordovaLib.saveDSFormLocally();
			},
			validate: function() {
				var idsToValidate = view.questionsData.getSectionIds();
				for(var i in idsToValidate) {
					view.removeError(idsToValidate[i]);
//					var field = $('#' + idsToValidate[i]).closest(':jqmData(role="fieldcontain")');
//					field.removeClass('flagged');
//					field.find('.error').html('');
				}
				var errs = FormRules.getValidationLogic().validate(idsToValidate, HomesVet.getCurrentVet().toJSON());
				if (errs && errs.length > 0) {
					var firstId;
					for (var i in errs) {
						var err = errs[i];
						if (err.elementId) {
							if (typeof err.elementId === 'string') {
								if (!firstId) firstId = err.elementId;
								view.addError(err.elementId,err.msgText);
//								var field = $('#' + err.elementId).closest(':jqmData(role="fieldcontain")');
//								field.addClass('flagged');
//								field.find('.error').html(err.msgText);
							}
						}
					}
					if (firstId) {
						// for some reason scroll is not working here, so lets do focus instead
						console.log('scroll to ' + firstId);
						view.scrollTo(firstId,false);

						// the scrollTo function does not leave this field in view every time
						if (firstId == 'txtAssessmentDate') {
							Global.getUtilities().putToTop();
						}
					}
					return false;
				} else {
					return true;
				}
			},
			scrollTo: function(moveToId,moveToNext) {
				if (moveToId) {
					if (moveToNext) {
						moveToId = view.findNextQuestion(moveToId);
					}
					console.log('KEYBOARD scrollTo calling scroll');
					Global.getUtilities().scroll(moveToId);
				}
			},
			messageCallBack: function() {
				if (App.getState().okMessagePressed) {
					Global.getUtilities().eventAggregator.invoke('homesVeteranSummary');
				}
			}
		});
		return HDSView;
	});
