define([
	'Global', 'App', 'Responses', 'HomesVet',
	'HomesUser', 'Forms', 'HomesLib', 'HomesSoap', 'MainView',
	'Text!../html/templates/panes/homes/history.html',
	'Text!../html/templates/panes/homes/historyItem.html',
    'Text!../html/templates/panes/homes/historyEmptyItem.html'
	],
	function(Global, App, Responses, HomesVet,
		HomesUser, Forms, HomesLib, HomesSoap, MainView,
		template, itemTemplate, emptyTemplate) {
		'use strict';
		var view;
		var HistoryView = Backbone.View.extend({
			id: 'history',
			tagName:		'div',
			attributes : {
				'class':	'history'
			},
			initialize: function(metaData) {
				view = this;
				view.template = _.template(template);
				view.itemTemplate = _.template(itemTemplate);
                view.emptyTemplate = _.template(emptyTemplate);
				view.metaData = metaData ? metaData : {};
			},
			render: function() {
				view.$el.empty();
				view.$el.append(view.template(view.metaData));
				var $container = $('#main').find(":jqmData(role='content')");
				$container.empty().append(view.$el).trigger('create');
				if (HomesVet.getCurrentVet()) {
					HomesLib.loadVetInfo(
						function() {
							view.buildList();
							view.delegateEvents();
						}
					);
				}
				return view;
			},
			events: {
				'tap li':				'viewForm'
			},
			buildList:	function() {
				var forms = Forms.instance();
				if (forms.models.length > 0) {
					var index = 0;
					forms.models.forEach(function(model) {
						var data = model.toJSON();
						var classes = 'line-item';
						data.classes = classes;
						if (data.FormType==App.getConstants().ASSESSMENT_FORM) {
							view.$el.find('#assessmentList').append(view.itemTemplate(data))
							.children(':last').attr('data-index', index);
						} else if (data.FormType==App.getConstants().DS_FORM) {
							view.$el.find('#dsList').append(view.itemTemplate(data))
							.children(':last').attr('data-index', index);
						}
						model.index = index++;
					});


                    if (!view.$el.find('#assessmentList').children().length) {
                         view.$el.find('#assessmentList').append(view.emptyTemplate())
							.children(':last').attr('data-index', -1);
                    }
                    if (!view.$el.find('#dsList').children().length) {
                        view.$el.find('#dsList').append(view.emptyTemplate())
							.children(':last').attr('data-index', -1);
                    }

				} else {
                    view.$el.find('#assessmentList').append(view.emptyTemplate())
							.children(':last').attr('data-index', -1);
                    view.$el.find('#dsList').append(view.emptyTemplate())
							.children(':last').attr('data-index', -1);
                }
			},
			viewForm: function(e) {
				e.preventDefault();
				e.stopPropagation();
				var index = $(e.currentTarget).attr('data-index');
                if (index != -1) {
                    var forms = Forms.instance();
                    var form = null;
                    if (forms && forms.models.length > index) {
                        form = forms.models[index];
                    }
                    console.log('tapped index=' + index);
                    console.log('form=' + JSON.stringify(form));
                    if (form.get('FormType')===App.getConstants().DS_FORM) {
                        HomesSoap.getDS(form,function(obj) {
                            if (obj.success) {
                                var data = obj.successData;
                                data.FormType = App.getConstants().DS_FORM;
                                console.log('data=' + JSON.stringify(data));
                                view.updateFromVA(obj.successData);
                                App.getState().historyForm = data;
                                Global.getUtilities().eventAggregator.invoke('form');
                            } else {
                                if (App.getState().isHomesOffline()) {
                                    Global.getUtilities().reportError('HOMES is Offline. You will need to reconnect to HOMES to view Veteran History', obj.errorData);
                                    MainView.refresh();
                                } else {
                                    Global.getUtilities().reportError('HOMES error',obj.errorData);
                                }
                                MainView.refresh();
                            }
                        });
                    } else if (form.get('FormType')===App.getConstants().ASSESSMENT_FORM) {
                        HomesSoap.getAssessment(form,function(obj) {
                            if (obj.success) {
                                var data = obj.successData;
                                data.FormType = App.getConstants().ASSESSMENT_FORM;
                                console.log('data=' + JSON.stringify(data));
                                view.updateFromVA(obj.successData);
                                App.getState().historyForm = data;
                                Global.getUtilities().eventAggregator.invoke('form');
                            } else {
                                if (App.getState().isHomesOffline()) {
                                    Global.getUtilities().reportError('HOMES is Offline. You will need to reconnect to HOMES to view Veteran History', obj.errorData);
                                    MainView.refresh();
                                } else {
                                    Global.getUtilities().reportError('HOMES error',obj.errorData);
                                }
                                MainView.refresh();
                            }
                        });
                    }
                }
			},
			updateFromVA: function(vaData) {
				console.log('updateFromVA ' + JSON.stringify(vaData));
				Responses.newInstance();
				for (var i in vaData.Responses.Response) {
					var vaId = vaData.Responses.Response[i].QuestionID;
					var answer = vaData.Responses.Response[i].ResponseValue;
					Responses.instance().fromVAforHistory(vaId,answer);
				}
			}
		});
		return HistoryView;
	}
);
