define(
	[ 'Global', 'App', 'CordovaLib', 'HomesLib', 'HomesSoap',
	'Templates', 'FormRules', 'DateView', 'HomesUser', 'HomesVet',
	'Responses', 'Response', 'SideBar', 'MainView',
	'QuestionsData', 'FormData',
	'Text!../html/templates/panes/homes/assessmentList.html',
	'Text!../html/templates/panes/homes/assessmentListEmpty.html',
	'Text!../html/templates/panes/homes/assessmentListItem.html'
	],
	function(Global, App, CordovaLib, HomesLib, HomesSoap,
			Templates, FormRules, DateView, HomesUser, HomesVet,
			Responses, Response, SideBar, MainView,
			QuestionsData, FormData,
			template, templateEmpty, templateItem) {
		'use strict';
		var view;
		var HomesAssessView = Backbone.View.extend({
			id:				'homes-assessment',
			tagName:		'div',
			attributes : {
				'class':	'homes-assessment'
			},
			initialize : function(metaData) {
				view = this;
				view.questionsData = new QuestionsData({form: 'HOMES'});
				view.formData = new FormData({
					matchingVeteran:	null,
					ready:				true,
					waitCount:			0,
					timeStamp:			0
				});
				view.template = _.template(template);
				view.templateEmpty = _.template(templateEmpty);
				view.templateItem = _.template(templateItem);
				view.metaData = metaData ? metaData : {};
//				timer = new (App.getUtilities().timer)();
			},
			render : function() {
				view.formData.set('ready',false);
				console.log('Assessment render');
				HomesLib.readAssessment(function(success) {
					if (success) {
						view.questionsData.chooseSection();
						view.$el.empty();
						view.metaData.title = view.questionsData.get('section').title;
						view.metaData.assessmentDate = formatDateNoTime(Responses.instance().getAnswer('txtAssessmentDate'));
						view.$el.append(view.template(view.metaData))
						var $container = $('#main').find(":jqmData(role='content')");
						$container.empty().append(view.$el).trigger('create');
						view.buildList();
						SideBar.modifyButtons();
                        MainView.refresh();
					} else {
						view.$el.empty();
						view.metaData.title = 'Cannot Edit/Create Assessment';
						view.$el.append(view.templateEmpty(view.metaData))
						var $container = $('#main').find(":jqmData(role='content')");
						$container.empty().append(view.$el).trigger('create');
						SideBar.modifyButtons();
                        MainView.refresh();
					}
					view.formData.set('ready',true);
				});
				return view;
			},
			events: {
				'tap a':							'showQuestion',
				'tap #prev-btn':					'prev',
				'click #prev-btn':					'prev',
				'tap #next-btn':					'next',
				'click #next-btn':					'next',
				'tap #submit-btn':					'submit',
				'click #submit-btn':				'submit',
				'tap .accept-btn':					'accept',
				'click .accept-btn':				'accept',
				"tap #txtAssessmentDate":			'pickDate',
				"click #txtAssessmentDate":			'pickDate',
				'blur input[type="text"]':			'blur'
			},
			showQuestion: function(e) {
				// this code simply paints a better title on the popup box
				setTimeout(function() {
					$('div.ui-popup-active').each(function() {
						var id = $(this).attr('id');
						var result = /(.*?)-listbox-popup$/.exec(id);
						if (result) {
							id = result[1] + '-question';
							var newText = $('#' + id).html();
							if (newText.length > 0) {
								$(this).find('.ui-title').html(newText);
							}
						}
					});
				}, 500);
			},
			pickDate : function(e) {
				e.preventDefault();
				e.stopPropagation();

				var applyDate = function (date) {
					console.log('applyDate date=' + date);
					if (typeof date === 'string') {
						$('#txtAssessmentDate-fieldcontain .ui-btn-text').html(formatDateNoTime(date));
						$('#txtAssessmentDate').val(formatDateNoTime(date));
						view.updateAnswers();
						view.buildList();
					}
				};

				new DateView($.extend(App.getMetaData()['assessDate'], {
					'date':			formatDateNoTime(Responses.instance().getAnswer('txtAssessmentDate')),
					'onApply':		applyDate,
					'yearsBack':	1,
					'caller':		$(e.currentTarget)
				})).render();
			},
			accept:	function(e) {
				e.preventDefault();
				e.stopPropagation();
				view.wait(function() {
					var $target = $(e.currentTarget);
					var vaId = $target.attr('data-vaid');
					Responses.instance().acceptAnswer(vaId);
					CordovaLib.saveAssessmentLocally()
					view.delayedUpdate(vaId,true);
				});
			},
			blur:	function(e) {
				var $target = $(e.currentTarget);
				var vaId = $target.attr('name');
				console.log('EVENT blur ' + vaId);
                if ($target.val() != '') {
                    Responses.instance().erasePrev(vaId);
                    $('#' + vaId + '-accept')
                        .addClass('hidden')
                        .parents('li').eq(0).removeClass('typeIV');
                }
				view.updateAnswers();
				view.updateDOM();
				var warning = FormRules.getValidationLogic().getWarning(vaId,HomesVet.getCurrentVet().toJSON());
				if (warning) {
					$target.parents(":jqmData(role='fieldcontain')").find('.warning').html(warning);
				}
			},
			selectWork: function(vaId) {
				console.log('selectWork');
				view.wait(function() {
					Responses.instance().erasePrev(vaId);
					view.delayedUpdate(vaId,false);
					view.formData.set('ready',true);
				});
			},
			handleSelect: function(e) {
				var id = $(e.currentTarget).attr('id');
				var result = /((?:txtHSA|ddlHSA).*?)-.*/.exec(id);
				if (result) {
					view.selectWork(result[1]);
				}
			},
			pagehide: function(e) {
//				console.log('pagehide on currentTarget=' + $(e.currentTarget).attr('id')
//						+ ', id=' + $(e.target).attr('id')
//						+ ', timestamp=' + e.timeStamp);
				var id = $(e.target).attr('id');
				var result = /((?:txtHSA|ddlHSA).*?)-dialog$/.exec(id);
				if (result) {
					if (e.timeStamp != view.formData.get('timeStamp')) {  // we get a bunch of of these with the same timeStamp
//						console.log('new pagehide');
						view.formData.set('timeStamp',e.timeStamp)
						view.selectWork(result[1]);
					}
				}
			},
			pageshow: function(e) {
//				console.log('pageshow on currentTarget id=' + $(e.currentTarget).attr('id')
//						+ ', target id=' + $(e.target).attr('id')
//						+ ', timestamp=' + e.timeStamp);
				var id = $(e.target).attr('id');
				var result = /((?:txtHSA|ddlHSA).*?)-dialog$/.exec(id);
				if (result) {
					if (e.timeStamp != view.formData.get('timeStamp')) {  // we get a bunch of of these with the same timeStamp
//						console.log('new pageshow');
						view.formData.set('timeStamp',e.timeStamp)
						$('#' + id + ' a[title="Close"]').on('tap click', function(e) {
							$('#' + id).dialog('close');
						});
					}
				}
			},
			delayedUpdate: function(vaId,fromAccept) {
				setTimeout(function() {
					if (!fromAccept) view.updateAnswers();
					view.updateList(vaId,fromAccept);
					view.formData.set('ready',true);
				},50);
			},
			buildList:	function(moveToId) {
//				var localTimer = new (App.getUtilities().timer)();
//				localTimer.startWaiting();
				console.log('Assessment buildList');
                MainView.refresh();
                if (App.getState().isHomesOffline() && view.questionsData.get('section').index == 8) {
                    Global.getUtilities().eventAggregator.trigger('message', 'cantSubmitAssessment');
                    console.log('SHOWING MESSAGE');
                }
				view.$el.find('ul').empty();

				var possibleAnswers = {};
				var reassess = view.questionsData.isReassess();
				var questions = view.questionsData.get('questions');

//				localTimer.markTime();
				for (var i in questions) {
					var question,userAnswer,classes,warning,prev,display,subText;
					question = questions[i];
					userAnswer = Responses.instance().getAnswer(question.vaId);

					classes = '';
					if (FormRules.getDisplayLogic().isSubQuestion(question)) {
						classes += ' indent';
					}
					if (!FormRules.getDisplayLogic().isQuestionDisplayed(question)) {
						classes += ' hidden';
					}
					warning = FormRules.getValidationLogic().getWarning(question.vaId,HomesVet.getCurrentVet().toJSON());
					possibleAnswers[question.vaId] = view.getPossibleAnswers(question)
					prev = '';
					display = '';
					var model = Responses.instance().get(question.vaId);
					if (model && reassess) {
						if (question.prevAllowed) {
							var prevAnswer = model.get('prev');
							if (isBlank(prevAnswer)) {
								if (isBlank(userAnswer)) {
									classes += ' ' + 'typeI';
									display = 1;
								}
							} else {
								if ($.isArray(prevAnswer)) {
									for (var i in prevAnswer) {
										if (i > 0) prev += ', ';
										prev += view.translateToText(prevAnswer[i],possibleAnswers[question.vaId]);
									}
								} else {
									prev = view.translateToText(prevAnswer,possibleAnswers[question.vaId]);
								}
								userAnswer = '';
								Responses.instance().eraseResponse(question.vaId);
								classes += ' ' + 'typeIV';
								display = 4;
							}
						} else {
							userAnswer = '';
							Responses.instance().eraseResponse(question.vaId);
							Responses.instance().erasePrev(question.vaId);
						}
					}
					view.$el.find('ul').append(view.templateItem({
						question:		question.text,
						vaId:			question.vaId,
						id:				question.id,
						type:			question.type,
						answer:			userAnswer,
						prev:			prev,
						display:		display,
						warning:		warning,
						classes:		classes,
						parenthetical:	''
					}));
				}
//				localTimer.writeTimeSinceMark('question loop ');

				view.buildOptions(possibleAnswers);
				view.setAnswers();
//				localTimer.writeTimeSinceMark('setAnswers ');

				view.$el.find('.title').html(view.questionsData.get('section').title);
//				localTimer.writeTimeSinceMark('title ');

				view.setDisable();
//				localTimer.writeTimeSinceMark('setDisable ');

//				localTimer.markTime();
				view.$el.find('ul').trigger('create');
//				localTimer.writeTimeSinceMark('trigger ');

				view.updateDOM();
//				localTimer.writeTimeSinceMark('updateDOM ');

				view.handleMultipleSelect();
				view.$el.find('.sectionCount').html(view.questionsData.calculateSectionCount());

//				localTimer.markTime();
				$('select').selectmenu('refresh');
//				localTimer.writeTimeSinceMark('selectmenu ');

				view.scrollTo(moveToId,true);
//				localTimer.writeTimeSinceStart('total time ');
				$("div[id$='-listbox-popup']").on('popupafterclose', function(event) {
					view.handleSelect(event);
				});
				$("*").on('pageshow', function(event) {
					view.pageshow(event);
				});
				$("*").on('pagehide', function(event) {
					view.pagehide(event);
				});
				Global.getUtilities().setupKeyEvents();

				setTimeout(function(){
					$.mobile.loading('hide');
				}, 1);
			},
			updateList:	function(vaId,fromAccept) {
				console.log('Assessment updateList');
                MainView.refresh();

				// erase error and accept markings on element that changed
				var field = $('#' + vaId).closest(':jqmData(role="fieldcontain")');
				field.removeClass('flagged');
				field.find('.error').html('');

				$('#' + vaId + '-accept')
					.addClass('hidden')
					.parents('li').eq(0).removeClass('typeIV');

				var possibleAnswers = {};
				var reassess = view.questionsData.isReassess();
				var questions = view.questionsData.get('questions');

				for (var i in questions) {
					var question,userAnswer,subText;
					question = questions[i];
					userAnswer = Responses.instance().getAnswer(question.vaId);

					// hide or show question
					$('#' + question.vaId + '-li').removeClass('hidden');
					if (!FormRules.getDisplayLogic().isQuestionDisplayed(question)) {
						$('#' + question.vaId + '-li').addClass('hidden');
					}

					// update warning
					var warning = FormRules.getValidationLogic().getWarning(question.vaId,HomesVet.getCurrentVet().toJSON());
					$('#' + question.vaId + '-warning').html(warning);
					possibleAnswers[question.vaId] = view.getPossibleAnswers(question)

					// hide or show previous question
					var model = Responses.instance().get(question.vaId);
					if (model && reassess) {
						var prevAnswer = model.get('prev');
						$('#' + question.vaId + '-accept').removeClass('hidden');
						if (isBlank(prevAnswer)) {
							$('#' + question.vaId + '-accept').addClass('hidden');
						}
					}
				}
				if(fromAccept) {
					view.setAnswersOnDOM(vaId,possibleAnswers);
				}
				view.setAnswers();
				view.$el.find('.title').html(view.questionsData.get('section').title);
				view.setDisable();
				view.updateDOM();
				view.$el.find('.sectionCount').html(view.questionsData.calculateSectionCount());
				view.scrollTo(vaId,true);

				setTimeout(function(){
					$.mobile.loading('hide');
				}, 1);
			},
			scrollTo: function(moveToId,moveToNext) {
				if (moveToId) {
					if (moveToNext) {
						moveToId = view.findNextQuestion(moveToId);
					}
					console.log('KEYBOARD scrollTo calling scroll');
					Global.getUtilities().scroll(moveToId);
				}
			},
			findNextQuestion: function(vaId) {
				var questions = view.questionsData.get('questions');
				for (var i in questions) {
					var id = questions[i].vaId;
					if (vaId==='txtAssessmentDate') {
						return id;
					} else if (id===vaId) {
						var $collection = $('#' + id + '-fieldcontain')
							.parent('li')
							.nextAll('li:not(.hidden)')
							.find(':jqmData(role="fieldcontain")');
						if ($collection.length > 0) {
							var result =  /(.*)-.*/.exec($($collection[0]).attr('id'));
							if (result) {
								return result[1];
							}
						}
						return vaId;
					}
				}
				return vaId;
			},
			handleMultipleSelect:	function() {
				$('select[multiple]').siblings('a').find('span.ui-btn-text').html('Select All That Apply');
			},
			buildOptions: function(possibleAnswers) {
				var $formSelects = view.$el.find('select');
				$formSelects.each(function () {
					var $this = $(this);
					var selections = possibleAnswers[$this.attr('name')];
					if (selections) {
						for(var i=0; i<selections.length; i++) {
							$this
								.append($('<option>', { value : selections[i].value })
								.text(selections[i].text));
						}
					}
				});
			},
			setAnswers: function() {
				var $formSelects = view.$el.find('select');
				$formSelects.each(function () {
					var $this = $(this);
					var userAnswer = Responses.instance().getAnswer($this.attr('name'));
					if ($.isArray(userAnswer)) {
						for(var i=0; i<userAnswer.length; i++) {
							$this.find('option[value="' + userAnswer[i] + '"]').attr("selected", "selected");
						}
					} else {
						$this.find('option[value="' + userAnswer + '"]').attr("selected", "selected");
					}
				});
				var $inputs = view.$el.find('input');
				$inputs.each(function () {
					var $this = $(this);
					var userAnswer = Responses.instance().getAnswer($this.attr('name'));
					// HMLS-807 says don't prepopulate the assessment date
//					if ($this.attr('name')==='txtAssessmentDate' && !userAnswer) {
//						userAnswer = formatDate(new Date());
//						Responses.instance().setOrCreate('txtAssessmentDate',userAnswer,null)
//					}
					$this.val(userAnswer);
				});
			},
			setAnswersOnDOM: function(vaId,possibleAnswers) {
				var isIn = function(val,answer,vaId) {
					if ($.isArray(answer)) {
						for (var i in answer) {
							if (val == view.translateToText(answer[i],possibleAnswers[vaId])) return true;
						}
					} else {
						if (val == view.translateToText(answer,possibleAnswers[vaId])) return true;
					}
				};
				var getBtnText = function(answer,vaId) {
					var btnText = '';
					if ($.isArray(answer)) {
						for (var i in answer) {
							if (i > 0) btnText += ', ';
							btnText += view.translateToText(answer[i],possibleAnswers[vaId])
						}
					} else {
						btnText = view.translateToText(answer,possibleAnswers[vaId])
					}
					return btnText;
				};
				var $formSelects = view.$el.find('select');
				$formSelects.each(function () {
					var $this = $(this);
					var id = $this.attr('id');
					if (id==vaId) {
						var userAnswer = Responses.instance().getAnswer(vaId);
						$('#' + vaId).siblings('a').find('span.ui-btn-text').html(getBtnText(userAnswer,vaId));
						$('#' + vaId + '-menu li').each(function() {
							$(this).removeClass('ui-btn-active');
							var val = $(this).find('a').html();
							if (isIn(val,userAnswer,vaId)) {
								$(this).addClass('ui-btn-active');
							}
						});
					}
				});
				var $inputs = view.$el.find('input');
				$inputs.each(function () {
					var $this = $(this);
					var id = $this.attr('id');
					if (id==vaId) {
						var userAnswer = Responses.instance().getAnswer(vaId);
						$this.val(userAnswer);
					}
				});
			},
			translateToText: function(val,possibleAnswers) {
				for (var i in possibleAnswers) {
					if (view.compareAnswers(possibleAnswers[i].value,val)) {
						return possibleAnswers[i].text;
					}
				}
				return val;
			},
			compareAnswers: function(a,b) {
				if (typeof a === 'string') {
					a = a.toLowerCase();
				}
				if (typeof b === 'string') {
					b = b.toLowerCase();
				}
				return a == b;
			},
			setDisable: function() {
				if (view.questionsData.get('sectionHash').hasPrev()) {
					view.$el.find('#prev-btn').removeClass('ui-disabled');
				} else {
					view.$el.find('#prev-btn').addClass('ui-disabled');
				}

				if (view.questionsData.get('sectionHash').hasNext()) {
					view.$el.find('#next-btn').removeClass('ui-disabled');
					view.$el.find('#submit-btn').addClass('ui-disabled');
				} else {
					view.$el.find('#next-btn').addClass('ui-disabled');
					if (App.getState().isHomesOnline()) {
						view.$el.find('#submit-btn').removeClass('ui-disabled');
					} else {
                        MainView.refresh();
						view.$el.find('#submit-btn').addClass('ui-disabled');
					}
				}
			},
			getPossibleAnswers: function(question) {
				var answerHash = view.questionsData.get('answerHash');
				var answers = FormRules.getDisplayLogic().getPossibleAnswers(answerHash, question.answerBase);
				if (question.vaId==='txtHSA_Q12a' || question.vaId==='txtHSA_Q12b') {
					var vet = HomesVet.getCurrentVet();
					if (vet) {
						var age = getYearAge();
						var date = new Date(vet.get('dateOfBirth'));
						var startYear = date.getFullYear() + 17;
						var newlist = [];
						for (var i in answers) {
							if (isNaN(answers[i].value) || (answers[i].value >= startYear)) {
								newlist.push(answers[i]);
							}
						}
						answers = newlist;
					}
				}
				return answers;
			},
			updateDOM: function() {
				var questions = view.questionsData.get('questions');
				for (var i in questions) {
					var question = questions[i];
					if (question.vaId) {
						if (question.vaId==='txtHSA_Q10') {
							var $target = $('#' + question.vaId);
							if ($target.val()==='N') {
								$('#txtHSA_Q10a').val('N');
							} else {
								var max = parseInt($target.val());
								$('#txtHSA_Q10a-menu li').each(function() {
									$(this).removeClass('hidden');
									var val = parseInt($(this).find('a').html());
									if (val > max) {
										$(this).addClass('hidden');
									}
								});
							}
							if ($target.val()==='0') {
								$('#txtHSA_Q10a').parents(":jqmData(role='fieldcontain')").addClass('hidden');
							} else {
								$('#txtHSA_Q10a').parents(":jqmData(role='fieldcontain')").removeClass('hidden');
							}
						} else if (question.vaId==='txtHSA_Q12a') {
							var $target = $('#' + question.vaId);
							var min = parseInt($target.val());
							if (isNumber(min)) {
								$('#txtHSA_Q12b-menu li').each(function() {
									$(this).removeClass('hidden');
									var val = parseInt($(this).find('a').html());
									if (val < min) {
										$(this).addClass('hidden');
									}
								});
							}
						} else if (question.vaId==='ddlHSA_Q15') {
							var max;
							var $target = $('#' + question.vaId);
							if ($target.val()==='1') {
								max = 9;
							} else if ($target.val()==='2') {
								max = 5;
							} else if ($target.val()==='3') {
								max = 10;
							}
							if (max) {
								$('#txtHSA_Q16-menu li').each(function() {
									$(this).removeClass('hidden');
									var val = parseInt($(this).find('a').html());
									if (val > max) {
										$(this).addClass('hidden');
									}
								});
							}
						} else if (question.vaId==='txtHSA_Q20_Total') {
//							timer.markTime();
							var sum = FormRules.getValidationLogic().getSum('Q20');
//							timer.writeTimeSinceMark('calculate time ');
							console.log('sum for txtHSA_Q20_Total ' + sum);
							$('label[for="txtHSA_Q20_Total"]').html(question.text + ' ' + sum);
							var letters = 'abcdefghijklmnopqrst';
							for (var i in letters) {
								var val = parseInt($('#txtHSA_Q20' + letters[i]).val());
								if (isNaN(val)) val = 0;
								$('#txtHSA_Q20' + letters[i] + '-menu li').each(function() {
									$(this).removeClass('hidden');
									var option = parseInt($(this).find('a').html());
									if (option > val && option+sum-val > 30) {
										$(this).addClass('hidden');
									}
								});
							}
//							timer.writeTimeSinceMark('GUI time ');
						} else if (question.vaId.indexOf('txtHSA_Q30_Total')==0) {
							var sum = FormRules.getValidationLogic().getSum('Q30');
							$('label[for="txtHSA_Q30_Total"]').html(question.text + ' ' + sum);
						} else if (question.vaId==='txtHSA_Q38') {
							var $target = $('#' + question.vaId);
							var max = parseInt($target.val());
							$('#txtHSA_Q39-menu li').each(function() {
								$(this).removeClass('hidden');
								var val = parseInt($(this).find('a').html());
								if (val > max) {
									$(this).addClass('hidden');
								}
							});
						}
					}
				}
			},
			prev: function(e) {
				e.preventDefault();
				e.stopPropagation();
				view.questionsData.prev();
				view.buildList();
				Global.getUtilities().putToTop();
			},
			next: function(e) {
				e.preventDefault();
				e.stopPropagation();
				view.updateAnswers();
				var valid = view.validate();
				if (valid) {
					view.questionsData.next();
					view.buildList();
					Global.getUtilities().putToTop();
				}
			},
			submit:	function(e) {
				if (e) {
					e.preventDefault();
					e.stopPropagation();
				}
				view.updateAnswers({submit: true});
				if (view.validate()) {
					if (App.getState().isHomesOffline()) {
                        Global.getUtilities().eventAggregator.trigger('message', 'cantSubmitAssessment');
                        MainView.refresh();
					} else {
						HomesSoap.checkVeteranHomesEpisodeStatus(view.episodeCallback);
					}
				}
			},
			episodeCallback: function(obj) { // continuation of the submit process
				if (obj.success) {
					var currentVeteran = HomesVet.getCurrentVet();
					if (currentVeteran.get('isHomesEpisodeOpen')) {
						// this should not happen anymore as HMLS-813 prevents us from entering this screen when there is an episode
						if (currentVeteran.get('homesEpisodeVISN') == HomesUser.instance().get('VISN')) {
							Global.getUtilities().eventAggregator.trigger('message', 'openEpisodeInProgressInVISN',
									'There is already an open HOMES Episode in your VISN ('
									+ currentVeteran.get('homesEpisodeVISN') + ') for this Veteran'
									, view);
						} else {
							Global.getUtilities().eventAggregator.trigger('message', 'openEpisodeInProgressOutsideVISN',
										['There is already an open HOMES Episode in VISN '
										+ currentVeteran.get('homesEpisodeVISN') + ' for this Veteran.',
										'If you wish to complete this Assessment, you will need to transfer the Veteran data to your VISN ('
										+ HomesUser.instance().get('VISN') + ').'
										], view);
						}
						MainView.refresh();
					} else {
						// backend needs the misspelled date, so create it here
						var newFormatDate = formatDateMMDDYYYY(Responses.instance().getAnswer('txtAssessmentDate'));
						Responses.instance().setOrCreate('txtAssesmentDate',newFormatDate,null); // notice the missing s

						view.questionsData.get('section').done = true;
						App.getState().completeForm = true;
						var keepList = ['txtAssesmentDate'];
						view.questionsData.removeUnusedAnswers(keepList);
						HomesLib.sendAssessment(FormRules.getSoapLogic().toVAStorage(), view.sendAssessmentCallback);
					}
				} else {
					if (App.getState().isHomesOffline()) {
                        Global.getUtilities().eventAggregator.trigger('message', 'cantSubmitAssessment');
					} else {
						Global.getUtilities().reportError('HOMES error', obj.errorData);
					}
					MainView.refresh();
				}
			},
			// this will remove blank answers irregardless of the existance of previous answers presents
            updateAnswers: function(options) {
				var forSubmit = typeof options != 'undefined' && typeof options.submit != 'undefined' && options.submit;

                if (forSubmit) {
                	Responses.instance().erasePrev('txtAssessmentDate');
                }
				Responses.instance().setFieldNoPrevChange('txtAssessmentDate', $('#txtAssessmentDate').val());

				$.each($('#questionForm input').serializeArray(), function(_, kv) {
					if (kv.value === 'n') kv.value = 'N';
                    if (forSubmit) {
                    	Responses.instance().erasePrev(kv.name);
                    }
					Responses.instance().setFieldNoPrevChange(kv.name, kv.value)
				});
				$('#questionForm select').each(function() {
					var val;
					if ($(this).attr('multiple')) {
						val = $(this).val();
					} else {
						// when we accept answers, for some reason the 'choose option' option is still selected
						val = $(this).find('option:selected:last').val();
					}
					Responses.instance().setFieldNoPrevChange($(this).attr('name'), val);
				});

				if (forSubmit) {
					var sum = FormRules.getValidationLogic().getSum('Q20');
					Responses.instance().setOrCreate('txtHSA_Q20_Total',sum,null)

					sum = FormRules.getValidationLogic().getSum('Q30');
					Responses.instance().setOrCreate('txtHSA_Q30_Total',sum,null)
				}

				CordovaLib.saveAssessmentLocally()
			},

			validate: function() {
				var idsToValidate = view.questionsData.getSectionIds();
				for(var i in idsToValidate) {
					var field = $('#' + idsToValidate[i]).closest(':jqmData(role="fieldcontain")');
					field.removeClass('flagged');
					field.find('.error').html('');
				}
				var errs = FormRules.getValidationLogic().validate(idsToValidate, HomesVet.getCurrentVet().toJSON());
				if (errs && errs.length > 0) {
					var firstId;
					for (var i in errs) {
						var err = errs[i];
						if (err.elementId) {
							if (typeof err.elementId === 'string') {
								if (!firstId) firstId = err.elementId;
								var field = $('#' + err.elementId).closest(':jqmData(role="fieldcontain")');
								field.addClass('flagged');
								field.find('.error').html(err.msgText);
							}
						}
					}
					if (firstId) {
						// for some reason scroll is not working here, so lets do focus instead
						console.log('scroll to ' + firstId);
						view.scrollTo(firstId,false);

						// the scrollTo function does not leave this field in view every time
						if (firstId == 'txtAssessmentDate') {
							Global.getUtilities().putToTop();
						}
					}
					return false;
				} else {
					return true;
				}
			},
			sendAssessmentCallback: function(obj) {
				if (obj.success) {
					console.log("Success sending" + obj.successData);
					Global.getUtilities().eventAggregator.trigger('message', 'successfullySubmittedAssessment', '', view);

					var queryParameters;
					var selectedVeteran = HomesVet.getCurrentVet();
					if (selectedVeteran.get('veteranID') && selectedVeteran.get('veteranLocalID')) {
						queryParameters = {
							'veteranHomesId': selectedVeteran.get('veteranID'),
							'veteranLocalID': selectedVeteran.get('veteranLocalID')
						};
					} else if (selectedVeteran.get('veteranID')) {
						queryParameters =  {'veteranHomesId': selectedVeteran.get('veteranID')};
					} else {
						queryParameters =  {'veteranLocalID': selectedVeteran.get('veteranLocalID')};
					}

					CordovaLib.deleteAssessment(queryParameters, null);
					HomesSoap.checkVeteranHomesEpisodeStatus(MainView.refresh);
				} else if (obj.matchFound) {
					view.formData.set('matchingVeteran',obj.homesVeteranMatch);
					Global.getUtilities().eventAggregator.trigger('message', 'submitMatchFoundForm', obj.dialogText, view);
				} else {
					console.log("Error sending: " + obj.errorData);
					if (App.getState().isHomesOffline()) {
                        Global.getUtilities().eventAggregator.trigger('message', 'cantSubmitAssessment');
					} else {
						Global.getUtilities().reportError('Could not save the Assessment Form', obj.errorData);
					}
					MainView.refresh();
				}
			},
			messageCallBack: function() {
				var matchingVeteran = view.formData.get('matchingVeteran');
				if (matchingVeteran) {
					if (App.getState().okMessagePressed) {
						var veteranLocalID = HomesVet.getCurrentVet().get('veteranLocalID');
						matchingVeteran.set('veteranLocalID', veteranLocalID);
						HomesVet.setCurrentVet(matchingVeteran);
						view.formData.set('matchingVeteran',null);
						MainView.refresh();
						view.submit();
						// HMLS-809 removed this functionality
//					} else if (App.getState().secondaryActionPressed) {
//						matchingVeteran = null;
//						Global.getUtilities().eventAggregator.trigger('message', 'manualFormSubmitReminder');
//						Global.getUtilities().eventAggregator.invoke('homesVeteranSummary');
                    } else if (App.getState().cancelMessagePressed) {
						view.formData.set('matchingVeteran',null);
                    }
                } else {
					if (App.getState().okMessagePressed) {
                        if (App.getState().isHomesOnline()) {
                            Global.getUtilities().eventAggregator.invoke('homesVeteranSummary');
                        }
					}
				}
			},
			wait: function(callback) {
				if (view.formData.get('ready')) {
					view.formData.set('waitCount', 50);
					callback();
				} else {
					var waitCount = view.formData.get('waitCount');
					if (waitCount > 0) {
						console.log('going to wait again ' + waitCount);
						waitCount--;
						view.formData.set('waitCount', waitCount);
						setTimeout(function() {
							view.wait(callback);
						}, 50);
					} else {
						console.log('TIMEOUT!');
						view.formData.set('ready',false);
						view.formData.set('waitCount', 50);
						callback();
					}
				}
			}
		});
		return HomesAssessView;
	});
