define([
	'Global', 'App',
	'HomesUser', 'HomesVet', 'HomesVets', 'HomesSoap', 'MainView', 'CordovaLib',
	'Text!../html/templates/panes/homes/homesList.html',
	'Text!../html/templates/panes/homes/homesListItem.html',
	'Text!../html/templates/panes/homes/homesListItem_empty.html'
	],
	function(Global, App,
		HomesUser, HomesVet, HomesVets, HomesSoap, MainView, CordovaLib,
		template, itemTemplate, emptyTemplate) {
		'use strict';
		var view;
		var tempVeteran;
		var HomesListView = Backbone.View.extend({
			id: 'homesListView',
			attributes : {
				'class': 'vetList'
			},
			initialize: function(metaData) {
//				console.log('HMLS-40 HomesListView initialize called');
				view = this;
				view.template = _.template(template);
				view.itemTemplate = _.template(itemTemplate);
				view.emptyTemplate = _.template(emptyTemplate);
				view.metaData = metaData ? metaData : {};
			},
			render : function() {
				App.getState().isLoaderShown = true;
				var $container = $('#main').find(":jqmData(role='content')");
				$container.empty();

				if (!HomesUser.instance().isLoggedIn()) {
					Global.getUtilities().eventAggregator.invoke('homesLogin');
					return view;
				}
				view.cleanModels();

				var homesVets = HomesVets.instance();
				view.metaData.showAdd = true;// HMLS-820 (homesVets.length == 0);
				view.metaData.titleWithCount = view.metaData.title + ' (' + homesVets.length + ')';

				view.computeView();
				view.delegateEvents();
				$container.append(view.$el).trigger('create');

				return view;
			},
			cleanModels: function() {
				var homesVets = HomesVets.instance();
				homesVets.models.forEach(function(model) {
					if (model.get('veteranID') === '') {
						homesVets.remove(model);
					}
				});
			},
			computeView: function() {
				var homesVets = HomesVets.instance();
				var listElements = [];
				if (homesVets.length > 0) {
					var index = 0;
					homesVets.models.forEach(function(model) {
						var data = model.toJSON();
						data.dateOfBirth = formatDateNoTime(model.get('dateOfBirth'));
						data.leftAgeData = view.leftAgeData(model);
						data.rightAgeData = view.rightAgeData(model);
						data.ssn = displaySSN(model.get('sSN'));
						data.homesID = model.get('veteranID');
						data.index = index;
						listElements.push(view.itemTemplate(data));
						model.index = index++;
					});
				} else {
					listElements.push(view.emptyTemplate());
				}

				view.$el.empty().append(view.template(view.metaData));

				// tweak DOM directly
//				console.log('HMLS-40 tweaking DOM')
				view.$el.find('ul').append(listElements);
				if (view.metaData.showAdd) {
					if (App.getState().isWorkingOffline()) {
						view.$("#add-btn").html("Add Veteran Offline");
					} else {
						view.$("#add-btn").html("Add Veteran to HOMES");
					}
				}
			},
			events: {
				'click #add-btn':								'add',
				'click [data-cell-right] #selectVet-btn':		'edit',
				'click [data-cell-left]':						'show'
			},
			add: function(e) {
				e.preventDefault();
				e.stopPropagation();
				HomesVet.setCurrentVet(null);
				Global.getUtilities().eventAggregator.invoke('homesVeteranSummary', {});
			},
			show: function(e) {
				e.preventDefault();
				e.stopPropagation();
				$(e.currentTarget).parents('li').find('[data-cell-right]').children().toggleClass('hidden');
			},
			edit: function(e) {
				e.preventDefault();
				e.stopPropagation();
				var index = $(e.currentTarget).parents('li').attr('data-index');

				var vets = HomesVets.instance();
				var vet = null;
				if (vets && vets.models.length > index) {
					vet = vets.models[index];
				}

				tempVeteran = vet;

				var queryParameters =  {'veteranHomesId': vet.get('veteranID')};

				CordovaLib.fetchOfflineJSON(queryParameters, function(obj) {

					var shouldContinue = true;
					if (obj.success) {
						if (obj.successData != '') {
							var veteranData = $.parseJSON(obj.successData.veteranJSON);

							if (veteranData.hasUnsubmittedChanges) {
								shouldContinue = false;
							}
						}
					}

					if (shouldContinue) {
						view.contineToSummary();
					} else {
						Global.getUtilities().eventAggregator.trigger('message', 'confirmOverwriteLocalChanges', '', view);
					}
				});
			},
			contineToSummary: function() {
				var vet = tempVeteran;
				HomesVet.setCurrentVet(vet);
				App.getState().showBackButton = true;

				if (vet) {
					HomesSoap.checkVeteranHomesEpisodeStatus(view.episodeCallback);
				}
			},
			episodeCallback: function(obj) {
				if (!obj.success) {
					if (App.getState().isHomesOffline()) {
						Global.getUtilities().reportError('HOMES is Offline', obj.errorData);
					} else {
						Global.getUtilities().reportError('HOMES error', obj.errorData);
					}
				}
				MainView.refresh();
				Global.getUtilities().eventAggregator.invoke('homesVeteranSummary');
			},
			leftAgeData: function(vet) { // same as method in headerView
				var slashDate = ISOtoDateStr(vet.get('dateOfBirth'))
				var age = getYearAge(slashDate);
				var text = slashDate + ' (' + age + ' yo) ' + genderLetter(vet.get('gender'));
				return text;
			},
			rightAgeData: function(vet) { // same as method in headerView
				var slashDate = ISOtoDateStr(vet.get('dateOfBirth'))
				var age = getYearAge(slashDate);
				var text = slashDate + ' Age ' + age;
				return text;
			},
			messageCallBack: function() {
				if (App.getState().okMessagePressed) {
					view.contineToSummary();
				}
			}
		});
		return HomesListView;
	}
);
