define([
	'Global', 'App', 'MainView', 'Security',
	'HomesUser', 'HomesSoap', 'Metric',
	'Text!../html/templates/navbars/homes/homesHeader.html',
	'Text!../html/templates/panes/homes/homesLogin.html'
	],
	function(Global, App, MainView, Security,
			HomesUser, HomesSoap, Metric,
			headerTemplate, loginTemplate) {
		'use strict';
		var view;
		var HomesLoginView = Backbone.View.extend({
			id: 'homesLoginView',
			initialize: function(metaData) {
				view = this;
				view.header = _.template(headerTemplate);
				view.login = _.template(loginTemplate);
				view.metaData = metaData ? metaData : {};
				$(view).ajaxSuccess(function(event, xhr, settings) {
					console.log('ajax VIEW success status=' + xhr.status);
				});
			},
			render: function() {
				if (HomesUser.instance().isLoggedIn()) {
					if (App.getState().isHomesOnline()) {
						Global.getUtilities().eventAggregator.invoke('blank');
					} else {
						Global.getUtilities().eventAggregator.invoke('homesSavedData');
					}
					return;
				}
				view.$el
					.append(view.header(view.metaData))
					.append(view.login());
				var $container = $('#main').find(":jqmData(role='content')");
				$container.empty().append(view.$el);
				$container.trigger('create');
				view.installStoredCreds();

				view.delegateEvents();

				cordova.exec(
					function(height) {
						console.log("KEYBOARD Keyboard is UP height=" + height);
						App.getState().contentHeight = height;
						App.getState().isKeyboardShown = true;
						Global.getUtilities().resizeUseSaved();
					},
					function() {
						console.log("KEYBOARD ERROR show");
					},
					"KeyboardNotification","keyboardShow",[]);

				cordova.exec(
					function(height) {
						console.log("KEYBOARD keyboard is DOWN height=" + height);
						App.getState().contentHeight = height;
//						App.getState().maxHeight = height;
						App.getState().isKeyboardShown = false;
						Global.getUtilities().resizeUseSaved();
					},
					function() {
						console.log("KEYBOARD ERROR hide");
					},
					"KeyboardNotification","keyboardHide",[]);
				cordova.exec(
					function(height) {
						console.log("KEYBOARD screenHeight height=" + height);
						App.getState().contentHeight = height;
					},
					function() {
						console.log("KEYBOARD screenHeight ERROR");
					},
					"KeyboardNotification","screenHeight",[]);
				return $container;
			},
			events: {
				'submit #logonForm':	'submit',
				'click #resetLogin':	'clear',
				'click #testData':		'testLocalData'
			},
			clear: function(e) {
				e.preventDefault();
				e.stopPropagation();
				$('#username', view.$el).val('');
				$('#password', view.$el).val('');
			},
			testLocalData: function() {
				// TEST local data
				var testJson = {
					value:  Math.floor((Math.random() * 100)).toString() + ' '
				};
				var data = {
						metricJSON:    testJson
					};
				cordova.exec(
	                    function(storedMetrics) {
	                        console.log('fetchTestData TestData=' + storedMetrics);
	        				console.log('found ' + storedMetrics.length + ' stored metrics');
	        				var text = 'stored data:';
	        				if ($.isArray(storedMetrics)) {
	        					for (var i=0; i<storedMetrics.length; i++) {
	        						var metric = $.parseJSON(storedMetrics[i].metricJSON);
	        						var metricID = storedMetrics[i].metricID;
	        						console.log('metric ' + metricID + ', value=' + metric.value);
	        						text = text + metric.value+ ' ';
	        					}
	        				}
	        				$('#testDataValues').text(text);
	                    },
	                    function(errorValue) {
							console.log("fetchTestData ERROR");
	                    },
						"CoreDataStorage", "fetchMetrics", []);

				cordova.exec(
	                    function(response) {
	                        console.log('saveTestData response=' + response);
	                    },
	                    function(errorValue) {
							console.log("saveTestData ERROR");
	                    },
	                    "CoreDataStorage", "saveMetric", [data]);
			},
			submit: function(e) {
				e.preventDefault();
				e.stopPropagation();

				HomesUser.clearInstance();
				var homesUser = HomesUser.instance({
					username: $('#username').val(),
					password: $('#password').val()
				});
				console.log('creds: ' + HomesUser.instance().get('username'));

				if (App.getState().isHomesOffline() || App.getState().isWorkingOffline()) {
					view.doOfflineLogin();
				} else {
					view.doOnlineLogin();
				}
			},
			doOfflineLogin: function() {
				var user = HomesUser.instance();
				var data = Security.getLocalStorage().isCredInStorage(user.get('username'),user.get('password'));
				if (data) {
					var dataObj = JSON.parse(data);
					if (App.getUtilities().isReadonly(dataObj.Roles)) {
						view.handleReadonly();
					} else {
						HomesUser.clearInstance();
						user = HomesUser.instance(dataObj);
						user.set('loggedIn',true);
						view.writeMetric(user);
						view.redirect();
					}
				} else {
					Global.getUtilities().eventAggregator.trigger('message', 'invalidLoginAttempt',
							['Please try again',
							'Since you are not connected to Homes, authentication is',
							'being done against local storage which may not have',
							'your Homes credentials stored']);
				}
			},
			doOnlineLogin: function() {
				HomesSoap.login(HomesUser.instance(), view.loginCallback);
			},
			loginCallback: function(obj) {
				var model = HomesUser.instance();
				console.log('creds: ' + model.get('username'));
				if (obj.success) {
					var loginResult = obj.successData;
					if (loginResult) {
						var obj = loginResult.User.Roles.string
						var roles = Array.isArray(obj) ? obj : [obj];
						if (App.getUtilities().isReadonly(roles)) {
							view.handleReadonly();
						} else if (loginResult.SessionID) {
								model.set('SessionID',			loginResult.SessionID);
								model.set('StaffID',			loginResult.User.StaffID);
								model.set('StaffName',			loginResult.User.StaffName);
								model.set('Authenticated',		loginResult.User.Authenticated);
								model.set('SiteCode',			loginResult.User.Facility);
								model.set('RoleObj',			loginResult.User.Roles);
								model.set('Roles',				roles);
								model.set('Email',				loginResult.User.StaffEmail);
								model.set('SUDAcceptedDate',	loginResult.User.SUDAcceptedDate);
								model.set('VISN',               loginResult.User.VISN);
								model.set('loggedIn',           true);
								Security.getLocalStorage().storeCred(model.get('username'),model.get('password'),JSON.stringify(model));
								model.rememberCreds();
								view.writeMetric(model);
								view.redirect();
						} else {
							view.badLogin(obj.errorData);
						}
					}
				} else {
					view.badLogin(obj.errorData);
				}
			},
			writeMetric: function(model) {
				console.log('METRIC creating metric');
				var metric = new Metric();
				metric.push(model);
			},
			redirect:	function() {
				if (App.getState().isHomesOnline()) {
					view.redirectOnline();
				} else {
					view.redirectOffline();
				}
			},
			redirectOnline: function() {
				var sudDate = HomesUser.instance().get('SUDAcceptedDate');
				var sudOk = Global.getUtilities().sudOk(sudDate);
				console.log('sudOk=' + sudOk + ', sudDate=' + sudDate + ', online=' + App.getState().isHomesOnline());
				if (sudOk) {
					Global.getUtilities().eventAggregator.invoke('blank');
				} else {
					Global.getUtilities().eventAggregator.invoke('sud');
				}
			},
			redirectOffline: function() {
				Global.getUtilities().eventAggregator.invoke('homesSavedData');
			},
			badLogin: function(errorData) {
				MainView.refresh();
				if (App.getState().isHomesOffline()) {
					Global.getUtilities().eventAggregator.trigger('message', 'offline',
							'To login in using your stored credentials, tap \"Sign In\" again.');
				} else {
					if (App.getState().checkForLockout(errorData)) {
						Global.getUtilities().eventAggregator.trigger('message', 'lockoutdLogin');
					} else {
						Global.getUtilities().eventAggregator.trigger('message', 'invalidLoginAttempt', 'Please try again');
					}
				}
			},
			handleReadonly: function(errorData) {
				MainView.refresh();
				Global.getUtilities().eventAggregator.trigger('message', 'readonlyLoginAttempt');
			},
			installStoredCreds: function() {
				if (App.getState().haveBeenOffline) {
					var username = App.getState().username;
//					var password = App.getState().password;
					if (username) {
						$('#username').val(username);
//						$('#password').val(password);
					}
				}
			}
		});
		return HomesLoginView;
	});
