define(
	[ 'App', 'Global', 'jQM', 'CordovaLib',
	'HomesUser', 'HomesVet', 'Forms', 'MainView', 'HomesLib',
	'Text!../html/templates/panes/homes/homesSavedData.html',
	'Text!../html/templates/panes/homes/homesSavedAssessmentListItem.html',
	'Text!../html/templates/panes/homes/homesSavedVeteranListItem.html',
	'Text!../html/templates/panes/homes/homesSavedEmptyItem.html'],
	function(App, Global, jQM, CordovaLib,
		HomesUser, HomesVet, Forms, MainView, HomesLib,
		template, assessmentItemTemplate, veteranItemTemplate, emptyItemTemplate) {
		'use strict';
		var view;
		var storedAssessments;
		var storedDSForms;
		var storedVeterans;
		var dataReady = false;
		var deleteConfirm;
		var deleteConfirmVeteran = 1;
		var deleteConfirmAssessment = 2;
		var deleteConfirmDSForm = 3;
		var objectToDelete;
		var dataParsed;

		var HomesSavedDataView = Backbone.View.extend({
			id:				'saved_data',
			tagName:		'div',
			attributes : {
				'class':	'saved-data'
			},
			initialize : function(metaData) {
				console.log('initialize');
				view = this;
				view.template = _.template(template);
				view.assessmentItemTemplate = _.template(assessmentItemTemplate);
				view.veteranItemTemplate = _.template(veteranItemTemplate);
				view.emptyItemTemplate = _.template(emptyItemTemplate);

				view.metaData = metaData ? metaData : {};
				view.metaData.isOffline = App.getState().isHomesOffline();
				view.metaData.sortBy = 1;
				view.metaData.sortByName = 1;
				view.metaData.sortByDate = 2;
				view.metaData.loading = true;

				dataParsed = false;
				view.loadData(view.displayData);
			},
			render : function() {
				view.$el.append(view.template(view.metaData));

				var $container = $('#main').find(":jqmData(role='content')");
				$container.empty().append(view.$el).trigger('create');
				console.log('render done');
				return view;
			},
			displayData: function() {
				console.log('displayData started');
				view.metaData.loading = false;

				view.$el.empty().append(view.template(view.metaData));

				$("select option").filter(function() {
					return $(this).val() == view.metaData.sortBy;
				}).prop('selected', true);

				var formOrder = new Object();

				if (!dataParsed) {
					for (var i=0; i<storedVeterans.length; i++) {
						var veteran = storedVeterans[i];
						veteran.veteranJSON = $.parseJSON(veteran.veteranJSON);
					}
					dataParsed = true;
				}

				if (view.metaData.sortBy == view.metaData.sortByName) {
					storedVeterans.sort(compareVetByName);
				} else if (view.metaData.sortBy == view.metaData.sortByDate){
					storedVeterans.sort(compareVetByDate);
				}

				for (var i=0; i<storedVeterans.length; i++) {
					var veteran = storedVeterans[i];
					veteran.veteranJSON.classes = 'line-item veteranListItem';
					var matchingAssessment = $.grep(storedAssessments, function(e){ return e.veteranLocalID == veteran.veteranLocalID; });
					if (matchingAssessment) {
						var veteranLocalID = veteran.veteranLocalID;
						formOrder[veteranLocalID] = i;
					}

					var matchingDSForm = $.grep(storedDSForms, function(e){ return e.veteranLocalID == veteran.veteranLocalID; });
					if (matchingDSForm) {
						var veteranLocalID = veteran.veteranLocalID;
						formOrder[veteranLocalID] = i;
					}

					var model = veteran.veteranJSON;
					model.displayName = model.lastName + ', ' + model.firstName + ' ' + model.middleInitial;
					model.displaySSN = displaySSN(model.sSN);
					model.displayDOB = view.displayDOB(model.dateOfBirth);
					model.displayGender = genderLetter(model.gender);
					model.displayVeteranID = view.displayVeteranID(model.veteranID);

					view.$el.find('#veteransList')
						.append(view.veteranItemTemplate(model))
						.children(':last').attr('data-index', i).attr('data-type', 'veteran');
				}

				if (storedVeterans.length == 0) {
					view.$el.find('#veteransList')
					.append(view.emptyItemTemplate({classes: '', text: 'No Veterans Saved Locally'}));
				}

				for (var i=0; i<storedAssessments.length; i++) {
					storedAssessments[i].orderNumber = formOrder[storedAssessments[i].veteranLocalID];
				}

				for (var i=0; i<storedDSForms.length; i++) {
					storedDSForms[i].orderNumber = formOrder[storedDSForms[i].veteranLocalID];
				}

				storedAssessments.sort(function(a, b){
					var orderNumberA = a.orderNumber,
					orderNumberB = b.orderNumber;

					if(orderNumberA < orderNumberB) return -1;
					if(orderNumberA > orderNumberB) return 1;
					return 0;
				});

				storedDSForms.sort(function(a, b){
					var orderNumberA = a.orderNumber,
					orderNumberB = b.orderNumber;

					if(orderNumberA < orderNumberB) return -1;
					if(orderNumberA > orderNumberB) return 1;
					return 0;
				});

				for (var i=0; i<storedAssessments.length; i++) {
					var assessment = storedAssessments[i];
					assessment.classes = 'line-item assessmentListItem';

					var model = assessment;
					var veteran = storedVeterans[assessment.orderNumber].veteranJSON;
					model.displayName = veteran.lastName + ', ' + veteran.firstName + ' ' + veteran.middleInitial;
					model.displaySSN = displaySSN(veteran.sSN);
					model.displayDOB = view.displayDOB(veteran.dateOfBirth);
					model.displayGender = genderLetter(veteran.gender);
					model.displayVeteranID = view.displayVeteranID(veteran.veteranHomesId);
					view.$el.find('#assessmentsList')
						.append(view.assessmentItemTemplate(model))
						.children(':last').attr('data-index', i).attr('data-type', 'assessment');
				}

				if (storedAssessments.length == 0) {
					view.$el.find('#assessmentsList')
					.append(view.emptyItemTemplate({classes: '', text: 'No Assessments Saved Locally'}));
				}

				for (var i=0; i<storedDSForms.length; i++) {
					var dsForm = storedDSForms[i];
					dsForm.classes = 'line-item assessmentListItem';

					var model = dsForm;
					var veteran = storedVeterans[dsForm.orderNumber].veteranJSON;

					model.displayName = veteran.lastName + ', ' + veteran.firstName + ' ' + veteran.middleInitial;
					model.displaySSN = displaySSN(veteran.sSN);
					model.displayDOB = view.displayDOB(veteran.dateOfBirth);
					model.displayGender = genderLetter(veteran.gender);
					model.displayVeteranID = view.displayVeteranID(veteran.veteranHomesId);

					view.$el.find('#dsFormsList')
						.append(view.assessmentItemTemplate(model))
						.children(':last').attr('data-index', i).attr('data-type', 'dsForm');
				}
				if (storedDSForms.length == 0) {
					view.$el.find('#dsFormsList')
					.append(view.emptyItemTemplate({classes: '', text: 'No Decision Support Forms Saved Locally'}));
				}

				var $container = $('#main').find(":jqmData(role='content')");
				$container.empty().append(view.$el).trigger('create');
				view.delegateEvents();
				console.log('displayData finished');
			},
			events: {
				'click #veteransList li':                       'loadVeteran',
				'click #assessmentsList li':                    'loadAssessment',
				'click #dsFormsList li':                        'loadAssessment',
				'click #add-offline-vet':                       'addOfflineVet',
				'click #assessmentsList li .trash-btn':         'confirmDeleteAssessment',
				'click #dsFormsList li .trash-btn':             'confirmDeleteDSForm',
				'click #veteransList li .trash-btn':            'confirmDeleteVeteran',
				'change select':                                'updateSort'

			},
			loadData: function(callback) {
				dataReady = false;
				CordovaLib.fetchAllOfflineJSON(
					function(obj) {
						if (obj.success) {
							storedVeterans = obj.successData;
							console.log('storedVeterans from IOS lenth=' + storedVeterans.length);
						} else {
							storedVeterans = [];
						}

						CordovaLib.fetchAllAssessments(
							function(obj) {
								if (obj.success) {
									storedAssessments = obj.successData;
									console.log('storedAssessment from IOS length=' + storedAssessments.length);
								} else {
									storedAssessments = [];
								}

								CordovaLib.fetchAllDSForms(
									function(obj) {
										if (obj.success) {
											storedDSForms = obj.successData;
											console.log('storedDS from IOS lenth=' + storedDSForms.length);
										} else {
											storedDSForms = [];
										}
										callback();
									}
								);
							}
						);
					}
				);
			},
			loadAssessment: function(e) {
				var data = {index: $(e.currentTarget).attr('data-index'),
							section: $(e.currentTarget).attr('data-type')};
				App.getUtilities().alert('tapped data=' + JSON.stringify(data));

				var selectedAssessment;
				if (data.section == 'dsForm') {
					selectedAssessment = storedDSForms[data.index];
				} else if (data.section == 'assessment') {
					selectedAssessment = storedAssessments[data.index];
				}

				view.setUpVeteran(
					selectedAssessment.orderNumber,
					function() {
						if (selectedAssessment.isDecisionSupport) {
							Global.getUtilities().eventAggregator.invoke('hds');
						} else {
							Global.getUtilities().eventAggregator.invoke('homesAssessment');
						}
					});
			},
			loadVeteran: function(e) {
				var index = $(e.currentTarget).attr('data-index');
				view.setUpVeteran(index, function() {
					Global.getUtilities().eventAggregator.invoke('homesVeteranSummary');
				});
			},
			setUpVeteran: function(index,callback) {
				var selectedVeteran = storedVeterans[index];
				var json = $.parseJSON(selectedVeteran.assessmentJSON);
				selectedVeteran.veteranJSON.veteranLocalID = selectedVeteran.veteranJSON.veteranLocalID;

				var currentVet = new HomesVet();
				for (var veteranKey in selectedVeteran.veteranJSON) {
					currentVet.set(veteranKey, selectedVeteran.veteranJSON[veteranKey]);
				}

				if (json) {
					console.log('data=' + JSON.stringify(json));
					currentVet.set('assessmentJSON', json);
				}

				HomesVet.setCurrentVet(currentVet);
				App.getState().vetIndex = null;
				HomesLib.loadVetInfo(callback);
			},
			confirmDeleteVeteran: function(e) {
				e.preventDefault();
				e.stopPropagation();

				var data = {index:      $(e.currentTarget).parents('li').attr('data-index')};
				var selectedVeteran = storedVeterans[data.index];
				deleteConfirm = deleteConfirmVeteran;
				objectToDelete = selectedVeteran;

				Global.getUtilities().eventAggregator.trigger('message', 'confirmVeteranDeletion', 'It will also remove all Assessment data and Decision Support Form data on this device for ' + selectedVeteran.veteranJSON['firstName'] + ' ' + selectedVeteran.veteranJSON['lastName'], view);
			},
			deleteVeteran: function() {

				var selectedVeteran = objectToDelete;
				var queryParameters;

				if (selectedVeteran.veteranHomesId && selectedVeteran.veteranLocalID) {
					queryParameters = {
						'veteranHomesId': selectedVeteran.veteranHomesId,
						'veteranLocalID': selectedVeteran.veteranLocalID
					};
					if (HomesVet.getCurrentVet()) {
						if (HomesVet.getCurrentVet().get('veteranLocalID') == selectedVeteran.veteranLocalID || HomesVet.getCurrentVet().get('veteranHomesId') == selectedVeteran.veteranHomesId) {
							HomesVet.setCurrentVet(null);
							MainView.refresh();
						}
					}

				} else if (selectedVeteran.veteranHomesId) {
					queryParameters =  {'veteranHomesId': selectedVeteran.veteranHomesId};
					if (HomesVet.getCurrentVet()) {
						if (HomesVet.getCurrentVet().get('veteranHomesId') == selectedVeteran.veteranHomesId) {
							HomesVet.setCurrentVet(null);
							MainView.refresh();
						}
					}

				} else {
					queryParameters =  {'veteranLocalID': selectedVeteran.veteranLocalID};

					if (HomesVet.getCurrentVet()) {
						if (HomesVet.getCurrentVet().get('veteranLocalID') == selectedVeteran.veteranLocalID) {
							HomesVet.setCurrentVet(null);
							MainView.refresh();
						}
					}
				}

				CordovaLib.deleteOfflineJSON(queryParameters, function(obj) {
					if (obj.success) {
						CordovaLib.deleteAssessment(queryParameters, null);
						CordovaLib.deleteDSForm(queryParameters, null);
						Global.getUtilities().eventAggregator.invoke('homesSavedData');
					}
				});

			},

			confirmDeleteAssessment: function(e) {
				e.preventDefault();
				e.stopPropagation();

				var data = {index:      $(e.currentTarget).parents('li').attr('data-index')};
				var selectedAssessment = storedAssessments[data.index];
				deleteConfirm = deleteConfirmAssessment;
				objectToDelete = selectedAssessment;

				Global.getUtilities().eventAggregator.trigger('message', 'confirmAssessmentDeletion', 'This will remove all Assessment data on this device for ' + selectedAssessment['veteranFirstName'] + ' ' + selectedAssessment['veteranLastName'], view);
			},

			deleteAssessment: function(e) {
				var selectedAssessment = objectToDelete;
				var queryParameters;

				if (selectedAssessment.veteranHomesId && selectedAssessment.veteranLocalID) {
					queryParameters = {
						'veteranHomesId': selectedAssessment.veteranHomesId,
						'veteranLocalID': selectedAssessment.veteranLocalID
					};
				} else if (selectedAssessment.veteranHomesId) {
					queryParameters =  {'veteranHomesId': selectedAssessment.veteranHomesId};
				} else {
					queryParameters =  {'veteranLocalID': selectedAssessment.veteranLocalID};
				}

				CordovaLib.deleteAssessment(queryParameters, function(obj) {
					if (obj.success) {
						Global.getUtilities().eventAggregator.invoke('homesSavedData');
					}
				});
			},
			confirmDeleteDSForm: function(e) {
				e.preventDefault();
				e.stopPropagation();

				var data = {index:      $(e.currentTarget).parents('li').attr('data-index')};
				var selectedDSForm = storedDSForms[data.index];

				deleteConfirm = deleteConfirmDSForm;
				objectToDelete = selectedDSForm;

				Global.getUtilities().eventAggregator.trigger('message', 'confirmDSFormDeletion', 'This will remove all Decision Support Form data on this device for ' + selectedDSForm['veteranFirstName'] + ' ' + selectedDSForm['veteranLastName'], view);
			},

			deleteDSForm: function(e) {


				var selectedAssessment = objectToDelete;

				var queryParameters;

				if (selectedAssessment.veteranHomesId && selectedAssessment.veteranLocalID) {
					queryParameters = {
						'veteranHomesId': selectedAssessment.veteranHomesId,
						'veteranLocalID': selectedAssessment.veteranLocalID
					};
				} else if (selectedAssessment.veteranHomesId) {
					queryParameters =  {'veteranHomesId': selectedAssessment.veteranHomesId};
				} else {
					queryParameters =  {'veteranLocalID': selectedAssessment.veteranLocalID};
				}


				CordovaLib.deleteDSForm(queryParameters, function(obj) {
					if (obj.success) {
						Global.getUtilities().eventAggregator.invoke('homesSavedData');
					}
				});

			},
			updateSort: function(e) {
				view.metaData.sortBy = $(e.currentTarget).val();
				view.displayData();
			},
			addOfflineVet: function(e) {
				HomesVet.setCurrentVet(null);
				App.getState().vetIndex = null;
				Global.getUtilities().eventAggregator.invoke('homesVeteranSummary');

			},
			displayDOB: function(dateOfBirth) {
				var slashDate = ISOtoDateStr(dateOfBirth)
				var age = getYearAge(slashDate);
				var text = slashDate + ' (' + age + ' yo) ';
				return text;
			},
			displayVeteranID: function(veteranID) {
				var text = '';
				if (veteranID && veteranID > 0) {
					text = 'HOMES ID: ' + veteranID;
				}

				return text;
			},
			messageCallBack: function() {
				if (deleteConfirm == deleteConfirmVeteran) {
					if (App.getState().okMessagePressed) {
						view.deleteVeteran();
					}
				} else if (deleteConfirm == deleteConfirmAssessment) {
					if (App.getState().okMessagePressed) {
						view.deleteAssessment();
					}
				} else if (deleteConfirm == deleteConfirmDSForm) {
					if (App.getState().okMessagePressed) {
						view.deleteDSForm();
					}
				}
			}
		});

		return HomesSavedDataView;
	}
);
