define(
	[ 'App', 'Global', 'jQM', 'CordovaLib', 'HomesSoap', 'HomesLib',
	'DateView',
	'HomesVets', 'HomesVet', 'HomesUser', 'MainView', 'Forms',
	'Text!../html/templates/panes/homes/summaryHeader.html',
	'Text!../html/templates/panes/homes/summary.html',
	'Text!../html/templates/panes/homes/summaryReadonly.html' ],
	function(App, Global, jQM, CordovaLib, HomesSoap, HomesLib,
			DateView,
			HomesVets, HomesVet, HomesUser, MainView, Forms,
			templateHeader,templateEdit,templateReadonly) {
		'use strict';
		var view;
		var statesList;
		var soapSuccess;
		var newVet;
		var matchingVeteran;
		var noInternetErrorMessage = 'You are not connected to the internet';

		var pOCRelationshipMap = {
			'':			'',
			"1":        "Sibling",
			"2":        "Spouse",
			"3":        "Parent",
			"4":        "Legal Guardian",
			"5":        "Other"
		};

		var isEditing = false, initialLoad=true;
		var HomesSummaryView = Backbone.View.extend({
			id: 'homesSummary',
			tagName: 'div',
			attributes : {
				'class': 'homesSummary'
			},
			model: HomesVet,
			initialize : function(metaData) {
				view = this;
				view.templateHeader = _.template(templateHeader);
				view.templateEdit = _.template(templateEdit);
				view.templateReadonly = _.template(templateReadonly);
				view.metaData = metaData ? metaData : {};
				isEditing = false;
				App.getState().veteranSummaryIsEditing = false;

				initialLoad=true;

				if (HomesVet.getCurrentVet()) {
					view.vet = HomesVet.getCurrentVet();
					newVet = false;
				} else {
					view.vet = new HomesVet();
					newVet = true;
				}

				view.getStates();
			},
			render : function() {
				var parms = {};
				parms.label = view.metaData;
				parms.label.submitLabel = view.getSubmitLabel();
				parms.vet = view.vet.toJSON();
				parms.vet.displayHomesID = view.getDisplayHomesID(view.vet);
				parms.vet.pOCRelationshipDisplay = pOCRelationshipMap[parms.vet.pOCRelationship];
				parms.readonly = !isEditing && HomesVet.getCurrentVet();
				App.getState().veteranSummaryIsEditing = !parms.readonly;

				parms.showBack = App.getState().showBackButton;

				parms.dateOfBirthDisplay = formatDateNoTime(parms.vet.dateOfBirth);

				console.log('vet summary parms.data=' + JSON.stringify(parms.vet));
				view.$el.empty().append(this.templateHeader(parms));
				if (parms.readonly) {
					view.$el.append(this.templateReadonly(parms));
				} else {
					view.$el.append(this.templateEdit(parms));
				}
				var $container = $('#main').find(":jqmData(role='content')");
				$container.empty().append(view.$el).trigger('create');
				view.populateStateSelector();

				if (initialLoad) {
					if (HomesVet.getCurrentVet()) {
						HomesLib.loadVetInfo( function() {
							view.needConfirmation = newVet;
							view.populate();
							Global.getUtilities().putToTop();
						});
					}
				}
				if (App.getState().isHomesOffline()) {
					$('#back-btn').addClass('ui-disabled');
				}

				if (App.getState().showBackButton == false) {
					$('#back-btn').addClass('hidden');
				}

				Global.getUtilities().setupKeyEvents();

				initialLoad = false;
				return view;
			},
			events: {
				'click #add-btn':				'addOrUpdateVet',
				'click #back-btn':				'back',
				'click #edit-btn':   			'beginEditVet',
				'tap .ui-btn':					'selectEdit',
				'change select':				'updateModel',
				'change #ssnConfirm':			'updateSsnConfirm',
				"tap #dateOfBirth" :			'pickDate',
				"tap #sSN":						'editingSSN',
			},
			updateSsnConfirm: function(e) {
				e.preventDefault();
				e.stopPropagation();
				view.needConfirmation = !$('#ssnConfirm').prop('checked');
			},
			getDisplayHomesID: function(vet) {
				if (vet.isLocal() || vet.get('veteranID') == '-1') {
					return 'N/A - Saved Locally';
				} else {
					return vet.get('veteranID');
				}
			},
			getSubmitLabel: function() {
				if (newVet) {
					if (App.getState().isHomesOffline()) {
						return 'Save Veteran Locally';
					} else {
						return 'Submit Veteran';
					}
				} else {
					if (App.getState().isHomesOffline()) {
						return 'Save Edit Locally';
					} else {
						if (HomesVet.isVetLocal()) {
							return 'Submit Veteran';
						} else {
							return 'Submit Edit';
						}
					}
				}
			},
			pickDate : function(e) {
				e.preventDefault();
				e.stopPropagation();

				var applyDate = function (date) {
					console.log('applyDate date=' + date);
					if (typeof date === 'string') {
						$('#dateOfBirth').val(formatDateNoTime(date));
					}
				};

				var date = $('#dateOfBirth').val();
				if (!date) {
					date = new Date();
					date.setYear(date.getYear()-19);
					date = formatDateNoTime(date);
				}
				new DateView($.extend(App.getMetaData()['dob'], {
					'date':			formatDateNoTime(date),
					'onApply':		applyDate,
					'yearsBack':	100,
					'caller':		$(e.currentTarget)
				})).render();
			},
			editingSSN: function(e) {
				view.needConfirmation = true;
				view.populate();
			},
			updateAnswers: function() {
				$('#editVeteranForm input').each(function() {
					var key = $(this).attr('id');
					var val = $(this).val();
//					console.log('key=' + key + ', val=' + val);
					view.vet.set(key,val);
				});
				$('#editVeteranForm select').each(function() {
					var key = $(this).attr('id');
					var val = $(this).val();
//					console.log('key=' + key + ', val=' + val);
					view.vet.set(key,val);
				});
				view.needConfirmation = !$('#ssnConfirm').prop('checked');
				console.log('needConfirmation=' + view.needConfirmation);
			},
			selectEdit: function(e) {
				console.log('selectEdit');
				var id = $(e.currentTarget).attr('id');
				console.log('selectEdit id=' + id);
				if (id==='gender-button') {
					Global.getUtilities().setNextFocus('address1');
				}
				if (id==='state-button') {
					Global.getUtilities().setNextFocus('zipCode');
				}
			},
			updateModel: function (e) {
				e.preventDefault();
				e.stopPropagation();
				view.updateAnswers();
				console.log('updateModel type' + e.type);
				if ($('.ui-page-active[data-role="dialog"]').length==0) {
					console.log('not on dialog, can call render');
					view.render();
				} else {
					console.log('on dialog, cannot call render');
				}

				view.populate();
				view.delegateEvents(view.events); // why we lose the event handlers I do not know
			},
			populate: function() {
				console.log('summary screen populate vet: ' + JSON.stringify(view.vet));
				$('#ssnConfirm').prop('checked', !view.needConfirmation).checkboxradio('refresh');

				$('#gender option').prop('selected', false);
				$('#gender option[value="' + view.vet.get('gender') + '"]').prop('selected', true);
				$('#gender').selectmenu('refresh');

				$('#state option').prop('selected', false);
				$('#state option[value="' + view.vet.get('state') + '"]').prop('selected', true);
				$('#state').selectmenu('refresh');

				$('#pOCRelationship option').prop('selected', false);
				$('#pOCRelationship option[value="' + view.vet.get('pOCRelationship') + '"]').prop('selected', true);
				$('#pOCRelationship').selectmenu('refresh');
			},

			addOrUpdateVet: function(e) {
				if (e) {
					e.preventDefault();
					e.stopPropagation();
				}

				var vet = this.collectVeteranInfo();
				if (view.validateVet(vet)) {
					var currentDateTime = new Date();
					vet['username']       = HomesUser.instance().get('username');
					vet['lastActionDate'] = currentDateTime;
					if (isEditing) {
						vet['creationDate']   = view.vet.get('creationDate');
						vet['veteranID']      = view.vet.get('veteranID');
						if (App.getState().isHomesOffline()) {
							view.updateVetOffline(vet);
						} else {
							vet['sessionId'] = HomesUser.instance().get('SessionID');
							if (HomesVet.isVetLocal()) {
								vet['creationDate'] = currentDateTime;
								view.addNewVetOnline(vet);
							} else {
								view.updateVetOnline(vet)
							}
						}
					} else {
						vet['creationDate'] = currentDateTime;
						if (App.getState().isHomesOffline()) {
							view.addNewVetOffline(vet);
						} else {
							vet['sessionId'] = HomesUser.instance().get('SessionID');
							view.addNewVetOnline(vet);
						}
					}
				}
			},
			updateVetOnline: function(editedVeteran) {
				var originalVet = HomesVet.getCurrentVet();

				HomesSoap.updateVet(editedVeteran, function(obj) {
					if (obj.success) {
						isEditing = false;
						App.getState().veteranSummaryIsEditing = false;
						var data = obj.successData;
						console.log('vet data=' + JSON.stringify(data));
						HomesVets.clearInstance();
						HomesVets.instance([]);

						var vet = new HomesVet(data);

						vet.set('isHomesEpisodeOpen', originalVet.get('isHomesEpisodeOpen'));
						if (originalVet.get('isHomesEpisodeOpen')) {
							vet.set('homesEpisodeVISN', originalVet.get('homesEpisodeVISN'));
						}

						vet.set('veteranLocalID', HomesVet.getCurrentVet().get('veteranLocalID'));
						HomesVet.setCurrentVet(vet);
						soapSuccess = true;
						Global.getUtilities().eventAggregator.trigger('message', 'successfullyEditedVetOnline', '', view);
						HomesVet.getCurrentVet().set('hasUnsubmittedChanges', false);
						console.log('calling storVet');
						CordovaLib.storeVet();
						MainView.refresh();
					} else {
						if (App.getState().isHomesOffline()) {
							Global.getUtilities().reportError('HOMES is Offline. Please save the Veteran info changes locally so you can submit to HOMES when reconnected.', obj.errorData);
							$('#add-btn .ui-btn-text').text(view.getSubmitLabel());
						} else {
							Global.getUtilities().reportError('Could not edit the Veteran in HOMES.',obj.errorData);
						}
						MainView.refresh();
					}
				});
			},
			addNewVetOnline: function(vet) {
				var originalVet = HomesVet.getCurrentVet();

				console.log('addNewVetOnline vet=' + JSON.stringify(vet));
				HomesSoap.addVet(vet, function(obj) {
					if (obj.success) {
						var data = obj.successData;
						var vet = new HomesVet(data);

						if (originalVet) {
							if (originalVet.get('veteranLocalID')) {
								vet.set('veteranLocalID', originalVet.get('veteranLocalID'));
							}
						}
						HomesVets.clearInstance();
						HomesVets.instance([]);

						HomesVet.setCurrentVet(vet);
						HomesVet.getCurrentVet().set('hasUnsubmittedChanges', false);

						soapSuccess = true;
						Global.getUtilities().eventAggregator.trigger('message', 'successfullyAddedVetOnline', '', view);
						console.log('calling storVet');
						CordovaLib.storeVet(function(obj) {
							if (obj.success) {
								isEditing = false;
								App.getState().veteranSummaryIsEditing = false;

								var data = obj.successData;
								console.log('Successfully added vet locally, ' + data);

								if (isNaN(data)) {
									console.log('BAD LOCAL VETERAN ID! summary view add new vet online');
									alert('BAD LOCAL VETERAN ID! summary view add new vet online');
								}

								HomesVet.getCurrentVet().set('veteranLocalID', data);

								MainView.refresh();
							} else {
								HomesVet.setCurrentVet(originalVet);
								MainView.refresh();
								Global.getUtilities().eventAggregator.trigger('message', 'errorEditingVetOffline', obj.errorData);
							}
						});

					} else if (obj.matchFound) {
						matchingVeteran = obj.homesVeteranMatch;
						Global.getUtilities().eventAggregator.trigger('message', 'submitMatchFoundSummary', obj.dialogText, view);
					} else {
						if (App.getState().isHomesOffline()) {
							Global.getUtilities().reportError('HOMES is Offline. Please save the Veteran info locally so you can submit to HOMES when reconnected.', obj.errorData);
							$('#add-btn .ui-btn-text').text(view.getSubmitLabel());
						} else {
							Global.getUtilities().reportError('Could not save the Veteran.',obj.errorData);
						}
						MainView.refresh();

					}

				});
			},
			updateVetOffline: function(vet) {
				MainView.refresh();
				var originalVet = HomesVet.getCurrentVet();
				vet.veteranLocalID = originalVet.get('veteranLocalID');
				vet.veteranID = originalVet.get('veteranID');
				HomesVet.setCurrentVet(new HomesVet(vet));
				HomesVet.getCurrentVet().set('hasUnsubmittedChanges', true);

				CordovaLib.storeVet(function(obj) {
					if (obj.success) {
						isEditing = false;
						App.getState().veteranSummaryIsEditing = false;
						var data = obj.successData;
						console.log('Successfully made edits to vet locally, ' + data);
						HomesVet.getCurrentVet().set('veteranLocalID', data);
						if (isNaN(data)) {
							console.log('BAD LOCAL VETERAN ID! veteran summary view update vet offline');
						}

						HomesVets.clearInstance();
						HomesVets.instance([]);

						var revisedVet = new HomesVet(vet);
						HomesVet.setCurrentVet(revisedVet);
						soapSuccess = true;

						Global.getUtilities().eventAggregator.trigger('message', 'successfullyEditedVetOffline', '', view);
						MainView.refresh();
					} else {
						HomesVet.setCurrentVet(originalVet);
						MainView.refresh();
						Global.getUtilities().eventAggregator.trigger('message', 'errorEditingVetOffline', obj.errorData);
					}
				});
			},
			addNewVetOffline: function(vet) {
				MainView.refresh();
				var originalVet = HomesVet.getCurrentVet();
				HomesVet.setCurrentVet(new HomesVet(vet));
				HomesVet.getCurrentVet().set('hasUnsubmittedChanges', true);
				console.log('calling storVet');
				CordovaLib.storeVet(function(obj) {
					if (obj.success) {
						var successValue = obj.successData;
						console.log('Success adding vet locally, ' + successValue);
						HomesVet.setCurrentVet(new HomesVet(vet));
						HomesVet.getCurrentVet().set('veteranLocalID', successValue);


						if (isNaN(successValue)) {
							console.log('BAD LOCAL VETERAN ID! veteran summary view add new vet offline');
						}
						soapSuccess = true;
						Global.getUtilities().eventAggregator.trigger('message', 'successfullyAddedVetOffline', '', view);
						MainView.refresh();
					} else {
						HomesVet.setCurrentVet(originalVet);
						MainView.refresh();
						var errorValue = obj.errorData;
						Global.getUtilities().eventAggregator.trigger('message', 'errorAddingVetOffline', errorValue);
						console.log('Error Adding Veteran!, ' + errorValue);
					}
				});
			},
			validateVet: function(displayedVeteran) {
				if (!displayedVeteran.firstName	|| !displayedVeteran.lastName) {
					Global.getUtilities().eventAggregator.trigger('message',
							'veteranSummaryValidation', "Please enter the Veteran's first and last name");
					return false;
				}
				if (!displayedVeteran.sSN || displayedVeteran.sSN.length!=9 || !isNumber(displayedVeteran.sSN)) {
					Global.getUtilities().eventAggregator.trigger('message',
							'veteranSummaryValidation', "Please enter the Veteran's nine digit SSN with no dashes");
					return false;
				}
				if (!$('input:checkbox[name=ssnConfirm]').is(':checked')) {
					Global.getUtilities().eventAggregator.trigger('message',
							'veteranSummaryValidation', "Please indicate that you have confirmed the Veteran's SSN");
					return false;
				}
				if (!displayedVeteran.dateOfBirth) {
					Global.getUtilities().eventAggregator.trigger('message',
							'veteranSummaryValidation', "Please enter the Veteran's DOB");
					return false;
				}
				if (!displayedVeteran.gender) {
					Global.getUtilities().eventAggregator.trigger('message',
							'veteranSummaryValidation', "Please enter the Veteran's gender");
					return false;
				}
				if (getYearAge(displayedVeteran.dateOfBirth) < 18) {
					Global.getUtilities().eventAggregator.trigger('message',
							'veteranSummaryValidation', "DOB indicates Veteran is less than 18");
					return false;
				}
				if(displayedVeteran.zipCode && !isValidUSZipNoExtension(displayedVeteran.zipCode)) {
					Global.getUtilities().eventAggregator.trigger('message', 'veteranSummaryZipCodeValidation');
					return false;
				}
				if (displayedVeteran.pOCRelationship && displayedVeteran.pOCRelationship === '5') {
					if (!displayedVeteran.pOCRelationshipOther) {
						Global.getUtilities().eventAggregator.trigger('message',
								'veteranSummaryValidation', "Please enter a Relationship in the 'POC Relationship Other' field");
						return false;
					}
				}
				return true;
			},
			beginEditVet: function(e) {
				e.preventDefault();
				e.stopPropagation();
				isEditing = true;
				App.getState().veteranSummaryIsEditing = true;
				view.render();
				view.populate();
				view.delegateEvents(view.events); // why we lose the event handlers I do not know
				Global.getUtilities().putToTop();
			},
			back: function(e) {
				e.preventDefault();
				e.stopPropagation();
				if (HomesVet.getCurrentVet()) {
					if (HomesVet.isVetLocal()) {
						Global.getUtilities().eventAggregator.invoke('homesSavedData');
					} else {
						Global.getUtilities().eventAggregator.invoke('homesList');
					}
				} else {
					Global.getUtilities().eventAggregator.invoke('homesList');
				}
			},
			collectVeteranInfo: function() {
				var currentVet = new HomesVet();

				currentVet.set('lastName', $("#lastName").val());
				currentVet.set('firstName', $("#firstName").val());
				currentVet.set('middleInitial', $("#middleInitial").val());
				currentVet.set('sSN', $("#sSN").val());
				currentVet.set('dateOfBirth', ISOdate($("#dateOfBirth").val()));
				currentVet.set('gender', $("#gender").val());
				currentVet.set('address1', $("#address1").val());
				currentVet.set('address2', $("#address2").val());
				currentVet.set('address3', $("#address3").val());
				currentVet.set('city', $("#city").val());
				currentVet.set('state', $("#state").val());
				currentVet.set('zipCode', $("#zipCode").val());
				currentVet.set('telephonePrimary', $("#telephonePrimary").val());
				currentVet.set('telephoneSecondary', $("#telephoneSecondary").val());
				currentVet.set('email', $("#email").val());
				currentVet.set('pOCName', $("#pOCName").val());
				currentVet.set('pOCPhone', $("#pOCPhone").val());
				currentVet.set('pOCRelationship', $("#pOCRelationship").val());
				if ($("#pOCRelationshipOther").val()) {
					currentVet.set('pOCRelationshipOther', $("#pOCRelationshipOther").val());
				} else {
					currentVet.set('pOCRelationshipOther', '');
				}
				return currentVet.attributes;
			},
			getStates: function() {
				var statesFileName = "res/states.txt";
				var request = new XMLHttpRequest();
				request.open("GET", statesFileName, false);
				request.send(null);
				statesList = request.responseText.split("\n");
			},
			populateStateSelector : function () {
				var stateSelector = $('#state');
				for (var i = 0, len = statesList.length; i < len; i++) {
					stateSelector.append("<option value='" + statesList[i] + "'>" + statesList[i] + "</option>");
				}
			},
			messageCallBack: function() {
				if (matchingVeteran && App.getState().okMessagePressed) {

					var veteranLocalID;
					if (HomesVet.getCurrentVet()) {
						veteranLocalID = HomesVet.getCurrentVet().get('veteranLocalID');
						matchingVeteran.set('veteranLocalID', veteranLocalID);
					}

					HomesVet.setCurrentVet(matchingVeteran);
					matchingVeteran = null;
					isEditing = false;
					App.getState().veteranSummaryIsEditing = false;

					HomesSoap.checkVeteranHomesEpisodeStatus(function() {
						MainView.refresh();
						Global.getUtilities().eventAggregator.invoke('homesVeteranSummary');
					});

					// HMLS-809 removed this functionality
//				} else if (matchingVeteran && App.getState().secondaryActionPressed) {
//					if (HomesVet.getCurrentVet()) {
//							HomesVet.getCurrentVet().set('veteranID', matchingVeteran.get('veteranID'));
//							HomesVet.getCurrentVet().set('creationDate', matchingVeteran.get('creationDate'));
//							view.vet = HomesVet.getCurrentVet();
//					} else {
//						view.vet.set('veteranID', matchingVeteran.get('veteranID'));
//						view.vet.set('creationDate', matchingVeteran.get('creationDate'));
//						HomesVet.setCurrentVet(view.vet);
//					}
//					isEditing = true;
//					App.getState().veteranSummaryIsEditing = true;
//					matchingVeteran = null;
//
//					view.checkOpenEpisode();
//
//					MainView.refresh();
//					view.addOrUpdateVet();
				} else if (matchingVeteran && App.getState().cancelMessagePressed) {
					matchingVeteran = null;
				} else {
					if (soapSuccess) {
						if (App.getState().okMessagePressed) {
							App.getState().showBackButton = false;
							Global.getUtilities().eventAggregator.invoke('homesVeteranSummary');
						}
					}
				}
			}
		});
		return HomesSummaryView;
	}
);
