define(
	[ 'Global', 'App', 'HomesLib', 'CordovaLib',
	  'Templates', 'FormRules',
	  'Responses', 'Response', 'MainView', 'HomesVet',
	  'Text!../html/templates/panes/homes/recommend.html'
	],
	function(Global, App, HomesLib, CordovaLib,
			Templates, FormRules,
			Responses, Response, MainView, HomesVet, template) {
		'use strict';
		var view;
		var section,questions;
		var alerts = [];
		var recommendations = {};
        var matchingVeteran;
		var alerts;
		var recommendations;
		var RecommendView = Backbone.View.extend({
			id:				'recommend',
			tagName:		'div',
			attributes : {
				'class':	'recommend'
			},
			initialize : function(metaData) {
				view = this;
				view.template = _.template(template);
				view.metaData = metaData ? metaData : {};
				alerts = [];
				recommendations = {};
				view.findAlerts();
				view.metaData.alerts = alerts;
				view.calculateScores();
                matchingVeteran = null;
			},
			render : function() {
				view.$el.empty();
				view.metaData.recommendations = recommendations;
				view.metaData.offline = App.getState().isHomesOffline();
				view.$el.append(view.template(view.metaData));
				var $container = $('#main').find(":jqmData(role='content')");
				$container.empty().append(view.$el).trigger('create');
				Global.getUtilities().putToTop();
				Global.getUtilities().setupKeyEvents();
				$("div[id$='-listbox-popup']").on('popupafterclose', function(event) {
					view.rejectText(event);
				});
				return view;
			},
			events: {
				'click .accept-btn':				'accept',
				'click .reject-btn':				'reject',
				'click #back-btn':					'back',
				'click #submit-btn':				'submit'
			},
			update: function() {
				view.$el.empty();
				view.metaData.recommendations = recommendations;
				view.metaData.offline = App.getState().isHomesOffline();
				view.$el.append(view.template(view.metaData));
				var $container = $('#main').find(":jqmData(role='content')");
				$container.empty().append(view.$el).trigger('create');
				$("div[id$='-listbox-popup']").on('popupafterclose', function(event) {
					view.rejectText(event);
				});
			},
			accept:	function(e) {
				console.log('accept');
				e.preventDefault();
				e.stopPropagation();
				var $target = $(e.currentTarget);
				var id = $target.attr('data-id');
				recommendations[id].accepted = true;
				recommendations[id].rejected = false;

				view.update();
				Global.getUtilities().scroll('reject');
				view.delegateEvents(view.events); // why we lose the event handlers I do not know
			},
			reject:	function(e) {
				console.log('reject');
				e.preventDefault();
				e.stopPropagation();
				var $target = $(e.currentTarget);
				var id = $target.attr('data-id');
				recommendations[id].rejected = true;
				recommendations[id].accepted = false;

				view.update();
				Global.getUtilities().scroll('rejectReason');
				view.delegateEvents(view.events);
			},
			rejectText: function(e) {
				console.log('rejectText');
				var scrollToId = $('#rejectText input').attr('id');
				if (scrollToId) {
					setTimeout(function() {
						Global.getUtilities().scroll(scrollToId);
					},100);
				}
			},
			back: function(e) {
				e.preventDefault();
				e.stopPropagation();
				Global.getUtilities().eventAggregator.invoke('hds');
			},
			submit: function(e) {
                if (e) {
                    e.preventDefault();
                    e.stopPropagation();
                }
				$("[id$='-rejectReason']").each(function() {
					var id = $(this).attr('id');
					var regex = /(.*)-rejectReason/;
					id = regex.exec(id)[1];
					recommendations[id].rejectReason = $(this).val();
					console.log('rejectReason: ' + id + '=' + recommendations[id].rejectReason);
				});
				$("[id$='-rejectText']").each(function() {
					var id = $(this).attr('id');
					var regex = /(.*)-rejectText/;
					id = regex.exec(id)[1];
					recommendations[id].rejectText = $(this).val();
					console.log('rejectText: ' + id + '=' + recommendations[id].rejectText);
				});
				if (App.getState().isHomesOffline()) {
                        Global.getUtilities().eventAggregator.trigger('message', 'cantSubmitDS');
                        MainView.refresh();
				} else {
					if (view.validate()) {
						HomesLib.sendDS(
							FormRules.getSoapLogic().toVAStorage(),
							FormRules.getSoapLogic().alertToVAStorage(alerts),
							FormRules.getSoapLogic().recommendToVAStorage(recommendations),
							view.sendDSFormCallback
						);
					}
				}
			},
			sendDSFormCallback: function(obj) {
				if (obj.success) {
					console.log("Success sending" + obj.successData);
					Global.getUtilities().eventAggregator.trigger('message', 'successfullySubmittedDSForm', '', view);

                    var queryParameters;
                    var selectedVeteran = HomesVet.getCurrentVet();
                    if (selectedVeteran.get('veteranID') && selectedVeteran.get('veteranLocalID')) {
                         queryParameters = {
                            'veteranHomesId': selectedVeteran.get('veteranID'),
                            'veteranLocalID': selectedVeteran.get('veteranLocalID')
                        };
                    } else if (selectedVeteran.get('veteranID')) {
                        queryParameters =  {'veteranHomesId': selectedVeteran.get('veteranID')};
                    } else {
                        queryParameters =  {'veteranLocalID': selectedVeteran.get('veteranLocalID')};
                    }
					CordovaLib.deleteDSForm(queryParameters, null);
				} else if (obj.matchFound) {
                    matchingVeteran = obj.homesVeteranMatch;
                    Global.getUtilities().eventAggregator.trigger('message', 'submitMatchFoundForm', obj.dialogText, view);

                } else {
					console.log("Error sending: " + obj.errorData);
					if (App.getState().isHomesOffline()) {
                        Global.getUtilities().eventAggregator.trigger('message', 'cantSubmitDS');
					} else {
						Global.getUtilities().reportError('Could not save the Decision Support Form',obj.errorData);
					}
					MainView.refresh();
				}
			},
			findAlerts: function() {
				alerts = [];
				var alertHash = Templates.alertHash_HDS;
				for (var key in alertHash) {
					var alert = alertHash[key];
					if (alert.questionId) {
						if (FormRules.getDisplayLogic().isQuestionDisplayed(alert.questionId)) {
							var answer = Responses.instance().getAnswer(alert.questionId);
							if (answer == alert.answer) {
								var newText = alert.text;
								var addText = true;
								for (var i in alerts) {
									if (newText===alerts[i]) {
										addText = false;
										break;
									}
								}
								if (addText) {
									alerts.push(alert.text);
								}
							}
						}
					}
				}
			},
			calculateScores: function() {
				var ruleHash = Templates.ruleHash_HDS;
				var prevention=0,rapid=0,transitional=0,permanent=0,caseMgmt=0,shortTerm=0;
				for (var key in ruleHash) {
					var rule = ruleHash[key];
					var ansA='', ansB='';
					var ruleAnsA='', ruleAnsB='';
					if (rule.quesA) {
						if (FormRules.getDisplayLogic().isQuestionDisplayed(rule.quesA)) {
							ansA = Responses.instance().getAnswer(rule.quesA);
							ruleAnsA = rule.ansA;
						}
					}
					if (rule.quesB) {
						if (FormRules.getDisplayLogic().isQuestionDisplayed(rule.quesB)) {
							ansB = Responses.instance().getAnswer(rule.quesB);
							ruleAnsB = rule.ansB;
						}
					}
//					console.log('key=' + key + ', ruleAnsA=' + ruleAnsA + ', ruleAnsB=' + ruleAnsB + ', ansA=' + ansA + ', ansB=' + ansB)
					prevention   += view.calculateScore(ansA, ruleAnsA, ansB, ruleAnsB, rule.prevention, key);
					rapid        += view.calculateScore(ansA, ruleAnsA, ansB, ruleAnsB, rule.rapid, key);
					transitional += view.calculateScore(ansA, ruleAnsA, ansB, ruleAnsB, rule.transitional, key);
					permanent    += view.calculateScore(ansA, ruleAnsA, ansB, ruleAnsB, rule.permanent, key);
					caseMgmt     += view.calculateScore(ansA, ruleAnsA, ansB, ruleAnsB, rule.caseMgmt, key);
					shortTerm    += view.calculateScore(ansA, ruleAnsA, ansB, ruleAnsB, rule.shortTerm, key);

				}
				console.log('prevention ' + prevention + ', rapid ' + rapid + ', transitional ' + transitional +
						', permanent ' + permanent + ', caseMgmt ' + caseMgmt + ', shortTerm ' + shortTerm);
				var getScore = function(id) {
					if (id === 'Prevention') {
						return prevention;
					} else if (id === 'Rapid') {
						return rapid;
					} else if (id === 'Transitional') {
						return transitional;
					} else if (id === 'Permanent') {
						return permanent;
					} else if (id === 'CaseMgmt') {
						return caseMgmt;
					} else if (id === 'ShortTerm') {
						return shortTerm;
					} else {
						return 0;
					}

				};
				var buildRecommendList = function() {
					var recommendHash = Templates.recommendHash_HDS;

					var maxScore = 0;
					for (var id in recommendHash) {
						maxScore = Math.max(maxScore,getScore(id));
					}
					if (maxScore > 0) {
						for (var id in recommendHash) {
							var score = getScore(id);
							if (getScore(id) === maxScore) {
								var hashVal = recommendHash[id];
								recommendations[id] = {
										title:			hashVal.displayName,
										score:			maxScore,
										text:			convertHTML(hashVal.text),
										id:				id,
										accepted:		false,
										rejected:		false,
										rejectReason:	'',
										rejectText:		''
									};
							}
						}
					}
				};
				var buildRecommendListThresholded = function() {
					var recommendHash = Templates.recommendHash_HDS;
					for (var id in recommendHash) {
						var hashVal = recommendHash[id];
						var score = getScore(id);
						if (score >= hashVal.threshold) {
							recommendations[id] = {
								title:			hashVal.displayName,
								score:			score,
								text:			convertHTML(hashVal.text),
								id:				id,
								accepted:		false,
								rejected:		false,
								rejectReason:	'',
								rejectText:		''
							};
						}
					}
				};
				buildRecommendList();
			},
			calculateScore: function(ansA,ruleAnsA,ansB,ruleAnsB,score,key) {
				var val = 0;
				if (!(isBlank(score) || score===0 || isBlank(ansA) || isBlank(ruleAnsA)) && ansA==ruleAnsA) {
					if (isBlank(ruleAnsB)) {
						console.log('adding ' + score + ' for ansA=' + ansA + ', rule=' + key);
						val = score;
					} else {
						if (!isBlank(ansB) && ansB==ruleAnsB) {
							console.log('adding ' + score + ' for ansA=' + ansA + ' ruleAnsA=' + ruleAnsA +
									' ansB=' + ansB + ' ruleAnsB=' + ruleAnsB + ', rule=' + key);
							val = score;
						}
					}
				}
				return val;
			},
			validate: function() {
				for (var key in recommendations) {
					var recommendation = recommendations[key]
					if (!recommendation.accepted && !recommendation.rejected) {
						var field = $('#' + recommendation.id + '-fieldSet');
						field.find('.acceptOrRejectQuestion').addClass('flagged');
						field.find('.acceptOrRejectError').html('You must accept or reject');
						return false;
					}
					if (recommendation.rejected && !recommendation.rejectReason) {
						var field = $('#' + recommendation.id + '-fieldSet');
						field.find('.reasonsQuestion').addClass('flagged');
						field.find('.reasonsError').html('You must select one or more reasons');
						return false;
					}
				}
				return true;
			},
			messageCallBack: function() {
                if (matchingVeteran) {
                    if (App.getState().okMessagePressed) {
                        var veteranLocalID = HomesVet.getCurrentVet().get('veteranLocalID');
                        matchingVeteran.set('veteranLocalID', veteranLocalID);
                        HomesVet.setCurrentVet(matchingVeteran);
                        matchingVeteran = null;
                        MainView.refresh();
                        view.submit();

    					// HMLS-809 removed this functionality
//                    } else if (App.getState().secondaryActionPressed) {
//                        matchingVeteran = null;
//                        Global.getUtilities().eventAggregator.trigger('message', 'manualFormSubmitReminder');
//                        Global.getUtilities().eventAggregator.invoke('homesVeteranSummary');
//
                    } else if (App.getState().cancelMessagePressed) {
                        matchingVeteran = null;
                    }

                } else {
                    if (App.getState().okMessagePressed) {
                        if (App.getState().isHomesOnline()) {
                            Global.getUtilities().eventAggregator.invoke('homesVeteranSummary');
                        }
                    }
                }
			}
		});
		return RecommendView;
	});
