﻿Imports System.Web
Imports System.Web.Services
Imports System.Web.Services.Protocols
Imports System.DirectoryServices
Imports System.Net
Imports System.Net.Http
Imports System.Configuration.ConfigurationManager
Imports System.Security.Principal
Imports com.agilex.homes.domain.va
Imports com.agilex.homes.dao
Imports System.IO.StreamReader
Imports System.IO
Imports System.Diagnostics
Imports com.agilex.homes.domain
Imports com.agilex.homes.domain.mock

Namespace com.agilex.mbb.homes.webservice

    <WebService(Namespace:="http://mbb.agilex.com/HDSSvc")> _
    <WebServiceBinding(ConformsTo:=WsiProfiles.BasicProfile1_1)> _
    <Global.Microsoft.VisualBasic.CompilerServices.DesignerGenerated()> _
    Public Class HDSSvc
        Inherits System.Web.Services.WebService

        <WebMethod(Description:="login with active directory account", EnableSession:=True)> _
        Public Function LogIn(ByVal UserLogin As String, ByVal UserPassword As String) As va.UserSession
            ' get LDAP path from Web.config
            Dim ldapPath As String = ConfigurationManager.AppSettings("ldapPath")
            Dim entry As New DirectoryEntry(ldapPath, UserLogin, UserPassword)
            Dim searcher As New DirectorySearcher(entry)
            Dim userSession As va.UserSession = Nothing
            Dim cn As String = UserLogin

            If cn.StartsWith("MDM\") Then
                cn = cn.Substring(4)
            End If
            searcher.Filter = String.Format("(SAMAccountName={0})", cn)
            searcher.PropertiesToLoad.Add("CN")
            Try
                Dim result As SearchResult = searcher.FindOne()
                If (result Is Nothing) Then
                    Context.Response.StatusCode = HttpStatusCode.Forbidden
                    Context.Response.StatusDescription = "Jackie StatusDescription."
                    Context.Response.End()
                    Return userSession
                End If

                userSession = New va.UserSession
                userSession.SessionID = 1
                Dim userDao As New UserDao
                userSession.User = userDao.fetchUser(cn)
                userSession.User.Authenticated = True
                userDao.Dispose()
                Session("authenticatedUser") = UserLogin
                Context.Response.StatusCode = HttpStatusCode.OK
            Catch ds As DirectoryServicesCOMException
                Context.Response.StatusCode = HttpStatusCode.Unauthorized
                Context.Response.StatusDescription = "Jackie StatusDescription."
                Context.Response.End()
                Return userSession
            Catch ex As Exception
                Throw ex
            End Try
            Return userSession

        End Function

        Private Function searchVeteran(ByVal username As String, ByVal sessionId As Long, ByVal searchCriteria As mock.VeteranSearchCriteria) As List(Of va.Veteran)
            validateUser(username, sessionId)
            Context.Response.AppendToLog(String.Format(" mock:searchCriteria[SSN={0}; HomesId={1}; FN={2}; LN={3}]", searchCriteria.SSN, searchCriteria.HomesId, searchCriteria.FirstName, searchCriteria.LastName))
            Dim last4 As String = searchCriteria.Last4
            If Not String.IsNullOrEmpty(last4) Then
                Dim ssnBegin As Integer = last4.Length - 4
                searchCriteria.SSN = last4.Substring(ssnBegin)
                searchCriteria.LastName = last4.Substring(0, ssnBegin)
            End If
            Dim myVeteranDao As New VeteranDao
            Dim resultList As List(Of va.Veteran) = myVeteranDao.searchVeteran(searchCriteria)
            myVeteranDao.Dispose()
            Return resultList

        End Function

        <WebMethod(Description:="Search for veteran by the first initial of the last name and last 4 digits of the SSN", EnableSession:=True)> _
        Public Function GetVeteranListByLast4(ByVal UserLogin As String, ByVal SessionID As Long, ByVal Last4 As String) As List(Of va.Veteran)
            If String.IsNullOrEmpty(Last4) Then
                Throw New ArgumentException("Must enter the first initial of the last name and last 4 digits of the SSN.")
            End If
            Dim ssnBegin As Integer = Last4.Length - 4
            If ssnBegin <= 0 Then
                Throw New ArgumentException("Must enter at least one letter for the last name.")
            End If
            Dim searchCriteria As New VeteranSearchCriteria
            searchCriteria.Last4 = Last4
            Return searchVeteran(UserLogin, SessionID, searchCriteria)
        End Function

        <WebMethod(Description:="Search for veteran by the SSN", EnableSession:=True)> _
        Public Function GetVeteranListBySSN(ByVal UserLogin As String, ByVal SessionID As Long, ByVal SSN As String) As List(Of va.Veteran)
            If String.IsNullOrEmpty(SSN) Then
                Throw New ArgumentException("Must enter the SSN.")
            End If
            Dim searchCriteria As New VeteranSearchCriteria
            searchCriteria.SSN = SSN
            Return searchVeteran(UserLogin, SessionID, searchCriteria)
        End Function

        <WebMethod(Description:="Search for veteran by the first or last name", EnableSession:=True)> _
        Public Function GetVeteranListByName(ByVal UserLogin As String, ByVal SessionID As Long, ByVal Name As String) As List(Of va.Veteran)
            If String.IsNullOrEmpty(Name) Then
                Throw New ArgumentException("Must enter a name.")
            End If
            Dim searchCriteria As New VeteranSearchCriteria
            searchCriteria.LastName = Name
            searchCriteria.FirstName = Name
            Return searchVeteran(UserLogin, SessionID, searchCriteria)
        End Function

        <WebMethod(Description:="Search for veteran by the HOMES ID", EnableSession:=True)> _
        Public Function GetVeteran(ByVal UserLogin As String, ByVal SessionID As Long, ByVal VeteranID As Long) As va.Veteran
            Dim searchCriteria As New VeteranSearchCriteria
            searchCriteria.HomesId = VeteranID
            Dim resultList As List(Of va.Veteran) = searchVeteran(UserLogin, SessionID, searchCriteria)
            Dim vet As va.Veteran = Nothing
            If Not resultList Is Nothing Then
                vet = resultList.Item(0)
            End If
            Return vet
        End Function

        <WebMethod(Description:="Add or Edit Veteran", EnableSession:=True)> _
        Public Function AddOrEditVeteran(ByVal veteran As va.Veteran, ByVal sessionID As Long, ByVal userLogin As String) As va.Veteran
            validateUser(userLogin, sessionID)
            Dim myVeteranDao As New VeteranDao
            If veteran.VeteranID = Nothing Then
                veteran.VeteranID = myVeteranDao.addVeteran(veteran)
            Else
                myVeteranDao.updateVeteran(veteran)
            End If
            myVeteranDao.Dispose()
            Context.Response.StatusCode = HttpStatusCode.OK
            Return veteran
        End Function

        Private Sub validateUser(ByVal username As String, ByVal sessionId As Long)
            Dim authSession As String = vbNullString
            Context.Response.AppendToLog(" HDSSvc:request[" & Context.Request.ServerVariables("ALL_RAW") & "]")
            If Session("authenticatedUser") Is Nothing Then
                authSession = "Session is nothing"
            Else
                If (Not Session("authenticatedUser") = username) Then
                    authSession = String.Format(" mock:[inputSession={0}; session object={1}]", sessionId, Session.ToString)
                End If
            End If
            If Not String.IsNullOrEmpty(authSession) Then
                Context.Request.InputStream.Position = 0
                Dim requestSoap As String = New StreamReader(Context.Request.InputStream).ReadToEnd()
                Throw New SoapException(requestSoap, SoapException.ClientFaultCode, New AccessViolationException(authSession))
            End If
        End Sub

        <WebMethod(Description:="Submit completed assessment form. Readonly fields: VISN, FormId.", EnableSession:=True)> _
        Public Function SubmitAssessmentForm(ByVal UserLogin As String, ByVal SessionID As Long, ByVal Assessment As VeteranAssessment) As Long
            validateUser(UserLogin, SessionID)
            Dim assDao As New AssessmentDao
            Dim formId As Long = assDao.submitAssessmentForm(Assessment)
            assDao.Dispose()
            Context.Response.StatusCode = HttpStatusCode.OK
            Return formId
        End Function

        <WebMethod(Description:="Fetch completed assessments for the veteran", EnableSession:=True)> _
        Public Function GetAssessmentForms(ByVal UserLogin As String, ByVal SessionID As Long, ByVal VeteranID As Long, ByVal YearsOfAssessmentHistory As Integer) As List(Of VeteranAssessment)
            validateUser(UserLogin, SessionID)
            Dim assDao As New AssessmentDao
            Dim resultList As List(Of VeteranAssessment) = assDao.getVeteranAssessments(VeteranID, YearsOfAssessmentHistory)
            assDao.Dispose()
            Context.Response.StatusCode = HttpStatusCode.OK
            Return resultList
        End Function
        <WebMethod(Description:="Get assessment form for a given FormID. Readonly fields: VISN, FormId.", EnableSession:=True)> _
        Public Function GetAssessmentForm(ByVal UserLogin As String, ByVal SessionID As Long, ByVal FormID As Long) As VeteranAssessment
            validateUser(UserLogin, SessionID)
            Dim assDao As New AssessmentDao
            Dim form As VeteranAssessment = assDao.getAssessment(FormID)
            assDao.Dispose()
            Context.Response.StatusCode = HttpStatusCode.OK
            Return form
        End Function
        <WebMethod(Description:="Get Veteran lastest assessment. Readonly fields: VISN, FormId.", EnableSession:=True)> _
        Public Function GetVeteranLastestAssessmentForm(ByVal UserLogin As String, ByVal SessionID As Long, ByVal VeteranID As Long) As VeteranAssessment
            validateUser(UserLogin, SessionID)
            Dim assDao As New AssessmentDao
            Dim form As VeteranAssessment = assDao.getVeteranLatestAssessment(VeteranID)
            assDao.Dispose()
            Context.Response.StatusCode = HttpStatusCode.OK
            Return form
        End Function

        <WebMethod(Description:="Submit completed decision support form. Readonly fields: VISN, FormId.", EnableSession:=True)> _
        Public Function SubmitDecisionSupportForm(ByVal UserLogin As String, ByVal SessionID As Long, ByVal DecisionSupport As VeteranDecisionSupportForm) As Long
            validateUser(UserLogin, SessionID)
            Dim dsDao As New DecisionSupportDao
            Dim formId As Long = dsDao.submitForm(DecisionSupport)
            dsDao.Dispose()
            Context.Response.StatusCode = HttpStatusCode.OK
            Return formId
        End Function

        <WebMethod(Description:="Get completed decision support form. Readonly fields: VISN, FormId.", EnableSession:=True)> _
        Public Function GetDecisionSuportForms(ByVal UserLogin As String, ByVal SessionID As Long, ByVal VeteranID As Long, ByVal YearsOfAssessmentHistory As Integer) As List(Of VeteranDecisionSupportForm)
            validateUser(UserLogin, SessionID)
            Dim dsDao As New DecisionSupportDao
            Dim list As List(Of VeteranDecisionSupportForm) = dsDao.getVeteranDSFs(VeteranID, YearsOfAssessmentHistory)
            dsDao.Dispose()
            Context.Response.StatusCode = HttpStatusCode.OK
            Return list
        End Function

        <WebMethod(Description:="Get Decision Support form for a given FormID. Readonly fields: VISN, FormId.", EnableSession:=True)> _
        Public Function GetDecisionSuportForm(ByVal UserLogin As String, ByVal SessionID As Long, ByVal FormID As Long) As VeteranDecisionSupportForm
            validateUser(UserLogin, SessionID)
            Dim dsDao As New DecisionSupportDao
            Dim form As VeteranDecisionSupportForm = dsDao.getDSF(FormID)
            dsDao.Dispose()
            Context.Response.StatusCode = HttpStatusCode.OK
            Return form
        End Function

    End Class
End Namespace