﻿Imports Microsoft.VisualBasic
Imports com.agilex.homes.domain.va
Imports System.Data.SqlClient
Imports System.ArgumentException
Imports System.Data
Namespace com.agilex.homes.dao

    Public Class AssessmentDao
        Inherits Connection

        Private Const Assessment_DateOfAssessment_QId As String = "txtAssesmentDate"
        Private Const ASSESSMENT_FORM_TYPE_ID As Integer = 2

        Private Enum AssessmentColumn As Integer
            FormId = 0
            VetId
            StationNumber
            VISN
            SubmitDate
            AssessmentDate
        End Enum
        Public Function submitAssessmentForm(ByRef form As VeteranAssessment) As Long
            Dim formId As Long
            Const sql As String = "insert into dbo.tbl_VeteranFormsMaster(VeteranID,FormType,Stationnumber,DtEntered,FormActive)" _
                                & " values (@vetId, @formType, @stationNo, @dtEntered, 1);SELECT @@Identity"
            Const sqlResponse As String = "insert into dbo.tbl_VeteranFormResponses(FormID,VeteranID,FormQuestionID,FormQuestionResponse)" _
                                          & " values (@formId, @vetId,@questionId,@response);"
            validateAssessmentForm(form)
            Me.connect()
            Dim transaction As SqlTransaction = MyBase.createTransaction
            Try
                Dim cmd As SqlCommand = MyBase.createSqlCommand(sql)
                cmd.Transaction = transaction
                Dim params As SqlParameterCollection = cmd.Parameters
                params.Add("@vetId", SqlDbType.BigInt).Value = form.VeteranID
                params.Add("@formType", SqlDbType.SmallInt).Value = ASSESSMENT_FORM_TYPE_ID
                params.Add("@stationNo", SqlDbType.VarChar, 50).Value = form.SiteID
                params.Add("@dtEntered", SqlDbType.DateTime).Value = form.SubmitDate
                formId = cmd.ExecuteScalar()

                Dim responseCmd As SqlCommand = MyBase.createSqlCommand(sqlResponse)
                responseCmd.Transaction = transaction
                Dim responseParams As SqlParameterCollection = responseCmd.Parameters
                responseParams.Add("@formId", SqlDbType.BigInt).Value = formId
                responseParams.Add("@vetId", SqlDbType.BigInt).Value = form.VeteranID
                responseParams.Add("@questionId", SqlDbType.VarChar, 50)
                responseParams.Add("@response", SqlDbType.VarChar, 1000)
                Dim responses As List(Of Response) = form.Responses

                For Each response In responses
                    responseParams("@questionId").Value = response.QuestionID
                    responseParams("@response").Value = response.ResponseValue
                    responseCmd.ExecuteNonQuery()
                Next
                transaction.Commit()
            Catch ex As Exception
                transaction.Rollback()
                Throw ex
            Finally
                transaction.Dispose()
            End Try
            Return formId
        End Function

        Private Sub validateAssessmentForm(ByRef form As VeteranAssessment)
            Dim responses As List(Of Response) = form.Responses
            If responses Is Nothing Then
                Throw New ArgumentException("Responses list is null.")
            End If
            If responses.Count = 0 Then
                Throw New ArgumentException("Responses list is empty.")
            End If
            For Each response In responses
                If String.IsNullOrEmpty(response.QuestionID) Then
                    Throw New ArgumentException("Question ID is null or empty.")
                End If
                If String.IsNullOrEmpty(response.ResponseValue) Then
                    Throw New ArgumentException(String.Format("Answer value for question ID={0}, is null or empty.", response.QuestionID))
                End If
            Next
            If form.SubmitDate = Nothing Then
                form.SubmitDate = Date.Today.ToShortDateString
            End If
            Dim dateOfAssessment As Response = responsesHasQuestion(Assessment_DateOfAssessment_QId, responses)
            If dateOfAssessment Is Nothing Then
                Throw New ArgumentException("Date of assessment is null.")
            End If

        End Sub

        Private Function responsesHasQuestion(ByVal questionId As String, ByRef responses As List(Of Response)) As Response
            For Each response In responses
                If response.QuestionId = questionId Then
                    Return response
                End If
            Next
            Return Nothing
        End Function

        Private Function createResponse(ByVal questionId As String, ByVal answerValue As String) As Response
            Dim resp As New Response()
            resp.QuestionId = questionId
            resp.ResponseValue = answerValue
            Return resp
        End Function

        Public Function getVeteranAssessments(ByVal veteranHomesId As Long, ByVal numberOfYearsFromToday As Integer) As List(Of VeteranAssessment)
            Const SQL As String = "SELECT FormID,VeteranID,Stationnumber,VISN,submit_date,assessment_date from dbo.completed_assessment where VeteranID=@vetId and assessment_date > DATEADD(year, @X, GETDATE()) order by assessment_date desc"
            Me.connect()
            Dim cmd As SqlCommand = MyBase.createSqlCommand(SQL)
            Dim params As SqlParameterCollection = cmd.Parameters

            params.AddWithValue("@vetId", veteranHomesId)
            params.AddWithValue("@X", -numberOfYearsFromToday)
            Dim dr As SqlDataReader = cmd.ExecuteReader()
            Return translateToAssessments(dr)
        End Function

        Public Function getAssessment(ByVal formId As Long) As VeteranAssessment
            Const SQL As String = "SELECT FormID,VeteranID,Stationnumber,VISN,submit_date,assessment_date from dbo.completed_assessment where FormID=@formId"
            Me.connect()
            Dim cmd As SqlCommand = MyBase.createSqlCommand(SQL)
            Dim params As SqlParameterCollection = cmd.Parameters

            params.AddWithValue("@formId", formId)
            Dim dr As SqlDataReader = cmd.ExecuteReader()
            Dim resultList As List(Of VeteranAssessment) = translateToAssessments(dr)
            Dim form As VeteranAssessment = Nothing
            If resultList.Count > 0 Then
                form = resultList.Item(0)
                form.Responses = fetchResponses(form.FormID, form.VeteranID)
            End If
            Return form
        End Function

        Public Function getVeteranLatestAssessment(ByVal veteranHomesId As Long) As VeteranAssessment
            Const SQL As String = "SELECT FormID from dbo.latest_assessment where VeteranID=@vetId"
            Me.connect()
            Dim cmd As SqlCommand = MyBase.createSqlCommand(SQL)
            Dim params As SqlParameterCollection = cmd.Parameters
            params.AddWithValue("@vetId", veteranHomesId)
            Dim formId As Long = cmd.ExecuteScalar
            Dim form As VeteranAssessment = Nothing
            If formId = Nothing Then
                Return form
            Else
                Return getAssessment(formId)
            End If
        End Function

        Public Function translateToAssessments(ByRef dr As SqlDataReader) As List(Of VeteranAssessment)

            Dim resultList As New List(Of VeteranAssessment)()
            If dr IsNot Nothing Then
                Do While dr.Read()
                    Dim ass As New VeteranAssessment
                    ass.FormId = Convert.ToInt64(dr.GetValue(AssessmentColumn.FormId))
                    ass.VeteranID = Convert.ToInt64(dr.GetValue(AssessmentColumn.VetId))
                    ass.SubmitDate = dr.GetDateTime(AssessmentColumn.SubmitDate)
                    ass.SiteID = dr.GetString(AssessmentColumn.StationNumber)
                    If Not IsDBNull(dr.GetString(AssessmentColumn.VISN)) Then
                        ass.VISN = Convert.ToInt32(dr.GetString(AssessmentColumn.VISN))
                    End If
                    'ass.Responses = fetchFormResponses(ass.FormID, ass.VeteranID)
                    resultList.Add(ass)
                Loop
            End If
            Return resultList
        End Function
        Private Function fetchResponses(ByVal formId As Long, ByVal vetId As Long) As List(Of Response)
            Const sqlResponse As String = "select FormQuestionID,FormQuestionResponse from dbo.tbl_VeteranFormResponses where FormID=@formId and VeteranID=@vetId;"

            Dim cmd As SqlCommand = MyBase.createSqlCommand(sqlResponse)
            Dim params As SqlParameterCollection = cmd.Parameters
            params.AddWithValue("@formId", formId)
            params.AddWithValue("@vetId", vetId)
            Dim dr As SqlDataReader = cmd.ExecuteReader()
            Dim resultList = New List(Of Response)()
            If dr IsNot Nothing Then
                Do While dr.Read()
                    Dim resp As New Response
                    resp.QuestionId = dr.GetString(0)
                    resp.ResponseValue = dr.GetValue(1)
                    resultList.Add(resp)
                Loop
            End If
            Return resultList
        End Function
    End Class
End Namespace

