﻿Imports Microsoft.VisualBasic
Imports System.Timers
Imports System.Configuration.ConfigurationManager
Imports System.Data.SqlClient
Imports System.Data
Imports System.TimeoutException

Namespace com.agilex.homes.dao

    Public MustInherit Class Connection
        Implements IDisposable

        Private _timedOut As Boolean = False
        Private _timer As Timer
        Private _connection As SqlConnection
        Private _transaction As SqlTransaction

        Sub New()
            _connection = New SqlConnection(ConfigurationManager.ConnectionStrings("homesConnectionString").ConnectionString)
        End Sub
        Public Sub connect()
            If Me._timedOut Then
                Throw New TimeoutException("Connection timeout, please login and try again.")
            End If

            If Me.isAlive Then
                resetTimer()
            Else
                _connection.Open()
                setTimeout(ConfigurationManager.AppSettings("connectionTimeoutInMilliSec"))
            End If
        End Sub

        Public Function createTransaction() As SqlTransaction
            _transaction = _connection.BeginTransaction()
            Return _transaction
        End Function

        Public Function isAlive() As Boolean
            Dim alive As Boolean = True
            If _timedOut Or _connection.State <> ConnectionState.Open Then
                Return (Not alive)
            End If
            Return alive
        End Function

        Private Sub resetTimer()
            If Not _timer Is Nothing Then
                _timer.Stop()
                _timer.Start()
            End If
        End Sub

        Private Sub setTimeout(ByVal timeout As Integer)
            _timer = New Timer(timeout)
            AddHandler _timer.Elapsed, New ElapsedEventHandler(AddressOf Timer_Handler)
            _timer.Enabled = True
        End Sub

        Public Sub Timer_Handler(ByVal sender As Object, ByVal e As ElapsedEventArgs)
            _timedOut = True
            Dispose()
        End Sub

        Public Function createSqlReader(ByVal sql As String) As SqlDataReader
            Dim cmd As New SqlCommand(sql, _connection)
            Return cmd.ExecuteReader()
        End Function

        Public Function createSqlCommand(ByVal sql As String) As SqlCommand
            Return New SqlCommand(sql, _connection)
        End Function
        Public Function createSqlCommand(ByVal sql As String, transaction As SqlTransaction) As SqlCommand
            Return New SqlCommand(sql, _connection, transaction)
        End Function
        Public Overloads Sub Dispose() Implements System.IDisposable.Dispose
            If Not (_timer Is Nothing) Then
                _timer.Stop()
                _timer.Dispose()
            End If
            If Not _transaction Is Nothing Then
                _transaction.Dispose()
            End If

            If _connection.State = ConnectionState.Open Then
                _connection.Close()
            End If
            _connection.Dispose()

            GC.SuppressFinalize(Me)
        End Sub

        Protected Overrides Sub Finalize()
            MyBase.Finalize()
        End Sub
    End Class

End Namespace

