﻿Imports Microsoft.VisualBasic
Imports System.Data.SqlClient
Imports System.ArgumentException
Imports System.Data
Imports com.agilex.homes.domain.va

Namespace com.agilex.homes.dao

    Public Class DecisionSupportDao
        Inherits Connection

        Private Const DATE_OF_ASSESSMENT_Q_ID As String = "txtAssesmentDate"
        Private Const DECISION_SUPPORT_FORM_TYPE_ID As Integer = 23

        Private Enum AssessmentColumn As Integer
            FormId = 0
            VetId
            StationNumber
            VISN
            SubmitDate
            AssessmentDate
            Version
        End Enum
        Public Function submitForm(ByRef form As VeteranDecisionSupportForm) As Long
            Dim formId As Long

            validateForm(form)
            Me.connect()
            Dim transaction As SqlTransaction = MyBase.createTransaction
            Try
                Const FORM_INSERT As String = "insert into dbo.tbl_VeteranFormsMaster(VeteranID,FormType,Stationnumber,DtEntered,FormActive,form_version)" _
                                & " values (@vetId, @formType, @stationNo, @dtEntered, 1, @version);SELECT @@Identity"
                Dim cmd As SqlCommand = MyBase.createSqlCommand(FORM_INSERT, transaction)
                Dim params As SqlParameterCollection = cmd.Parameters
                params.Add("@vetId", SqlDbType.BigInt).Value = form.VeteranID
                params.Add("@formType", SqlDbType.SmallInt).Value = DECISION_SUPPORT_FORM_TYPE_ID
                params.Add("@stationNo", SqlDbType.VarChar, 50).Value = form.SiteID
                params.Add("@dtEntered", SqlDbType.DateTime).Value = form.SubmitDate
                params.Add("@version", SqlDbType.VarChar, 30).Value = form.Version
                formId = cmd.ExecuteScalar()

                Const RESPONSE_INSERT As String = "insert into dbo.tbl_VeteranFormResponses(FormID,VeteranID,FormQuestionID,FormQuestionResponse)" _
                                              & " values (@formId, @vetId,@questionId,@response);"
                cmd = MyBase.createSqlCommand(RESPONSE_INSERT, transaction)
                params = cmd.Parameters
                params.Add("@formId", SqlDbType.BigInt).Value = formId
                params.Add("@vetId", SqlDbType.BigInt).Value = form.VeteranID
                params.Add("@questionId", SqlDbType.VarChar, 50)
                params.Add("@response", SqlDbType.VarChar, 1000)

                Dim responses As List(Of Response) = form.Responses
                For Each response In responses
                    params("@questionId").Value = response.QuestionID
                    params("@response").Value = response.ResponseValue
                    cmd.ExecuteNonQuery()
                Next

                saveAlertDecisions(formId, form.AlertDecisions, transaction)
                saveProgramScores(formId, form.ProgramScores, transaction)
                transaction.Commit()
            Catch ex As Exception
                transaction.Rollback()
                Throw ex
            Finally
                transaction.Dispose()
            End Try
            Return formId
        End Function

        Private Sub saveAlertDecisions(formId As Long, alerts As List(Of AlertDecision), transaction As SqlTransaction)

            Const ALERT_INSERT As String = "insert into dbo.ALERT_DECISION(ALERT_MESSAGE,FormID) values (@alert, @formId);SELECT @@Identity"
            Const ALERT_RESPONSE_INSERT As String = "insert into dbo.ALERT_DECISION_RESPONSE(ALERT_DECISION_ID,FormQuestionID,FormQuestionResponse)" _
                                                    & " values (@alertId, @questionId,@response);"
            If alerts Is Nothing Then
                Return
            End If
            Dim cmd As SqlCommand = MyBase.createSqlCommand(ALERT_INSERT, transaction)
            Dim params As SqlParameterCollection = cmd.Parameters
            params.Add("@formId", SqlDbType.BigInt).Value = formId
            params.Add("@alert", SqlDbType.VarChar, 200)
            For Each alert In alerts
                params("@alert").Value = alert.AlertMessage
                Dim alertId As Long = cmd.ExecuteScalar()

                Dim responses As List(Of Response) = alert.Responses
                Dim childCmd = MyBase.createSqlCommand(ALERT_RESPONSE_INSERT, transaction)
                Dim childParams As SqlParameterCollection = childCmd.Parameters
                childParams.Add("@alertId", SqlDbType.BigInt).Value = alertId
                childParams.Add("@questionId", SqlDbType.VarChar, 50)
                childParams.Add("@response", SqlDbType.VarChar, 1000)
                For Each response In responses
                    childParams("@questionId").Value = response.QuestionId
                    childParams("@response").Value = response.ResponseValue
                    childCmd.ExecuteNonQuery()
                Next
            Next
        End Sub
        Private Sub saveProgramScores(formId As Long, scores As List(Of ProgramScore), transaction As SqlTransaction)

            Const SCORE_INSERT As String = "insert into dbo.PROGRAM_SCORE(SCORE,PROGRAM_TYPE,FormID) values (@score, @type, @formId);SELECT @@Identity"
            Const SCORE_RESPONSE_INSERT As String = "insert into dbo.PROGRAM_SCORE_RESPONSE(PROGRAM_SCORE_ID,FormQuestionID,FormQuestionResponse)" _
                                                    & " values (@scoreId, @questionId,@response);"
            If scores Is Nothing Then
                Return
            End If
            Dim cmd As SqlCommand = MyBase.createSqlCommand(SCORE_INSERT, transaction)
            Dim params As SqlParameterCollection = cmd.Parameters
            params.Add("@formId", SqlDbType.BigInt).Value = formId
            params.Add("@score", SqlDbType.Int)
            params.Add("@type", SqlDbType.VarChar, 30)
            For Each score In scores
                params("@score").Value = score.Score
                params("@type").Value = score.ProgramType
                Dim scoreId As Long = cmd.ExecuteScalar()

                Dim responses As List(Of Response) = score.Responses
                Dim childCmd = MyBase.createSqlCommand(SCORE_RESPONSE_INSERT, transaction)
                Dim childParams As SqlParameterCollection = childCmd.Parameters
                childParams.Add("@scoreId", SqlDbType.BigInt).Value = scoreId
                childParams.Add("@questionId", SqlDbType.VarChar, 50)
                childParams.Add("@response", SqlDbType.VarChar, 1000)
                For Each response In responses
                    childParams("@questionId").Value = response.QuestionId
                    childParams("@response").Value = response.ResponseValue
                    childCmd.ExecuteNonQuery()
                Next
            Next
        End Sub
        Private Sub validateForm(ByRef form As VeteranDecisionSupportForm)
            Dim responses As List(Of Response) = form.Responses
            If responses Is Nothing Then
                Throw New ArgumentException("Responses list is null.")
            End If
            If responses.Count = 0 Then
                Throw New ArgumentException("Responses list is empty.")
            End If
            For Each response In responses
                If String.IsNullOrEmpty(response.QuestionID) Then
                    Throw New ArgumentException("Question ID is null or empty.")
                End If
                If String.IsNullOrEmpty(response.ResponseValue) Then
                    Throw New ArgumentException(String.Format("Answer value for question ID={0}, is null or empty.", response.QuestionID))
                End If
            Next
            If form.SubmitDate = Nothing Then
                form.SubmitDate = Date.Today.ToShortDateString
            End If
            Dim dateOfAssessment As Response = responsesHasQuestion(DATE_OF_ASSESSMENT_Q_ID, responses)
            If dateOfAssessment Is Nothing Then
                Throw New ArgumentException("Date of assessment is null.")
            End If

        End Sub

        Private Function responsesHasQuestion(ByVal questionId As String, ByRef responses As List(Of Response)) As Response
            For Each response In responses
                If response.QuestionID = questionId Then
                    Return response
                End If
            Next
            Return Nothing
        End Function

        Private Function createResponse(ByVal questionId As String, ByVal answerValue As String) As Response
            Dim resp As New Response()
            resp.QuestionID = questionId
            resp.ResponseValue = answerValue
            Return resp
        End Function

        Public Function getVeteranDSFs(ByVal veteranHomesId As Long, ByVal numberOfYearsFromToday As Integer) As List(Of VeteranDecisionSupportForm)
            Const SQL As String = "SELECT FormID,VeteranID,Stationnumber,VISN,submit_date,assessment_date,form_version from dbo.completed_ds_form where VeteranID=@vetId and assessment_date > DATEADD(year, @X, GETDATE()) order by assessment_date desc"
            Me.connect()
            Dim cmd As SqlCommand = MyBase.createSqlCommand(SQL)
            Dim params As SqlParameterCollection = cmd.Parameters
            params.AddWithValue("@vetId", veteranHomesId)
            params.AddWithValue("@X", -numberOfYearsFromToday)
            Dim dr As SqlDataReader = cmd.ExecuteReader()
            Return translateToDSF(dr)
        End Function

        Public Function getDSF(ByVal formId As Long) As VeteranDecisionSupportForm
            Const SQL As String = "SELECT FormID,VeteranID,Stationnumber,VISN,submit_date,assessment_date,form_version from dbo.completed_ds_form where FormID=@formId"
            Me.connect()
            Dim cmd As SqlCommand = MyBase.createSqlCommand(SQL)
            Dim params As SqlParameterCollection = cmd.Parameters
            params.AddWithValue("@formId", formId)
            Dim dr As SqlDataReader = cmd.ExecuteReader()
            Dim resultList As List(Of VeteranDecisionSupportForm) = translateToDSF(dr)
            Dim form As VeteranDecisionSupportForm = Nothing
            If resultList.Count > 0 Then
                form = resultList.Item(0)
                form.Responses = fetchResponses(form.FormID, form.VeteranID)
                form.AlertDecisions = fetchAlertDecisions(form.FormID)
                form.ProgramScores = fetchProgramScores(form.FormID)
            End If
            Return form
        End Function

        Private Function translateToDSF(ByRef dr As SqlDataReader) As List(Of VeteranDecisionSupportForm)

            Dim resultList = New List(Of VeteranDecisionSupportForm)()
            If dr IsNot Nothing Then
                Do While dr.Read()
                    Dim dsForm As New VeteranDecisionSupportForm
                    dsForm.FormId = Convert.ToInt64(dr.GetValue(AssessmentColumn.FormId))
                    dsForm.VeteranID = Convert.ToInt64(dr.GetValue(AssessmentColumn.VetId))
                    dsForm.SubmitDate = dr.GetDateTime(AssessmentColumn.SubmitDate)
                    dsForm.SiteID = dr.GetString(AssessmentColumn.StationNumber)
                    If Not IsDBNull(dr.GetString(AssessmentColumn.Version)) Then
                        dsForm.Version = dr.GetString(AssessmentColumn.Version)
                    End If
                    If Not IsDBNull(dr.GetString(AssessmentColumn.VISN)) Then
                        dsForm.VISN = Convert.ToInt32(dr.GetString(AssessmentColumn.VISN))
                    End If
                    'dsForm.Responses = fetchFormResponses(dsForm.FormID, dsForm.VeteranID)
                    'dsForm.AlertDecisions = fetchAlertDecisions(dsForm.FormID)
                    'dsForm.ProgramScores = fetchProgramScores(dsForm.FormID)
                    resultList.Add(dsForm)
                Loop
            End If
            Return resultList
        End Function

        Private Function fetchResponses(ByVal formId As Long, ByVal vetId As Long) As List(Of Response)
            Const sqlResponse As String = "select FormQuestionID,FormQuestionResponse from dbo.tbl_VeteranFormResponses where FormID=@formId and VeteranID=@vetId;"

            Dim cmd As SqlCommand = MyBase.createSqlCommand(sqlResponse)
            Dim params As SqlParameterCollection = cmd.Parameters
            params.AddWithValue("@formId", formId)
            params.AddWithValue("@vetId", vetId)
            Dim dr As SqlDataReader = cmd.ExecuteReader()
            Dim resultList = New List(Of Response)()
            If dr IsNot Nothing Then
                Do While dr.Read()
                    Dim resp As New Response
                    resp.QuestionId = dr.GetString(0)
                    resp.ResponseValue = dr.GetValue(1)
                    resultList.Add(resp)
                Loop
            End If
            Return resultList
        End Function
        Private Function fetchAlertDecisions(ByVal formId As Long) As List(Of AlertDecision)
            Const SQL As String = "select a.ALERT_DECISION_ID,a.ALERT_MESSAGE,r.FormQuestionID,r.FormQuestionResponse" _
                    & " from dbo.ALERT_DECISION a inner join dbo.ALERT_DECISION_RESPONSE r" _
                    & " on a.ALERT_DECISION_ID = r.ALERT_DECISION_ID where a.FormID = @formId order by a.ALERT_DECISION_ID"
            Dim cmd As SqlCommand = MyBase.createSqlCommand(SQL)
            Dim params As SqlParameterCollection = cmd.Parameters
            params.AddWithValue("@formId", formId)

            Dim dr As SqlDataReader = cmd.ExecuteReader()
            Dim resultList As New List(Of AlertDecision)()
            If dr IsNot Nothing Then
                Dim alertDecisionId As Long = 0
                Dim alert As AlertDecision = Nothing
                Do While dr.Read()
                    If alertDecisionId <> dr.GetInt64(0) Then
                        alert = New AlertDecision
                        alert.AlertMessage = dr.GetString(1)

                        resultList.Add(alert)
                        alertDecisionId = dr.GetInt64(0)
                    End If
                    alert.Responses.Add(New Response(dr.GetString(2), dr.GetString(3)))
                Loop
            End If
            Return resultList
        End Function
        Private Function fetchProgramScores(ByVal formId As Long) As List(Of ProgramScore)
            Const SQL As String = "SELECT a.PROGRAM_SCORE_ID, a.SCORE, r.FormQuestionID, r.FormQuestionResponse" _
                                & " FROM dbo.PROGRAM_SCORE a inner join dbo.PROGRAM_SCORE_RESPONSE r on a.PROGRAM_SCORE_ID = r.PROGRAM_SCORE_ID" _
                                & " where a.FormID = @formId order by a.PROGRAM_SCORE_ID"
            Dim cmd As SqlCommand = MyBase.createSqlCommand(SQL)
            Dim params As SqlParameterCollection = cmd.Parameters
            params.AddWithValue("@formId", formId)

            Dim dr As SqlDataReader = cmd.ExecuteReader()
            Dim resultList As New List(Of ProgramScore)()
            If dr IsNot Nothing Then
                Dim scoreId As Long = 0
                Dim score As ProgramScore = Nothing
                Do While dr.Read()
                    If scoreId <> dr.GetInt64(0) Then
                        score = New ProgramScore
                        score.Score = dr.GetInt32(1)

                        resultList.Add(score)
                        scoreId = dr.GetInt64(0)
                    End If
                    score.Responses.Add(New Response(dr.GetString(2), dr.GetString(3)))
                Loop
            End If
            Return resultList
        End Function
    End Class
End Namespace

