﻿Imports Microsoft.VisualBasic
Imports com.agilex.homes.domain
Imports System.Data.SqlClient
Imports System.ArgumentException
Imports System.Data

Namespace com.agilex.homes.dao

    Public Class VeteranDao
        Inherits Connection

        Private Enum PatientColumn
            HomesIdColumn = 0
            FirstNameColumn
            LastNameColumn
            MiddleInitialColumn
            SSNColumn
            SSNTypeColumn
            DoBColumn
            GenderColumn
            AddressLine1Column
            AddressLine2Column
            AddressLine3Column
            CityColumn
            StateColumn
            ZipColumn
            PrimaryPhoneColumn
            SecondaryPhoneColumn
            EmailColumn
            POCnameColumn
            POCphoneColumn
            POCrelationshipColumn
        End Enum
        Public Function searchVeteran(ByVal searchCriteria As mock.VeteranSearchCriteria) As List(Of va.Veteran)

            Dim sql As String = "select v.VeteranID,FirstName,LastName,MiddleInitial,SSN,RealSSN,DateOfBirth,GENDER" _
                                & ",c.Address1,c.Address2,c.Address3,c.City,c.State,c.Zipcode,c.PrimaryTelNumber,c.SecTelNumber" _
                                & ",c.EmailAddress,c.POCName,c.POCPhone,c.POCRelation FROM dbo.tbl_Veteran v" _
                                & " left join dbo.tbl_VeteranContactInformation c on v.VeteranID = c.VeteranID"
            Dim whereClause As New StringBuilder
            Dim addHomesId As Boolean = False
            Dim addSSN As Boolean = False
            Dim addLN As Boolean = False
            Dim addFN As Boolean = False

            If Not searchCriteria.HomesId = Nothing Then
                whereClause.Append(" where v.VeteranID = @id")
                addHomesId = True
            ElseIf Not String.IsNullOrEmpty(searchCriteria.SSN) Then
                If searchCriteria.SSN.Length = 4 Then
                    If String.IsNullOrEmpty(searchCriteria.LastName) Then
                        Throw New ArgumentException("Must enter at least one letter for the last name.")
                    End If
                    whereClause.Append(" where Right(SSN, 4) = @ssn AND LastName like @ln")
                    addLN = True
                Else
                    If searchCriteria.SSN.Length <> 9 Then
                        Throw New ArgumentException("SSN must be 9 digits.")
                    End If
                    whereClause.Append(" where SSN = @ssn")
                End If
                addSSN = True
            ElseIf Not String.IsNullOrEmpty(searchCriteria.LastName) Or Not String.IsNullOrEmpty(searchCriteria.FirstName) Then
                whereClause.Append(" where")
                If Not String.IsNullOrEmpty(searchCriteria.LastName) Then
                    whereClause.Append(" LastName like @ln")
                    addLN = True
                End If
                If Not String.IsNullOrEmpty(searchCriteria.FirstName) Then
                    If addLN Then
                        whereClause.Append(" OR")
                    End If
                    whereClause.Append(" FirstName like @fn")
                    addFN = True
                End If
            End If

            sql = sql & whereClause.ToString()
            Dim cmd As SqlCommand = Me.createSqlCommand(sql)

            If addHomesId Then
                cmd.Parameters.AddWithValue("@id", searchCriteria.HomesId)
            End If
            If addSSN Then
                cmd.Parameters.AddWithValue("@ssn", searchCriteria.SSN)
            End If
            If addLN Then
                cmd.Parameters.AddWithValue("@ln", String.Format("{0}%", searchCriteria.LastName))
            End If
            If addFN Then
                cmd.Parameters.AddWithValue("@fn", String.Format("{0}%", searchCriteria.FirstName))
            End If
            Me.connect()
            Dim dr As SqlDataReader = cmd.ExecuteReader()
            Return translateToVeteran(dr)

        End Function
        Private Function translateToVeteran(ByRef dr As SqlDataReader) As List(Of va.Veteran)

            Dim resultList = New List(Of va.Veteran)()
            If dr IsNot Nothing Then
                Do While dr.Read()
                    Dim patient = New va.Veteran With {.VeteranID = Convert.ToInt64(dr.GetValue(PatientColumn.HomesIdColumn)),
                                                    .FirstName = dr.GetString(PatientColumn.FirstNameColumn),
                                                    .LastName = dr.GetString(PatientColumn.LastNameColumn),
                                                    .SSN = dr.GetString(PatientColumn.SSNColumn),
                                                    .DateOfBirth = Convert.ToDateTime(dr.GetString(PatientColumn.DoBColumn)),
                                                    .Gender = dr.GetString(PatientColumn.GenderColumn)
                                                   }
                    If Not dr.IsDBNull(PatientColumn.MiddleInitialColumn) Then
                        patient.MiddleInitial = dr.GetString(PatientColumn.MiddleInitialColumn)
                    End If
                    If Not dr.IsDBNull(PatientColumn.AddressLine1Column) Then
                        patient.Address1 = dr.GetString(PatientColumn.AddressLine1Column)
                    End If
                    If Not dr.IsDBNull(PatientColumn.AddressLine2Column) Then
                        patient.Address2 = dr.GetString(PatientColumn.AddressLine2Column)
                    End If
                    If Not dr.IsDBNull(PatientColumn.AddressLine3Column) Then
                        patient.Address3 = dr.GetString(PatientColumn.AddressLine3Column)
                    End If
                    If Not dr.IsDBNull(PatientColumn.CityColumn) Then
                        patient.City = dr.GetString(PatientColumn.CityColumn)
                    End If
                    If Not dr.IsDBNull(PatientColumn.StateColumn) Then
                        patient.State = dr.GetString(PatientColumn.StateColumn)
                    End If
                    If Not dr.IsDBNull(PatientColumn.ZipColumn) Then
                        patient.ZipCode = dr.GetString(PatientColumn.ZipColumn)
                    End If
                    If Not dr.IsDBNull(PatientColumn.PrimaryPhoneColumn) Then
                        patient.TelephonePrimary = dr.GetString(PatientColumn.PrimaryPhoneColumn)
                    End If
                    If Not dr.IsDBNull(PatientColumn.SecondaryPhoneColumn) Then
                        patient.TelephoneSecondary = dr.GetString(PatientColumn.SecondaryPhoneColumn)
                    End If
                    If Not dr.IsDBNull(PatientColumn.EmailColumn) Then
                        patient.Email = dr.GetString(PatientColumn.EmailColumn)
                    End If
                    If Not dr.IsDBNull(PatientColumn.POCnameColumn) Then
                        patient.POCname = dr.GetString(PatientColumn.POCnameColumn)
                    End If
                    If Not dr.IsDBNull(PatientColumn.POCphoneColumn) Then
                        patient.POCPhone = dr.GetString(PatientColumn.POCphoneColumn)
                    End If
                    If Not dr.IsDBNull(PatientColumn.POCrelationshipColumn) Then
                        patient.POCRelationship = dr.GetString(PatientColumn.POCrelationshipColumn)
                    End If
                    resultList.Add(patient)
                Loop

            End If
            Return resultList

        End Function
        Public Function addVeteran(ByRef vet As va.Veteran) As Long

            Const VET_SQL As String = "insert into dbo.tbl_Veteran(FirstName,LastName,MiddleInitial,SSN,DateOfBirth,GENDER,DtEdited,DtAdded)" _
                                    & " values (@fn, @ln, @mi, @ssn, @dob, @gender, @lastActionDate, @createDate);SELECT @@Identity"
            Const CONTACT_SQL As String = "insert into dbo.tbl_VeteranContactInformation(VeteranID,Address1,Address2,Address3,CITY,STATE,Zipcode" _
                                        & ",PrimaryTelNumber,SecTelNumber,EmailAddress,POCName,POCPhone,POCRelation,EditedDate,AddedDate)" _
                                        & " values (@id, @line1, @line2, @line3, @city, @state, @zip, @primPhone, @secondPhone, @email, @pocNm, @pocPhone, @pocRelationship, @lastActionDate, @createDate);"
            Dim vetHomesId As Long
            validateVetInput(vet)

            Me.connect()
            Dim transaction As SqlTransaction = MyBase.createTransaction
            Try
                Dim cmd As SqlCommand = createCommandForVet(vet, VET_SQL)
                cmd.Transaction = transaction
                vetHomesId = cmd.ExecuteScalar()
                vet.VeteranID = vetHomesId

                Dim contactCmd As SqlCommand = createCommandForContact(vet, CONTACT_SQL, True)
                contactCmd.Transaction = transaction
                contactCmd.ExecuteNonQuery()

                transaction.Commit()
            Catch ex As Exception
                transaction.Rollback()
                Throw ex
            Finally
                transaction.Dispose()
            End Try

            Return vetHomesId
        End Function
        Private Function createCommandForVet(ByRef vet As va.Veteran, ByVal sql As String) As SqlCommand
            Dim cmd As SqlCommand = Me.createSqlCommand(sql)
            Dim params As SqlParameterCollection = cmd.Parameters
            If vet.VeteranID > 0 Then
                params.Add("@id", SqlDbType.BigInt).Value = vet.VeteranID
            Else
                If vet.CreationDate = Nothing Then
                    vet.CreationDate = Date.Today
                End If
                params.Add("@createDate", SqlDbType.SmallDateTime).Value = vet.CreationDate
            End If
            params.Add("@fn", SqlDbType.VarChar, 100).Value = vet.FirstName
            params.Add("@ln", SqlDbType.VarChar, 100).Value = vet.LastName
            params.Add("@mi", SqlDbType.VarChar, 25).Value = IIf(String.IsNullOrEmpty(vet.MiddleInitial), DBNull.Value, vet.MiddleInitial)
            params.Add("@ssn", SqlDbType.VarChar, 20).Value = vet.SSN
            params.Add("@dob", SqlDbType.VarChar).Value = vet.DateOfBirth.ToShortDateString
            params.Add("@gender", SqlDbType.VarChar, 30).Value = vet.Gender
            If vet.LastActionDate = Nothing Then
                vet.LastActionDate = DateTime.Today
            End If
            params.Add("@lastActionDate", SqlDbType.SmallDateTime).Value = vet.LastActionDate
            Return cmd
        End Function
        Private Function createCommandForContact(ByRef vet As va.Veteran, ByVal sql As String, ByVal newContact As Boolean) As SqlCommand

            Dim cmd As SqlCommand = Me.createSqlCommand(sql)
            Dim params As SqlParameterCollection = cmd.Parameters
            params.Add("@id", SqlDbType.BigInt).Value = vet.VeteranID
            If newContact Then
                params.Add("@createDate", SqlDbType.SmallDateTime).Value = vet.CreationDate
            End If
            params.Add("@lastActionDate", SqlDbType.SmallDateTime).Value = vet.LastActionDate

            params.Add("@line1", SqlDbType.VarChar, 100).Value = IIf(String.IsNullOrEmpty(vet.Address1), DBNull.Value, vet.Address1)
            params.Add("@line2", SqlDbType.VarChar, 100).Value = IIf(String.IsNullOrEmpty(vet.Address2), DBNull.Value, vet.Address2)
            params.Add("@line3", SqlDbType.VarChar, 100).Value = IIf(String.IsNullOrEmpty(vet.Address3), DBNull.Value, vet.Address3)
            params.Add("@city", SqlDbType.VarChar, 100).Value = IIf(String.IsNullOrEmpty(vet.City), DBNull.Value, vet.City)
            params.Add("@state", SqlDbType.VarChar, 10).Value = IIf(String.IsNullOrEmpty(vet.State), DBNull.Value, vet.State)
            params.Add("@zip", SqlDbType.VarChar, 20).Value = IIf(String.IsNullOrEmpty(vet.ZipCode), DBNull.Value, vet.ZipCode)
            params.Add("@primPhone", SqlDbType.VarChar, 20).Value = IIf(String.IsNullOrEmpty(vet.TelephonePrimary), DBNull.Value, vet.TelephonePrimary)
            params.Add("@secondPhone", SqlDbType.VarChar, 20).Value = IIf(String.IsNullOrEmpty(vet.TelephoneSecondary), DBNull.Value, vet.TelephoneSecondary)
            params.Add("@email", SqlDbType.VarChar, 100).Value = IIf(String.IsNullOrEmpty(vet.Email), DBNull.Value, vet.Email)
            params.Add("@pocNm", SqlDbType.VarChar, 150).Value = IIf(String.IsNullOrEmpty(vet.POCname), DBNull.Value, vet.POCname)
            params.Add("@pocPhone", SqlDbType.VarChar, 20).Value = IIf(String.IsNullOrEmpty(vet.POCPhone), DBNull.Value, vet.POCPhone)
            params.Add("@pocRelationship", SqlDbType.VarChar, 10).Value = IIf(String.IsNullOrEmpty(vet.POCRelationship), DBNull.Value, vet.POCRelationship)
            Return cmd

        End Function
        Public Function updateVeteran(ByRef vet As va.Veteran) As Long
            Const VET_SQL As String = "update dbo.tbl_Veteran set DtEdited = @lastActionDate, FirstName = @fn, LastName = @ln" _
                                & ", MiddleInitial = @mi, SSN = @ssn, DateOfBirth = @dob, GENDER = @gender" _
                                & " where VeteranID = @id"
            Const CONTACT_SQL As String = "update dbo.tbl_VeteranContactInformation set EditedDate = @lastActionDate, Address1 = @line1, Address2 = @line2, Address3 = @line3, CITY = @city, STATE = @state, Zipcode = @zip" _
                                & ", PrimaryTelNumber = @primPhone, SecTelNumber = @secondPhone, EmailAddress = @email, POCName = @pocNm, POCPhone = @pocPhone, POCRelation = @pocRelationship" _
                                & " where VeteranID = @id"
            validateVetInput(vet)
            Me.connect()
            Dim transaction As SqlTransaction = Me.createTransaction
            Try
                Dim cmd As SqlCommand = createCommandForVet(vet, VET_SQL)
                cmd.Transaction = transaction
                cmd.ExecuteNonQuery()

                Dim contactCmd = createCommandForContact(vet, CONTACT_SQL, False)
                contactCmd.Transaction = transaction
                contactCmd.ExecuteNonQuery()

                transaction.Commit()
            Catch ex As Exception
                transaction.Rollback()
                Throw ex
            Finally
                transaction.Dispose()
            End Try

            Return vet.VeteranID
        End Function

        Private Sub validateVetInput(ByRef vet As va.Veteran)
            Dim enumValue As Integer
            If vet.DateOfBirth = Nothing Then
                Throw New ArgumentException("Date of Birth is null.")
            Else
                If vet.DateOfBirth.CompareTo(DateTime.Now) >= 0 Then
                    Throw New ArgumentException("Date of Birth cannot be in the future.")
                End If
            End If

            If Not String.IsNullOrEmpty(vet.POCRelationship) Then
                If Not IsNumeric(vet.POCRelationship) Then
                    Throw New ArgumentException(String.Format("Expected an underlying numeric value of the POCRelationships enumeration, but not '{0}'", vet.POCRelationship))
                End If
                enumValue = Integer.Parse(vet.POCRelationship)
                If Not [Enum].IsDefined(GetType(va.POCRelationships), enumValue) Then
                    Throw New ArgumentException(String.Format("{0} is not an underlying value of the POCRelationships enumeration.", vet.POCRelationship))
                End If
            End If
        End Sub
    End Class
End Namespace

